/* SPDX-FileCopyrightText: © 2024 Decompollaborate */
/* SPDX-License-Identifier: MIT */

#include "rabbitizer.h"

#include <string.h>
#include <stdlib.h>
#include <assert.h>

#include "expected_disasm_utils.h"

#define TEST_ENTRY_C(word, imm, expected) TEST_ENTRY(RABBITIZER_INSTRCAT_R4000ALLEGREX, word, imm, expected, )

const TestEntry test_entries[] = {
    /* vector_zero */
    TEST_ENTRY_C(0xD0060000, NULL, "vzero.s     S000"),
    TEST_ENTRY_C(0xD0060001, NULL, "vzero.s     S010"),
    TEST_ENTRY_C(0xD0060002, NULL, "vzero.s     S020"),
    TEST_ENTRY_C(0xD0060003, NULL, "vzero.s     S030"),
    TEST_ENTRY_C(0xD0060004, NULL, "vzero.s     S100"),
    TEST_ENTRY_C(0xD0060005, NULL, "vzero.s     S110"),
    TEST_ENTRY_C(0xD0060006, NULL, "vzero.s     S120"),
    TEST_ENTRY_C(0xD0060007, NULL, "vzero.s     S130"),
    TEST_ENTRY_C(0xD0060008, NULL, "vzero.s     S200"),
    TEST_ENTRY_C(0xD0060009, NULL, "vzero.s     S210"),
    TEST_ENTRY_C(0xD006000A, NULL, "vzero.s     S220"),
    TEST_ENTRY_C(0xD006000B, NULL, "vzero.s     S230"),
    TEST_ENTRY_C(0xD006000C, NULL, "vzero.s     S300"),
    TEST_ENTRY_C(0xD006000D, NULL, "vzero.s     S310"),
    TEST_ENTRY_C(0xD006000E, NULL, "vzero.s     S320"),
    TEST_ENTRY_C(0xD006000F, NULL, "vzero.s     S330"),
    TEST_ENTRY_C(0xD0060010, NULL, "vzero.s     S400"),
    TEST_ENTRY_C(0xD0060011, NULL, "vzero.s     S410"),
    TEST_ENTRY_C(0xD0060012, NULL, "vzero.s     S420"),
    TEST_ENTRY_C(0xD0060013, NULL, "vzero.s     S430"),
    TEST_ENTRY_C(0xD0060014, NULL, "vzero.s     S500"),
    TEST_ENTRY_C(0xD0060015, NULL, "vzero.s     S510"),
    TEST_ENTRY_C(0xD0060016, NULL, "vzero.s     S520"),
    TEST_ENTRY_C(0xD0060017, NULL, "vzero.s     S530"),
    TEST_ENTRY_C(0xD0060018, NULL, "vzero.s     S600"),
    TEST_ENTRY_C(0xD0060019, NULL, "vzero.s     S610"),
    TEST_ENTRY_C(0xD006001A, NULL, "vzero.s     S620"),
    TEST_ENTRY_C(0xD006001B, NULL, "vzero.s     S630"),
    TEST_ENTRY_C(0xD006001C, NULL, "vzero.s     S700"),
    TEST_ENTRY_C(0xD006001D, NULL, "vzero.s     S710"),
    TEST_ENTRY_C(0xD006001E, NULL, "vzero.s     S720"),
    TEST_ENTRY_C(0xD006001F, NULL, "vzero.s     S730"),
    TEST_ENTRY_C(0xD0060020, NULL, "vzero.s     S001"),
    TEST_ENTRY_C(0xD0060021, NULL, "vzero.s     S011"),
    TEST_ENTRY_C(0xD0060022, NULL, "vzero.s     S021"),
    TEST_ENTRY_C(0xD0060023, NULL, "vzero.s     S031"),
    TEST_ENTRY_C(0xD0060024, NULL, "vzero.s     S101"),
    TEST_ENTRY_C(0xD0060025, NULL, "vzero.s     S111"),
    TEST_ENTRY_C(0xD0060026, NULL, "vzero.s     S121"),
    TEST_ENTRY_C(0xD0060027, NULL, "vzero.s     S131"),
    TEST_ENTRY_C(0xD0060028, NULL, "vzero.s     S201"),
    TEST_ENTRY_C(0xD0060029, NULL, "vzero.s     S211"),
    TEST_ENTRY_C(0xD006002A, NULL, "vzero.s     S221"),
    TEST_ENTRY_C(0xD006002B, NULL, "vzero.s     S231"),
    TEST_ENTRY_C(0xD006002C, NULL, "vzero.s     S301"),
    TEST_ENTRY_C(0xD006002D, NULL, "vzero.s     S311"),
    TEST_ENTRY_C(0xD006002E, NULL, "vzero.s     S321"),
    TEST_ENTRY_C(0xD006002F, NULL, "vzero.s     S331"),
    TEST_ENTRY_C(0xD0060030, NULL, "vzero.s     S401"),
    TEST_ENTRY_C(0xD0060031, NULL, "vzero.s     S411"),
    TEST_ENTRY_C(0xD0060032, NULL, "vzero.s     S421"),
    TEST_ENTRY_C(0xD0060033, NULL, "vzero.s     S431"),
    TEST_ENTRY_C(0xD0060034, NULL, "vzero.s     S501"),
    TEST_ENTRY_C(0xD0060035, NULL, "vzero.s     S511"),
    TEST_ENTRY_C(0xD0060036, NULL, "vzero.s     S521"),
    TEST_ENTRY_C(0xD0060037, NULL, "vzero.s     S531"),
    TEST_ENTRY_C(0xD0060038, NULL, "vzero.s     S601"),
    TEST_ENTRY_C(0xD0060039, NULL, "vzero.s     S611"),
    TEST_ENTRY_C(0xD006003A, NULL, "vzero.s     S621"),
    TEST_ENTRY_C(0xD006003B, NULL, "vzero.s     S631"),
    TEST_ENTRY_C(0xD006003C, NULL, "vzero.s     S701"),
    TEST_ENTRY_C(0xD006003D, NULL, "vzero.s     S711"),
    TEST_ENTRY_C(0xD006003E, NULL, "vzero.s     S721"),
    TEST_ENTRY_C(0xD006003F, NULL, "vzero.s     S731"),
    TEST_ENTRY_C(0xD0060040, NULL, "vzero.s     S002"),
    TEST_ENTRY_C(0xD0060041, NULL, "vzero.s     S012"),
    TEST_ENTRY_C(0xD0060042, NULL, "vzero.s     S022"),
    TEST_ENTRY_C(0xD0060043, NULL, "vzero.s     S032"),
    TEST_ENTRY_C(0xD0060044, NULL, "vzero.s     S102"),
    TEST_ENTRY_C(0xD0060045, NULL, "vzero.s     S112"),
    TEST_ENTRY_C(0xD0060046, NULL, "vzero.s     S122"),
    TEST_ENTRY_C(0xD0060047, NULL, "vzero.s     S132"),
    TEST_ENTRY_C(0xD0060048, NULL, "vzero.s     S202"),
    TEST_ENTRY_C(0xD0060049, NULL, "vzero.s     S212"),
    TEST_ENTRY_C(0xD006004A, NULL, "vzero.s     S222"),
    TEST_ENTRY_C(0xD006004B, NULL, "vzero.s     S232"),
    TEST_ENTRY_C(0xD006004C, NULL, "vzero.s     S302"),
    TEST_ENTRY_C(0xD006004D, NULL, "vzero.s     S312"),
    TEST_ENTRY_C(0xD006004E, NULL, "vzero.s     S322"),
    TEST_ENTRY_C(0xD006004F, NULL, "vzero.s     S332"),
    TEST_ENTRY_C(0xD0060050, NULL, "vzero.s     S402"),
    TEST_ENTRY_C(0xD0060051, NULL, "vzero.s     S412"),
    TEST_ENTRY_C(0xD0060052, NULL, "vzero.s     S422"),
    TEST_ENTRY_C(0xD0060053, NULL, "vzero.s     S432"),
    TEST_ENTRY_C(0xD0060054, NULL, "vzero.s     S502"),
    TEST_ENTRY_C(0xD0060055, NULL, "vzero.s     S512"),
    TEST_ENTRY_C(0xD0060056, NULL, "vzero.s     S522"),
    TEST_ENTRY_C(0xD0060057, NULL, "vzero.s     S532"),
    TEST_ENTRY_C(0xD0060058, NULL, "vzero.s     S602"),
    TEST_ENTRY_C(0xD0060059, NULL, "vzero.s     S612"),
    TEST_ENTRY_C(0xD006005A, NULL, "vzero.s     S622"),
    TEST_ENTRY_C(0xD006005B, NULL, "vzero.s     S632"),
    TEST_ENTRY_C(0xD006005C, NULL, "vzero.s     S702"),
    TEST_ENTRY_C(0xD006005D, NULL, "vzero.s     S712"),
    TEST_ENTRY_C(0xD006005E, NULL, "vzero.s     S722"),
    TEST_ENTRY_C(0xD006005F, NULL, "vzero.s     S732"),
    TEST_ENTRY_C(0xD0060060, NULL, "vzero.s     S003"),
    TEST_ENTRY_C(0xD0060061, NULL, "vzero.s     S013"),
    TEST_ENTRY_C(0xD0060062, NULL, "vzero.s     S023"),
    TEST_ENTRY_C(0xD0060063, NULL, "vzero.s     S033"),
    TEST_ENTRY_C(0xD0060064, NULL, "vzero.s     S103"),
    TEST_ENTRY_C(0xD0060065, NULL, "vzero.s     S113"),
    TEST_ENTRY_C(0xD0060066, NULL, "vzero.s     S123"),
    TEST_ENTRY_C(0xD0060067, NULL, "vzero.s     S133"),
    TEST_ENTRY_C(0xD0060068, NULL, "vzero.s     S203"),
    TEST_ENTRY_C(0xD0060069, NULL, "vzero.s     S213"),
    TEST_ENTRY_C(0xD006006A, NULL, "vzero.s     S223"),
    TEST_ENTRY_C(0xD006006B, NULL, "vzero.s     S233"),
    TEST_ENTRY_C(0xD006006C, NULL, "vzero.s     S303"),
    TEST_ENTRY_C(0xD006006D, NULL, "vzero.s     S313"),
    TEST_ENTRY_C(0xD006006E, NULL, "vzero.s     S323"),
    TEST_ENTRY_C(0xD006006F, NULL, "vzero.s     S333"),
    TEST_ENTRY_C(0xD0060070, NULL, "vzero.s     S403"),
    TEST_ENTRY_C(0xD0060071, NULL, "vzero.s     S413"),
    TEST_ENTRY_C(0xD0060072, NULL, "vzero.s     S423"),
    TEST_ENTRY_C(0xD0060073, NULL, "vzero.s     S433"),
    TEST_ENTRY_C(0xD0060074, NULL, "vzero.s     S503"),
    TEST_ENTRY_C(0xD0060075, NULL, "vzero.s     S513"),
    TEST_ENTRY_C(0xD0060076, NULL, "vzero.s     S523"),
    TEST_ENTRY_C(0xD0060077, NULL, "vzero.s     S533"),
    TEST_ENTRY_C(0xD0060078, NULL, "vzero.s     S603"),
    TEST_ENTRY_C(0xD0060079, NULL, "vzero.s     S613"),
    TEST_ENTRY_C(0xD006007A, NULL, "vzero.s     S623"),
    TEST_ENTRY_C(0xD006007B, NULL, "vzero.s     S633"),
    TEST_ENTRY_C(0xD006007C, NULL, "vzero.s     S703"),
    TEST_ENTRY_C(0xD006007D, NULL, "vzero.s     S713"),
    TEST_ENTRY_C(0xD006007E, NULL, "vzero.s     S723"),
    TEST_ENTRY_C(0xD006007F, NULL, "vzero.s     S733"),
    TEST_ENTRY_C(0xD0060080, NULL, "vzero.p     C000"),
    TEST_ENTRY_C(0xD0060081, NULL, "vzero.p     C010"),
    TEST_ENTRY_C(0xD0060082, NULL, "vzero.p     C020"),
    TEST_ENTRY_C(0xD0060083, NULL, "vzero.p     C030"),
    TEST_ENTRY_C(0xD0060084, NULL, "vzero.p     C100"),
    TEST_ENTRY_C(0xD0060085, NULL, "vzero.p     C110"),
    TEST_ENTRY_C(0xD0060086, NULL, "vzero.p     C120"),
    TEST_ENTRY_C(0xD0060087, NULL, "vzero.p     C130"),
    TEST_ENTRY_C(0xD0060088, NULL, "vzero.p     C200"),
    TEST_ENTRY_C(0xD0060089, NULL, "vzero.p     C210"),
    TEST_ENTRY_C(0xD006008A, NULL, "vzero.p     C220"),
    TEST_ENTRY_C(0xD006008B, NULL, "vzero.p     C230"),
    TEST_ENTRY_C(0xD006008C, NULL, "vzero.p     C300"),
    TEST_ENTRY_C(0xD006008D, NULL, "vzero.p     C310"),
    TEST_ENTRY_C(0xD006008E, NULL, "vzero.p     C320"),
    TEST_ENTRY_C(0xD006008F, NULL, "vzero.p     C330"),
    TEST_ENTRY_C(0xD0060090, NULL, "vzero.p     C400"),
    TEST_ENTRY_C(0xD0060091, NULL, "vzero.p     C410"),
    TEST_ENTRY_C(0xD0060092, NULL, "vzero.p     C420"),
    TEST_ENTRY_C(0xD0060093, NULL, "vzero.p     C430"),
    TEST_ENTRY_C(0xD0060094, NULL, "vzero.p     C500"),
    TEST_ENTRY_C(0xD0060095, NULL, "vzero.p     C510"),
    TEST_ENTRY_C(0xD0060096, NULL, "vzero.p     C520"),
    TEST_ENTRY_C(0xD0060097, NULL, "vzero.p     C530"),
    TEST_ENTRY_C(0xD0060098, NULL, "vzero.p     C600"),
    TEST_ENTRY_C(0xD0060099, NULL, "vzero.p     C610"),
    TEST_ENTRY_C(0xD006009A, NULL, "vzero.p     C620"),
    TEST_ENTRY_C(0xD006009B, NULL, "vzero.p     C630"),
    TEST_ENTRY_C(0xD006009C, NULL, "vzero.p     C700"),
    TEST_ENTRY_C(0xD006009D, NULL, "vzero.p     C710"),
    TEST_ENTRY_C(0xD006009E, NULL, "vzero.p     C720"),
    TEST_ENTRY_C(0xD006009F, NULL, "vzero.p     C730"),
    TEST_ENTRY_C(0xD00600A0, NULL, "vzero.p     R000"),
    TEST_ENTRY_C(0xD00600A1, NULL, "vzero.p     R001"),
    TEST_ENTRY_C(0xD00600A2, NULL, "vzero.p     R002"),
    TEST_ENTRY_C(0xD00600A3, NULL, "vzero.p     R003"),
    TEST_ENTRY_C(0xD00600A4, NULL, "vzero.p     R100"),
    TEST_ENTRY_C(0xD00600A5, NULL, "vzero.p     R101"),
    TEST_ENTRY_C(0xD00600A6, NULL, "vzero.p     R102"),
    TEST_ENTRY_C(0xD00600A7, NULL, "vzero.p     R103"),
    TEST_ENTRY_C(0xD00600A8, NULL, "vzero.p     R200"),
    TEST_ENTRY_C(0xD00600A9, NULL, "vzero.p     R201"),
    TEST_ENTRY_C(0xD00600AA, NULL, "vzero.p     R202"),
    TEST_ENTRY_C(0xD00600AB, NULL, "vzero.p     R203"),
    TEST_ENTRY_C(0xD00600AC, NULL, "vzero.p     R300"),
    TEST_ENTRY_C(0xD00600AD, NULL, "vzero.p     R301"),
    TEST_ENTRY_C(0xD00600AE, NULL, "vzero.p     R302"),
    TEST_ENTRY_C(0xD00600AF, NULL, "vzero.p     R303"),
    TEST_ENTRY_C(0xD00600B0, NULL, "vzero.p     R400"),
    TEST_ENTRY_C(0xD00600B1, NULL, "vzero.p     R401"),
    TEST_ENTRY_C(0xD00600B2, NULL, "vzero.p     R402"),
    TEST_ENTRY_C(0xD00600B3, NULL, "vzero.p     R403"),
    TEST_ENTRY_C(0xD00600B4, NULL, "vzero.p     R500"),
    TEST_ENTRY_C(0xD00600B5, NULL, "vzero.p     R501"),
    TEST_ENTRY_C(0xD00600B6, NULL, "vzero.p     R502"),
    TEST_ENTRY_C(0xD00600B7, NULL, "vzero.p     R503"),
    TEST_ENTRY_C(0xD00600B8, NULL, "vzero.p     R600"),
    TEST_ENTRY_C(0xD00600B9, NULL, "vzero.p     R601"),
    TEST_ENTRY_C(0xD00600BA, NULL, "vzero.p     R602"),
    TEST_ENTRY_C(0xD00600BB, NULL, "vzero.p     R603"),
    TEST_ENTRY_C(0xD00600BC, NULL, "vzero.p     R700"),
    TEST_ENTRY_C(0xD00600BD, NULL, "vzero.p     R701"),
    TEST_ENTRY_C(0xD00600BE, NULL, "vzero.p     R702"),
    TEST_ENTRY_C(0xD00600BF, NULL, "vzero.p     R703"),
    TEST_ENTRY_C(0xD00600C0, NULL, "vzero.p     C002"),
    TEST_ENTRY_C(0xD00600C1, NULL, "vzero.p     C012"),
    TEST_ENTRY_C(0xD00600C2, NULL, "vzero.p     C022"),
    TEST_ENTRY_C(0xD00600C3, NULL, "vzero.p     C032"),
    TEST_ENTRY_C(0xD00600C4, NULL, "vzero.p     C102"),
    TEST_ENTRY_C(0xD00600C5, NULL, "vzero.p     C112"),
    TEST_ENTRY_C(0xD00600C6, NULL, "vzero.p     C122"),
    TEST_ENTRY_C(0xD00600C7, NULL, "vzero.p     C132"),
    TEST_ENTRY_C(0xD00600C8, NULL, "vzero.p     C202"),
    TEST_ENTRY_C(0xD00600C9, NULL, "vzero.p     C212"),
    TEST_ENTRY_C(0xD00600CA, NULL, "vzero.p     C222"),
    TEST_ENTRY_C(0xD00600CB, NULL, "vzero.p     C232"),
    TEST_ENTRY_C(0xD00600CC, NULL, "vzero.p     C302"),
    TEST_ENTRY_C(0xD00600CD, NULL, "vzero.p     C312"),
    TEST_ENTRY_C(0xD00600CE, NULL, "vzero.p     C322"),
    TEST_ENTRY_C(0xD00600CF, NULL, "vzero.p     C332"),
    TEST_ENTRY_C(0xD00600D0, NULL, "vzero.p     C402"),
    TEST_ENTRY_C(0xD00600D1, NULL, "vzero.p     C412"),
    TEST_ENTRY_C(0xD00600D2, NULL, "vzero.p     C422"),
    TEST_ENTRY_C(0xD00600D3, NULL, "vzero.p     C432"),
    TEST_ENTRY_C(0xD00600D4, NULL, "vzero.p     C502"),
    TEST_ENTRY_C(0xD00600D5, NULL, "vzero.p     C512"),
    TEST_ENTRY_C(0xD00600D6, NULL, "vzero.p     C522"),
    TEST_ENTRY_C(0xD00600D7, NULL, "vzero.p     C532"),
    TEST_ENTRY_C(0xD00600D8, NULL, "vzero.p     C602"),
    TEST_ENTRY_C(0xD00600D9, NULL, "vzero.p     C612"),
    TEST_ENTRY_C(0xD00600DA, NULL, "vzero.p     C622"),
    TEST_ENTRY_C(0xD00600DB, NULL, "vzero.p     C632"),
    TEST_ENTRY_C(0xD00600DC, NULL, "vzero.p     C702"),
    TEST_ENTRY_C(0xD00600DD, NULL, "vzero.p     C712"),
    TEST_ENTRY_C(0xD00600DE, NULL, "vzero.p     C722"),
    TEST_ENTRY_C(0xD00600DF, NULL, "vzero.p     C732"),
    TEST_ENTRY_C(0xD00600E0, NULL, "vzero.p     R020"),
    TEST_ENTRY_C(0xD00600E1, NULL, "vzero.p     R021"),
    TEST_ENTRY_C(0xD00600E2, NULL, "vzero.p     R022"),
    TEST_ENTRY_C(0xD00600E3, NULL, "vzero.p     R023"),
    TEST_ENTRY_C(0xD00600E4, NULL, "vzero.p     R120"),
    TEST_ENTRY_C(0xD00600E5, NULL, "vzero.p     R121"),
    TEST_ENTRY_C(0xD00600E6, NULL, "vzero.p     R122"),
    TEST_ENTRY_C(0xD00600E7, NULL, "vzero.p     R123"),
    TEST_ENTRY_C(0xD00600E8, NULL, "vzero.p     R220"),
    TEST_ENTRY_C(0xD00600E9, NULL, "vzero.p     R221"),
    TEST_ENTRY_C(0xD00600EA, NULL, "vzero.p     R222"),
    TEST_ENTRY_C(0xD00600EB, NULL, "vzero.p     R223"),
    TEST_ENTRY_C(0xD00600EC, NULL, "vzero.p     R320"),
    TEST_ENTRY_C(0xD00600ED, NULL, "vzero.p     R321"),
    TEST_ENTRY_C(0xD00600EE, NULL, "vzero.p     R322"),
    TEST_ENTRY_C(0xD00600EF, NULL, "vzero.p     R323"),
    TEST_ENTRY_C(0xD00600F0, NULL, "vzero.p     R420"),
    TEST_ENTRY_C(0xD00600F1, NULL, "vzero.p     R421"),
    TEST_ENTRY_C(0xD00600F2, NULL, "vzero.p     R422"),
    TEST_ENTRY_C(0xD00600F3, NULL, "vzero.p     R423"),
    TEST_ENTRY_C(0xD00600F4, NULL, "vzero.p     R520"),
    TEST_ENTRY_C(0xD00600F5, NULL, "vzero.p     R521"),
    TEST_ENTRY_C(0xD00600F6, NULL, "vzero.p     R522"),
    TEST_ENTRY_C(0xD00600F7, NULL, "vzero.p     R523"),
    TEST_ENTRY_C(0xD00600F8, NULL, "vzero.p     R620"),
    TEST_ENTRY_C(0xD00600F9, NULL, "vzero.p     R621"),
    TEST_ENTRY_C(0xD00600FA, NULL, "vzero.p     R622"),
    TEST_ENTRY_C(0xD00600FB, NULL, "vzero.p     R623"),
    TEST_ENTRY_C(0xD00600FC, NULL, "vzero.p     R720"),
    TEST_ENTRY_C(0xD00600FD, NULL, "vzero.p     R721"),
    TEST_ENTRY_C(0xD00600FE, NULL, "vzero.p     R722"),
    TEST_ENTRY_C(0xD00600FF, NULL, "vzero.p     R723"),
    TEST_ENTRY_C(0xD0068000, NULL, "vzero.t     C000"),
    TEST_ENTRY_C(0xD0068001, NULL, "vzero.t     C010"),
    TEST_ENTRY_C(0xD0068002, NULL, "vzero.t     C020"),
    TEST_ENTRY_C(0xD0068003, NULL, "vzero.t     C030"),
    TEST_ENTRY_C(0xD0068004, NULL, "vzero.t     C100"),
    TEST_ENTRY_C(0xD0068005, NULL, "vzero.t     C110"),
    TEST_ENTRY_C(0xD0068006, NULL, "vzero.t     C120"),
    TEST_ENTRY_C(0xD0068007, NULL, "vzero.t     C130"),
    TEST_ENTRY_C(0xD0068008, NULL, "vzero.t     C200"),
    TEST_ENTRY_C(0xD0068009, NULL, "vzero.t     C210"),
    TEST_ENTRY_C(0xD006800A, NULL, "vzero.t     C220"),
    TEST_ENTRY_C(0xD006800B, NULL, "vzero.t     C230"),
    TEST_ENTRY_C(0xD006800C, NULL, "vzero.t     C300"),
    TEST_ENTRY_C(0xD006800D, NULL, "vzero.t     C310"),
    TEST_ENTRY_C(0xD006800E, NULL, "vzero.t     C320"),
    TEST_ENTRY_C(0xD006800F, NULL, "vzero.t     C330"),
    TEST_ENTRY_C(0xD0068010, NULL, "vzero.t     C400"),
    TEST_ENTRY_C(0xD0068011, NULL, "vzero.t     C410"),
    TEST_ENTRY_C(0xD0068012, NULL, "vzero.t     C420"),
    TEST_ENTRY_C(0xD0068013, NULL, "vzero.t     C430"),
    TEST_ENTRY_C(0xD0068014, NULL, "vzero.t     C500"),
    TEST_ENTRY_C(0xD0068015, NULL, "vzero.t     C510"),
    TEST_ENTRY_C(0xD0068016, NULL, "vzero.t     C520"),
    TEST_ENTRY_C(0xD0068017, NULL, "vzero.t     C530"),
    TEST_ENTRY_C(0xD0068018, NULL, "vzero.t     C600"),
    TEST_ENTRY_C(0xD0068019, NULL, "vzero.t     C610"),
    TEST_ENTRY_C(0xD006801A, NULL, "vzero.t     C620"),
    TEST_ENTRY_C(0xD006801B, NULL, "vzero.t     C630"),
    TEST_ENTRY_C(0xD006801C, NULL, "vzero.t     C700"),
    TEST_ENTRY_C(0xD006801D, NULL, "vzero.t     C710"),
    TEST_ENTRY_C(0xD006801E, NULL, "vzero.t     C720"),
    TEST_ENTRY_C(0xD006801F, NULL, "vzero.t     C730"),
    TEST_ENTRY_C(0xD0068020, NULL, "vzero.t     R000"),
    TEST_ENTRY_C(0xD0068021, NULL, "vzero.t     R001"),
    TEST_ENTRY_C(0xD0068022, NULL, "vzero.t     R002"),
    TEST_ENTRY_C(0xD0068023, NULL, "vzero.t     R003"),
    TEST_ENTRY_C(0xD0068024, NULL, "vzero.t     R100"),
    TEST_ENTRY_C(0xD0068025, NULL, "vzero.t     R101"),
    TEST_ENTRY_C(0xD0068026, NULL, "vzero.t     R102"),
    TEST_ENTRY_C(0xD0068027, NULL, "vzero.t     R103"),
    TEST_ENTRY_C(0xD0068028, NULL, "vzero.t     R200"),
    TEST_ENTRY_C(0xD0068029, NULL, "vzero.t     R201"),
    TEST_ENTRY_C(0xD006802A, NULL, "vzero.t     R202"),
    TEST_ENTRY_C(0xD006802B, NULL, "vzero.t     R203"),
    TEST_ENTRY_C(0xD006802C, NULL, "vzero.t     R300"),
    TEST_ENTRY_C(0xD006802D, NULL, "vzero.t     R301"),
    TEST_ENTRY_C(0xD006802E, NULL, "vzero.t     R302"),
    TEST_ENTRY_C(0xD006802F, NULL, "vzero.t     R303"),
    TEST_ENTRY_C(0xD0068030, NULL, "vzero.t     R400"),
    TEST_ENTRY_C(0xD0068031, NULL, "vzero.t     R401"),
    TEST_ENTRY_C(0xD0068032, NULL, "vzero.t     R402"),
    TEST_ENTRY_C(0xD0068033, NULL, "vzero.t     R403"),
    TEST_ENTRY_C(0xD0068034, NULL, "vzero.t     R500"),
    TEST_ENTRY_C(0xD0068035, NULL, "vzero.t     R501"),
    TEST_ENTRY_C(0xD0068036, NULL, "vzero.t     R502"),
    TEST_ENTRY_C(0xD0068037, NULL, "vzero.t     R503"),
    TEST_ENTRY_C(0xD0068038, NULL, "vzero.t     R600"),
    TEST_ENTRY_C(0xD0068039, NULL, "vzero.t     R601"),
    TEST_ENTRY_C(0xD006803A, NULL, "vzero.t     R602"),
    TEST_ENTRY_C(0xD006803B, NULL, "vzero.t     R603"),
    TEST_ENTRY_C(0xD006803C, NULL, "vzero.t     R700"),
    TEST_ENTRY_C(0xD006803D, NULL, "vzero.t     R701"),
    TEST_ENTRY_C(0xD006803E, NULL, "vzero.t     R702"),
    TEST_ENTRY_C(0xD006803F, NULL, "vzero.t     R703"),
    TEST_ENTRY_C(0xD0068040, NULL, "vzero.t     C001"),
    TEST_ENTRY_C(0xD0068041, NULL, "vzero.t     C011"),
    TEST_ENTRY_C(0xD0068042, NULL, "vzero.t     C021"),
    TEST_ENTRY_C(0xD0068043, NULL, "vzero.t     C031"),
    TEST_ENTRY_C(0xD0068044, NULL, "vzero.t     C101"),
    TEST_ENTRY_C(0xD0068045, NULL, "vzero.t     C111"),
    TEST_ENTRY_C(0xD0068046, NULL, "vzero.t     C121"),
    TEST_ENTRY_C(0xD0068047, NULL, "vzero.t     C131"),
    TEST_ENTRY_C(0xD0068048, NULL, "vzero.t     C201"),
    TEST_ENTRY_C(0xD0068049, NULL, "vzero.t     C211"),
    TEST_ENTRY_C(0xD006804A, NULL, "vzero.t     C221"),
    TEST_ENTRY_C(0xD006804B, NULL, "vzero.t     C231"),
    TEST_ENTRY_C(0xD006804C, NULL, "vzero.t     C301"),
    TEST_ENTRY_C(0xD006804D, NULL, "vzero.t     C311"),
    TEST_ENTRY_C(0xD006804E, NULL, "vzero.t     C321"),
    TEST_ENTRY_C(0xD006804F, NULL, "vzero.t     C331"),
    TEST_ENTRY_C(0xD0068050, NULL, "vzero.t     C401"),
    TEST_ENTRY_C(0xD0068051, NULL, "vzero.t     C411"),
    TEST_ENTRY_C(0xD0068052, NULL, "vzero.t     C421"),
    TEST_ENTRY_C(0xD0068053, NULL, "vzero.t     C431"),
    TEST_ENTRY_C(0xD0068054, NULL, "vzero.t     C501"),
    TEST_ENTRY_C(0xD0068055, NULL, "vzero.t     C511"),
    TEST_ENTRY_C(0xD0068056, NULL, "vzero.t     C521"),
    TEST_ENTRY_C(0xD0068057, NULL, "vzero.t     C531"),
    TEST_ENTRY_C(0xD0068058, NULL, "vzero.t     C601"),
    TEST_ENTRY_C(0xD0068059, NULL, "vzero.t     C611"),
    TEST_ENTRY_C(0xD006805A, NULL, "vzero.t     C621"),
    TEST_ENTRY_C(0xD006805B, NULL, "vzero.t     C631"),
    TEST_ENTRY_C(0xD006805C, NULL, "vzero.t     C701"),
    TEST_ENTRY_C(0xD006805D, NULL, "vzero.t     C711"),
    TEST_ENTRY_C(0xD006805E, NULL, "vzero.t     C721"),
    TEST_ENTRY_C(0xD006805F, NULL, "vzero.t     C731"),
    TEST_ENTRY_C(0xD0068060, NULL, "vzero.t     R010"),
    TEST_ENTRY_C(0xD0068061, NULL, "vzero.t     R011"),
    TEST_ENTRY_C(0xD0068062, NULL, "vzero.t     R012"),
    TEST_ENTRY_C(0xD0068063, NULL, "vzero.t     R013"),
    TEST_ENTRY_C(0xD0068064, NULL, "vzero.t     R110"),
    TEST_ENTRY_C(0xD0068065, NULL, "vzero.t     R111"),
    TEST_ENTRY_C(0xD0068066, NULL, "vzero.t     R112"),
    TEST_ENTRY_C(0xD0068067, NULL, "vzero.t     R113"),
    TEST_ENTRY_C(0xD0068068, NULL, "vzero.t     R210"),
    TEST_ENTRY_C(0xD0068069, NULL, "vzero.t     R211"),
    TEST_ENTRY_C(0xD006806A, NULL, "vzero.t     R212"),
    TEST_ENTRY_C(0xD006806B, NULL, "vzero.t     R213"),
    TEST_ENTRY_C(0xD006806C, NULL, "vzero.t     R310"),
    TEST_ENTRY_C(0xD006806D, NULL, "vzero.t     R311"),
    TEST_ENTRY_C(0xD006806E, NULL, "vzero.t     R312"),
    TEST_ENTRY_C(0xD006806F, NULL, "vzero.t     R313"),
    TEST_ENTRY_C(0xD0068070, NULL, "vzero.t     R410"),
    TEST_ENTRY_C(0xD0068071, NULL, "vzero.t     R411"),
    TEST_ENTRY_C(0xD0068072, NULL, "vzero.t     R412"),
    TEST_ENTRY_C(0xD0068073, NULL, "vzero.t     R413"),
    TEST_ENTRY_C(0xD0068074, NULL, "vzero.t     R510"),
    TEST_ENTRY_C(0xD0068075, NULL, "vzero.t     R511"),
    TEST_ENTRY_C(0xD0068076, NULL, "vzero.t     R512"),
    TEST_ENTRY_C(0xD0068077, NULL, "vzero.t     R513"),
    TEST_ENTRY_C(0xD0068078, NULL, "vzero.t     R610"),
    TEST_ENTRY_C(0xD0068079, NULL, "vzero.t     R611"),
    TEST_ENTRY_C(0xD006807A, NULL, "vzero.t     R612"),
    TEST_ENTRY_C(0xD006807B, NULL, "vzero.t     R613"),
    TEST_ENTRY_C(0xD006807C, NULL, "vzero.t     R710"),
    TEST_ENTRY_C(0xD006807D, NULL, "vzero.t     R711"),
    TEST_ENTRY_C(0xD006807E, NULL, "vzero.t     R712"),
    TEST_ENTRY_C(0xD006807F, NULL, "vzero.t     R713"),
    TEST_ENTRY_C(0xD0068080, NULL, "vzero.q     C000"),
    TEST_ENTRY_C(0xD0068081, NULL, "vzero.q     C010"),
    TEST_ENTRY_C(0xD0068082, NULL, "vzero.q     C020"),
    TEST_ENTRY_C(0xD0068083, NULL, "vzero.q     C030"),
    TEST_ENTRY_C(0xD0068084, NULL, "vzero.q     C100"),
    TEST_ENTRY_C(0xD0068085, NULL, "vzero.q     C110"),
    TEST_ENTRY_C(0xD0068086, NULL, "vzero.q     C120"),
    TEST_ENTRY_C(0xD0068087, NULL, "vzero.q     C130"),
    TEST_ENTRY_C(0xD0068088, NULL, "vzero.q     C200"),
    TEST_ENTRY_C(0xD0068089, NULL, "vzero.q     C210"),
    TEST_ENTRY_C(0xD006808A, NULL, "vzero.q     C220"),
    TEST_ENTRY_C(0xD006808B, NULL, "vzero.q     C230"),
    TEST_ENTRY_C(0xD006808C, NULL, "vzero.q     C300"),
    TEST_ENTRY_C(0xD006808D, NULL, "vzero.q     C310"),
    TEST_ENTRY_C(0xD006808E, NULL, "vzero.q     C320"),
    TEST_ENTRY_C(0xD006808F, NULL, "vzero.q     C330"),
    TEST_ENTRY_C(0xD0068090, NULL, "vzero.q     C400"),
    TEST_ENTRY_C(0xD0068091, NULL, "vzero.q     C410"),
    TEST_ENTRY_C(0xD0068092, NULL, "vzero.q     C420"),
    TEST_ENTRY_C(0xD0068093, NULL, "vzero.q     C430"),
    TEST_ENTRY_C(0xD0068094, NULL, "vzero.q     C500"),
    TEST_ENTRY_C(0xD0068095, NULL, "vzero.q     C510"),
    TEST_ENTRY_C(0xD0068096, NULL, "vzero.q     C520"),
    TEST_ENTRY_C(0xD0068097, NULL, "vzero.q     C530"),
    TEST_ENTRY_C(0xD0068098, NULL, "vzero.q     C600"),
    TEST_ENTRY_C(0xD0068099, NULL, "vzero.q     C610"),
    TEST_ENTRY_C(0xD006809A, NULL, "vzero.q     C620"),
    TEST_ENTRY_C(0xD006809B, NULL, "vzero.q     C630"),
    TEST_ENTRY_C(0xD006809C, NULL, "vzero.q     C700"),
    TEST_ENTRY_C(0xD006809D, NULL, "vzero.q     C710"),
    TEST_ENTRY_C(0xD006809E, NULL, "vzero.q     C720"),
    TEST_ENTRY_C(0xD006809F, NULL, "vzero.q     C730"),
    TEST_ENTRY_C(0xD00680A0, NULL, "vzero.q     R000"),
    TEST_ENTRY_C(0xD00680A1, NULL, "vzero.q     R001"),
    TEST_ENTRY_C(0xD00680A2, NULL, "vzero.q     R002"),
    TEST_ENTRY_C(0xD00680A3, NULL, "vzero.q     R003"),
    TEST_ENTRY_C(0xD00680A4, NULL, "vzero.q     R100"),
    TEST_ENTRY_C(0xD00680A5, NULL, "vzero.q     R101"),
    TEST_ENTRY_C(0xD00680A6, NULL, "vzero.q     R102"),
    TEST_ENTRY_C(0xD00680A7, NULL, "vzero.q     R103"),
    TEST_ENTRY_C(0xD00680A8, NULL, "vzero.q     R200"),
    TEST_ENTRY_C(0xD00680A9, NULL, "vzero.q     R201"),
    TEST_ENTRY_C(0xD00680AA, NULL, "vzero.q     R202"),
    TEST_ENTRY_C(0xD00680AB, NULL, "vzero.q     R203"),
    TEST_ENTRY_C(0xD00680AC, NULL, "vzero.q     R300"),
    TEST_ENTRY_C(0xD00680AD, NULL, "vzero.q     R301"),
    TEST_ENTRY_C(0xD00680AE, NULL, "vzero.q     R302"),
    TEST_ENTRY_C(0xD00680AF, NULL, "vzero.q     R303"),
    TEST_ENTRY_C(0xD00680B0, NULL, "vzero.q     R400"),
    TEST_ENTRY_C(0xD00680B1, NULL, "vzero.q     R401"),
    TEST_ENTRY_C(0xD00680B2, NULL, "vzero.q     R402"),
    TEST_ENTRY_C(0xD00680B3, NULL, "vzero.q     R403"),
    TEST_ENTRY_C(0xD00680B4, NULL, "vzero.q     R500"),
    TEST_ENTRY_C(0xD00680B5, NULL, "vzero.q     R501"),
    TEST_ENTRY_C(0xD00680B6, NULL, "vzero.q     R502"),
    TEST_ENTRY_C(0xD00680B7, NULL, "vzero.q     R503"),
    TEST_ENTRY_C(0xD00680B8, NULL, "vzero.q     R600"),
    TEST_ENTRY_C(0xD00680B9, NULL, "vzero.q     R601"),
    TEST_ENTRY_C(0xD00680BA, NULL, "vzero.q     R602"),
    TEST_ENTRY_C(0xD00680BB, NULL, "vzero.q     R603"),
    TEST_ENTRY_C(0xD00680BC, NULL, "vzero.q     R700"),
    TEST_ENTRY_C(0xD00680BD, NULL, "vzero.q     R701"),
    TEST_ENTRY_C(0xD00680BE, NULL, "vzero.q     R702"),
    TEST_ENTRY_C(0xD00680BF, NULL, "vzero.q     R703"),
    TEST_ENTRY_C(0xD00680C0, NULL, "vzero.q     C002"),
    TEST_ENTRY_C(0xD00680C1, NULL, "vzero.q     C012"),
    TEST_ENTRY_C(0xD00680C2, NULL, "vzero.q     C022"),
    TEST_ENTRY_C(0xD00680C3, NULL, "vzero.q     C032"),
    TEST_ENTRY_C(0xD00680C4, NULL, "vzero.q     C102"),
    TEST_ENTRY_C(0xD00680C5, NULL, "vzero.q     C112"),
    TEST_ENTRY_C(0xD00680C6, NULL, "vzero.q     C122"),
    TEST_ENTRY_C(0xD00680C7, NULL, "vzero.q     C132"),
    TEST_ENTRY_C(0xD00680C8, NULL, "vzero.q     C202"),
    TEST_ENTRY_C(0xD00680C9, NULL, "vzero.q     C212"),
    TEST_ENTRY_C(0xD00680CA, NULL, "vzero.q     C222"),
    TEST_ENTRY_C(0xD00680CB, NULL, "vzero.q     C232"),
    TEST_ENTRY_C(0xD00680CC, NULL, "vzero.q     C302"),
    TEST_ENTRY_C(0xD00680CD, NULL, "vzero.q     C312"),
    TEST_ENTRY_C(0xD00680CE, NULL, "vzero.q     C322"),
    TEST_ENTRY_C(0xD00680CF, NULL, "vzero.q     C332"),
    TEST_ENTRY_C(0xD00680D0, NULL, "vzero.q     C402"),
    TEST_ENTRY_C(0xD00680D1, NULL, "vzero.q     C412"),
    TEST_ENTRY_C(0xD00680D2, NULL, "vzero.q     C422"),
    TEST_ENTRY_C(0xD00680D3, NULL, "vzero.q     C432"),
    TEST_ENTRY_C(0xD00680D4, NULL, "vzero.q     C502"),
    TEST_ENTRY_C(0xD00680D5, NULL, "vzero.q     C512"),
    TEST_ENTRY_C(0xD00680D6, NULL, "vzero.q     C522"),
    TEST_ENTRY_C(0xD00680D7, NULL, "vzero.q     C532"),
    TEST_ENTRY_C(0xD00680D8, NULL, "vzero.q     C602"),
    TEST_ENTRY_C(0xD00680D9, NULL, "vzero.q     C612"),
    TEST_ENTRY_C(0xD00680DA, NULL, "vzero.q     C622"),
    TEST_ENTRY_C(0xD00680DB, NULL, "vzero.q     C632"),
    TEST_ENTRY_C(0xD00680DC, NULL, "vzero.q     C702"),
    TEST_ENTRY_C(0xD00680DD, NULL, "vzero.q     C712"),
    TEST_ENTRY_C(0xD00680DE, NULL, "vzero.q     C722"),
    TEST_ENTRY_C(0xD00680DF, NULL, "vzero.q     C732"),
    TEST_ENTRY_C(0xD00680E0, NULL, "vzero.q     R020"),
    TEST_ENTRY_C(0xD00680E1, NULL, "vzero.q     R021"),
    TEST_ENTRY_C(0xD00680E2, NULL, "vzero.q     R022"),
    TEST_ENTRY_C(0xD00680E3, NULL, "vzero.q     R023"),
    TEST_ENTRY_C(0xD00680E4, NULL, "vzero.q     R120"),
    TEST_ENTRY_C(0xD00680E5, NULL, "vzero.q     R121"),
    TEST_ENTRY_C(0xD00680E6, NULL, "vzero.q     R122"),
    TEST_ENTRY_C(0xD00680E7, NULL, "vzero.q     R123"),
    TEST_ENTRY_C(0xD00680E8, NULL, "vzero.q     R220"),
    TEST_ENTRY_C(0xD00680E9, NULL, "vzero.q     R221"),
    TEST_ENTRY_C(0xD00680EA, NULL, "vzero.q     R222"),
    TEST_ENTRY_C(0xD00680EB, NULL, "vzero.q     R223"),
    TEST_ENTRY_C(0xD00680EC, NULL, "vzero.q     R320"),
    TEST_ENTRY_C(0xD00680ED, NULL, "vzero.q     R321"),
    TEST_ENTRY_C(0xD00680EE, NULL, "vzero.q     R322"),
    TEST_ENTRY_C(0xD00680EF, NULL, "vzero.q     R323"),
    TEST_ENTRY_C(0xD00680F0, NULL, "vzero.q     R420"),
    TEST_ENTRY_C(0xD00680F1, NULL, "vzero.q     R421"),
    TEST_ENTRY_C(0xD00680F2, NULL, "vzero.q     R422"),
    TEST_ENTRY_C(0xD00680F3, NULL, "vzero.q     R423"),
    TEST_ENTRY_C(0xD00680F4, NULL, "vzero.q     R520"),
    TEST_ENTRY_C(0xD00680F5, NULL, "vzero.q     R521"),
    TEST_ENTRY_C(0xD00680F6, NULL, "vzero.q     R522"),
    TEST_ENTRY_C(0xD00680F7, NULL, "vzero.q     R523"),
    TEST_ENTRY_C(0xD00680F8, NULL, "vzero.q     R620"),
    TEST_ENTRY_C(0xD00680F9, NULL, "vzero.q     R621"),
    TEST_ENTRY_C(0xD00680FA, NULL, "vzero.q     R622"),
    TEST_ENTRY_C(0xD00680FB, NULL, "vzero.q     R623"),
    TEST_ENTRY_C(0xD00680FC, NULL, "vzero.q     R720"),
    TEST_ENTRY_C(0xD00680FD, NULL, "vzero.q     R721"),
    TEST_ENTRY_C(0xD00680FE, NULL, "vzero.q     R722"),
    TEST_ENTRY_C(0xD00680FF, NULL, "vzero.q     R723"),

    /* matrix_zero */
    TEST_ENTRY_C(0xF3860080, NULL, "vmzero.p    M000"),
    TEST_ENTRY_C(0xF3860081, NULL, "vmzero.p    M010"),
    TEST_ENTRY_C(0xF3860082, NULL, "vmzero.p    M020"),
    TEST_ENTRY_C(0xF3860083, NULL, "vmzero.p    M030"),
    TEST_ENTRY_C(0xF3860084, NULL, "vmzero.p    M100"),
    TEST_ENTRY_C(0xF3860085, NULL, "vmzero.p    M110"),
    TEST_ENTRY_C(0xF3860086, NULL, "vmzero.p    M120"),
    TEST_ENTRY_C(0xF3860087, NULL, "vmzero.p    M130"),
    TEST_ENTRY_C(0xF3860088, NULL, "vmzero.p    M200"),
    TEST_ENTRY_C(0xF3860089, NULL, "vmzero.p    M210"),
    TEST_ENTRY_C(0xF386008A, NULL, "vmzero.p    M220"),
    TEST_ENTRY_C(0xF386008B, NULL, "vmzero.p    M230"),
    TEST_ENTRY_C(0xF386008C, NULL, "vmzero.p    M300"),
    TEST_ENTRY_C(0xF386008D, NULL, "vmzero.p    M310"),
    TEST_ENTRY_C(0xF386008E, NULL, "vmzero.p    M320"),
    TEST_ENTRY_C(0xF386008F, NULL, "vmzero.p    M330"),
    TEST_ENTRY_C(0xF3860090, NULL, "vmzero.p    M400"),
    TEST_ENTRY_C(0xF3860091, NULL, "vmzero.p    M410"),
    TEST_ENTRY_C(0xF3860092, NULL, "vmzero.p    M420"),
    TEST_ENTRY_C(0xF3860093, NULL, "vmzero.p    M430"),
    TEST_ENTRY_C(0xF3860094, NULL, "vmzero.p    M500"),
    TEST_ENTRY_C(0xF3860095, NULL, "vmzero.p    M510"),
    TEST_ENTRY_C(0xF3860096, NULL, "vmzero.p    M520"),
    TEST_ENTRY_C(0xF3860097, NULL, "vmzero.p    M530"),
    TEST_ENTRY_C(0xF3860098, NULL, "vmzero.p    M600"),
    TEST_ENTRY_C(0xF3860099, NULL, "vmzero.p    M610"),
    TEST_ENTRY_C(0xF386009A, NULL, "vmzero.p    M620"),
    TEST_ENTRY_C(0xF386009B, NULL, "vmzero.p    M630"),
    TEST_ENTRY_C(0xF386009C, NULL, "vmzero.p    M700"),
    TEST_ENTRY_C(0xF386009D, NULL, "vmzero.p    M710"),
    TEST_ENTRY_C(0xF386009E, NULL, "vmzero.p    M720"),
    TEST_ENTRY_C(0xF386009F, NULL, "vmzero.p    M730"),
    TEST_ENTRY_C(0xF38600A0, NULL, "vmzero.p    E000"),
    TEST_ENTRY_C(0xF38600A1, NULL, "vmzero.p    E001"),
    TEST_ENTRY_C(0xF38600A2, NULL, "vmzero.p    E002"),
    TEST_ENTRY_C(0xF38600A3, NULL, "vmzero.p    E003"),
    TEST_ENTRY_C(0xF38600A4, NULL, "vmzero.p    E100"),
    TEST_ENTRY_C(0xF38600A5, NULL, "vmzero.p    E101"),
    TEST_ENTRY_C(0xF38600A6, NULL, "vmzero.p    E102"),
    TEST_ENTRY_C(0xF38600A7, NULL, "vmzero.p    E103"),
    TEST_ENTRY_C(0xF38600A8, NULL, "vmzero.p    E200"),
    TEST_ENTRY_C(0xF38600A9, NULL, "vmzero.p    E201"),
    TEST_ENTRY_C(0xF38600AA, NULL, "vmzero.p    E202"),
    TEST_ENTRY_C(0xF38600AB, NULL, "vmzero.p    E203"),
    TEST_ENTRY_C(0xF38600AC, NULL, "vmzero.p    E300"),
    TEST_ENTRY_C(0xF38600AD, NULL, "vmzero.p    E301"),
    TEST_ENTRY_C(0xF38600AE, NULL, "vmzero.p    E302"),
    TEST_ENTRY_C(0xF38600AF, NULL, "vmzero.p    E303"),
    TEST_ENTRY_C(0xF38600B0, NULL, "vmzero.p    E400"),
    TEST_ENTRY_C(0xF38600B1, NULL, "vmzero.p    E401"),
    TEST_ENTRY_C(0xF38600B2, NULL, "vmzero.p    E402"),
    TEST_ENTRY_C(0xF38600B3, NULL, "vmzero.p    E403"),
    TEST_ENTRY_C(0xF38600B4, NULL, "vmzero.p    E500"),
    TEST_ENTRY_C(0xF38600B5, NULL, "vmzero.p    E501"),
    TEST_ENTRY_C(0xF38600B6, NULL, "vmzero.p    E502"),
    TEST_ENTRY_C(0xF38600B7, NULL, "vmzero.p    E503"),
    TEST_ENTRY_C(0xF38600B8, NULL, "vmzero.p    E600"),
    TEST_ENTRY_C(0xF38600B9, NULL, "vmzero.p    E601"),
    TEST_ENTRY_C(0xF38600BA, NULL, "vmzero.p    E602"),
    TEST_ENTRY_C(0xF38600BB, NULL, "vmzero.p    E603"),
    TEST_ENTRY_C(0xF38600BC, NULL, "vmzero.p    E700"),
    TEST_ENTRY_C(0xF38600BD, NULL, "vmzero.p    E701"),
    TEST_ENTRY_C(0xF38600BE, NULL, "vmzero.p    E702"),
    TEST_ENTRY_C(0xF38600BF, NULL, "vmzero.p    E703"),
    TEST_ENTRY_C(0xF38600C0, NULL, "vmzero.p    M002"),
    TEST_ENTRY_C(0xF38600C1, NULL, "vmzero.p    M012"),
    TEST_ENTRY_C(0xF38600C2, NULL, "vmzero.p    M022"),
    TEST_ENTRY_C(0xF38600C3, NULL, "vmzero.p    M032"),
    TEST_ENTRY_C(0xF38600C4, NULL, "vmzero.p    M102"),
    TEST_ENTRY_C(0xF38600C5, NULL, "vmzero.p    M112"),
    TEST_ENTRY_C(0xF38600C6, NULL, "vmzero.p    M122"),
    TEST_ENTRY_C(0xF38600C7, NULL, "vmzero.p    M132"),
    TEST_ENTRY_C(0xF38600C8, NULL, "vmzero.p    M202"),
    TEST_ENTRY_C(0xF38600C9, NULL, "vmzero.p    M212"),
    TEST_ENTRY_C(0xF38600CA, NULL, "vmzero.p    M222"),
    TEST_ENTRY_C(0xF38600CB, NULL, "vmzero.p    M232"),
    TEST_ENTRY_C(0xF38600CC, NULL, "vmzero.p    M302"),
    TEST_ENTRY_C(0xF38600CD, NULL, "vmzero.p    M312"),
    TEST_ENTRY_C(0xF38600CE, NULL, "vmzero.p    M322"),
    TEST_ENTRY_C(0xF38600CF, NULL, "vmzero.p    M332"),
    TEST_ENTRY_C(0xF38600D0, NULL, "vmzero.p    M402"),
    TEST_ENTRY_C(0xF38600D1, NULL, "vmzero.p    M412"),
    TEST_ENTRY_C(0xF38600D2, NULL, "vmzero.p    M422"),
    TEST_ENTRY_C(0xF38600D3, NULL, "vmzero.p    M432"),
    TEST_ENTRY_C(0xF38600D4, NULL, "vmzero.p    M502"),
    TEST_ENTRY_C(0xF38600D5, NULL, "vmzero.p    M512"),
    TEST_ENTRY_C(0xF38600D6, NULL, "vmzero.p    M522"),
    TEST_ENTRY_C(0xF38600D7, NULL, "vmzero.p    M532"),
    TEST_ENTRY_C(0xF38600D8, NULL, "vmzero.p    M602"),
    TEST_ENTRY_C(0xF38600D9, NULL, "vmzero.p    M612"),
    TEST_ENTRY_C(0xF38600DA, NULL, "vmzero.p    M622"),
    TEST_ENTRY_C(0xF38600DB, NULL, "vmzero.p    M632"),
    TEST_ENTRY_C(0xF38600DC, NULL, "vmzero.p    M702"),
    TEST_ENTRY_C(0xF38600DD, NULL, "vmzero.p    M712"),
    TEST_ENTRY_C(0xF38600DE, NULL, "vmzero.p    M722"),
    TEST_ENTRY_C(0xF38600DF, NULL, "vmzero.p    M732"),
    TEST_ENTRY_C(0xF38600E0, NULL, "vmzero.p    E020"),
    TEST_ENTRY_C(0xF38600E1, NULL, "vmzero.p    E021"),
    TEST_ENTRY_C(0xF38600E2, NULL, "vmzero.p    E022"),
    TEST_ENTRY_C(0xF38600E3, NULL, "vmzero.p    E023"),
    TEST_ENTRY_C(0xF38600E4, NULL, "vmzero.p    E120"),
    TEST_ENTRY_C(0xF38600E5, NULL, "vmzero.p    E121"),
    TEST_ENTRY_C(0xF38600E6, NULL, "vmzero.p    E122"),
    TEST_ENTRY_C(0xF38600E7, NULL, "vmzero.p    E123"),
    TEST_ENTRY_C(0xF38600E8, NULL, "vmzero.p    E220"),
    TEST_ENTRY_C(0xF38600E9, NULL, "vmzero.p    E221"),
    TEST_ENTRY_C(0xF38600EA, NULL, "vmzero.p    E222"),
    TEST_ENTRY_C(0xF38600EB, NULL, "vmzero.p    E223"),
    TEST_ENTRY_C(0xF38600EC, NULL, "vmzero.p    E320"),
    TEST_ENTRY_C(0xF38600ED, NULL, "vmzero.p    E321"),
    TEST_ENTRY_C(0xF38600EE, NULL, "vmzero.p    E322"),
    TEST_ENTRY_C(0xF38600EF, NULL, "vmzero.p    E323"),
    TEST_ENTRY_C(0xF38600F0, NULL, "vmzero.p    E420"),
    TEST_ENTRY_C(0xF38600F1, NULL, "vmzero.p    E421"),
    TEST_ENTRY_C(0xF38600F2, NULL, "vmzero.p    E422"),
    TEST_ENTRY_C(0xF38600F3, NULL, "vmzero.p    E423"),
    TEST_ENTRY_C(0xF38600F4, NULL, "vmzero.p    E520"),
    TEST_ENTRY_C(0xF38600F5, NULL, "vmzero.p    E521"),
    TEST_ENTRY_C(0xF38600F6, NULL, "vmzero.p    E522"),
    TEST_ENTRY_C(0xF38600F7, NULL, "vmzero.p    E523"),
    TEST_ENTRY_C(0xF38600F8, NULL, "vmzero.p    E620"),
    TEST_ENTRY_C(0xF38600F9, NULL, "vmzero.p    E621"),
    TEST_ENTRY_C(0xF38600FA, NULL, "vmzero.p    E622"),
    TEST_ENTRY_C(0xF38600FB, NULL, "vmzero.p    E623"),
    TEST_ENTRY_C(0xF38600FC, NULL, "vmzero.p    E720"),
    TEST_ENTRY_C(0xF38600FD, NULL, "vmzero.p    E721"),
    TEST_ENTRY_C(0xF38600FE, NULL, "vmzero.p    E722"),
    TEST_ENTRY_C(0xF38600FF, NULL, "vmzero.p    E723"),
    TEST_ENTRY_C(0xF3868000, NULL, "vmzero.t    M000"),
    TEST_ENTRY_C(0xF3868001, NULL, "vmzero.t    M010"),
    TEST_ENTRY_C(0xF3868002, NULL, "vmzero.t    M020"),
    TEST_ENTRY_C(0xF3868003, NULL, "vmzero.t    M030"),
    TEST_ENTRY_C(0xF3868004, NULL, "vmzero.t    M100"),
    TEST_ENTRY_C(0xF3868005, NULL, "vmzero.t    M110"),
    TEST_ENTRY_C(0xF3868006, NULL, "vmzero.t    M120"),
    TEST_ENTRY_C(0xF3868007, NULL, "vmzero.t    M130"),
    TEST_ENTRY_C(0xF3868008, NULL, "vmzero.t    M200"),
    TEST_ENTRY_C(0xF3868009, NULL, "vmzero.t    M210"),
    TEST_ENTRY_C(0xF386800A, NULL, "vmzero.t    M220"),
    TEST_ENTRY_C(0xF386800B, NULL, "vmzero.t    M230"),
    TEST_ENTRY_C(0xF386800C, NULL, "vmzero.t    M300"),
    TEST_ENTRY_C(0xF386800D, NULL, "vmzero.t    M310"),
    TEST_ENTRY_C(0xF386800E, NULL, "vmzero.t    M320"),
    TEST_ENTRY_C(0xF386800F, NULL, "vmzero.t    M330"),
    TEST_ENTRY_C(0xF3868010, NULL, "vmzero.t    M400"),
    TEST_ENTRY_C(0xF3868011, NULL, "vmzero.t    M410"),
    TEST_ENTRY_C(0xF3868012, NULL, "vmzero.t    M420"),
    TEST_ENTRY_C(0xF3868013, NULL, "vmzero.t    M430"),
    TEST_ENTRY_C(0xF3868014, NULL, "vmzero.t    M500"),
    TEST_ENTRY_C(0xF3868015, NULL, "vmzero.t    M510"),
    TEST_ENTRY_C(0xF3868016, NULL, "vmzero.t    M520"),
    TEST_ENTRY_C(0xF3868017, NULL, "vmzero.t    M530"),
    TEST_ENTRY_C(0xF3868018, NULL, "vmzero.t    M600"),
    TEST_ENTRY_C(0xF3868019, NULL, "vmzero.t    M610"),
    TEST_ENTRY_C(0xF386801A, NULL, "vmzero.t    M620"),
    TEST_ENTRY_C(0xF386801B, NULL, "vmzero.t    M630"),
    TEST_ENTRY_C(0xF386801C, NULL, "vmzero.t    M700"),
    TEST_ENTRY_C(0xF386801D, NULL, "vmzero.t    M710"),
    TEST_ENTRY_C(0xF386801E, NULL, "vmzero.t    M720"),
    TEST_ENTRY_C(0xF386801F, NULL, "vmzero.t    M730"),
    TEST_ENTRY_C(0xF3868020, NULL, "vmzero.t    E000"),
    TEST_ENTRY_C(0xF3868021, NULL, "vmzero.t    E001"),
    TEST_ENTRY_C(0xF3868022, NULL, "vmzero.t    E002"),
    TEST_ENTRY_C(0xF3868023, NULL, "vmzero.t    E003"),
    TEST_ENTRY_C(0xF3868024, NULL, "vmzero.t    E100"),
    TEST_ENTRY_C(0xF3868025, NULL, "vmzero.t    E101"),
    TEST_ENTRY_C(0xF3868026, NULL, "vmzero.t    E102"),
    TEST_ENTRY_C(0xF3868027, NULL, "vmzero.t    E103"),
    TEST_ENTRY_C(0xF3868028, NULL, "vmzero.t    E200"),
    TEST_ENTRY_C(0xF3868029, NULL, "vmzero.t    E201"),
    TEST_ENTRY_C(0xF386802A, NULL, "vmzero.t    E202"),
    TEST_ENTRY_C(0xF386802B, NULL, "vmzero.t    E203"),
    TEST_ENTRY_C(0xF386802C, NULL, "vmzero.t    E300"),
    TEST_ENTRY_C(0xF386802D, NULL, "vmzero.t    E301"),
    TEST_ENTRY_C(0xF386802E, NULL, "vmzero.t    E302"),
    TEST_ENTRY_C(0xF386802F, NULL, "vmzero.t    E303"),
    TEST_ENTRY_C(0xF3868030, NULL, "vmzero.t    E400"),
    TEST_ENTRY_C(0xF3868031, NULL, "vmzero.t    E401"),
    TEST_ENTRY_C(0xF3868032, NULL, "vmzero.t    E402"),
    TEST_ENTRY_C(0xF3868033, NULL, "vmzero.t    E403"),
    TEST_ENTRY_C(0xF3868034, NULL, "vmzero.t    E500"),
    TEST_ENTRY_C(0xF3868035, NULL, "vmzero.t    E501"),
    TEST_ENTRY_C(0xF3868036, NULL, "vmzero.t    E502"),
    TEST_ENTRY_C(0xF3868037, NULL, "vmzero.t    E503"),
    TEST_ENTRY_C(0xF3868038, NULL, "vmzero.t    E600"),
    TEST_ENTRY_C(0xF3868039, NULL, "vmzero.t    E601"),
    TEST_ENTRY_C(0xF386803A, NULL, "vmzero.t    E602"),
    TEST_ENTRY_C(0xF386803B, NULL, "vmzero.t    E603"),
    TEST_ENTRY_C(0xF386803C, NULL, "vmzero.t    E700"),
    TEST_ENTRY_C(0xF386803D, NULL, "vmzero.t    E701"),
    TEST_ENTRY_C(0xF386803E, NULL, "vmzero.t    E702"),
    TEST_ENTRY_C(0xF386803F, NULL, "vmzero.t    E703"),
    TEST_ENTRY_C(0xF3868040, NULL, "vmzero.t    M001"),
    TEST_ENTRY_C(0xF3868041, NULL, "vmzero.t    M011"),
    TEST_ENTRY_C(0xF3868042, NULL, "vmzero.t    M021"),
    TEST_ENTRY_C(0xF3868043, NULL, "vmzero.t    M031"),
    TEST_ENTRY_C(0xF3868044, NULL, "vmzero.t    M101"),
    TEST_ENTRY_C(0xF3868045, NULL, "vmzero.t    M111"),
    TEST_ENTRY_C(0xF3868046, NULL, "vmzero.t    M121"),
    TEST_ENTRY_C(0xF3868047, NULL, "vmzero.t    M131"),
    TEST_ENTRY_C(0xF3868048, NULL, "vmzero.t    M201"),
    TEST_ENTRY_C(0xF3868049, NULL, "vmzero.t    M211"),
    TEST_ENTRY_C(0xF386804A, NULL, "vmzero.t    M221"),
    TEST_ENTRY_C(0xF386804B, NULL, "vmzero.t    M231"),
    TEST_ENTRY_C(0xF386804C, NULL, "vmzero.t    M301"),
    TEST_ENTRY_C(0xF386804D, NULL, "vmzero.t    M311"),
    TEST_ENTRY_C(0xF386804E, NULL, "vmzero.t    M321"),
    TEST_ENTRY_C(0xF386804F, NULL, "vmzero.t    M331"),
    TEST_ENTRY_C(0xF3868050, NULL, "vmzero.t    M401"),
    TEST_ENTRY_C(0xF3868051, NULL, "vmzero.t    M411"),
    TEST_ENTRY_C(0xF3868052, NULL, "vmzero.t    M421"),
    TEST_ENTRY_C(0xF3868053, NULL, "vmzero.t    M431"),
    TEST_ENTRY_C(0xF3868054, NULL, "vmzero.t    M501"),
    TEST_ENTRY_C(0xF3868055, NULL, "vmzero.t    M511"),
    TEST_ENTRY_C(0xF3868056, NULL, "vmzero.t    M521"),
    TEST_ENTRY_C(0xF3868057, NULL, "vmzero.t    M531"),
    TEST_ENTRY_C(0xF3868058, NULL, "vmzero.t    M601"),
    TEST_ENTRY_C(0xF3868059, NULL, "vmzero.t    M611"),
    TEST_ENTRY_C(0xF386805A, NULL, "vmzero.t    M621"),
    TEST_ENTRY_C(0xF386805B, NULL, "vmzero.t    M631"),
    TEST_ENTRY_C(0xF386805C, NULL, "vmzero.t    M701"),
    TEST_ENTRY_C(0xF386805D, NULL, "vmzero.t    M711"),
    TEST_ENTRY_C(0xF386805E, NULL, "vmzero.t    M721"),
    TEST_ENTRY_C(0xF386805F, NULL, "vmzero.t    M731"),
    TEST_ENTRY_C(0xF3868060, NULL, "vmzero.t    E010"),
    TEST_ENTRY_C(0xF3868061, NULL, "vmzero.t    E011"),
    TEST_ENTRY_C(0xF3868062, NULL, "vmzero.t    E012"),
    TEST_ENTRY_C(0xF3868063, NULL, "vmzero.t    E013"),
    TEST_ENTRY_C(0xF3868064, NULL, "vmzero.t    E110"),
    TEST_ENTRY_C(0xF3868065, NULL, "vmzero.t    E111"),
    TEST_ENTRY_C(0xF3868066, NULL, "vmzero.t    E112"),
    TEST_ENTRY_C(0xF3868067, NULL, "vmzero.t    E113"),
    TEST_ENTRY_C(0xF3868068, NULL, "vmzero.t    E210"),
    TEST_ENTRY_C(0xF3868069, NULL, "vmzero.t    E211"),
    TEST_ENTRY_C(0xF386806A, NULL, "vmzero.t    E212"),
    TEST_ENTRY_C(0xF386806B, NULL, "vmzero.t    E213"),
    TEST_ENTRY_C(0xF386806C, NULL, "vmzero.t    E310"),
    TEST_ENTRY_C(0xF386806D, NULL, "vmzero.t    E311"),
    TEST_ENTRY_C(0xF386806E, NULL, "vmzero.t    E312"),
    TEST_ENTRY_C(0xF386806F, NULL, "vmzero.t    E313"),
    TEST_ENTRY_C(0xF3868070, NULL, "vmzero.t    E410"),
    TEST_ENTRY_C(0xF3868071, NULL, "vmzero.t    E411"),
    TEST_ENTRY_C(0xF3868072, NULL, "vmzero.t    E412"),
    TEST_ENTRY_C(0xF3868073, NULL, "vmzero.t    E413"),
    TEST_ENTRY_C(0xF3868074, NULL, "vmzero.t    E510"),
    TEST_ENTRY_C(0xF3868075, NULL, "vmzero.t    E511"),
    TEST_ENTRY_C(0xF3868076, NULL, "vmzero.t    E512"),
    TEST_ENTRY_C(0xF3868077, NULL, "vmzero.t    E513"),
    TEST_ENTRY_C(0xF3868078, NULL, "vmzero.t    E610"),
    TEST_ENTRY_C(0xF3868079, NULL, "vmzero.t    E611"),
    TEST_ENTRY_C(0xF386807A, NULL, "vmzero.t    E612"),
    TEST_ENTRY_C(0xF386807B, NULL, "vmzero.t    E613"),
    TEST_ENTRY_C(0xF386807C, NULL, "vmzero.t    E710"),
    TEST_ENTRY_C(0xF386807D, NULL, "vmzero.t    E711"),
    TEST_ENTRY_C(0xF386807E, NULL, "vmzero.t    E712"),
    TEST_ENTRY_C(0xF386807F, NULL, "vmzero.t    E713"),
    TEST_ENTRY_C(0xF3868080, NULL, "vmzero.q    M000"),
    TEST_ENTRY_C(0xF3868081, NULL, "vmzero.q    M010"),
    TEST_ENTRY_C(0xF3868082, NULL, "vmzero.q    M020"),
    TEST_ENTRY_C(0xF3868083, NULL, "vmzero.q    M030"),
    TEST_ENTRY_C(0xF3868084, NULL, "vmzero.q    M100"),
    TEST_ENTRY_C(0xF3868085, NULL, "vmzero.q    M110"),
    TEST_ENTRY_C(0xF3868086, NULL, "vmzero.q    M120"),
    TEST_ENTRY_C(0xF3868087, NULL, "vmzero.q    M130"),
    TEST_ENTRY_C(0xF3868088, NULL, "vmzero.q    M200"),
    TEST_ENTRY_C(0xF3868089, NULL, "vmzero.q    M210"),
    TEST_ENTRY_C(0xF386808A, NULL, "vmzero.q    M220"),
    TEST_ENTRY_C(0xF386808B, NULL, "vmzero.q    M230"),
    TEST_ENTRY_C(0xF386808C, NULL, "vmzero.q    M300"),
    TEST_ENTRY_C(0xF386808D, NULL, "vmzero.q    M310"),
    TEST_ENTRY_C(0xF386808E, NULL, "vmzero.q    M320"),
    TEST_ENTRY_C(0xF386808F, NULL, "vmzero.q    M330"),
    TEST_ENTRY_C(0xF3868090, NULL, "vmzero.q    M400"),
    TEST_ENTRY_C(0xF3868091, NULL, "vmzero.q    M410"),
    TEST_ENTRY_C(0xF3868092, NULL, "vmzero.q    M420"),
    TEST_ENTRY_C(0xF3868093, NULL, "vmzero.q    M430"),
    TEST_ENTRY_C(0xF3868094, NULL, "vmzero.q    M500"),
    TEST_ENTRY_C(0xF3868095, NULL, "vmzero.q    M510"),
    TEST_ENTRY_C(0xF3868096, NULL, "vmzero.q    M520"),
    TEST_ENTRY_C(0xF3868097, NULL, "vmzero.q    M530"),
    TEST_ENTRY_C(0xF3868098, NULL, "vmzero.q    M600"),
    TEST_ENTRY_C(0xF3868099, NULL, "vmzero.q    M610"),
    TEST_ENTRY_C(0xF386809A, NULL, "vmzero.q    M620"),
    TEST_ENTRY_C(0xF386809B, NULL, "vmzero.q    M630"),
    TEST_ENTRY_C(0xF386809C, NULL, "vmzero.q    M700"),
    TEST_ENTRY_C(0xF386809D, NULL, "vmzero.q    M710"),
    TEST_ENTRY_C(0xF386809E, NULL, "vmzero.q    M720"),
    TEST_ENTRY_C(0xF386809F, NULL, "vmzero.q    M730"),
    TEST_ENTRY_C(0xF38680A0, NULL, "vmzero.q    E000"),
    TEST_ENTRY_C(0xF38680A1, NULL, "vmzero.q    E001"),
    TEST_ENTRY_C(0xF38680A2, NULL, "vmzero.q    E002"),
    TEST_ENTRY_C(0xF38680A3, NULL, "vmzero.q    E003"),
    TEST_ENTRY_C(0xF38680A4, NULL, "vmzero.q    E100"),
    TEST_ENTRY_C(0xF38680A5, NULL, "vmzero.q    E101"),
    TEST_ENTRY_C(0xF38680A6, NULL, "vmzero.q    E102"),
    TEST_ENTRY_C(0xF38680A7, NULL, "vmzero.q    E103"),
    TEST_ENTRY_C(0xF38680A8, NULL, "vmzero.q    E200"),
    TEST_ENTRY_C(0xF38680A9, NULL, "vmzero.q    E201"),
    TEST_ENTRY_C(0xF38680AA, NULL, "vmzero.q    E202"),
    TEST_ENTRY_C(0xF38680AB, NULL, "vmzero.q    E203"),
    TEST_ENTRY_C(0xF38680AC, NULL, "vmzero.q    E300"),
    TEST_ENTRY_C(0xF38680AD, NULL, "vmzero.q    E301"),
    TEST_ENTRY_C(0xF38680AE, NULL, "vmzero.q    E302"),
    TEST_ENTRY_C(0xF38680AF, NULL, "vmzero.q    E303"),
    TEST_ENTRY_C(0xF38680B0, NULL, "vmzero.q    E400"),
    TEST_ENTRY_C(0xF38680B1, NULL, "vmzero.q    E401"),
    TEST_ENTRY_C(0xF38680B2, NULL, "vmzero.q    E402"),
    TEST_ENTRY_C(0xF38680B3, NULL, "vmzero.q    E403"),
    TEST_ENTRY_C(0xF38680B4, NULL, "vmzero.q    E500"),
    TEST_ENTRY_C(0xF38680B5, NULL, "vmzero.q    E501"),
    TEST_ENTRY_C(0xF38680B6, NULL, "vmzero.q    E502"),
    TEST_ENTRY_C(0xF38680B7, NULL, "vmzero.q    E503"),
    TEST_ENTRY_C(0xF38680B8, NULL, "vmzero.q    E600"),
    TEST_ENTRY_C(0xF38680B9, NULL, "vmzero.q    E601"),
    TEST_ENTRY_C(0xF38680BA, NULL, "vmzero.q    E602"),
    TEST_ENTRY_C(0xF38680BB, NULL, "vmzero.q    E603"),
    TEST_ENTRY_C(0xF38680BC, NULL, "vmzero.q    E700"),
    TEST_ENTRY_C(0xF38680BD, NULL, "vmzero.q    E701"),
    TEST_ENTRY_C(0xF38680BE, NULL, "vmzero.q    E702"),
    TEST_ENTRY_C(0xF38680BF, NULL, "vmzero.q    E703"),
    TEST_ENTRY_C(0xF38680C0, NULL, "vmzero.q    M002"),
    TEST_ENTRY_C(0xF38680C1, NULL, "vmzero.q    M012"),
    TEST_ENTRY_C(0xF38680C2, NULL, "vmzero.q    M022"),
    TEST_ENTRY_C(0xF38680C3, NULL, "vmzero.q    M032"),
    TEST_ENTRY_C(0xF38680C4, NULL, "vmzero.q    M102"),
    TEST_ENTRY_C(0xF38680C5, NULL, "vmzero.q    M112"),
    TEST_ENTRY_C(0xF38680C6, NULL, "vmzero.q    M122"),
    TEST_ENTRY_C(0xF38680C7, NULL, "vmzero.q    M132"),
    TEST_ENTRY_C(0xF38680C8, NULL, "vmzero.q    M202"),
    TEST_ENTRY_C(0xF38680C9, NULL, "vmzero.q    M212"),
    TEST_ENTRY_C(0xF38680CA, NULL, "vmzero.q    M222"),
    TEST_ENTRY_C(0xF38680CB, NULL, "vmzero.q    M232"),
    TEST_ENTRY_C(0xF38680CC, NULL, "vmzero.q    M302"),
    TEST_ENTRY_C(0xF38680CD, NULL, "vmzero.q    M312"),
    TEST_ENTRY_C(0xF38680CE, NULL, "vmzero.q    M322"),
    TEST_ENTRY_C(0xF38680CF, NULL, "vmzero.q    M332"),
    TEST_ENTRY_C(0xF38680D0, NULL, "vmzero.q    M402"),
    TEST_ENTRY_C(0xF38680D1, NULL, "vmzero.q    M412"),
    TEST_ENTRY_C(0xF38680D2, NULL, "vmzero.q    M422"),
    TEST_ENTRY_C(0xF38680D3, NULL, "vmzero.q    M432"),
    TEST_ENTRY_C(0xF38680D4, NULL, "vmzero.q    M502"),
    TEST_ENTRY_C(0xF38680D5, NULL, "vmzero.q    M512"),
    TEST_ENTRY_C(0xF38680D6, NULL, "vmzero.q    M522"),
    TEST_ENTRY_C(0xF38680D7, NULL, "vmzero.q    M532"),
    TEST_ENTRY_C(0xF38680D8, NULL, "vmzero.q    M602"),
    TEST_ENTRY_C(0xF38680D9, NULL, "vmzero.q    M612"),
    TEST_ENTRY_C(0xF38680DA, NULL, "vmzero.q    M622"),
    TEST_ENTRY_C(0xF38680DB, NULL, "vmzero.q    M632"),
    TEST_ENTRY_C(0xF38680DC, NULL, "vmzero.q    M702"),
    TEST_ENTRY_C(0xF38680DD, NULL, "vmzero.q    M712"),
    TEST_ENTRY_C(0xF38680DE, NULL, "vmzero.q    M722"),
    TEST_ENTRY_C(0xF38680DF, NULL, "vmzero.q    M732"),
    TEST_ENTRY_C(0xF38680E0, NULL, "vmzero.q    E020"),
    TEST_ENTRY_C(0xF38680E1, NULL, "vmzero.q    E021"),
    TEST_ENTRY_C(0xF38680E2, NULL, "vmzero.q    E022"),
    TEST_ENTRY_C(0xF38680E3, NULL, "vmzero.q    E023"),
    TEST_ENTRY_C(0xF38680E4, NULL, "vmzero.q    E120"),
    TEST_ENTRY_C(0xF38680E5, NULL, "vmzero.q    E121"),
    TEST_ENTRY_C(0xF38680E6, NULL, "vmzero.q    E122"),
    TEST_ENTRY_C(0xF38680E7, NULL, "vmzero.q    E123"),
    TEST_ENTRY_C(0xF38680E8, NULL, "vmzero.q    E220"),
    TEST_ENTRY_C(0xF38680E9, NULL, "vmzero.q    E221"),
    TEST_ENTRY_C(0xF38680EA, NULL, "vmzero.q    E222"),
    TEST_ENTRY_C(0xF38680EB, NULL, "vmzero.q    E223"),
    TEST_ENTRY_C(0xF38680EC, NULL, "vmzero.q    E320"),
    TEST_ENTRY_C(0xF38680ED, NULL, "vmzero.q    E321"),
    TEST_ENTRY_C(0xF38680EE, NULL, "vmzero.q    E322"),
    TEST_ENTRY_C(0xF38680EF, NULL, "vmzero.q    E323"),
    TEST_ENTRY_C(0xF38680F0, NULL, "vmzero.q    E420"),
    TEST_ENTRY_C(0xF38680F1, NULL, "vmzero.q    E421"),
    TEST_ENTRY_C(0xF38680F2, NULL, "vmzero.q    E422"),
    TEST_ENTRY_C(0xF38680F3, NULL, "vmzero.q    E423"),
    TEST_ENTRY_C(0xF38680F4, NULL, "vmzero.q    E520"),
    TEST_ENTRY_C(0xF38680F5, NULL, "vmzero.q    E521"),
    TEST_ENTRY_C(0xF38680F6, NULL, "vmzero.q    E522"),
    TEST_ENTRY_C(0xF38680F7, NULL, "vmzero.q    E523"),
    TEST_ENTRY_C(0xF38680F8, NULL, "vmzero.q    E620"),
    TEST_ENTRY_C(0xF38680F9, NULL, "vmzero.q    E621"),
    TEST_ENTRY_C(0xF38680FA, NULL, "vmzero.q    E622"),
    TEST_ENTRY_C(0xF38680FB, NULL, "vmzero.q    E623"),
    TEST_ENTRY_C(0xF38680FC, NULL, "vmzero.q    E720"),
    TEST_ENTRY_C(0xF38680FD, NULL, "vmzero.q    E721"),
    TEST_ENTRY_C(0xF38680FE, NULL, "vmzero.q    E722"),
    TEST_ENTRY_C(0xF38680FF, NULL, "vmzero.q    E723"),

    /* branches */
    TEST_ENTRY_C(0x49000002, NULL,              "bvf         0, . + 4 + (0x2 << 2)"),
    TEST_ENTRY_C(0x49050002, NULL,              "bvt         1, . + 4 + (0x2 << 2)"),
    TEST_ENTRY_C(0x490A0002, NULL,              "bvfl        2, . + 4 + (0x2 << 2)"),
    TEST_ENTRY_C(0x490F0002, NULL,              "bvtl        3, . + 4 + (0x2 << 2)"),
    TEST_ENTRY_C(0x49100002, NULL,              "bvf         4, . + 4 + (0x2 << 2)"),
    TEST_ENTRY_C(0x49150002, NULL,              "bvt         5, . + 4 + (0x2 << 2)"),
    TEST_ENTRY_C(0x49000002, "branches_end",    "bvf         0, branches_end"),

    /* load_move_store */
    TEST_ENTRY_C(0xC8800040, NULL, "lv.s        S000, 0x40($a0)"),
    TEST_ENTRY_C(0xC8810040, NULL, "lv.s        S010, 0x40($a0)"),
    TEST_ENTRY_C(0xC8820040, NULL, "lv.s        S020, 0x40($a0)"),
    TEST_ENTRY_C(0xC8830040, NULL, "lv.s        S030, 0x40($a0)"),
    TEST_ENTRY_C(0xC8840040, NULL, "lv.s        S100, 0x40($a0)"),
    TEST_ENTRY_C(0xC8850040, NULL, "lv.s        S110, 0x40($a0)"),
    TEST_ENTRY_C(0xC8860040, NULL, "lv.s        S120, 0x40($a0)"),
    TEST_ENTRY_C(0xC8870040, NULL, "lv.s        S130, 0x40($a0)"),
    TEST_ENTRY_C(0xC8880040, NULL, "lv.s        S200, 0x40($a0)"),
    TEST_ENTRY_C(0xC8890040, NULL, "lv.s        S210, 0x40($a0)"),
    TEST_ENTRY_C(0xC88A0040, NULL, "lv.s        S220, 0x40($a0)"),
    TEST_ENTRY_C(0xC88B0040, NULL, "lv.s        S230, 0x40($a0)"),
    TEST_ENTRY_C(0xC88C0040, NULL, "lv.s        S300, 0x40($a0)"),
    TEST_ENTRY_C(0xC88D0040, NULL, "lv.s        S310, 0x40($a0)"),
    TEST_ENTRY_C(0xC88E0040, NULL, "lv.s        S320, 0x40($a0)"),
    TEST_ENTRY_C(0xC88F0040, NULL, "lv.s        S330, 0x40($a0)"),
    TEST_ENTRY_C(0xC8900040, NULL, "lv.s        S400, 0x40($a0)"),
    TEST_ENTRY_C(0xC8910040, NULL, "lv.s        S410, 0x40($a0)"),
    TEST_ENTRY_C(0xC8920040, NULL, "lv.s        S420, 0x40($a0)"),
    TEST_ENTRY_C(0xC8930040, NULL, "lv.s        S430, 0x40($a0)"),
    TEST_ENTRY_C(0xC8940040, NULL, "lv.s        S500, 0x40($a0)"),
    TEST_ENTRY_C(0xC8950040, NULL, "lv.s        S510, 0x40($a0)"),
    TEST_ENTRY_C(0xC8960040, NULL, "lv.s        S520, 0x40($a0)"),
    TEST_ENTRY_C(0xC8970040, NULL, "lv.s        S530, 0x40($a0)"),
    TEST_ENTRY_C(0xC8980040, NULL, "lv.s        S600, 0x40($a0)"),
    TEST_ENTRY_C(0xC8990040, NULL, "lv.s        S610, 0x40($a0)"),
    TEST_ENTRY_C(0xC89A0040, NULL, "lv.s        S620, 0x40($a0)"),
    TEST_ENTRY_C(0xC89B0040, NULL, "lv.s        S630, 0x40($a0)"),
    TEST_ENTRY_C(0xC89C0040, NULL, "lv.s        S700, 0x40($a0)"),
    TEST_ENTRY_C(0xC89D0040, NULL, "lv.s        S710, 0x40($a0)"),
    TEST_ENTRY_C(0xC89E0040, NULL, "lv.s        S720, 0x40($a0)"),
    TEST_ENTRY_C(0xC89F0040, NULL, "lv.s        S730, 0x40($a0)"),
    TEST_ENTRY_C(0xC8800041, NULL, "lv.s        S001, 0x40($a0)"),
    TEST_ENTRY_C(0xC8810041, NULL, "lv.s        S011, 0x40($a0)"),
    TEST_ENTRY_C(0xC8820041, NULL, "lv.s        S021, 0x40($a0)"),
    TEST_ENTRY_C(0xC8830041, NULL, "lv.s        S031, 0x40($a0)"),
    TEST_ENTRY_C(0xC8840041, NULL, "lv.s        S101, 0x40($a0)"),
    TEST_ENTRY_C(0xC8850041, NULL, "lv.s        S111, 0x40($a0)"),
    TEST_ENTRY_C(0xC8860041, NULL, "lv.s        S121, 0x40($a0)"),
    TEST_ENTRY_C(0xC8870041, NULL, "lv.s        S131, 0x40($a0)"),
    TEST_ENTRY_C(0xC8880041, NULL, "lv.s        S201, 0x40($a0)"),
    TEST_ENTRY_C(0xC8890041, NULL, "lv.s        S211, 0x40($a0)"),
    TEST_ENTRY_C(0xC88A0041, NULL, "lv.s        S221, 0x40($a0)"),
    TEST_ENTRY_C(0xC88B0041, NULL, "lv.s        S231, 0x40($a0)"),
    TEST_ENTRY_C(0xC88C0041, NULL, "lv.s        S301, 0x40($a0)"),
    TEST_ENTRY_C(0xC88D0041, NULL, "lv.s        S311, 0x40($a0)"),
    TEST_ENTRY_C(0xC88E0041, NULL, "lv.s        S321, 0x40($a0)"),
    TEST_ENTRY_C(0xC88F0041, NULL, "lv.s        S331, 0x40($a0)"),
    TEST_ENTRY_C(0xC8900041, NULL, "lv.s        S401, 0x40($a0)"),
    TEST_ENTRY_C(0xC8910041, NULL, "lv.s        S411, 0x40($a0)"),
    TEST_ENTRY_C(0xC8920041, NULL, "lv.s        S421, 0x40($a0)"),
    TEST_ENTRY_C(0xC8930041, NULL, "lv.s        S431, 0x40($a0)"),
    TEST_ENTRY_C(0xC8940041, NULL, "lv.s        S501, 0x40($a0)"),
    TEST_ENTRY_C(0xC8950041, NULL, "lv.s        S511, 0x40($a0)"),
    TEST_ENTRY_C(0xC8960041, NULL, "lv.s        S521, 0x40($a0)"),
    TEST_ENTRY_C(0xC8970041, NULL, "lv.s        S531, 0x40($a0)"),
    TEST_ENTRY_C(0xC8980041, NULL, "lv.s        S601, 0x40($a0)"),
    TEST_ENTRY_C(0xC8990041, NULL, "lv.s        S611, 0x40($a0)"),
    TEST_ENTRY_C(0xC89A0041, NULL, "lv.s        S621, 0x40($a0)"),
    TEST_ENTRY_C(0xC89B0041, NULL, "lv.s        S631, 0x40($a0)"),
    TEST_ENTRY_C(0xC89C0041, NULL, "lv.s        S701, 0x40($a0)"),
    TEST_ENTRY_C(0xC89D0041, NULL, "lv.s        S711, 0x40($a0)"),
    TEST_ENTRY_C(0xC89E0041, NULL, "lv.s        S721, 0x40($a0)"),
    TEST_ENTRY_C(0xC89F0041, NULL, "lv.s        S731, 0x40($a0)"),
    TEST_ENTRY_C(0xC8800042, NULL, "lv.s        S002, 0x40($a0)"),
    TEST_ENTRY_C(0xC8810042, NULL, "lv.s        S012, 0x40($a0)"),
    TEST_ENTRY_C(0xC8820042, NULL, "lv.s        S022, 0x40($a0)"),
    TEST_ENTRY_C(0xC8830042, NULL, "lv.s        S032, 0x40($a0)"),
    TEST_ENTRY_C(0xC8840042, NULL, "lv.s        S102, 0x40($a0)"),
    TEST_ENTRY_C(0xC8850042, NULL, "lv.s        S112, 0x40($a0)"),
    TEST_ENTRY_C(0xC8860042, NULL, "lv.s        S122, 0x40($a0)"),
    TEST_ENTRY_C(0xC8870042, NULL, "lv.s        S132, 0x40($a0)"),
    TEST_ENTRY_C(0xC8880042, NULL, "lv.s        S202, 0x40($a0)"),
    TEST_ENTRY_C(0xC8890042, NULL, "lv.s        S212, 0x40($a0)"),
    TEST_ENTRY_C(0xC88A0042, NULL, "lv.s        S222, 0x40($a0)"),
    TEST_ENTRY_C(0xC88B0042, NULL, "lv.s        S232, 0x40($a0)"),
    TEST_ENTRY_C(0xC88C0042, NULL, "lv.s        S302, 0x40($a0)"),
    TEST_ENTRY_C(0xC88D0042, NULL, "lv.s        S312, 0x40($a0)"),
    TEST_ENTRY_C(0xC88E0042, NULL, "lv.s        S322, 0x40($a0)"),
    TEST_ENTRY_C(0xC88F0042, NULL, "lv.s        S332, 0x40($a0)"),
    TEST_ENTRY_C(0xC8900042, NULL, "lv.s        S402, 0x40($a0)"),
    TEST_ENTRY_C(0xC8910042, NULL, "lv.s        S412, 0x40($a0)"),
    TEST_ENTRY_C(0xC8920042, NULL, "lv.s        S422, 0x40($a0)"),
    TEST_ENTRY_C(0xC8930042, NULL, "lv.s        S432, 0x40($a0)"),
    TEST_ENTRY_C(0xC8940042, NULL, "lv.s        S502, 0x40($a0)"),
    TEST_ENTRY_C(0xC8950042, NULL, "lv.s        S512, 0x40($a0)"),
    TEST_ENTRY_C(0xC8960042, NULL, "lv.s        S522, 0x40($a0)"),
    TEST_ENTRY_C(0xC8970042, NULL, "lv.s        S532, 0x40($a0)"),
    TEST_ENTRY_C(0xC8980042, NULL, "lv.s        S602, 0x40($a0)"),
    TEST_ENTRY_C(0xC8990042, NULL, "lv.s        S612, 0x40($a0)"),
    TEST_ENTRY_C(0xC89A0042, NULL, "lv.s        S622, 0x40($a0)"),
    TEST_ENTRY_C(0xC89B0042, NULL, "lv.s        S632, 0x40($a0)"),
    TEST_ENTRY_C(0xC89C0042, NULL, "lv.s        S702, 0x40($a0)"),
    TEST_ENTRY_C(0xC89D0042, NULL, "lv.s        S712, 0x40($a0)"),
    TEST_ENTRY_C(0xC89E0042, NULL, "lv.s        S722, 0x40($a0)"),
    TEST_ENTRY_C(0xC89F0042, NULL, "lv.s        S732, 0x40($a0)"),
    TEST_ENTRY_C(0xC8800043, NULL, "lv.s        S003, 0x40($a0)"),
    TEST_ENTRY_C(0xC8810043, NULL, "lv.s        S013, 0x40($a0)"),
    TEST_ENTRY_C(0xC8820043, NULL, "lv.s        S023, 0x40($a0)"),
    TEST_ENTRY_C(0xC8830043, NULL, "lv.s        S033, 0x40($a0)"),
    TEST_ENTRY_C(0xC8840043, NULL, "lv.s        S103, 0x40($a0)"),
    TEST_ENTRY_C(0xC8850043, NULL, "lv.s        S113, 0x40($a0)"),
    TEST_ENTRY_C(0xC8860043, NULL, "lv.s        S123, 0x40($a0)"),
    TEST_ENTRY_C(0xC8870043, NULL, "lv.s        S133, 0x40($a0)"),
    TEST_ENTRY_C(0xC8880043, NULL, "lv.s        S203, 0x40($a0)"),
    TEST_ENTRY_C(0xC8890043, NULL, "lv.s        S213, 0x40($a0)"),
    TEST_ENTRY_C(0xC88A0043, NULL, "lv.s        S223, 0x40($a0)"),
    TEST_ENTRY_C(0xC88B0043, NULL, "lv.s        S233, 0x40($a0)"),
    TEST_ENTRY_C(0xC88C0043, NULL, "lv.s        S303, 0x40($a0)"),
    TEST_ENTRY_C(0xC88D0043, NULL, "lv.s        S313, 0x40($a0)"),
    TEST_ENTRY_C(0xC88E0043, NULL, "lv.s        S323, 0x40($a0)"),
    TEST_ENTRY_C(0xC88F0043, NULL, "lv.s        S333, 0x40($a0)"),
    TEST_ENTRY_C(0xC8900043, NULL, "lv.s        S403, 0x40($a0)"),
    TEST_ENTRY_C(0xC8910043, NULL, "lv.s        S413, 0x40($a0)"),
    TEST_ENTRY_C(0xC8920043, NULL, "lv.s        S423, 0x40($a0)"),
    TEST_ENTRY_C(0xC8930043, NULL, "lv.s        S433, 0x40($a0)"),
    TEST_ENTRY_C(0xC8940043, NULL, "lv.s        S503, 0x40($a0)"),
    TEST_ENTRY_C(0xC8950043, NULL, "lv.s        S513, 0x40($a0)"),
    TEST_ENTRY_C(0xC8960043, NULL, "lv.s        S523, 0x40($a0)"),
    TEST_ENTRY_C(0xC8970043, NULL, "lv.s        S533, 0x40($a0)"),
    TEST_ENTRY_C(0xC8980043, NULL, "lv.s        S603, 0x40($a0)"),
    TEST_ENTRY_C(0xC8990043, NULL, "lv.s        S613, 0x40($a0)"),
    TEST_ENTRY_C(0xC89A0043, NULL, "lv.s        S623, 0x40($a0)"),
    TEST_ENTRY_C(0xC89B0043, NULL, "lv.s        S633, 0x40($a0)"),
    TEST_ENTRY_C(0xC89C0043, NULL, "lv.s        S703, 0x40($a0)"),
    TEST_ENTRY_C(0xC89D0043, NULL, "lv.s        S713, 0x40($a0)"),
    TEST_ENTRY_C(0xC89E0043, NULL, "lv.s        S723, 0x40($a0)"),
    TEST_ENTRY_C(0xC89F0043, NULL, "lv.s        S733, 0x40($a0)"),
    TEST_ENTRY_C(0xD8800040, NULL, "lv.q        C000, 0x40($a0)"),
    TEST_ENTRY_C(0xD8810040, NULL, "lv.q        C010, 0x40($a0)"),
    TEST_ENTRY_C(0xD8820040, NULL, "lv.q        C020, 0x40($a0)"),
    TEST_ENTRY_C(0xD8830040, NULL, "lv.q        C030, 0x40($a0)"),
    TEST_ENTRY_C(0xD8840040, NULL, "lv.q        C100, 0x40($a0)"),
    TEST_ENTRY_C(0xD8850040, NULL, "lv.q        C110, 0x40($a0)"),
    TEST_ENTRY_C(0xD8860040, NULL, "lv.q        C120, 0x40($a0)"),
    TEST_ENTRY_C(0xD8870040, NULL, "lv.q        C130, 0x40($a0)"),
    TEST_ENTRY_C(0xD8880040, NULL, "lv.q        C200, 0x40($a0)"),
    TEST_ENTRY_C(0xD8890040, NULL, "lv.q        C210, 0x40($a0)"),
    TEST_ENTRY_C(0xD88A0040, NULL, "lv.q        C220, 0x40($a0)"),
    TEST_ENTRY_C(0xD88B0040, NULL, "lv.q        C230, 0x40($a0)"),
    TEST_ENTRY_C(0xD88C0040, NULL, "lv.q        C300, 0x40($a0)"),
    TEST_ENTRY_C(0xD88D0040, NULL, "lv.q        C310, 0x40($a0)"),
    TEST_ENTRY_C(0xD88E0040, NULL, "lv.q        C320, 0x40($a0)"),
    TEST_ENTRY_C(0xD88F0040, NULL, "lv.q        C330, 0x40($a0)"),
    TEST_ENTRY_C(0xD8900040, NULL, "lv.q        C400, 0x40($a0)"),
    TEST_ENTRY_C(0xD8910040, NULL, "lv.q        C410, 0x40($a0)"),
    TEST_ENTRY_C(0xD8920040, NULL, "lv.q        C420, 0x40($a0)"),
    TEST_ENTRY_C(0xD8930040, NULL, "lv.q        C430, 0x40($a0)"),
    TEST_ENTRY_C(0xD8940040, NULL, "lv.q        C500, 0x40($a0)"),
    TEST_ENTRY_C(0xD8950040, NULL, "lv.q        C510, 0x40($a0)"),
    TEST_ENTRY_C(0xD8960040, NULL, "lv.q        C520, 0x40($a0)"),
    TEST_ENTRY_C(0xD8970040, NULL, "lv.q        C530, 0x40($a0)"),
    TEST_ENTRY_C(0xD8980040, NULL, "lv.q        C600, 0x40($a0)"),
    TEST_ENTRY_C(0xD8990040, NULL, "lv.q        C610, 0x40($a0)"),
    TEST_ENTRY_C(0xD89A0040, NULL, "lv.q        C620, 0x40($a0)"),
    TEST_ENTRY_C(0xD89B0040, NULL, "lv.q        C630, 0x40($a0)"),
    TEST_ENTRY_C(0xD89C0040, NULL, "lv.q        C700, 0x40($a0)"),
    TEST_ENTRY_C(0xD89D0040, NULL, "lv.q        C710, 0x40($a0)"),
    TEST_ENTRY_C(0xD89E0040, NULL, "lv.q        C720, 0x40($a0)"),
    TEST_ENTRY_C(0xD89F0040, NULL, "lv.q        C730, 0x40($a0)"),
    TEST_ENTRY_C(0xD8800041, NULL, "lv.q        R000, 0x40($a0)"),
    TEST_ENTRY_C(0xD8810041, NULL, "lv.q        R001, 0x40($a0)"),
    TEST_ENTRY_C(0xD8820041, NULL, "lv.q        R002, 0x40($a0)"),
    TEST_ENTRY_C(0xD8830041, NULL, "lv.q        R003, 0x40($a0)"),
    TEST_ENTRY_C(0xD8840041, NULL, "lv.q        R100, 0x40($a0)"),
    TEST_ENTRY_C(0xD8850041, NULL, "lv.q        R101, 0x40($a0)"),
    TEST_ENTRY_C(0xD8860041, NULL, "lv.q        R102, 0x40($a0)"),
    TEST_ENTRY_C(0xD8870041, NULL, "lv.q        R103, 0x40($a0)"),
    TEST_ENTRY_C(0xD8880041, NULL, "lv.q        R200, 0x40($a0)"),
    TEST_ENTRY_C(0xD8890041, NULL, "lv.q        R201, 0x40($a0)"),
    TEST_ENTRY_C(0xD88A0041, NULL, "lv.q        R202, 0x40($a0)"),
    TEST_ENTRY_C(0xD88B0041, NULL, "lv.q        R203, 0x40($a0)"),
    TEST_ENTRY_C(0xD88C0041, NULL, "lv.q        R300, 0x40($a0)"),
    TEST_ENTRY_C(0xD88D0041, NULL, "lv.q        R301, 0x40($a0)"),
    TEST_ENTRY_C(0xD88E0041, NULL, "lv.q        R302, 0x40($a0)"),
    TEST_ENTRY_C(0xD88F0041, NULL, "lv.q        R303, 0x40($a0)"),
    TEST_ENTRY_C(0xD8900041, NULL, "lv.q        R400, 0x40($a0)"),
    TEST_ENTRY_C(0xD8910041, NULL, "lv.q        R401, 0x40($a0)"),
    TEST_ENTRY_C(0xD8920041, NULL, "lv.q        R402, 0x40($a0)"),
    TEST_ENTRY_C(0xD8930041, NULL, "lv.q        R403, 0x40($a0)"),
    TEST_ENTRY_C(0xD8940041, NULL, "lv.q        R500, 0x40($a0)"),
    TEST_ENTRY_C(0xD8950041, NULL, "lv.q        R501, 0x40($a0)"),
    TEST_ENTRY_C(0xD8960041, NULL, "lv.q        R502, 0x40($a0)"),
    TEST_ENTRY_C(0xD8970041, NULL, "lv.q        R503, 0x40($a0)"),
    TEST_ENTRY_C(0xD8980041, NULL, "lv.q        R600, 0x40($a0)"),
    TEST_ENTRY_C(0xD8990041, NULL, "lv.q        R601, 0x40($a0)"),
    TEST_ENTRY_C(0xD89A0041, NULL, "lv.q        R602, 0x40($a0)"),
    TEST_ENTRY_C(0xD89B0041, NULL, "lv.q        R603, 0x40($a0)"),
    TEST_ENTRY_C(0xD89C0041, NULL, "lv.q        R700, 0x40($a0)"),
    TEST_ENTRY_C(0xD89D0041, NULL, "lv.q        R701, 0x40($a0)"),
    TEST_ENTRY_C(0xD89E0041, NULL, "lv.q        R702, 0x40($a0)"),
    TEST_ENTRY_C(0xD89F0041, NULL, "lv.q        R703, 0x40($a0)"),

    // According to the manual those should be invalid, but the sn objdump is happy with them, so idk
#if 0
    TEST_ENTRY_C(0xD8800042, NULL, "lv.q        C000, 0x40($a0)"),
    TEST_ENTRY_C(0xD8810042, NULL, "lv.q        C010, 0x40($a0)"),
    TEST_ENTRY_C(0xD8820042, NULL, "lv.q        C020, 0x40($a0)"),
    TEST_ENTRY_C(0xD8830042, NULL, "lv.q        C030, 0x40($a0)"),
    TEST_ENTRY_C(0xD8840042, NULL, "lv.q        C100, 0x40($a0)"),
    TEST_ENTRY_C(0xD8850042, NULL, "lv.q        C110, 0x40($a0)"),
    TEST_ENTRY_C(0xD8860042, NULL, "lv.q        C120, 0x40($a0)"),
    TEST_ENTRY_C(0xD8870042, NULL, "lv.q        C130, 0x40($a0)"),
    TEST_ENTRY_C(0xD8880042, NULL, "lv.q        C200, 0x40($a0)"),
    TEST_ENTRY_C(0xD8890042, NULL, "lv.q        C210, 0x40($a0)"),
    TEST_ENTRY_C(0xD88A0042, NULL, "lv.q        C220, 0x40($a0)"),
    TEST_ENTRY_C(0xD88B0042, NULL, "lv.q        C230, 0x40($a0)"),
    TEST_ENTRY_C(0xD88C0042, NULL, "lv.q        C300, 0x40($a0)"),
    TEST_ENTRY_C(0xD88D0042, NULL, "lv.q        C310, 0x40($a0)"),
    TEST_ENTRY_C(0xD88E0042, NULL, "lv.q        C320, 0x40($a0)"),
    TEST_ENTRY_C(0xD88F0042, NULL, "lv.q        C330, 0x40($a0)"),
    TEST_ENTRY_C(0xD8900042, NULL, "lv.q        C400, 0x40($a0)"),
    TEST_ENTRY_C(0xD8910042, NULL, "lv.q        C410, 0x40($a0)"),
    TEST_ENTRY_C(0xD8920042, NULL, "lv.q        C420, 0x40($a0)"),
    TEST_ENTRY_C(0xD8930042, NULL, "lv.q        C430, 0x40($a0)"),
    TEST_ENTRY_C(0xD8940042, NULL, "lv.q        C500, 0x40($a0)"),
    TEST_ENTRY_C(0xD8950042, NULL, "lv.q        C510, 0x40($a0)"),
    TEST_ENTRY_C(0xD8960042, NULL, "lv.q        C520, 0x40($a0)"),
    TEST_ENTRY_C(0xD8970042, NULL, "lv.q        C530, 0x40($a0)"),
    TEST_ENTRY_C(0xD8980042, NULL, "lv.q        C600, 0x40($a0)"),
    TEST_ENTRY_C(0xD8990042, NULL, "lv.q        C610, 0x40($a0)"),
    TEST_ENTRY_C(0xD89A0042, NULL, "lv.q        C620, 0x40($a0)"),
    TEST_ENTRY_C(0xD89B0042, NULL, "lv.q        C630, 0x40($a0)"),
    TEST_ENTRY_C(0xD89C0042, NULL, "lv.q        C700, 0x40($a0)"),
    TEST_ENTRY_C(0xD89D0042, NULL, "lv.q        C710, 0x40($a0)"),
    TEST_ENTRY_C(0xD89E0042, NULL, "lv.q        C720, 0x40($a0)"),
    TEST_ENTRY_C(0xD89F0042, NULL, "lv.q        C730, 0x40($a0)"),
#else
    TEST_ENTRY_C(0xD8800042, NULL, ".word       0xD8800042                   # lv.q        C000, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD8810042, NULL, ".word       0xD8810042                   # lv.q        C010, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD8820042, NULL, ".word       0xD8820042                   # lv.q        C020, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD8830042, NULL, ".word       0xD8830042                   # lv.q        C030, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD8840042, NULL, ".word       0xD8840042                   # lv.q        C100, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD8850042, NULL, ".word       0xD8850042                   # lv.q        C110, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD8860042, NULL, ".word       0xD8860042                   # lv.q        C120, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD8870042, NULL, ".word       0xD8870042                   # lv.q        C130, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD8880042, NULL, ".word       0xD8880042                   # lv.q        C200, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD8890042, NULL, ".word       0xD8890042                   # lv.q        C210, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD88A0042, NULL, ".word       0xD88A0042                   # lv.q        C220, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD88B0042, NULL, ".word       0xD88B0042                   # lv.q        C230, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD88C0042, NULL, ".word       0xD88C0042                   # lv.q        C300, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD88D0042, NULL, ".word       0xD88D0042                   # lv.q        C310, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD88E0042, NULL, ".word       0xD88E0042                   # lv.q        C320, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD88F0042, NULL, ".word       0xD88F0042                   # lv.q        C330, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD8900042, NULL, ".word       0xD8900042                   # lv.q        C400, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD8910042, NULL, ".word       0xD8910042                   # lv.q        C410, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD8920042, NULL, ".word       0xD8920042                   # lv.q        C420, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD8930042, NULL, ".word       0xD8930042                   # lv.q        C430, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD8940042, NULL, ".word       0xD8940042                   # lv.q        C500, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD8950042, NULL, ".word       0xD8950042                   # lv.q        C510, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD8960042, NULL, ".word       0xD8960042                   # lv.q        C520, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD8970042, NULL, ".word       0xD8970042                   # lv.q        C530, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD8980042, NULL, ".word       0xD8980042                   # lv.q        C600, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD8990042, NULL, ".word       0xD8990042                   # lv.q        C610, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD89A0042, NULL, ".word       0xD89A0042                   # lv.q        C620, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD89B0042, NULL, ".word       0xD89B0042                   # lv.q        C630, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD89C0042, NULL, ".word       0xD89C0042                   # lv.q        C700, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD89D0042, NULL, ".word       0xD89D0042                   # lv.q        C710, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD89E0042, NULL, ".word       0xD89E0042                   # lv.q        C720, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
    TEST_ENTRY_C(0xD89F0042, NULL, ".word       0xD89F0042                   # lv.q        C730, 0x40($a0) # 00000002 <InstrIdType: R4000ALLEGREX_NORMAL>"),
#endif

    TEST_ENTRY_C(0x48640004, NULL, "mfv         $a0, S100"),
    // TEST_ENTRY_C(0x48640080, NULL, "mfvc        $a0, $128"),
    // TEST_ENTRY_C(0x48640080, NULL, "mfvc        $a0, VFPU_PFXS"),
    // TEST_ENTRY_C(0x48640084, NULL, "mfvc        $a0, $132"),
    // TEST_ENTRY_C(0x48640084, NULL, "mfvc        $a0, VFPU_INF4"),
    TEST_ENTRY_C(0x48E40004, NULL, "mtv         $a0, S100"),
    TEST_ENTRY_C(0x48E40084, NULL, "mtvc        $a0, $132"),
    // TEST_ENTRY_C(0x48E40084, NULL, "mtvc        $a0, VFPU_INF4"),
    TEST_ENTRY_C(0xE8800040, NULL, "sv.s        S000, 0x40($a0)"),
    TEST_ENTRY_C(0xF8800040, NULL, "sv.q        C000, 0x40($a0)"),
    TEST_ENTRY_C(0xF8800042, NULL, "sv.q        C000, 0x40($a0), wb"),
    TEST_ENTRY_C(0xF4800040, NULL, "svl.q       C000, 0x40($a0)"),
    TEST_ENTRY_C(0xF4800042, NULL, "svr.q       C000, 0x40($a0)"),

#if 0
    // According to the manual, bit 1 should be zero. The compiler handles it fine, but objdump ignores this bit, so idk
    TEST_ENTRY_C(0xD8800043, NULL, NULL),
    TEST_ENTRY_C(0xD8810043, NULL, NULL),
    TEST_ENTRY_C(0xD8820043, NULL, NULL),
    TEST_ENTRY_C(0xD8830043, NULL, NULL),
    TEST_ENTRY_C(0xD8840043, NULL, NULL),
    TEST_ENTRY_C(0xD8850043, NULL, NULL),
    TEST_ENTRY_C(0xD8860043, NULL, NULL),
    TEST_ENTRY_C(0xD8870043, NULL, NULL),
    TEST_ENTRY_C(0xD8880043, NULL, NULL),
    TEST_ENTRY_C(0xD8890043, NULL, NULL),
    TEST_ENTRY_C(0xD88A0043, NULL, NULL),
    TEST_ENTRY_C(0xD88B0043, NULL, NULL),
    TEST_ENTRY_C(0xD88C0043, NULL, NULL),
    TEST_ENTRY_C(0xD88D0043, NULL, NULL),
    TEST_ENTRY_C(0xD88E0043, NULL, NULL),
    TEST_ENTRY_C(0xD88F0043, NULL, NULL),
    TEST_ENTRY_C(0xD8900043, NULL, NULL),
    TEST_ENTRY_C(0xD8910043, NULL, NULL),
    TEST_ENTRY_C(0xD8920043, NULL, NULL),
    TEST_ENTRY_C(0xD8930043, NULL, NULL),
    TEST_ENTRY_C(0xD8940043, NULL, NULL),
    TEST_ENTRY_C(0xD8950043, NULL, NULL),
    TEST_ENTRY_C(0xD8960043, NULL, NULL),
    TEST_ENTRY_C(0xD8970043, NULL, NULL),
    TEST_ENTRY_C(0xD8980043, NULL, NULL),
    TEST_ENTRY_C(0xD8990043, NULL, NULL),
    TEST_ENTRY_C(0xD89A0043, NULL, NULL),
    TEST_ENTRY_C(0xD89B0043, NULL, NULL),
    TEST_ENTRY_C(0xD89C0043, NULL, NULL),
    TEST_ENTRY_C(0xD89D0043, NULL, NULL),
    TEST_ENTRY_C(0xD89E0043, NULL, NULL),
    TEST_ENTRY_C(0xD89F0043, NULL, NULL),
#endif

    /* scale_quad_x_quad_matrix */
    TEST_ENTRY_C(0xF2008080, NULL, "vmscl.q     M000, M000, S000"),
    TEST_ENTRY_C(0xF2018080, NULL, "vmscl.q     M000, M000, S010"),
    TEST_ENTRY_C(0xF2028080, NULL, "vmscl.q     M000, M000, S020"),
    TEST_ENTRY_C(0xF2038080, NULL, "vmscl.q     M000, M000, S030"),
    TEST_ENTRY_C(0xF2048080, NULL, "vmscl.q     M000, M000, S100"),
    TEST_ENTRY_C(0xF2058080, NULL, "vmscl.q     M000, M000, S110"),
    TEST_ENTRY_C(0xF2068080, NULL, "vmscl.q     M000, M000, S120"),
    TEST_ENTRY_C(0xF2078080, NULL, "vmscl.q     M000, M000, S130"),
    TEST_ENTRY_C(0xF2088080, NULL, "vmscl.q     M000, M000, S200"),
    TEST_ENTRY_C(0xF2098080, NULL, "vmscl.q     M000, M000, S210"),
    TEST_ENTRY_C(0xF20A8080, NULL, "vmscl.q     M000, M000, S220"),
    TEST_ENTRY_C(0xF20B8080, NULL, "vmscl.q     M000, M000, S230"),
    TEST_ENTRY_C(0xF20C8080, NULL, "vmscl.q     M000, M000, S300"),
    TEST_ENTRY_C(0xF20D8080, NULL, "vmscl.q     M000, M000, S310"),
    TEST_ENTRY_C(0xF20E8080, NULL, "vmscl.q     M000, M000, S320"),
    TEST_ENTRY_C(0xF20F8080, NULL, "vmscl.q     M000, M000, S330"),
    TEST_ENTRY_C(0xF2108080, NULL, "vmscl.q     M000, M000, S400"),
    TEST_ENTRY_C(0xF2118080, NULL, "vmscl.q     M000, M000, S410"),
    TEST_ENTRY_C(0xF2128080, NULL, "vmscl.q     M000, M000, S420"),
    TEST_ENTRY_C(0xF2138080, NULL, "vmscl.q     M000, M000, S430"),
    TEST_ENTRY_C(0xF2148080, NULL, "vmscl.q     M000, M000, S500"),
    TEST_ENTRY_C(0xF2158080, NULL, "vmscl.q     M000, M000, S510"),
    TEST_ENTRY_C(0xF2168080, NULL, "vmscl.q     M000, M000, S520"),
    TEST_ENTRY_C(0xF2178080, NULL, "vmscl.q     M000, M000, S530"),
    TEST_ENTRY_C(0xF2188080, NULL, "vmscl.q     M000, M000, S600"),
    TEST_ENTRY_C(0xF2198080, NULL, "vmscl.q     M000, M000, S610"),
    TEST_ENTRY_C(0xF21A8080, NULL, "vmscl.q     M000, M000, S620"),
    TEST_ENTRY_C(0xF21B8080, NULL, "vmscl.q     M000, M000, S630"),
    TEST_ENTRY_C(0xF21C8080, NULL, "vmscl.q     M000, M000, S700"),
    TEST_ENTRY_C(0xF21D8080, NULL, "vmscl.q     M000, M000, S710"),
    TEST_ENTRY_C(0xF21E8080, NULL, "vmscl.q     M000, M000, S720"),
    TEST_ENTRY_C(0xF21F8080, NULL, "vmscl.q     M000, M000, S730"),
    TEST_ENTRY_C(0xF2208080, NULL, "vmscl.q     M000, M000, S001"),
    TEST_ENTRY_C(0xF2218080, NULL, "vmscl.q     M000, M000, S011"),
    TEST_ENTRY_C(0xF2228080, NULL, "vmscl.q     M000, M000, S021"),
    TEST_ENTRY_C(0xF2238080, NULL, "vmscl.q     M000, M000, S031"),
    TEST_ENTRY_C(0xF2248080, NULL, "vmscl.q     M000, M000, S101"),
    TEST_ENTRY_C(0xF2258080, NULL, "vmscl.q     M000, M000, S111"),
    TEST_ENTRY_C(0xF2268080, NULL, "vmscl.q     M000, M000, S121"),
    TEST_ENTRY_C(0xF2278080, NULL, "vmscl.q     M000, M000, S131"),
    TEST_ENTRY_C(0xF2288080, NULL, "vmscl.q     M000, M000, S201"),
    TEST_ENTRY_C(0xF2298080, NULL, "vmscl.q     M000, M000, S211"),
    TEST_ENTRY_C(0xF22A8080, NULL, "vmscl.q     M000, M000, S221"),
    TEST_ENTRY_C(0xF22B8080, NULL, "vmscl.q     M000, M000, S231"),
    TEST_ENTRY_C(0xF22C8080, NULL, "vmscl.q     M000, M000, S301"),
    TEST_ENTRY_C(0xF22D8080, NULL, "vmscl.q     M000, M000, S311"),
    TEST_ENTRY_C(0xF22E8080, NULL, "vmscl.q     M000, M000, S321"),
    TEST_ENTRY_C(0xF22F8080, NULL, "vmscl.q     M000, M000, S331"),
    TEST_ENTRY_C(0xF2308080, NULL, "vmscl.q     M000, M000, S401"),
    TEST_ENTRY_C(0xF2318080, NULL, "vmscl.q     M000, M000, S411"),
    TEST_ENTRY_C(0xF2328080, NULL, "vmscl.q     M000, M000, S421"),
    TEST_ENTRY_C(0xF2338080, NULL, "vmscl.q     M000, M000, S431"),
    TEST_ENTRY_C(0xF2348080, NULL, "vmscl.q     M000, M000, S501"),
    TEST_ENTRY_C(0xF2358080, NULL, "vmscl.q     M000, M000, S511"),
    TEST_ENTRY_C(0xF2368080, NULL, "vmscl.q     M000, M000, S521"),
    TEST_ENTRY_C(0xF2378080, NULL, "vmscl.q     M000, M000, S531"),
    TEST_ENTRY_C(0xF2388080, NULL, "vmscl.q     M000, M000, S601"),
    TEST_ENTRY_C(0xF2398080, NULL, "vmscl.q     M000, M000, S611"),
    TEST_ENTRY_C(0xF23A8080, NULL, "vmscl.q     M000, M000, S621"),
    TEST_ENTRY_C(0xF23B8080, NULL, "vmscl.q     M000, M000, S631"),
    TEST_ENTRY_C(0xF23C8080, NULL, "vmscl.q     M000, M000, S701"),
    TEST_ENTRY_C(0xF23D8080, NULL, "vmscl.q     M000, M000, S711"),
    TEST_ENTRY_C(0xF23E8080, NULL, "vmscl.q     M000, M000, S721"),
    TEST_ENTRY_C(0xF23F8080, NULL, "vmscl.q     M000, M000, S731"),
    TEST_ENTRY_C(0xF2408080, NULL, "vmscl.q     M000, M000, S002"),
    TEST_ENTRY_C(0xF2418080, NULL, "vmscl.q     M000, M000, S012"),
    TEST_ENTRY_C(0xF2428080, NULL, "vmscl.q     M000, M000, S022"),
    TEST_ENTRY_C(0xF2438080, NULL, "vmscl.q     M000, M000, S032"),
    TEST_ENTRY_C(0xF2448080, NULL, "vmscl.q     M000, M000, S102"),
    TEST_ENTRY_C(0xF2458080, NULL, "vmscl.q     M000, M000, S112"),
    TEST_ENTRY_C(0xF2468080, NULL, "vmscl.q     M000, M000, S122"),
    TEST_ENTRY_C(0xF2478080, NULL, "vmscl.q     M000, M000, S132"),
    TEST_ENTRY_C(0xF2488080, NULL, "vmscl.q     M000, M000, S202"),
    TEST_ENTRY_C(0xF2498080, NULL, "vmscl.q     M000, M000, S212"),
    TEST_ENTRY_C(0xF24A8080, NULL, "vmscl.q     M000, M000, S222"),
    TEST_ENTRY_C(0xF24B8080, NULL, "vmscl.q     M000, M000, S232"),
    TEST_ENTRY_C(0xF24C8080, NULL, "vmscl.q     M000, M000, S302"),
    TEST_ENTRY_C(0xF24D8080, NULL, "vmscl.q     M000, M000, S312"),
    TEST_ENTRY_C(0xF24E8080, NULL, "vmscl.q     M000, M000, S322"),
    TEST_ENTRY_C(0xF24F8080, NULL, "vmscl.q     M000, M000, S332"),
    TEST_ENTRY_C(0xF2508080, NULL, "vmscl.q     M000, M000, S402"),
    TEST_ENTRY_C(0xF2518080, NULL, "vmscl.q     M000, M000, S412"),
    TEST_ENTRY_C(0xF2528080, NULL, "vmscl.q     M000, M000, S422"),
    TEST_ENTRY_C(0xF2538080, NULL, "vmscl.q     M000, M000, S432"),
    TEST_ENTRY_C(0xF2548080, NULL, "vmscl.q     M000, M000, S502"),
    TEST_ENTRY_C(0xF2558080, NULL, "vmscl.q     M000, M000, S512"),
    TEST_ENTRY_C(0xF2568080, NULL, "vmscl.q     M000, M000, S522"),
    TEST_ENTRY_C(0xF2578080, NULL, "vmscl.q     M000, M000, S532"),
    TEST_ENTRY_C(0xF2588080, NULL, "vmscl.q     M000, M000, S602"),
    TEST_ENTRY_C(0xF2598080, NULL, "vmscl.q     M000, M000, S612"),
    TEST_ENTRY_C(0xF25A8080, NULL, "vmscl.q     M000, M000, S622"),
    TEST_ENTRY_C(0xF25B8080, NULL, "vmscl.q     M000, M000, S632"),
    TEST_ENTRY_C(0xF25C8080, NULL, "vmscl.q     M000, M000, S702"),
    TEST_ENTRY_C(0xF25D8080, NULL, "vmscl.q     M000, M000, S712"),
    TEST_ENTRY_C(0xF25E8080, NULL, "vmscl.q     M000, M000, S722"),
    TEST_ENTRY_C(0xF25F8080, NULL, "vmscl.q     M000, M000, S732"),
    TEST_ENTRY_C(0xF2608080, NULL, "vmscl.q     M000, M000, S003"),
    TEST_ENTRY_C(0xF2618080, NULL, "vmscl.q     M000, M000, S013"),
    TEST_ENTRY_C(0xF2628080, NULL, "vmscl.q     M000, M000, S023"),
    TEST_ENTRY_C(0xF2638080, NULL, "vmscl.q     M000, M000, S033"),
    TEST_ENTRY_C(0xF2648080, NULL, "vmscl.q     M000, M000, S103"),
    TEST_ENTRY_C(0xF2658080, NULL, "vmscl.q     M000, M000, S113"),
    TEST_ENTRY_C(0xF2668080, NULL, "vmscl.q     M000, M000, S123"),
    TEST_ENTRY_C(0xF2678080, NULL, "vmscl.q     M000, M000, S133"),
    TEST_ENTRY_C(0xF2688080, NULL, "vmscl.q     M000, M000, S203"),
    TEST_ENTRY_C(0xF2698080, NULL, "vmscl.q     M000, M000, S213"),
    TEST_ENTRY_C(0xF26A8080, NULL, "vmscl.q     M000, M000, S223"),
    TEST_ENTRY_C(0xF26B8080, NULL, "vmscl.q     M000, M000, S233"),
    TEST_ENTRY_C(0xF26C8080, NULL, "vmscl.q     M000, M000, S303"),
    TEST_ENTRY_C(0xF26D8080, NULL, "vmscl.q     M000, M000, S313"),
    TEST_ENTRY_C(0xF26E8080, NULL, "vmscl.q     M000, M000, S323"),
    TEST_ENTRY_C(0xF26F8080, NULL, "vmscl.q     M000, M000, S333"),
    TEST_ENTRY_C(0xF2708080, NULL, "vmscl.q     M000, M000, S403"),
    TEST_ENTRY_C(0xF2718080, NULL, "vmscl.q     M000, M000, S413"),
    TEST_ENTRY_C(0xF2728080, NULL, "vmscl.q     M000, M000, S423"),
    TEST_ENTRY_C(0xF2738080, NULL, "vmscl.q     M000, M000, S433"),
    TEST_ENTRY_C(0xF2748080, NULL, "vmscl.q     M000, M000, S503"),
    TEST_ENTRY_C(0xF2758080, NULL, "vmscl.q     M000, M000, S513"),
    TEST_ENTRY_C(0xF2768080, NULL, "vmscl.q     M000, M000, S523"),
    TEST_ENTRY_C(0xF2778080, NULL, "vmscl.q     M000, M000, S533"),
    TEST_ENTRY_C(0xF2788080, NULL, "vmscl.q     M000, M000, S603"),
    TEST_ENTRY_C(0xF2798080, NULL, "vmscl.q     M000, M000, S613"),
    TEST_ENTRY_C(0xF27A8080, NULL, "vmscl.q     M000, M000, S623"),
    TEST_ENTRY_C(0xF27B8080, NULL, "vmscl.q     M000, M000, S633"),
    TEST_ENTRY_C(0xF27C8080, NULL, "vmscl.q     M000, M000, S703"),
    TEST_ENTRY_C(0xF27D8080, NULL, "vmscl.q     M000, M000, S713"),
    TEST_ENTRY_C(0xF27E8080, NULL, "vmscl.q     M000, M000, S723"),
    TEST_ENTRY_C(0xF27F8080, NULL, "vmscl.q     M000, M000, S733"),
    TEST_ENTRY_C(0xF2008081, NULL, "vmscl.q     M010, M000, S000"),
    TEST_ENTRY_C(0xF2008082, NULL, "vmscl.q     M020, M000, S000"),
    TEST_ENTRY_C(0xF2008083, NULL, "vmscl.q     M030, M000, S000"),
    TEST_ENTRY_C(0xF2008084, NULL, "vmscl.q     M100, M000, S000"),
    TEST_ENTRY_C(0xF2008085, NULL, "vmscl.q     M110, M000, S000"),
    TEST_ENTRY_C(0xF2008086, NULL, "vmscl.q     M120, M000, S000"),
    TEST_ENTRY_C(0xF2008087, NULL, "vmscl.q     M130, M000, S000"),
    TEST_ENTRY_C(0xF2008088, NULL, "vmscl.q     M200, M000, S000"),
    TEST_ENTRY_C(0xF2008089, NULL, "vmscl.q     M210, M000, S000"),
    TEST_ENTRY_C(0xF200808A, NULL, "vmscl.q     M220, M000, S000"),
    TEST_ENTRY_C(0xF200808B, NULL, "vmscl.q     M230, M000, S000"),
    TEST_ENTRY_C(0xF200808C, NULL, "vmscl.q     M300, M000, S000"),
    TEST_ENTRY_C(0xF200808D, NULL, "vmscl.q     M310, M000, S000"),
    TEST_ENTRY_C(0xF200808E, NULL, "vmscl.q     M320, M000, S000"),
    TEST_ENTRY_C(0xF200808F, NULL, "vmscl.q     M330, M000, S000"),
    TEST_ENTRY_C(0xF2008090, NULL, "vmscl.q     M400, M000, S000"),
    TEST_ENTRY_C(0xF2008091, NULL, "vmscl.q     M410, M000, S000"),
    TEST_ENTRY_C(0xF2008092, NULL, "vmscl.q     M420, M000, S000"),
    TEST_ENTRY_C(0xF2008093, NULL, "vmscl.q     M430, M000, S000"),
    TEST_ENTRY_C(0xF2008094, NULL, "vmscl.q     M500, M000, S000"),
    TEST_ENTRY_C(0xF2008095, NULL, "vmscl.q     M510, M000, S000"),
    TEST_ENTRY_C(0xF2008096, NULL, "vmscl.q     M520, M000, S000"),
    TEST_ENTRY_C(0xF2008097, NULL, "vmscl.q     M530, M000, S000"),
    TEST_ENTRY_C(0xF2008098, NULL, "vmscl.q     M600, M000, S000"),
    TEST_ENTRY_C(0xF2008099, NULL, "vmscl.q     M610, M000, S000"),
    TEST_ENTRY_C(0xF200809A, NULL, "vmscl.q     M620, M000, S000"),
    TEST_ENTRY_C(0xF200809B, NULL, "vmscl.q     M630, M000, S000"),
    TEST_ENTRY_C(0xF200809C, NULL, "vmscl.q     M700, M000, S000"),
    TEST_ENTRY_C(0xF200809D, NULL, "vmscl.q     M710, M000, S000"),
    TEST_ENTRY_C(0xF200809E, NULL, "vmscl.q     M720, M000, S000"),
    TEST_ENTRY_C(0xF200809F, NULL, "vmscl.q     M730, M000, S000"),
    TEST_ENTRY_C(0xF20080A0, NULL, "vmscl.q     E000, M000, S000"),
    TEST_ENTRY_C(0xF20080A1, NULL, "vmscl.q     E001, M000, S000"),
    TEST_ENTRY_C(0xF20080A2, NULL, "vmscl.q     E002, M000, S000"),
    TEST_ENTRY_C(0xF20080A3, NULL, "vmscl.q     E003, M000, S000"),
    TEST_ENTRY_C(0xF20080A4, NULL, "vmscl.q     E100, M000, S000"),
    TEST_ENTRY_C(0xF20080A5, NULL, "vmscl.q     E101, M000, S000"),
    TEST_ENTRY_C(0xF20080A6, NULL, "vmscl.q     E102, M000, S000"),
    TEST_ENTRY_C(0xF20080A7, NULL, "vmscl.q     E103, M000, S000"),
    TEST_ENTRY_C(0xF20080A8, NULL, "vmscl.q     E200, M000, S000"),
    TEST_ENTRY_C(0xF20080A9, NULL, "vmscl.q     E201, M000, S000"),
    TEST_ENTRY_C(0xF20080AA, NULL, "vmscl.q     E202, M000, S000"),
    TEST_ENTRY_C(0xF20080AB, NULL, "vmscl.q     E203, M000, S000"),
    TEST_ENTRY_C(0xF20080AC, NULL, "vmscl.q     E300, M000, S000"),
    TEST_ENTRY_C(0xF20080AD, NULL, "vmscl.q     E301, M000, S000"),
    TEST_ENTRY_C(0xF20080AE, NULL, "vmscl.q     E302, M000, S000"),
    TEST_ENTRY_C(0xF20080AF, NULL, "vmscl.q     E303, M000, S000"),
    TEST_ENTRY_C(0xF20080B0, NULL, "vmscl.q     E400, M000, S000"),
    TEST_ENTRY_C(0xF20080B1, NULL, "vmscl.q     E401, M000, S000"),
    TEST_ENTRY_C(0xF20080B2, NULL, "vmscl.q     E402, M000, S000"),
    TEST_ENTRY_C(0xF20080B3, NULL, "vmscl.q     E403, M000, S000"),
    TEST_ENTRY_C(0xF20080B4, NULL, "vmscl.q     E500, M000, S000"),
    TEST_ENTRY_C(0xF20080B5, NULL, "vmscl.q     E501, M000, S000"),
    TEST_ENTRY_C(0xF20080B6, NULL, "vmscl.q     E502, M000, S000"),
    TEST_ENTRY_C(0xF20080B7, NULL, "vmscl.q     E503, M000, S000"),
    TEST_ENTRY_C(0xF20080B8, NULL, "vmscl.q     E600, M000, S000"),
    TEST_ENTRY_C(0xF20080B9, NULL, "vmscl.q     E601, M000, S000"),
    TEST_ENTRY_C(0xF20080BA, NULL, "vmscl.q     E602, M000, S000"),
    TEST_ENTRY_C(0xF20080BB, NULL, "vmscl.q     E603, M000, S000"),
    TEST_ENTRY_C(0xF20080BC, NULL, "vmscl.q     E700, M000, S000"),
    TEST_ENTRY_C(0xF20080BD, NULL, "vmscl.q     E701, M000, S000"),
    TEST_ENTRY_C(0xF20080BE, NULL, "vmscl.q     E702, M000, S000"),
    TEST_ENTRY_C(0xF20080BF, NULL, "vmscl.q     E703, M000, S000"),
    TEST_ENTRY_C(0xF20080C0, NULL, "vmscl.q     M002, M000, S000"),
    TEST_ENTRY_C(0xF20080C1, NULL, "vmscl.q     M012, M000, S000"),
    TEST_ENTRY_C(0xF20080C2, NULL, "vmscl.q     M022, M000, S000"),
    TEST_ENTRY_C(0xF20080C3, NULL, "vmscl.q     M032, M000, S000"),
    TEST_ENTRY_C(0xF20080C4, NULL, "vmscl.q     M102, M000, S000"),
    TEST_ENTRY_C(0xF20080C5, NULL, "vmscl.q     M112, M000, S000"),
    TEST_ENTRY_C(0xF20080C6, NULL, "vmscl.q     M122, M000, S000"),
    TEST_ENTRY_C(0xF20080C7, NULL, "vmscl.q     M132, M000, S000"),
    TEST_ENTRY_C(0xF20080C8, NULL, "vmscl.q     M202, M000, S000"),
    TEST_ENTRY_C(0xF20080C9, NULL, "vmscl.q     M212, M000, S000"),
    TEST_ENTRY_C(0xF20080CA, NULL, "vmscl.q     M222, M000, S000"),
    TEST_ENTRY_C(0xF20080CB, NULL, "vmscl.q     M232, M000, S000"),
    TEST_ENTRY_C(0xF20080CC, NULL, "vmscl.q     M302, M000, S000"),
    TEST_ENTRY_C(0xF20080CD, NULL, "vmscl.q     M312, M000, S000"),
    TEST_ENTRY_C(0xF20080CE, NULL, "vmscl.q     M322, M000, S000"),
    TEST_ENTRY_C(0xF20080CF, NULL, "vmscl.q     M332, M000, S000"),
    TEST_ENTRY_C(0xF20080D0, NULL, "vmscl.q     M402, M000, S000"),
    TEST_ENTRY_C(0xF20080D1, NULL, "vmscl.q     M412, M000, S000"),
    TEST_ENTRY_C(0xF20080D2, NULL, "vmscl.q     M422, M000, S000"),
    TEST_ENTRY_C(0xF20080D3, NULL, "vmscl.q     M432, M000, S000"),
    TEST_ENTRY_C(0xF20080D4, NULL, "vmscl.q     M502, M000, S000"),
    TEST_ENTRY_C(0xF20080D5, NULL, "vmscl.q     M512, M000, S000"),
    TEST_ENTRY_C(0xF20080D6, NULL, "vmscl.q     M522, M000, S000"),
    TEST_ENTRY_C(0xF20080D7, NULL, "vmscl.q     M532, M000, S000"),
    TEST_ENTRY_C(0xF20080D8, NULL, "vmscl.q     M602, M000, S000"),
    TEST_ENTRY_C(0xF20080D9, NULL, "vmscl.q     M612, M000, S000"),
    TEST_ENTRY_C(0xF20080DA, NULL, "vmscl.q     M622, M000, S000"),
    TEST_ENTRY_C(0xF20080DB, NULL, "vmscl.q     M632, M000, S000"),
    TEST_ENTRY_C(0xF20080DC, NULL, "vmscl.q     M702, M000, S000"),
    TEST_ENTRY_C(0xF20080DD, NULL, "vmscl.q     M712, M000, S000"),
    TEST_ENTRY_C(0xF20080DE, NULL, "vmscl.q     M722, M000, S000"),
    TEST_ENTRY_C(0xF20080DF, NULL, "vmscl.q     M732, M000, S000"),
    TEST_ENTRY_C(0xF20080E0, NULL, "vmscl.q     E020, M000, S000"),
    TEST_ENTRY_C(0xF20080E1, NULL, "vmscl.q     E021, M000, S000"),
    TEST_ENTRY_C(0xF20080E2, NULL, "vmscl.q     E022, M000, S000"),
    TEST_ENTRY_C(0xF20080E3, NULL, "vmscl.q     E023, M000, S000"),
    TEST_ENTRY_C(0xF20080E4, NULL, "vmscl.q     E120, M000, S000"),
    TEST_ENTRY_C(0xF20080E5, NULL, "vmscl.q     E121, M000, S000"),
    TEST_ENTRY_C(0xF20080E6, NULL, "vmscl.q     E122, M000, S000"),
    TEST_ENTRY_C(0xF20080E7, NULL, "vmscl.q     E123, M000, S000"),
    TEST_ENTRY_C(0xF20080E8, NULL, "vmscl.q     E220, M000, S000"),
    TEST_ENTRY_C(0xF20080E9, NULL, "vmscl.q     E221, M000, S000"),
    TEST_ENTRY_C(0xF20080EA, NULL, "vmscl.q     E222, M000, S000"),
    TEST_ENTRY_C(0xF20080EB, NULL, "vmscl.q     E223, M000, S000"),
    TEST_ENTRY_C(0xF20080EC, NULL, "vmscl.q     E320, M000, S000"),
    TEST_ENTRY_C(0xF20080ED, NULL, "vmscl.q     E321, M000, S000"),
    TEST_ENTRY_C(0xF20080EE, NULL, "vmscl.q     E322, M000, S000"),
    TEST_ENTRY_C(0xF20080EF, NULL, "vmscl.q     E323, M000, S000"),
    TEST_ENTRY_C(0xF20080F0, NULL, "vmscl.q     E420, M000, S000"),
    TEST_ENTRY_C(0xF20080F1, NULL, "vmscl.q     E421, M000, S000"),
    TEST_ENTRY_C(0xF20080F2, NULL, "vmscl.q     E422, M000, S000"),
    TEST_ENTRY_C(0xF20080F3, NULL, "vmscl.q     E423, M000, S000"),
    TEST_ENTRY_C(0xF20080F4, NULL, "vmscl.q     E520, M000, S000"),
    TEST_ENTRY_C(0xF20080F5, NULL, "vmscl.q     E521, M000, S000"),
    TEST_ENTRY_C(0xF20080F6, NULL, "vmscl.q     E522, M000, S000"),
    TEST_ENTRY_C(0xF20080F7, NULL, "vmscl.q     E523, M000, S000"),
    TEST_ENTRY_C(0xF20080F8, NULL, "vmscl.q     E620, M000, S000"),
    TEST_ENTRY_C(0xF20080F9, NULL, "vmscl.q     E621, M000, S000"),
    TEST_ENTRY_C(0xF20080FA, NULL, "vmscl.q     E622, M000, S000"),
    TEST_ENTRY_C(0xF20080FB, NULL, "vmscl.q     E623, M000, S000"),
    TEST_ENTRY_C(0xF20080FC, NULL, "vmscl.q     E720, M000, S000"),
    TEST_ENTRY_C(0xF20080FD, NULL, "vmscl.q     E721, M000, S000"),
    TEST_ENTRY_C(0xF20080FE, NULL, "vmscl.q     E722, M000, S000"),
    TEST_ENTRY_C(0xF20080FF, NULL, "vmscl.q     E723, M000, S000"),

    /* abs all */
    TEST_ENTRY_C(0xD0010000, NULL, "vabs.s      S000, S000"),
    TEST_ENTRY_C(0xD0010101, NULL, "vabs.s      S010, S010"),
    TEST_ENTRY_C(0xD0010202, NULL, "vabs.s      S020, S020"),
    TEST_ENTRY_C(0xD0010303, NULL, "vabs.s      S030, S030"),
    TEST_ENTRY_C(0xD0010404, NULL, "vabs.s      S100, S100"),
    TEST_ENTRY_C(0xD0010505, NULL, "vabs.s      S110, S110"),
    TEST_ENTRY_C(0xD0010606, NULL, "vabs.s      S120, S120"),
    TEST_ENTRY_C(0xD0010707, NULL, "vabs.s      S130, S130"),
    TEST_ENTRY_C(0xD0010808, NULL, "vabs.s      S200, S200"),
    TEST_ENTRY_C(0xD0010909, NULL, "vabs.s      S210, S210"),
    TEST_ENTRY_C(0xD0010A0A, NULL, "vabs.s      S220, S220"),
    TEST_ENTRY_C(0xD0010B0B, NULL, "vabs.s      S230, S230"),
    TEST_ENTRY_C(0xD0010C0C, NULL, "vabs.s      S300, S300"),
    TEST_ENTRY_C(0xD0010D0D, NULL, "vabs.s      S310, S310"),
    TEST_ENTRY_C(0xD0010E0E, NULL, "vabs.s      S320, S320"),
    TEST_ENTRY_C(0xD0010F0F, NULL, "vabs.s      S330, S330"),
    TEST_ENTRY_C(0xD0011010, NULL, "vabs.s      S400, S400"),
    TEST_ENTRY_C(0xD0011111, NULL, "vabs.s      S410, S410"),
    TEST_ENTRY_C(0xD0011212, NULL, "vabs.s      S420, S420"),
    TEST_ENTRY_C(0xD0011313, NULL, "vabs.s      S430, S430"),
    TEST_ENTRY_C(0xD0011414, NULL, "vabs.s      S500, S500"),
    TEST_ENTRY_C(0xD0011515, NULL, "vabs.s      S510, S510"),
    TEST_ENTRY_C(0xD0011616, NULL, "vabs.s      S520, S520"),
    TEST_ENTRY_C(0xD0011717, NULL, "vabs.s      S530, S530"),
    TEST_ENTRY_C(0xD0011818, NULL, "vabs.s      S600, S600"),
    TEST_ENTRY_C(0xD0011919, NULL, "vabs.s      S610, S610"),
    TEST_ENTRY_C(0xD0011A1A, NULL, "vabs.s      S620, S620"),
    TEST_ENTRY_C(0xD0011B1B, NULL, "vabs.s      S630, S630"),
    TEST_ENTRY_C(0xD0011C1C, NULL, "vabs.s      S700, S700"),
    TEST_ENTRY_C(0xD0011D1D, NULL, "vabs.s      S710, S710"),
    TEST_ENTRY_C(0xD0011E1E, NULL, "vabs.s      S720, S720"),
    TEST_ENTRY_C(0xD0011F1F, NULL, "vabs.s      S730, S730"),
    TEST_ENTRY_C(0xD0012020, NULL, "vabs.s      S001, S001"),
    TEST_ENTRY_C(0xD0012121, NULL, "vabs.s      S011, S011"),
    TEST_ENTRY_C(0xD0012222, NULL, "vabs.s      S021, S021"),
    TEST_ENTRY_C(0xD0012323, NULL, "vabs.s      S031, S031"),
    TEST_ENTRY_C(0xD0012424, NULL, "vabs.s      S101, S101"),
    TEST_ENTRY_C(0xD0012525, NULL, "vabs.s      S111, S111"),
    TEST_ENTRY_C(0xD0012626, NULL, "vabs.s      S121, S121"),
    TEST_ENTRY_C(0xD0012727, NULL, "vabs.s      S131, S131"),
    TEST_ENTRY_C(0xD0012828, NULL, "vabs.s      S201, S201"),
    TEST_ENTRY_C(0xD0012929, NULL, "vabs.s      S211, S211"),
    TEST_ENTRY_C(0xD0012A2A, NULL, "vabs.s      S221, S221"),
    TEST_ENTRY_C(0xD0012B2B, NULL, "vabs.s      S231, S231"),
    TEST_ENTRY_C(0xD0012C2C, NULL, "vabs.s      S301, S301"),
    TEST_ENTRY_C(0xD0012D2D, NULL, "vabs.s      S311, S311"),
    TEST_ENTRY_C(0xD0012E2E, NULL, "vabs.s      S321, S321"),
    TEST_ENTRY_C(0xD0012F2F, NULL, "vabs.s      S331, S331"),
    TEST_ENTRY_C(0xD0013030, NULL, "vabs.s      S401, S401"),
    TEST_ENTRY_C(0xD0013131, NULL, "vabs.s      S411, S411"),
    TEST_ENTRY_C(0xD0013232, NULL, "vabs.s      S421, S421"),
    TEST_ENTRY_C(0xD0013333, NULL, "vabs.s      S431, S431"),
    TEST_ENTRY_C(0xD0013434, NULL, "vabs.s      S501, S501"),
    TEST_ENTRY_C(0xD0013535, NULL, "vabs.s      S511, S511"),
    TEST_ENTRY_C(0xD0013636, NULL, "vabs.s      S521, S521"),
    TEST_ENTRY_C(0xD0013737, NULL, "vabs.s      S531, S531"),
    TEST_ENTRY_C(0xD0013838, NULL, "vabs.s      S601, S601"),
    TEST_ENTRY_C(0xD0013939, NULL, "vabs.s      S611, S611"),
    TEST_ENTRY_C(0xD0013A3A, NULL, "vabs.s      S621, S621"),
    TEST_ENTRY_C(0xD0013B3B, NULL, "vabs.s      S631, S631"),
    TEST_ENTRY_C(0xD0013C3C, NULL, "vabs.s      S701, S701"),
    TEST_ENTRY_C(0xD0013D3D, NULL, "vabs.s      S711, S711"),
    TEST_ENTRY_C(0xD0013E3E, NULL, "vabs.s      S721, S721"),
    TEST_ENTRY_C(0xD0013F3F, NULL, "vabs.s      S731, S731"),
    TEST_ENTRY_C(0xD0014040, NULL, "vabs.s      S002, S002"),
    TEST_ENTRY_C(0xD0014141, NULL, "vabs.s      S012, S012"),
    TEST_ENTRY_C(0xD0014242, NULL, "vabs.s      S022, S022"),
    TEST_ENTRY_C(0xD0014343, NULL, "vabs.s      S032, S032"),
    TEST_ENTRY_C(0xD0014444, NULL, "vabs.s      S102, S102"),
    TEST_ENTRY_C(0xD0014545, NULL, "vabs.s      S112, S112"),
    TEST_ENTRY_C(0xD0014646, NULL, "vabs.s      S122, S122"),
    TEST_ENTRY_C(0xD0014747, NULL, "vabs.s      S132, S132"),
    TEST_ENTRY_C(0xD0014848, NULL, "vabs.s      S202, S202"),
    TEST_ENTRY_C(0xD0014949, NULL, "vabs.s      S212, S212"),
    TEST_ENTRY_C(0xD0014A4A, NULL, "vabs.s      S222, S222"),
    TEST_ENTRY_C(0xD0014B4B, NULL, "vabs.s      S232, S232"),
    TEST_ENTRY_C(0xD0014C4C, NULL, "vabs.s      S302, S302"),
    TEST_ENTRY_C(0xD0014D4D, NULL, "vabs.s      S312, S312"),
    TEST_ENTRY_C(0xD0014E4E, NULL, "vabs.s      S322, S322"),
    TEST_ENTRY_C(0xD0014F4F, NULL, "vabs.s      S332, S332"),
    TEST_ENTRY_C(0xD0015050, NULL, "vabs.s      S402, S402"),
    TEST_ENTRY_C(0xD0015151, NULL, "vabs.s      S412, S412"),
    TEST_ENTRY_C(0xD0015252, NULL, "vabs.s      S422, S422"),
    TEST_ENTRY_C(0xD0015353, NULL, "vabs.s      S432, S432"),
    TEST_ENTRY_C(0xD0015454, NULL, "vabs.s      S502, S502"),
    TEST_ENTRY_C(0xD0015555, NULL, "vabs.s      S512, S512"),
    TEST_ENTRY_C(0xD0015656, NULL, "vabs.s      S522, S522"),
    TEST_ENTRY_C(0xD0015757, NULL, "vabs.s      S532, S532"),
    TEST_ENTRY_C(0xD0015858, NULL, "vabs.s      S602, S602"),
    TEST_ENTRY_C(0xD0015959, NULL, "vabs.s      S612, S612"),
    TEST_ENTRY_C(0xD0015A5A, NULL, "vabs.s      S622, S622"),
    TEST_ENTRY_C(0xD0015B5B, NULL, "vabs.s      S632, S632"),
    TEST_ENTRY_C(0xD0015C5C, NULL, "vabs.s      S702, S702"),
    TEST_ENTRY_C(0xD0015D5D, NULL, "vabs.s      S712, S712"),
    TEST_ENTRY_C(0xD0015E5E, NULL, "vabs.s      S722, S722"),
    TEST_ENTRY_C(0xD0015F5F, NULL, "vabs.s      S732, S732"),
    TEST_ENTRY_C(0xD0016060, NULL, "vabs.s      S003, S003"),
    TEST_ENTRY_C(0xD0016161, NULL, "vabs.s      S013, S013"),
    TEST_ENTRY_C(0xD0016262, NULL, "vabs.s      S023, S023"),
    TEST_ENTRY_C(0xD0016363, NULL, "vabs.s      S033, S033"),
    TEST_ENTRY_C(0xD0016464, NULL, "vabs.s      S103, S103"),
    TEST_ENTRY_C(0xD0016565, NULL, "vabs.s      S113, S113"),
    TEST_ENTRY_C(0xD0016666, NULL, "vabs.s      S123, S123"),
    TEST_ENTRY_C(0xD0016767, NULL, "vabs.s      S133, S133"),
    TEST_ENTRY_C(0xD0016868, NULL, "vabs.s      S203, S203"),
    TEST_ENTRY_C(0xD0016969, NULL, "vabs.s      S213, S213"),
    TEST_ENTRY_C(0xD0016A6A, NULL, "vabs.s      S223, S223"),
    TEST_ENTRY_C(0xD0016B6B, NULL, "vabs.s      S233, S233"),
    TEST_ENTRY_C(0xD0016C6C, NULL, "vabs.s      S303, S303"),
    TEST_ENTRY_C(0xD0016D6D, NULL, "vabs.s      S313, S313"),
    TEST_ENTRY_C(0xD0016E6E, NULL, "vabs.s      S323, S323"),
    TEST_ENTRY_C(0xD0016F6F, NULL, "vabs.s      S333, S333"),
    TEST_ENTRY_C(0xD0017070, NULL, "vabs.s      S403, S403"),
    TEST_ENTRY_C(0xD0017171, NULL, "vabs.s      S413, S413"),
    TEST_ENTRY_C(0xD0017272, NULL, "vabs.s      S423, S423"),
    TEST_ENTRY_C(0xD0017373, NULL, "vabs.s      S433, S433"),
    TEST_ENTRY_C(0xD0017474, NULL, "vabs.s      S503, S503"),
    TEST_ENTRY_C(0xD0017575, NULL, "vabs.s      S513, S513"),
    TEST_ENTRY_C(0xD0017676, NULL, "vabs.s      S523, S523"),
    TEST_ENTRY_C(0xD0017777, NULL, "vabs.s      S533, S533"),
    TEST_ENTRY_C(0xD0017878, NULL, "vabs.s      S603, S603"),
    TEST_ENTRY_C(0xD0017979, NULL, "vabs.s      S613, S613"),
    TEST_ENTRY_C(0xD0017A7A, NULL, "vabs.s      S623, S623"),
    TEST_ENTRY_C(0xD0017B7B, NULL, "vabs.s      S633, S633"),
    TEST_ENTRY_C(0xD0017C7C, NULL, "vabs.s      S703, S703"),
    TEST_ENTRY_C(0xD0017D7D, NULL, "vabs.s      S713, S713"),
    TEST_ENTRY_C(0xD0017E7E, NULL, "vabs.s      S723, S723"),
    TEST_ENTRY_C(0xD0017F7F, NULL, "vabs.s      S733, S733"),
    TEST_ENTRY_C(0xD0010080, NULL, "vabs.p      C000, C000"),
    TEST_ENTRY_C(0xD0010181, NULL, "vabs.p      C010, C010"),
    TEST_ENTRY_C(0xD0010282, NULL, "vabs.p      C020, C020"),
    TEST_ENTRY_C(0xD0010383, NULL, "vabs.p      C030, C030"),
    TEST_ENTRY_C(0xD0010484, NULL, "vabs.p      C100, C100"),
    TEST_ENTRY_C(0xD0010585, NULL, "vabs.p      C110, C110"),
    TEST_ENTRY_C(0xD0010686, NULL, "vabs.p      C120, C120"),
    TEST_ENTRY_C(0xD0010787, NULL, "vabs.p      C130, C130"),
    TEST_ENTRY_C(0xD0010888, NULL, "vabs.p      C200, C200"),
    TEST_ENTRY_C(0xD0010989, NULL, "vabs.p      C210, C210"),
    TEST_ENTRY_C(0xD0010A8A, NULL, "vabs.p      C220, C220"),
    TEST_ENTRY_C(0xD0010B8B, NULL, "vabs.p      C230, C230"),
    TEST_ENTRY_C(0xD0010C8C, NULL, "vabs.p      C300, C300"),
    TEST_ENTRY_C(0xD0010D8D, NULL, "vabs.p      C310, C310"),
    TEST_ENTRY_C(0xD0010E8E, NULL, "vabs.p      C320, C320"),
    TEST_ENTRY_C(0xD0010F8F, NULL, "vabs.p      C330, C330"),
    TEST_ENTRY_C(0xD0011090, NULL, "vabs.p      C400, C400"),
    TEST_ENTRY_C(0xD0011191, NULL, "vabs.p      C410, C410"),
    TEST_ENTRY_C(0xD0011292, NULL, "vabs.p      C420, C420"),
    TEST_ENTRY_C(0xD0011393, NULL, "vabs.p      C430, C430"),
    TEST_ENTRY_C(0xD0011494, NULL, "vabs.p      C500, C500"),
    TEST_ENTRY_C(0xD0011595, NULL, "vabs.p      C510, C510"),
    TEST_ENTRY_C(0xD0011696, NULL, "vabs.p      C520, C520"),
    TEST_ENTRY_C(0xD0011797, NULL, "vabs.p      C530, C530"),
    TEST_ENTRY_C(0xD0011898, NULL, "vabs.p      C600, C600"),
    TEST_ENTRY_C(0xD0011999, NULL, "vabs.p      C610, C610"),
    TEST_ENTRY_C(0xD0011A9A, NULL, "vabs.p      C620, C620"),
    TEST_ENTRY_C(0xD0011B9B, NULL, "vabs.p      C630, C630"),
    TEST_ENTRY_C(0xD0011C9C, NULL, "vabs.p      C700, C700"),
    TEST_ENTRY_C(0xD0011D9D, NULL, "vabs.p      C710, C710"),
    TEST_ENTRY_C(0xD0011E9E, NULL, "vabs.p      C720, C720"),
    TEST_ENTRY_C(0xD0011F9F, NULL, "vabs.p      C730, C730"),
    TEST_ENTRY_C(0xD00120A0, NULL, "vabs.p      R000, R000"),
    TEST_ENTRY_C(0xD00121A1, NULL, "vabs.p      R001, R001"),
    TEST_ENTRY_C(0xD00122A2, NULL, "vabs.p      R002, R002"),
    TEST_ENTRY_C(0xD00123A3, NULL, "vabs.p      R003, R003"),
    TEST_ENTRY_C(0xD00124A4, NULL, "vabs.p      R100, R100"),
    TEST_ENTRY_C(0xD00125A5, NULL, "vabs.p      R101, R101"),
    TEST_ENTRY_C(0xD00126A6, NULL, "vabs.p      R102, R102"),
    TEST_ENTRY_C(0xD00127A7, NULL, "vabs.p      R103, R103"),
    TEST_ENTRY_C(0xD00128A8, NULL, "vabs.p      R200, R200"),
    TEST_ENTRY_C(0xD00129A9, NULL, "vabs.p      R201, R201"),
    TEST_ENTRY_C(0xD0012AAA, NULL, "vabs.p      R202, R202"),
    TEST_ENTRY_C(0xD0012BAB, NULL, "vabs.p      R203, R203"),
    TEST_ENTRY_C(0xD0012CAC, NULL, "vabs.p      R300, R300"),
    TEST_ENTRY_C(0xD0012DAD, NULL, "vabs.p      R301, R301"),
    TEST_ENTRY_C(0xD0012EAE, NULL, "vabs.p      R302, R302"),
    TEST_ENTRY_C(0xD0012FAF, NULL, "vabs.p      R303, R303"),
    TEST_ENTRY_C(0xD00130B0, NULL, "vabs.p      R400, R400"),
    TEST_ENTRY_C(0xD00131B1, NULL, "vabs.p      R401, R401"),
    TEST_ENTRY_C(0xD00132B2, NULL, "vabs.p      R402, R402"),
    TEST_ENTRY_C(0xD00133B3, NULL, "vabs.p      R403, R403"),
    TEST_ENTRY_C(0xD00134B4, NULL, "vabs.p      R500, R500"),
    TEST_ENTRY_C(0xD00135B5, NULL, "vabs.p      R501, R501"),
    TEST_ENTRY_C(0xD00136B6, NULL, "vabs.p      R502, R502"),
    TEST_ENTRY_C(0xD00137B7, NULL, "vabs.p      R503, R503"),
    TEST_ENTRY_C(0xD00138B8, NULL, "vabs.p      R600, R600"),
    TEST_ENTRY_C(0xD00139B9, NULL, "vabs.p      R601, R601"),
    TEST_ENTRY_C(0xD0013ABA, NULL, "vabs.p      R602, R602"),
    TEST_ENTRY_C(0xD0013BBB, NULL, "vabs.p      R603, R603"),
    TEST_ENTRY_C(0xD0013CBC, NULL, "vabs.p      R700, R700"),
    TEST_ENTRY_C(0xD0013DBD, NULL, "vabs.p      R701, R701"),
    TEST_ENTRY_C(0xD0013EBE, NULL, "vabs.p      R702, R702"),
    TEST_ENTRY_C(0xD0013FBF, NULL, "vabs.p      R703, R703"),
    TEST_ENTRY_C(0xD00140C0, NULL, "vabs.p      C002, C002"),
    TEST_ENTRY_C(0xD00141C1, NULL, "vabs.p      C012, C012"),
    TEST_ENTRY_C(0xD00142C2, NULL, "vabs.p      C022, C022"),
    TEST_ENTRY_C(0xD00143C3, NULL, "vabs.p      C032, C032"),
    TEST_ENTRY_C(0xD00144C4, NULL, "vabs.p      C102, C102"),
    TEST_ENTRY_C(0xD00145C5, NULL, "vabs.p      C112, C112"),
    TEST_ENTRY_C(0xD00146C6, NULL, "vabs.p      C122, C122"),
    TEST_ENTRY_C(0xD00147C7, NULL, "vabs.p      C132, C132"),
    TEST_ENTRY_C(0xD00148C8, NULL, "vabs.p      C202, C202"),
    TEST_ENTRY_C(0xD00149C9, NULL, "vabs.p      C212, C212"),
    TEST_ENTRY_C(0xD0014ACA, NULL, "vabs.p      C222, C222"),
    TEST_ENTRY_C(0xD0014BCB, NULL, "vabs.p      C232, C232"),
    TEST_ENTRY_C(0xD0014CCC, NULL, "vabs.p      C302, C302"),
    TEST_ENTRY_C(0xD0014DCD, NULL, "vabs.p      C312, C312"),
    TEST_ENTRY_C(0xD0014ECE, NULL, "vabs.p      C322, C322"),
    TEST_ENTRY_C(0xD0014FCF, NULL, "vabs.p      C332, C332"),
    TEST_ENTRY_C(0xD00150D0, NULL, "vabs.p      C402, C402"),
    TEST_ENTRY_C(0xD00151D1, NULL, "vabs.p      C412, C412"),
    TEST_ENTRY_C(0xD00152D2, NULL, "vabs.p      C422, C422"),
    TEST_ENTRY_C(0xD00153D3, NULL, "vabs.p      C432, C432"),
    TEST_ENTRY_C(0xD00154D4, NULL, "vabs.p      C502, C502"),
    TEST_ENTRY_C(0xD00155D5, NULL, "vabs.p      C512, C512"),
    TEST_ENTRY_C(0xD00156D6, NULL, "vabs.p      C522, C522"),
    TEST_ENTRY_C(0xD00157D7, NULL, "vabs.p      C532, C532"),
    TEST_ENTRY_C(0xD00158D8, NULL, "vabs.p      C602, C602"),
    TEST_ENTRY_C(0xD00159D9, NULL, "vabs.p      C612, C612"),
    TEST_ENTRY_C(0xD0015ADA, NULL, "vabs.p      C622, C622"),
    TEST_ENTRY_C(0xD0015BDB, NULL, "vabs.p      C632, C632"),
    TEST_ENTRY_C(0xD0015CDC, NULL, "vabs.p      C702, C702"),
    TEST_ENTRY_C(0xD0015DDD, NULL, "vabs.p      C712, C712"),
    TEST_ENTRY_C(0xD0015EDE, NULL, "vabs.p      C722, C722"),
    TEST_ENTRY_C(0xD0015FDF, NULL, "vabs.p      C732, C732"),
    TEST_ENTRY_C(0xD00160E0, NULL, "vabs.p      R020, R020"),
    TEST_ENTRY_C(0xD00161E1, NULL, "vabs.p      R021, R021"),
    TEST_ENTRY_C(0xD00162E2, NULL, "vabs.p      R022, R022"),
    TEST_ENTRY_C(0xD00163E3, NULL, "vabs.p      R023, R023"),
    TEST_ENTRY_C(0xD00164E4, NULL, "vabs.p      R120, R120"),
    TEST_ENTRY_C(0xD00165E5, NULL, "vabs.p      R121, R121"),
    TEST_ENTRY_C(0xD00166E6, NULL, "vabs.p      R122, R122"),
    TEST_ENTRY_C(0xD00167E7, NULL, "vabs.p      R123, R123"),
    TEST_ENTRY_C(0xD00168E8, NULL, "vabs.p      R220, R220"),
    TEST_ENTRY_C(0xD00169E9, NULL, "vabs.p      R221, R221"),
    TEST_ENTRY_C(0xD0016AEA, NULL, "vabs.p      R222, R222"),
    TEST_ENTRY_C(0xD0016BEB, NULL, "vabs.p      R223, R223"),
    TEST_ENTRY_C(0xD0016CEC, NULL, "vabs.p      R320, R320"),
    TEST_ENTRY_C(0xD0016DED, NULL, "vabs.p      R321, R321"),
    TEST_ENTRY_C(0xD0016EEE, NULL, "vabs.p      R322, R322"),
    TEST_ENTRY_C(0xD0016FEF, NULL, "vabs.p      R323, R323"),
    TEST_ENTRY_C(0xD00170F0, NULL, "vabs.p      R420, R420"),
    TEST_ENTRY_C(0xD00171F1, NULL, "vabs.p      R421, R421"),
    TEST_ENTRY_C(0xD00172F2, NULL, "vabs.p      R422, R422"),
    TEST_ENTRY_C(0xD00173F3, NULL, "vabs.p      R423, R423"),
    TEST_ENTRY_C(0xD00174F4, NULL, "vabs.p      R520, R520"),
    TEST_ENTRY_C(0xD00175F5, NULL, "vabs.p      R521, R521"),
    TEST_ENTRY_C(0xD00176F6, NULL, "vabs.p      R522, R522"),
    TEST_ENTRY_C(0xD00177F7, NULL, "vabs.p      R523, R523"),
    TEST_ENTRY_C(0xD00178F8, NULL, "vabs.p      R620, R620"),
    TEST_ENTRY_C(0xD00179F9, NULL, "vabs.p      R621, R621"),
    TEST_ENTRY_C(0xD0017AFA, NULL, "vabs.p      R622, R622"),
    TEST_ENTRY_C(0xD0017BFB, NULL, "vabs.p      R623, R623"),
    TEST_ENTRY_C(0xD0017CFC, NULL, "vabs.p      R720, R720"),
    TEST_ENTRY_C(0xD0017DFD, NULL, "vabs.p      R721, R721"),
    TEST_ENTRY_C(0xD0017EFE, NULL, "vabs.p      R722, R722"),
    TEST_ENTRY_C(0xD0017FFF, NULL, "vabs.p      R723, R723"),
    TEST_ENTRY_C(0xD0018000, NULL, "vabs.t      C000, C000"),
    TEST_ENTRY_C(0xD0018101, NULL, "vabs.t      C010, C010"),
    TEST_ENTRY_C(0xD0018202, NULL, "vabs.t      C020, C020"),
    TEST_ENTRY_C(0xD0018303, NULL, "vabs.t      C030, C030"),
    TEST_ENTRY_C(0xD0018404, NULL, "vabs.t      C100, C100"),
    TEST_ENTRY_C(0xD0018505, NULL, "vabs.t      C110, C110"),
    TEST_ENTRY_C(0xD0018606, NULL, "vabs.t      C120, C120"),
    TEST_ENTRY_C(0xD0018707, NULL, "vabs.t      C130, C130"),
    TEST_ENTRY_C(0xD0018808, NULL, "vabs.t      C200, C200"),
    TEST_ENTRY_C(0xD0018909, NULL, "vabs.t      C210, C210"),
    TEST_ENTRY_C(0xD0018A0A, NULL, "vabs.t      C220, C220"),
    TEST_ENTRY_C(0xD0018B0B, NULL, "vabs.t      C230, C230"),
    TEST_ENTRY_C(0xD0018C0C, NULL, "vabs.t      C300, C300"),
    TEST_ENTRY_C(0xD0018D0D, NULL, "vabs.t      C310, C310"),
    TEST_ENTRY_C(0xD0018E0E, NULL, "vabs.t      C320, C320"),
    TEST_ENTRY_C(0xD0018F0F, NULL, "vabs.t      C330, C330"),
    TEST_ENTRY_C(0xD0019010, NULL, "vabs.t      C400, C400"),
    TEST_ENTRY_C(0xD0019111, NULL, "vabs.t      C410, C410"),
    TEST_ENTRY_C(0xD0019212, NULL, "vabs.t      C420, C420"),
    TEST_ENTRY_C(0xD0019313, NULL, "vabs.t      C430, C430"),
    TEST_ENTRY_C(0xD0019414, NULL, "vabs.t      C500, C500"),
    TEST_ENTRY_C(0xD0019515, NULL, "vabs.t      C510, C510"),
    TEST_ENTRY_C(0xD0019616, NULL, "vabs.t      C520, C520"),
    TEST_ENTRY_C(0xD0019717, NULL, "vabs.t      C530, C530"),
    TEST_ENTRY_C(0xD0019818, NULL, "vabs.t      C600, C600"),
    TEST_ENTRY_C(0xD0019919, NULL, "vabs.t      C610, C610"),
    TEST_ENTRY_C(0xD0019A1A, NULL, "vabs.t      C620, C620"),
    TEST_ENTRY_C(0xD0019B1B, NULL, "vabs.t      C630, C630"),
    TEST_ENTRY_C(0xD0019C1C, NULL, "vabs.t      C700, C700"),
    TEST_ENTRY_C(0xD0019D1D, NULL, "vabs.t      C710, C710"),
    TEST_ENTRY_C(0xD0019E1E, NULL, "vabs.t      C720, C720"),
    TEST_ENTRY_C(0xD0019F1F, NULL, "vabs.t      C730, C730"),
    TEST_ENTRY_C(0xD001A020, NULL, "vabs.t      R000, R000"),
    TEST_ENTRY_C(0xD001A121, NULL, "vabs.t      R001, R001"),
    TEST_ENTRY_C(0xD001A222, NULL, "vabs.t      R002, R002"),
    TEST_ENTRY_C(0xD001A323, NULL, "vabs.t      R003, R003"),
    TEST_ENTRY_C(0xD001A424, NULL, "vabs.t      R100, R100"),
    TEST_ENTRY_C(0xD001A525, NULL, "vabs.t      R101, R101"),
    TEST_ENTRY_C(0xD001A626, NULL, "vabs.t      R102, R102"),
    TEST_ENTRY_C(0xD001A727, NULL, "vabs.t      R103, R103"),
    TEST_ENTRY_C(0xD001A828, NULL, "vabs.t      R200, R200"),
    TEST_ENTRY_C(0xD001A929, NULL, "vabs.t      R201, R201"),
    TEST_ENTRY_C(0xD001AA2A, NULL, "vabs.t      R202, R202"),
    TEST_ENTRY_C(0xD001AB2B, NULL, "vabs.t      R203, R203"),
    TEST_ENTRY_C(0xD001AC2C, NULL, "vabs.t      R300, R300"),
    TEST_ENTRY_C(0xD001AD2D, NULL, "vabs.t      R301, R301"),
    TEST_ENTRY_C(0xD001AE2E, NULL, "vabs.t      R302, R302"),
    TEST_ENTRY_C(0xD001AF2F, NULL, "vabs.t      R303, R303"),
    TEST_ENTRY_C(0xD001B030, NULL, "vabs.t      R400, R400"),
    TEST_ENTRY_C(0xD001B131, NULL, "vabs.t      R401, R401"),
    TEST_ENTRY_C(0xD001B232, NULL, "vabs.t      R402, R402"),
    TEST_ENTRY_C(0xD001B333, NULL, "vabs.t      R403, R403"),
    TEST_ENTRY_C(0xD001B434, NULL, "vabs.t      R500, R500"),
    TEST_ENTRY_C(0xD001B535, NULL, "vabs.t      R501, R501"),
    TEST_ENTRY_C(0xD001B636, NULL, "vabs.t      R502, R502"),
    TEST_ENTRY_C(0xD001B737, NULL, "vabs.t      R503, R503"),
    TEST_ENTRY_C(0xD001B838, NULL, "vabs.t      R600, R600"),
    TEST_ENTRY_C(0xD001B939, NULL, "vabs.t      R601, R601"),
    TEST_ENTRY_C(0xD001BA3A, NULL, "vabs.t      R602, R602"),
    TEST_ENTRY_C(0xD001BB3B, NULL, "vabs.t      R603, R603"),
    TEST_ENTRY_C(0xD001BC3C, NULL, "vabs.t      R700, R700"),
    TEST_ENTRY_C(0xD001BD3D, NULL, "vabs.t      R701, R701"),
    TEST_ENTRY_C(0xD001BE3E, NULL, "vabs.t      R702, R702"),
    TEST_ENTRY_C(0xD001BF3F, NULL, "vabs.t      R703, R703"),
    TEST_ENTRY_C(0xD001C040, NULL, "vabs.t      C001, C001"),
    TEST_ENTRY_C(0xD001C141, NULL, "vabs.t      C011, C011"),
    TEST_ENTRY_C(0xD001C242, NULL, "vabs.t      C021, C021"),
    TEST_ENTRY_C(0xD001C343, NULL, "vabs.t      C031, C031"),
    TEST_ENTRY_C(0xD001C444, NULL, "vabs.t      C101, C101"),
    TEST_ENTRY_C(0xD001C545, NULL, "vabs.t      C111, C111"),
    TEST_ENTRY_C(0xD001C646, NULL, "vabs.t      C121, C121"),
    TEST_ENTRY_C(0xD001C747, NULL, "vabs.t      C131, C131"),
    TEST_ENTRY_C(0xD001C848, NULL, "vabs.t      C201, C201"),
    TEST_ENTRY_C(0xD001C949, NULL, "vabs.t      C211, C211"),
    TEST_ENTRY_C(0xD001CA4A, NULL, "vabs.t      C221, C221"),
    TEST_ENTRY_C(0xD001CB4B, NULL, "vabs.t      C231, C231"),
    TEST_ENTRY_C(0xD001CC4C, NULL, "vabs.t      C301, C301"),
    TEST_ENTRY_C(0xD001CD4D, NULL, "vabs.t      C311, C311"),
    TEST_ENTRY_C(0xD001CE4E, NULL, "vabs.t      C321, C321"),
    TEST_ENTRY_C(0xD001CF4F, NULL, "vabs.t      C331, C331"),
    TEST_ENTRY_C(0xD001D050, NULL, "vabs.t      C401, C401"),
    TEST_ENTRY_C(0xD001D151, NULL, "vabs.t      C411, C411"),
    TEST_ENTRY_C(0xD001D252, NULL, "vabs.t      C421, C421"),
    TEST_ENTRY_C(0xD001D353, NULL, "vabs.t      C431, C431"),
    TEST_ENTRY_C(0xD001D454, NULL, "vabs.t      C501, C501"),
    TEST_ENTRY_C(0xD001D555, NULL, "vabs.t      C511, C511"),
    TEST_ENTRY_C(0xD001D656, NULL, "vabs.t      C521, C521"),
    TEST_ENTRY_C(0xD001D757, NULL, "vabs.t      C531, C531"),
    TEST_ENTRY_C(0xD001D858, NULL, "vabs.t      C601, C601"),
    TEST_ENTRY_C(0xD001D959, NULL, "vabs.t      C611, C611"),
    TEST_ENTRY_C(0xD001DA5A, NULL, "vabs.t      C621, C621"),
    TEST_ENTRY_C(0xD001DB5B, NULL, "vabs.t      C631, C631"),
    TEST_ENTRY_C(0xD001DC5C, NULL, "vabs.t      C701, C701"),
    TEST_ENTRY_C(0xD001DD5D, NULL, "vabs.t      C711, C711"),
    TEST_ENTRY_C(0xD001DE5E, NULL, "vabs.t      C721, C721"),
    TEST_ENTRY_C(0xD001DF5F, NULL, "vabs.t      C731, C731"),
    TEST_ENTRY_C(0xD001E060, NULL, "vabs.t      R010, R010"),
    TEST_ENTRY_C(0xD001E161, NULL, "vabs.t      R011, R011"),
    TEST_ENTRY_C(0xD001E262, NULL, "vabs.t      R012, R012"),
    TEST_ENTRY_C(0xD001E363, NULL, "vabs.t      R013, R013"),
    TEST_ENTRY_C(0xD001E464, NULL, "vabs.t      R110, R110"),
    TEST_ENTRY_C(0xD001E565, NULL, "vabs.t      R111, R111"),
    TEST_ENTRY_C(0xD001E666, NULL, "vabs.t      R112, R112"),
    TEST_ENTRY_C(0xD001E767, NULL, "vabs.t      R113, R113"),
    TEST_ENTRY_C(0xD001E868, NULL, "vabs.t      R210, R210"),
    TEST_ENTRY_C(0xD001E969, NULL, "vabs.t      R211, R211"),
    TEST_ENTRY_C(0xD001EA6A, NULL, "vabs.t      R212, R212"),
    TEST_ENTRY_C(0xD001EB6B, NULL, "vabs.t      R213, R213"),
    TEST_ENTRY_C(0xD001EC6C, NULL, "vabs.t      R310, R310"),
    TEST_ENTRY_C(0xD001ED6D, NULL, "vabs.t      R311, R311"),
    TEST_ENTRY_C(0xD001EE6E, NULL, "vabs.t      R312, R312"),
    TEST_ENTRY_C(0xD001EF6F, NULL, "vabs.t      R313, R313"),
    TEST_ENTRY_C(0xD001F070, NULL, "vabs.t      R410, R410"),
    TEST_ENTRY_C(0xD001F171, NULL, "vabs.t      R411, R411"),
    TEST_ENTRY_C(0xD001F272, NULL, "vabs.t      R412, R412"),
    TEST_ENTRY_C(0xD001F373, NULL, "vabs.t      R413, R413"),
    TEST_ENTRY_C(0xD001F474, NULL, "vabs.t      R510, R510"),
    TEST_ENTRY_C(0xD001F575, NULL, "vabs.t      R511, R511"),
    TEST_ENTRY_C(0xD001F676, NULL, "vabs.t      R512, R512"),
    TEST_ENTRY_C(0xD001F777, NULL, "vabs.t      R513, R513"),
    TEST_ENTRY_C(0xD001F878, NULL, "vabs.t      R610, R610"),
    TEST_ENTRY_C(0xD001F979, NULL, "vabs.t      R611, R611"),
    TEST_ENTRY_C(0xD001FA7A, NULL, "vabs.t      R612, R612"),
    TEST_ENTRY_C(0xD001FB7B, NULL, "vabs.t      R613, R613"),
    TEST_ENTRY_C(0xD001FC7C, NULL, "vabs.t      R710, R710"),
    TEST_ENTRY_C(0xD001FD7D, NULL, "vabs.t      R711, R711"),
    TEST_ENTRY_C(0xD001FE7E, NULL, "vabs.t      R712, R712"),
    TEST_ENTRY_C(0xD001FF7F, NULL, "vabs.t      R713, R713"),
    TEST_ENTRY_C(0xD0018080, NULL, "vabs.q      C000, C000"),
    TEST_ENTRY_C(0xD0018181, NULL, "vabs.q      C010, C010"),
    TEST_ENTRY_C(0xD0018282, NULL, "vabs.q      C020, C020"),
    TEST_ENTRY_C(0xD0018383, NULL, "vabs.q      C030, C030"),
    TEST_ENTRY_C(0xD0018484, NULL, "vabs.q      C100, C100"),
    TEST_ENTRY_C(0xD0018585, NULL, "vabs.q      C110, C110"),
    TEST_ENTRY_C(0xD0018686, NULL, "vabs.q      C120, C120"),
    TEST_ENTRY_C(0xD0018787, NULL, "vabs.q      C130, C130"),
    TEST_ENTRY_C(0xD0018888, NULL, "vabs.q      C200, C200"),
    TEST_ENTRY_C(0xD0018989, NULL, "vabs.q      C210, C210"),
    TEST_ENTRY_C(0xD0018A8A, NULL, "vabs.q      C220, C220"),
    TEST_ENTRY_C(0xD0018B8B, NULL, "vabs.q      C230, C230"),
    TEST_ENTRY_C(0xD0018C8C, NULL, "vabs.q      C300, C300"),
    TEST_ENTRY_C(0xD0018D8D, NULL, "vabs.q      C310, C310"),
    TEST_ENTRY_C(0xD0018E8E, NULL, "vabs.q      C320, C320"),
    TEST_ENTRY_C(0xD0018F8F, NULL, "vabs.q      C330, C330"),
    TEST_ENTRY_C(0xD0019090, NULL, "vabs.q      C400, C400"),
    TEST_ENTRY_C(0xD0019191, NULL, "vabs.q      C410, C410"),
    TEST_ENTRY_C(0xD0019292, NULL, "vabs.q      C420, C420"),
    TEST_ENTRY_C(0xD0019393, NULL, "vabs.q      C430, C430"),
    TEST_ENTRY_C(0xD0019494, NULL, "vabs.q      C500, C500"),
    TEST_ENTRY_C(0xD0019595, NULL, "vabs.q      C510, C510"),
    TEST_ENTRY_C(0xD0019696, NULL, "vabs.q      C520, C520"),
    TEST_ENTRY_C(0xD0019797, NULL, "vabs.q      C530, C530"),
    TEST_ENTRY_C(0xD0019898, NULL, "vabs.q      C600, C600"),
    TEST_ENTRY_C(0xD0019999, NULL, "vabs.q      C610, C610"),
    TEST_ENTRY_C(0xD0019A9A, NULL, "vabs.q      C620, C620"),
    TEST_ENTRY_C(0xD0019B9B, NULL, "vabs.q      C630, C630"),
    TEST_ENTRY_C(0xD0019C9C, NULL, "vabs.q      C700, C700"),
    TEST_ENTRY_C(0xD0019D9D, NULL, "vabs.q      C710, C710"),
    TEST_ENTRY_C(0xD0019E9E, NULL, "vabs.q      C720, C720"),
    TEST_ENTRY_C(0xD0019F9F, NULL, "vabs.q      C730, C730"),
    TEST_ENTRY_C(0xD001A0A0, NULL, "vabs.q      R000, R000"),
    TEST_ENTRY_C(0xD001A1A1, NULL, "vabs.q      R001, R001"),
    TEST_ENTRY_C(0xD001A2A2, NULL, "vabs.q      R002, R002"),
    TEST_ENTRY_C(0xD001A3A3, NULL, "vabs.q      R003, R003"),
    TEST_ENTRY_C(0xD001A4A4, NULL, "vabs.q      R100, R100"),
    TEST_ENTRY_C(0xD001A5A5, NULL, "vabs.q      R101, R101"),
    TEST_ENTRY_C(0xD001A6A6, NULL, "vabs.q      R102, R102"),
    TEST_ENTRY_C(0xD001A7A7, NULL, "vabs.q      R103, R103"),
    TEST_ENTRY_C(0xD001A8A8, NULL, "vabs.q      R200, R200"),
    TEST_ENTRY_C(0xD001A9A9, NULL, "vabs.q      R201, R201"),
    TEST_ENTRY_C(0xD001AAAA, NULL, "vabs.q      R202, R202"),
    TEST_ENTRY_C(0xD001ABAB, NULL, "vabs.q      R203, R203"),
    TEST_ENTRY_C(0xD001ACAC, NULL, "vabs.q      R300, R300"),
    TEST_ENTRY_C(0xD001ADAD, NULL, "vabs.q      R301, R301"),
    TEST_ENTRY_C(0xD001AEAE, NULL, "vabs.q      R302, R302"),
    TEST_ENTRY_C(0xD001AFAF, NULL, "vabs.q      R303, R303"),
    TEST_ENTRY_C(0xD001B0B0, NULL, "vabs.q      R400, R400"),
    TEST_ENTRY_C(0xD001B1B1, NULL, "vabs.q      R401, R401"),
    TEST_ENTRY_C(0xD001B2B2, NULL, "vabs.q      R402, R402"),
    TEST_ENTRY_C(0xD001B3B3, NULL, "vabs.q      R403, R403"),
    TEST_ENTRY_C(0xD001B4B4, NULL, "vabs.q      R500, R500"),
    TEST_ENTRY_C(0xD001B5B5, NULL, "vabs.q      R501, R501"),
    TEST_ENTRY_C(0xD001B6B6, NULL, "vabs.q      R502, R502"),
    TEST_ENTRY_C(0xD001B7B7, NULL, "vabs.q      R503, R503"),
    TEST_ENTRY_C(0xD001B8B8, NULL, "vabs.q      R600, R600"),
    TEST_ENTRY_C(0xD001B9B9, NULL, "vabs.q      R601, R601"),
    TEST_ENTRY_C(0xD001BABA, NULL, "vabs.q      R602, R602"),
    TEST_ENTRY_C(0xD001BBBB, NULL, "vabs.q      R603, R603"),
    TEST_ENTRY_C(0xD001BCBC, NULL, "vabs.q      R700, R700"),
    TEST_ENTRY_C(0xD001BDBD, NULL, "vabs.q      R701, R701"),
    TEST_ENTRY_C(0xD001BEBE, NULL, "vabs.q      R702, R702"),
    TEST_ENTRY_C(0xD001BFBF, NULL, "vabs.q      R703, R703"),
    TEST_ENTRY_C(0xD001C0C0, NULL, "vabs.q      C002, C002"),
    TEST_ENTRY_C(0xD001C1C1, NULL, "vabs.q      C012, C012"),
    TEST_ENTRY_C(0xD001C2C2, NULL, "vabs.q      C022, C022"),
    TEST_ENTRY_C(0xD001C3C3, NULL, "vabs.q      C032, C032"),
    TEST_ENTRY_C(0xD001C4C4, NULL, "vabs.q      C102, C102"),
    TEST_ENTRY_C(0xD001C5C5, NULL, "vabs.q      C112, C112"),
    TEST_ENTRY_C(0xD001C6C6, NULL, "vabs.q      C122, C122"),
    TEST_ENTRY_C(0xD001C7C7, NULL, "vabs.q      C132, C132"),
    TEST_ENTRY_C(0xD001C8C8, NULL, "vabs.q      C202, C202"),
    TEST_ENTRY_C(0xD001C9C9, NULL, "vabs.q      C212, C212"),
    TEST_ENTRY_C(0xD001CACA, NULL, "vabs.q      C222, C222"),
    TEST_ENTRY_C(0xD001CBCB, NULL, "vabs.q      C232, C232"),
    TEST_ENTRY_C(0xD001CCCC, NULL, "vabs.q      C302, C302"),
    TEST_ENTRY_C(0xD001CDCD, NULL, "vabs.q      C312, C312"),
    TEST_ENTRY_C(0xD001CECE, NULL, "vabs.q      C322, C322"),
    TEST_ENTRY_C(0xD001CFCF, NULL, "vabs.q      C332, C332"),
    TEST_ENTRY_C(0xD001D0D0, NULL, "vabs.q      C402, C402"),
    TEST_ENTRY_C(0xD001D1D1, NULL, "vabs.q      C412, C412"),
    TEST_ENTRY_C(0xD001D2D2, NULL, "vabs.q      C422, C422"),
    TEST_ENTRY_C(0xD001D3D3, NULL, "vabs.q      C432, C432"),
    TEST_ENTRY_C(0xD001D4D4, NULL, "vabs.q      C502, C502"),
    TEST_ENTRY_C(0xD001D5D5, NULL, "vabs.q      C512, C512"),
    TEST_ENTRY_C(0xD001D6D6, NULL, "vabs.q      C522, C522"),
    TEST_ENTRY_C(0xD001D7D7, NULL, "vabs.q      C532, C532"),
    TEST_ENTRY_C(0xD001D8D8, NULL, "vabs.q      C602, C602"),
    TEST_ENTRY_C(0xD001D9D9, NULL, "vabs.q      C612, C612"),
    TEST_ENTRY_C(0xD001DADA, NULL, "vabs.q      C622, C622"),
    TEST_ENTRY_C(0xD001DBDB, NULL, "vabs.q      C632, C632"),
    TEST_ENTRY_C(0xD001DCDC, NULL, "vabs.q      C702, C702"),
    TEST_ENTRY_C(0xD001DDDD, NULL, "vabs.q      C712, C712"),
    TEST_ENTRY_C(0xD001DEDE, NULL, "vabs.q      C722, C722"),
    TEST_ENTRY_C(0xD001DFDF, NULL, "vabs.q      C732, C732"),
    TEST_ENTRY_C(0xD001E0E0, NULL, "vabs.q      R020, R020"),
    TEST_ENTRY_C(0xD001E1E1, NULL, "vabs.q      R021, R021"),
    TEST_ENTRY_C(0xD001E2E2, NULL, "vabs.q      R022, R022"),
    TEST_ENTRY_C(0xD001E3E3, NULL, "vabs.q      R023, R023"),
    TEST_ENTRY_C(0xD001E4E4, NULL, "vabs.q      R120, R120"),
    TEST_ENTRY_C(0xD001E5E5, NULL, "vabs.q      R121, R121"),
    TEST_ENTRY_C(0xD001E6E6, NULL, "vabs.q      R122, R122"),
    TEST_ENTRY_C(0xD001E7E7, NULL, "vabs.q      R123, R123"),
    TEST_ENTRY_C(0xD001E8E8, NULL, "vabs.q      R220, R220"),
    TEST_ENTRY_C(0xD001E9E9, NULL, "vabs.q      R221, R221"),
    TEST_ENTRY_C(0xD001EAEA, NULL, "vabs.q      R222, R222"),
    TEST_ENTRY_C(0xD001EBEB, NULL, "vabs.q      R223, R223"),
    TEST_ENTRY_C(0xD001ECEC, NULL, "vabs.q      R320, R320"),
    TEST_ENTRY_C(0xD001EDED, NULL, "vabs.q      R321, R321"),
    TEST_ENTRY_C(0xD001EEEE, NULL, "vabs.q      R322, R322"),
    TEST_ENTRY_C(0xD001EFEF, NULL, "vabs.q      R323, R323"),
    TEST_ENTRY_C(0xD001F0F0, NULL, "vabs.q      R420, R420"),
    TEST_ENTRY_C(0xD001F1F1, NULL, "vabs.q      R421, R421"),
    TEST_ENTRY_C(0xD001F2F2, NULL, "vabs.q      R422, R422"),
    TEST_ENTRY_C(0xD001F3F3, NULL, "vabs.q      R423, R423"),
    TEST_ENTRY_C(0xD001F4F4, NULL, "vabs.q      R520, R520"),
    TEST_ENTRY_C(0xD001F5F5, NULL, "vabs.q      R521, R521"),
    TEST_ENTRY_C(0xD001F6F6, NULL, "vabs.q      R522, R522"),
    TEST_ENTRY_C(0xD001F7F7, NULL, "vabs.q      R523, R523"),
    TEST_ENTRY_C(0xD001F8F8, NULL, "vabs.q      R620, R620"),
    TEST_ENTRY_C(0xD001F9F9, NULL, "vabs.q      R621, R621"),
    TEST_ENTRY_C(0xD001FAFA, NULL, "vabs.q      R622, R622"),
    TEST_ENTRY_C(0xD001FBFB, NULL, "vabs.q      R623, R623"),
    TEST_ENTRY_C(0xD001FCFC, NULL, "vabs.q      R720, R720"),
    TEST_ENTRY_C(0xD001FDFD, NULL, "vabs.q      R721, R721"),
    TEST_ENTRY_C(0xD001FEFE, NULL, "vabs.q      R722, R722"),
    TEST_ENTRY_C(0xD001FFFF, NULL, "vabs.q      R723, R723"),

    /* VFPU0 */
    TEST_ENTRY_C(0x60001040, NULL, "vadd.s      S002, S400, S000"),
    TEST_ENTRY_C(0x600010C0, NULL, "vadd.p      C002, C400, C000"),
    TEST_ENTRY_C(0x60009040, NULL, "vadd.t      C001, C400, C000"),
    TEST_ENTRY_C(0x600090C0, NULL, "vadd.q      C002, C400, C000"),
    TEST_ENTRY_C(0x60801040, NULL, "vsub.s      S002, S400, S000"),
    TEST_ENTRY_C(0x608010C0, NULL, "vsub.p      C002, C400, C000"),
    TEST_ENTRY_C(0x60809040, NULL, "vsub.t      C001, C400, C000"),
    TEST_ENTRY_C(0x608090C0, NULL, "vsub.q      C002, C400, C000"),
    TEST_ENTRY_C(0x61001040, NULL, "vsbn.s      S002, S400, S000"),
    TEST_ENTRY_C(0x610010C0, NULL, ".word       0x610010C0                   # INVALID     $t0, $zero, 0x10C0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU0>"),
    TEST_ENTRY_C(0x61009040, NULL, ".word       0x61009040                   # INVALID     $t0, $zero, -0x6FC0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU0>"),
    TEST_ENTRY_C(0x610090C0, NULL, ".word       0x610090C0                   # INVALID     $t0, $zero, -0x6F40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU0>"),
    TEST_ENTRY_C(0x61801040, NULL, ".word       0x61801040                   # INVALID     $t4, $zero, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU0>"),
    TEST_ENTRY_C(0x618010C0, NULL, ".word       0x618010C0                   # INVALID     $t4, $zero, 0x10C0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU0>"),
    TEST_ENTRY_C(0x61809040, NULL, ".word       0x61809040                   # INVALID     $t4, $zero, -0x6FC0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU0>"),
    TEST_ENTRY_C(0x618090C0, NULL, ".word       0x618090C0                   # INVALID     $t4, $zero, -0x6F40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU0>"),
    TEST_ENTRY_C(0x62001040, NULL, ".word       0x62001040                   # INVALID     $s0, $zero, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU0>"),
    TEST_ENTRY_C(0x620010C0, NULL, ".word       0x620010C0                   # INVALID     $s0, $zero, 0x10C0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU0>"),
    TEST_ENTRY_C(0x62009040, NULL, ".word       0x62009040                   # INVALID     $s0, $zero, -0x6FC0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU0>"),
    TEST_ENTRY_C(0x620090C0, NULL, ".word       0x620090C0                   # INVALID     $s0, $zero, -0x6F40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU0>"),
    TEST_ENTRY_C(0x62801040, NULL, ".word       0x62801040                   # INVALID     $s4, $zero, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU0>"),
    TEST_ENTRY_C(0x628010C0, NULL, ".word       0x628010C0                   # INVALID     $s4, $zero, 0x10C0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU0>"),
    TEST_ENTRY_C(0x62809040, NULL, ".word       0x62809040                   # INVALID     $s4, $zero, -0x6FC0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU0>"),
    TEST_ENTRY_C(0x628090C0, NULL, ".word       0x628090C0                   # INVALID     $s4, $zero, -0x6F40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU0>"),
    TEST_ENTRY_C(0x63001040, NULL, ".word       0x63001040                   # INVALID     $t8, $zero, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU0>"),
    TEST_ENTRY_C(0x630010C0, NULL, ".word       0x630010C0                   # INVALID     $t8, $zero, 0x10C0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU0>"),
    TEST_ENTRY_C(0x63009040, NULL, ".word       0x63009040                   # INVALID     $t8, $zero, -0x6FC0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU0>"),
    TEST_ENTRY_C(0x630090C0, NULL, ".word       0x630090C0                   # INVALID     $t8, $zero, -0x6F40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU0>"),
    TEST_ENTRY_C(0x63801040, NULL, "vdiv.s      S002, S400, S000"),
    TEST_ENTRY_C(0x63801084, NULL, "vdiv.p      C100, C400, C000"),
    TEST_ENTRY_C(0x63809004, NULL, "vdiv.t      C100, C400, C000"),
    TEST_ENTRY_C(0x63809084, NULL, "vdiv.q      C100, C400, C000"),

    /* VFPU1 */
    TEST_ENTRY_C(0x64001040, NULL, "vmul.s      S002, S400, S000"),
    TEST_ENTRY_C(0x640010C0, NULL, "vmul.p      C002, C400, C000"),
    TEST_ENTRY_C(0x64009040, NULL, "vmul.t      C001, C400, C000"),
    TEST_ENTRY_C(0x640090C0, NULL, "vmul.q      C002, C400, C000"),
    TEST_ENTRY_C(0x64801040, NULL, ".word       0x64801040                   # INVALID     $a0, $zero, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU1>"),
    TEST_ENTRY_C(0x648010C0, NULL, "vdot.p      S002, C400, C000"),
    TEST_ENTRY_C(0x64809040, NULL, "vdot.t      S002, C400, C000"),
    TEST_ENTRY_C(0x648090C0, NULL, "vdot.q      S002, C400, C000"),
    TEST_ENTRY_C(0x65001040, NULL, ".word       0x65001040                   # INVALID     $t0, $zero, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU1>"),
    TEST_ENTRY_C(0x650010C0, NULL, "vscl.p      C002, C400, S000"),
    TEST_ENTRY_C(0x65009040, NULL, "vscl.t      C001, C400, S000"),
    TEST_ENTRY_C(0x650090C0, NULL, "vscl.q      C002, C400, S000"),
    TEST_ENTRY_C(0x65801040, NULL, ".word       0x65801040                   # INVALID     $t4, $zero, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU1>"),
    TEST_ENTRY_C(0x658010C0, NULL, ".word       0x658010C0                   # INVALID     $t4, $zero, 0x10C0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU1>"),
    TEST_ENTRY_C(0x65809040, NULL, ".word       0x65809040                   # INVALID     $t4, $zero, -0x6FC0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU1>"),
    TEST_ENTRY_C(0x658090C0, NULL, ".word       0x658090C0                   # INVALID     $t4, $zero, -0x6F40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU1>"),
    TEST_ENTRY_C(0x66001040, NULL, ".word       0x66001040                   # INVALID     $s0, $zero, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU1>"),
    TEST_ENTRY_C(0x660010C0, NULL, "vhdp.p      S002, C400, C000"),
    TEST_ENTRY_C(0x66009040, NULL, "vhdp.t      S002, C400, C000"),
    TEST_ENTRY_C(0x660090C0, NULL, "vhdp.q      S002, C400, C000"),
    TEST_ENTRY_C(0x66801040, NULL, ".word       0x66801040                   # INVALID     $s4, $zero, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU1>"),
    TEST_ENTRY_C(0x668010C0, NULL, ".word       0x668010C0                   # INVALID     $s4, $zero, 0x10C0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU1>"),
    TEST_ENTRY_C(0x66809040, NULL, "vcrs.t      C001, C400, C000"),
    TEST_ENTRY_C(0x668090C0, NULL, ".word       0x668090C0                   # INVALID     $s4, $zero, -0x6F40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU1>"),
    TEST_ENTRY_C(0x67001040, NULL, ".word       0x67001040                   # INVALID     $t8, $zero, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU1>"),
    TEST_ENTRY_C(0x670010C0, NULL, "vdet.p      S002, C400, C000"),
    TEST_ENTRY_C(0x67009040, NULL, ".word       0x67009040                   # INVALID     $t8, $zero, -0x6FC0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU1>"),
    TEST_ENTRY_C(0x670090C0, NULL, ".word       0x670090C0                   # INVALID     $t8, $zero, -0x6F40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU1>"),
    TEST_ENTRY_C(0x67801040, NULL, ".word       0x67801040                   # INVALID     $gp, $zero, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU1>"),
    TEST_ENTRY_C(0x678010C0, NULL, ".word       0x678010C0                   # INVALID     $gp, $zero, 0x10C0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU1>"),
    TEST_ENTRY_C(0x67809040, NULL, ".word       0x67809040                   # INVALID     $gp, $zero, -0x6FC0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU1>"),
    TEST_ENTRY_C(0x678090C0, NULL, ".word       0x678090C0                   # INVALID     $gp, $zero, -0x6F40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU1>"),

    /* VFPU3 */
    TEST_ENTRY_C(0x6C401001, NULL, "vcmp.s      eq, S400, S002"),
    TEST_ENTRY_C(0x6C401081, NULL, "vcmp.p      eq, C400, C002"),
    TEST_ENTRY_C(0x6C409001, NULL, "vcmp.t      eq, C400, C001"),
    TEST_ENTRY_C(0x6C409081, NULL, "vcmp.q      eq, C400, C002"),
    TEST_ENTRY_C(0x6CC01001, NULL, ".word       0x6CC01001                   # INVALID     $a2, $zero, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU3>"),
    TEST_ENTRY_C(0x6CC01081, NULL, ".word       0x6CC01081                   # INVALID     $a2, $zero, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU3>"),
    TEST_ENTRY_C(0x6CC09001, NULL, ".word       0x6CC09001                   # INVALID     $a2, $zero, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU3>"),
    TEST_ENTRY_C(0x6CC09081, NULL, ".word       0x6CC09081                   # INVALID     $a2, $zero, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU3>"),
    TEST_ENTRY_C(0x6D401001, NULL, "vmin.s      S010, S400, S002"),
    TEST_ENTRY_C(0x6D401081, NULL, "vmin.p      C010, C400, C002"),
    TEST_ENTRY_C(0x6D409001, NULL, "vmin.t      C010, C400, C001"),
    TEST_ENTRY_C(0x6D409081, NULL, "vmin.q      C010, C400, C002"),
    TEST_ENTRY_C(0x6DC01001, NULL, "vmax.s      S010, S400, S002"),
    TEST_ENTRY_C(0x6DC01081, NULL, "vmax.p      C010, C400, C002"),
    TEST_ENTRY_C(0x6DC09001, NULL, "vmax.t      C010, C400, C001"),
    TEST_ENTRY_C(0x6DC09081, NULL, "vmax.q      C010, C400, C002"),
    TEST_ENTRY_C(0x6E401001, NULL, ".word       0x6E401001                   # INVALID     $s2, $zero, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU3>"),
    TEST_ENTRY_C(0x6E401081, NULL, ".word       0x6E401081                   # INVALID     $s2, $zero, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU3>"),
    TEST_ENTRY_C(0x6E409001, NULL, ".word       0x6E409001                   # INVALID     $s2, $zero, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU3>"),
    TEST_ENTRY_C(0x6E409081, NULL, ".word       0x6E409081                   # INVALID     $s2, $zero, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU3>"),
    TEST_ENTRY_C(0x6EC01001, NULL, "vscmp.s     S010, S400, S002"),
    TEST_ENTRY_C(0x6EC01081, NULL, "vscmp.p     C010, C400, C002"),
    TEST_ENTRY_C(0x6EC09001, NULL, "vscmp.t     C010, C400, C001"),
    TEST_ENTRY_C(0x6EC09081, NULL, "vscmp.q     C010, C400, C002"),
    TEST_ENTRY_C(0x6F401001, NULL, "vsge.s      S010, S400, S002"),
    TEST_ENTRY_C(0x6F401081, NULL, "vsge.p      C010, C400, C002"),
    TEST_ENTRY_C(0x6F409001, NULL, "vsge.t      C010, C400, C001"),
    TEST_ENTRY_C(0x6F409081, NULL, "vsge.q      C010, C400, C002"),
    TEST_ENTRY_C(0x6FC01001, NULL, "vslt.s      S010, S400, S002"),
    TEST_ENTRY_C(0x6FC01081, NULL, "vslt.p      C010, C400, C002"),
    TEST_ENTRY_C(0x6FC09001, NULL, "vslt.t      C010, C400, C001"),
    TEST_ENTRY_C(0x6FC09081, NULL, "vslt.q      C010, C400, C002"),

    /* vcmp all conds */
    TEST_ENTRY_C(0x6C400800, NULL, "vcmp.s      fl, S200, S002"),
    TEST_ENTRY_C(0x6C400880, NULL, "vcmp.p      fl, C200, C002"),
    TEST_ENTRY_C(0x6C408800, NULL, "vcmp.t      fl, C200, C001"),
    TEST_ENTRY_C(0x6C408880, NULL, "vcmp.q      fl, C200, C002"),
    TEST_ENTRY_C(0x6C400801, NULL, "vcmp.s      eq, S200, S002"),
    TEST_ENTRY_C(0x6C400881, NULL, "vcmp.p      eq, C200, C002"),
    TEST_ENTRY_C(0x6C408801, NULL, "vcmp.t      eq, C200, C001"),
    TEST_ENTRY_C(0x6C408881, NULL, "vcmp.q      eq, C200, C002"),
    TEST_ENTRY_C(0x6C400802, NULL, "vcmp.s      lt, S200, S002"),
    TEST_ENTRY_C(0x6C400882, NULL, "vcmp.p      lt, C200, C002"),
    TEST_ENTRY_C(0x6C408802, NULL, "vcmp.t      lt, C200, C001"),
    TEST_ENTRY_C(0x6C408882, NULL, "vcmp.q      lt, C200, C002"),
    TEST_ENTRY_C(0x6C400803, NULL, "vcmp.s      le, S200, S002"),
    TEST_ENTRY_C(0x6C400883, NULL, "vcmp.p      le, C200, C002"),
    TEST_ENTRY_C(0x6C408803, NULL, "vcmp.t      le, C200, C001"),
    TEST_ENTRY_C(0x6C408883, NULL, "vcmp.q      le, C200, C002"),
    TEST_ENTRY_C(0x6C400804, NULL, "vcmp.s      tr, S200, S002"),
    TEST_ENTRY_C(0x6C400884, NULL, "vcmp.p      tr, C200, C002"),
    TEST_ENTRY_C(0x6C408804, NULL, "vcmp.t      tr, C200, C001"),
    TEST_ENTRY_C(0x6C408884, NULL, "vcmp.q      tr, C200, C002"),
    TEST_ENTRY_C(0x6C400805, NULL, "vcmp.s      ne, S200, S002"),
    TEST_ENTRY_C(0x6C400885, NULL, "vcmp.p      ne, C200, C002"),
    TEST_ENTRY_C(0x6C408805, NULL, "vcmp.t      ne, C200, C001"),
    TEST_ENTRY_C(0x6C408885, NULL, "vcmp.q      ne, C200, C002"),
    TEST_ENTRY_C(0x6C400806, NULL, "vcmp.s      ge, S200, S002"),
    TEST_ENTRY_C(0x6C400886, NULL, "vcmp.p      ge, C200, C002"),
    TEST_ENTRY_C(0x6C408806, NULL, "vcmp.t      ge, C200, C001"),
    TEST_ENTRY_C(0x6C408886, NULL, "vcmp.q      ge, C200, C002"),
    TEST_ENTRY_C(0x6C400807, NULL, "vcmp.s      gt, S200, S002"),
    TEST_ENTRY_C(0x6C400887, NULL, "vcmp.p      gt, C200, C002"),
    TEST_ENTRY_C(0x6C408807, NULL, "vcmp.t      gt, C200, C001"),
    TEST_ENTRY_C(0x6C408887, NULL, "vcmp.q      gt, C200, C002"),
    TEST_ENTRY_C(0x6C400808, NULL, "vcmp.s      ez, S200, S002"),
    TEST_ENTRY_C(0x6C400888, NULL, "vcmp.p      ez, C200, C002"),
    TEST_ENTRY_C(0x6C408808, NULL, "vcmp.t      ez, C200, C001"),
    TEST_ENTRY_C(0x6C408888, NULL, "vcmp.q      ez, C200, C002"),
    TEST_ENTRY_C(0x6C400809, NULL, "vcmp.s      en, S200, S002"),
    TEST_ENTRY_C(0x6C400889, NULL, "vcmp.p      en, C200, C002"),
    TEST_ENTRY_C(0x6C408809, NULL, "vcmp.t      en, C200, C001"),
    TEST_ENTRY_C(0x6C408889, NULL, "vcmp.q      en, C200, C002"),
    TEST_ENTRY_C(0x6C40080A, NULL, "vcmp.s      ei, S200, S002"),
    TEST_ENTRY_C(0x6C40088A, NULL, "vcmp.p      ei, C200, C002"),
    TEST_ENTRY_C(0x6C40880A, NULL, "vcmp.t      ei, C200, C001"),
    TEST_ENTRY_C(0x6C40888A, NULL, "vcmp.q      ei, C200, C002"),
    TEST_ENTRY_C(0x6C40080B, NULL, "vcmp.s      es, S200, S002"),
    TEST_ENTRY_C(0x6C40088B, NULL, "vcmp.p      es, C200, C002"),
    TEST_ENTRY_C(0x6C40880B, NULL, "vcmp.t      es, C200, C001"),
    TEST_ENTRY_C(0x6C40888B, NULL, "vcmp.q      es, C200, C002"),
    TEST_ENTRY_C(0x6C40080C, NULL, "vcmp.s      nz, S200, S002"),
    TEST_ENTRY_C(0x6C40088C, NULL, "vcmp.p      nz, C200, C002"),
    TEST_ENTRY_C(0x6C40880C, NULL, "vcmp.t      nz, C200, C001"),
    TEST_ENTRY_C(0x6C40888C, NULL, "vcmp.q      nz, C200, C002"),
    TEST_ENTRY_C(0x6C40080D, NULL, "vcmp.s      nn, S200, S002"),
    TEST_ENTRY_C(0x6C40088D, NULL, "vcmp.p      nn, C200, C002"),
    TEST_ENTRY_C(0x6C40880D, NULL, "vcmp.t      nn, C200, C001"),
    TEST_ENTRY_C(0x6C40888D, NULL, "vcmp.q      nn, C200, C002"),
    TEST_ENTRY_C(0x6C40080E, NULL, "vcmp.s      ni, S200, S002"),
    TEST_ENTRY_C(0x6C40088E, NULL, "vcmp.p      ni, C200, C002"),
    TEST_ENTRY_C(0x6C40880E, NULL, "vcmp.t      ni, C200, C001"),
    TEST_ENTRY_C(0x6C40888E, NULL, "vcmp.q      ni, C200, C002"),
    TEST_ENTRY_C(0x6C40080F, NULL, "vcmp.s      ns, S200, S002"),
    TEST_ENTRY_C(0x6C40088F, NULL, "vcmp.p      ns, C200, C002"),
    TEST_ENTRY_C(0x6C40880F, NULL, "vcmp.t      ns, C200, C001"),
    TEST_ENTRY_C(0x6C40888F, NULL, "vcmp.q      ns, C200, C002"),

    /* vcmp_zero */
    TEST_ENTRY_C(0x6C000000, NULL, "vcmp.s      fl"),
    TEST_ENTRY_C(0x6C000080, NULL, "vcmp.p      fl"),
    TEST_ENTRY_C(0x6C008000, NULL, "vcmp.t      fl"),
    TEST_ENTRY_C(0x6C008080, NULL, "vcmp.q      fl"),
    TEST_ENTRY_C(0x6C000001, NULL, "vcmp.s      eq, S000, S000"),
    TEST_ENTRY_C(0x6C000081, NULL, "vcmp.p      eq, C000, C000"),
    TEST_ENTRY_C(0x6C008001, NULL, "vcmp.t      eq, C000, C000"),
    TEST_ENTRY_C(0x6C008081, NULL, "vcmp.q      eq, C000, C000"),
    TEST_ENTRY_C(0x6C000002, NULL, "vcmp.s      lt, S000, S000"),
    TEST_ENTRY_C(0x6C000082, NULL, "vcmp.p      lt, C000, C000"),
    TEST_ENTRY_C(0x6C008002, NULL, "vcmp.t      lt, C000, C000"),
    TEST_ENTRY_C(0x6C008082, NULL, "vcmp.q      lt, C000, C000"),
    TEST_ENTRY_C(0x6C000003, NULL, "vcmp.s      le, S000, S000"),
    TEST_ENTRY_C(0x6C000083, NULL, "vcmp.p      le, C000, C000"),
    TEST_ENTRY_C(0x6C008003, NULL, "vcmp.t      le, C000, C000"),
    TEST_ENTRY_C(0x6C008083, NULL, "vcmp.q      le, C000, C000"),
    TEST_ENTRY_C(0x6C000004, NULL, "vcmp.s      tr"),
    TEST_ENTRY_C(0x6C000084, NULL, "vcmp.p      tr"),
    TEST_ENTRY_C(0x6C008004, NULL, "vcmp.t      tr"),
    TEST_ENTRY_C(0x6C008084, NULL, "vcmp.q      tr"),
    TEST_ENTRY_C(0x6C000005, NULL, "vcmp.s      ne, S000, S000"),
    TEST_ENTRY_C(0x6C000085, NULL, "vcmp.p      ne, C000, C000"),
    TEST_ENTRY_C(0x6C008005, NULL, "vcmp.t      ne, C000, C000"),
    TEST_ENTRY_C(0x6C008085, NULL, "vcmp.q      ne, C000, C000"),
    TEST_ENTRY_C(0x6C000006, NULL, "vcmp.s      ge, S000, S000"),
    TEST_ENTRY_C(0x6C000086, NULL, "vcmp.p      ge, C000, C000"),
    TEST_ENTRY_C(0x6C008006, NULL, "vcmp.t      ge, C000, C000"),
    TEST_ENTRY_C(0x6C008086, NULL, "vcmp.q      ge, C000, C000"),
    TEST_ENTRY_C(0x6C000007, NULL, "vcmp.s      gt, S000, S000"),
    TEST_ENTRY_C(0x6C000087, NULL, "vcmp.p      gt, C000, C000"),
    TEST_ENTRY_C(0x6C008007, NULL, "vcmp.t      gt, C000, C000"),
    TEST_ENTRY_C(0x6C008087, NULL, "vcmp.q      gt, C000, C000"),
    TEST_ENTRY_C(0x6C000008, NULL, "vcmp.s      ez, S000"),
    TEST_ENTRY_C(0x6C000088, NULL, "vcmp.p      ez, C000"),
    TEST_ENTRY_C(0x6C008008, NULL, "vcmp.t      ez, C000"),
    TEST_ENTRY_C(0x6C008088, NULL, "vcmp.q      ez, C000"),
    TEST_ENTRY_C(0x6C000009, NULL, "vcmp.s      en, S000"),
    TEST_ENTRY_C(0x6C000089, NULL, "vcmp.p      en, C000"),
    TEST_ENTRY_C(0x6C008009, NULL, "vcmp.t      en, C000"),
    TEST_ENTRY_C(0x6C008089, NULL, "vcmp.q      en, C000"),
    TEST_ENTRY_C(0x6C00000A, NULL, "vcmp.s      ei, S000"),
    TEST_ENTRY_C(0x6C00008A, NULL, "vcmp.p      ei, C000"),
    TEST_ENTRY_C(0x6C00800A, NULL, "vcmp.t      ei, C000"),
    TEST_ENTRY_C(0x6C00808A, NULL, "vcmp.q      ei, C000"),
    TEST_ENTRY_C(0x6C00000B, NULL, "vcmp.s      es, S000"),
    TEST_ENTRY_C(0x6C00008B, NULL, "vcmp.p      es, C000"),
    TEST_ENTRY_C(0x6C00800B, NULL, "vcmp.t      es, C000"),
    TEST_ENTRY_C(0x6C00808B, NULL, "vcmp.q      es, C000"),
    TEST_ENTRY_C(0x6C00000C, NULL, "vcmp.s      nz, S000"),
    TEST_ENTRY_C(0x6C00008C, NULL, "vcmp.p      nz, C000"),
    TEST_ENTRY_C(0x6C00800C, NULL, "vcmp.t      nz, C000"),
    TEST_ENTRY_C(0x6C00808C, NULL, "vcmp.q      nz, C000"),
    TEST_ENTRY_C(0x6C00000D, NULL, "vcmp.s      nn, S000"),
    TEST_ENTRY_C(0x6C00008D, NULL, "vcmp.p      nn, C000"),
    TEST_ENTRY_C(0x6C00800D, NULL, "vcmp.t      nn, C000"),
    TEST_ENTRY_C(0x6C00808D, NULL, "vcmp.q      nn, C000"),
    TEST_ENTRY_C(0x6C00000E, NULL, "vcmp.s      ni, S000"),
    TEST_ENTRY_C(0x6C00008E, NULL, "vcmp.p      ni, C000"),
    TEST_ENTRY_C(0x6C00800E, NULL, "vcmp.t      ni, C000"),
    TEST_ENTRY_C(0x6C00808E, NULL, "vcmp.q      ni, C000"),
    TEST_ENTRY_C(0x6C00000F, NULL, "vcmp.s      ns, S000"),
    TEST_ENTRY_C(0x6C00008F, NULL, "vcmp.p      ns, C000"),
    TEST_ENTRY_C(0x6C00800F, NULL, "vcmp.t      ns, C000"),
    TEST_ENTRY_C(0x6C00808F, NULL, "vcmp.q      ns, C000"),

    /* COP2 */
    TEST_ENTRY_C(0x48640080, NULL, "mfvc        $a0, $128"),
    TEST_ENTRY_C(0x48640081, NULL, "mfvc        $a0, $129"),
    TEST_ENTRY_C(0x48640082, NULL, "mfvc        $a0, $130"),
    TEST_ENTRY_C(0x48640083, NULL, "mfvc        $a0, $131"),
    TEST_ENTRY_C(0x48640084, NULL, "mfvc        $a0, $132"),
    TEST_ENTRY_C(0x48640085, NULL, "mfvc        $a0, $133"),
    TEST_ENTRY_C(0x48640086, NULL, "mfvc        $a0, $134"),
    TEST_ENTRY_C(0x48640087, NULL, "mfvc        $a0, $135"),
    TEST_ENTRY_C(0x48640088, NULL, "mfvc        $a0, $136"),
    TEST_ENTRY_C(0x48640089, NULL, "mfvc        $a0, $137"),
    TEST_ENTRY_C(0x4864008A, NULL, "mfvc        $a0, $138"),
    TEST_ENTRY_C(0x4864008B, NULL, "mfvc        $a0, $139"),
    TEST_ENTRY_C(0x4864008C, NULL, "mfvc        $a0, $140"),
    TEST_ENTRY_C(0x4864008D, NULL, "mfvc        $a0, $141"),
    TEST_ENTRY_C(0x4864008E, NULL, "mfvc        $a0, $142"),
    TEST_ENTRY_C(0x4864008F, NULL, "mfvc        $a0, $143"),
    TEST_ENTRY_C(0x48640090, NULL, ".word       0x48640090                   # INVALID     $v1, $a0, 0x90 # 00000000 <InstrIdType: R4000ALLEGREX_COP2_MFHC2_P>"),
    TEST_ENTRY_C(0x48640091, NULL, ".word       0x48640091                   # INVALID     $v1, $a0, 0x91 # 00000000 <InstrIdType: R4000ALLEGREX_COP2_MFHC2_P>"),
    TEST_ENTRY_C(0x48640092, NULL, ".word       0x48640092                   # INVALID     $v1, $a0, 0x92 # 00000000 <InstrIdType: R4000ALLEGREX_COP2_MFHC2_P>"),
    TEST_ENTRY_C(0x486000FF, NULL, "vsync2"),
    TEST_ENTRY_C(0x48650000, NULL, "mfv         $a1, S000"),
    TEST_ENTRY_C(0x48650081, NULL, "mfvc        $a1, $129"),
    TEST_ENTRY_C(0x48E50000, NULL, "mtv         $a1, S000"),
    TEST_ENTRY_C(0x48E50081, NULL, "mtvc        $a1, $129"),

    /* VFPU4 FMT0 */
    TEST_ENTRY_C(0xD0001001, NULL, "vmov.s      S010, S400"),
    TEST_ENTRY_C(0xD0001081, NULL, "vmov.p      C010, C400"),
    TEST_ENTRY_C(0xD0009001, NULL, "vmov.t      C010, C400"),
    TEST_ENTRY_C(0xD0009081, NULL, "vmov.q      C010, C400"),
    TEST_ENTRY_C(0xD0011001, NULL, "vabs.s      S010, S400"),
    TEST_ENTRY_C(0xD0011081, NULL, "vabs.p      C010, C400"),
    TEST_ENTRY_C(0xD0019001, NULL, "vabs.t      C010, C400"),
    TEST_ENTRY_C(0xD0019081, NULL, "vabs.q      C010, C400"),
    TEST_ENTRY_C(0xD0021001, NULL, "vneg.s      S010, S400"),
    TEST_ENTRY_C(0xD0021081, NULL, "vneg.p      C010, C400"),
    TEST_ENTRY_C(0xD0029001, NULL, "vneg.t      C010, C400"),
    TEST_ENTRY_C(0xD0029081, NULL, "vneg.q      C010, C400"),
    TEST_ENTRY_C(0xD0031001, NULL, ".word       0xD0031001                   # INVALID     $zero, $v1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT0>"),
    TEST_ENTRY_C(0xD0031081, NULL, ".word       0xD0031081                   # vidt.p      C010 # 00001000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT0>"),
    TEST_ENTRY_C(0xD0030081, NULL, "vidt.p      C010"),
    TEST_ENTRY_C(0xD0039001, NULL, ".word       0xD0039001                   # INVALID     $zero, $v1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT0>"),
    TEST_ENTRY_C(0xD0039081, NULL, ".word       0xD0039081                   # vidt.q      C010 # 00001000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT0>"),
    TEST_ENTRY_C(0xD0038081, NULL, "vidt.q      C010"),
    TEST_ENTRY_C(0xD0041001, NULL, "vsat0.s     S010, S400"),
    TEST_ENTRY_C(0xD0041081, NULL, "vsat0.p     C010, C400"),
    TEST_ENTRY_C(0xD0049001, NULL, "vsat0.t     C010, C400"),
    TEST_ENTRY_C(0xD0049081, NULL, "vsat0.q     C010, C400"),
    TEST_ENTRY_C(0xD0051001, NULL, "vsat1.s     S010, S400"),
    TEST_ENTRY_C(0xD0051081, NULL, "vsat1.p     C010, C400"),
    TEST_ENTRY_C(0xD0059001, NULL, "vsat1.t     C010, C400"),
    TEST_ENTRY_C(0xD0059081, NULL, "vsat1.q     C010, C400"),
    TEST_ENTRY_C(0xD0061001, NULL, ".word       0xD0061001                   # vzero.s     S010 # 00001000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT0>"),
    TEST_ENTRY_C(0xD0061081, NULL, ".word       0xD0061081                   # vzero.p     C010 # 00001000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT0>"),
    TEST_ENTRY_C(0xD0069001, NULL, ".word       0xD0069001                   # vzero.t     C010 # 00001000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT0>"),
    TEST_ENTRY_C(0xD0069081, NULL, ".word       0xD0069081                   # vzero.q     C010 # 00001000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT0>"),
    TEST_ENTRY_C(0xD0070001, NULL, "vone.s      S010"),
    TEST_ENTRY_C(0xD0070081, NULL, "vone.p      C010"),
    TEST_ENTRY_C(0xD0078001, NULL, "vone.t      C010"),
    TEST_ENTRY_C(0xD0078081, NULL, "vone.q      C010"),
    TEST_ENTRY_C(0xD0081001, NULL, ".word       0xD0081001                   # INVALID     $zero, $t0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0081081, NULL, ".word       0xD0081081                   # INVALID     $zero, $t0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0089001, NULL, ".word       0xD0089001                   # INVALID     $zero, $t0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0089081, NULL, ".word       0xD0089081                   # INVALID     $zero, $t0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0091001, NULL, ".word       0xD0091001                   # INVALID     $zero, $t1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0091081, NULL, ".word       0xD0091081                   # INVALID     $zero, $t1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0099001, NULL, ".word       0xD0099001                   # INVALID     $zero, $t1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0099081, NULL, ".word       0xD0099081                   # INVALID     $zero, $t1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD00A1001, NULL, ".word       0xD00A1001                   # INVALID     $zero, $t2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD00A1081, NULL, ".word       0xD00A1081                   # INVALID     $zero, $t2, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD00A9001, NULL, ".word       0xD00A9001                   # INVALID     $zero, $t2, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD00A9081, NULL, ".word       0xD00A9081                   # INVALID     $zero, $t2, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD00B1001, NULL, ".word       0xD00B1001                   # INVALID     $zero, $t3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD00B1081, NULL, ".word       0xD00B1081                   # INVALID     $zero, $t3, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD00B9001, NULL, ".word       0xD00B9001                   # INVALID     $zero, $t3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD00B9081, NULL, ".word       0xD00B9081                   # INVALID     $zero, $t3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD00C1001, NULL, ".word       0xD00C1001                   # INVALID     $zero, $t4, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD00C1081, NULL, ".word       0xD00C1081                   # INVALID     $zero, $t4, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD00C9001, NULL, ".word       0xD00C9001                   # INVALID     $zero, $t4, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD00C9081, NULL, ".word       0xD00C9081                   # INVALID     $zero, $t4, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD00D1001, NULL, ".word       0xD00D1001                   # INVALID     $zero, $t5, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD00D1081, NULL, ".word       0xD00D1081                   # INVALID     $zero, $t5, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD00D9001, NULL, ".word       0xD00D9001                   # INVALID     $zero, $t5, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD00D9081, NULL, ".word       0xD00D9081                   # INVALID     $zero, $t5, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD00E1001, NULL, ".word       0xD00E1001                   # INVALID     $zero, $t6, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD00E1081, NULL, ".word       0xD00E1081                   # INVALID     $zero, $t6, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD00E9001, NULL, ".word       0xD00E9001                   # INVALID     $zero, $t6, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD00E9081, NULL, ".word       0xD00E9081                   # INVALID     $zero, $t6, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD00F1001, NULL, ".word       0xD00F1001                   # INVALID     $zero, $t7, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD00F1081, NULL, ".word       0xD00F1081                   # INVALID     $zero, $t7, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD00F9001, NULL, ".word       0xD00F9001                   # INVALID     $zero, $t7, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD00F9081, NULL, ".word       0xD00F9081                   # INVALID     $zero, $t7, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0101001, NULL, "vrcp.s      S010, S400"),
    TEST_ENTRY_C(0xD0101081, NULL, "vrcp.p      C010, C400"),
    TEST_ENTRY_C(0xD0109001, NULL, "vrcp.t      C010, C400"),
    TEST_ENTRY_C(0xD0109081, NULL, "vrcp.q      C010, C400"),
    TEST_ENTRY_C(0xD0111001, NULL, "vrsq.s      S010, S400"),
    TEST_ENTRY_C(0xD0111081, NULL, "vrsq.p      C010, C400"),
    TEST_ENTRY_C(0xD0119001, NULL, "vrsq.t      C010, C400"),
    TEST_ENTRY_C(0xD0119081, NULL, "vrsq.q      C010, C400"),
    TEST_ENTRY_C(0xD0121001, NULL, "vsin.s      S010, S400"),
    TEST_ENTRY_C(0xD0121081, NULL, "vsin.p      C010, C400"),
    TEST_ENTRY_C(0xD0129001, NULL, "vsin.t      C010, C400"),
    TEST_ENTRY_C(0xD0129081, NULL, "vsin.q      C010, C400"),
    TEST_ENTRY_C(0xD0131001, NULL, "vcos.s      S010, S400"),
    TEST_ENTRY_C(0xD0131081, NULL, "vcos.p      C010, C400"),
    TEST_ENTRY_C(0xD0139001, NULL, "vcos.t      C010, C400"),
    TEST_ENTRY_C(0xD0139081, NULL, "vcos.q      C010, C400"),
    TEST_ENTRY_C(0xD0141001, NULL, "vexp2.s     S010, S400"),
    TEST_ENTRY_C(0xD0141081, NULL, "vexp2.p     C010, C400"),
    TEST_ENTRY_C(0xD0149001, NULL, "vexp2.t     C010, C400"),
    TEST_ENTRY_C(0xD0149081, NULL, "vexp2.q     C010, C400"),
    TEST_ENTRY_C(0xD0151001, NULL, "vlog2.s     S010, S400"),
    TEST_ENTRY_C(0xD0151081, NULL, "vlog2.p     C010, C400"),
    TEST_ENTRY_C(0xD0159001, NULL, "vlog2.t     C010, C400"),
    TEST_ENTRY_C(0xD0159081, NULL, "vlog2.q     C010, C400"),
    TEST_ENTRY_C(0xD0161001, NULL, "vsqrt.s     S010, S400"),
    TEST_ENTRY_C(0xD0161081, NULL, "vsqrt.p     C010, C400"),
    TEST_ENTRY_C(0xD0169001, NULL, "vsqrt.t     C010, C400"),
    TEST_ENTRY_C(0xD0169081, NULL, "vsqrt.q     C010, C400"),
    TEST_ENTRY_C(0xD0171001, NULL, "vasin.s     S010, S400"),
    TEST_ENTRY_C(0xD0171081, NULL, "vasin.p     C010, C400"),
    TEST_ENTRY_C(0xD0179001, NULL, "vasin.t     C010, C400"),
    TEST_ENTRY_C(0xD0179081, NULL, "vasin.q     C010, C400"),
    TEST_ENTRY_C(0xD0181001, NULL, "vnrcp.s     S010, S400"),
    TEST_ENTRY_C(0xD0181081, NULL, "vnrcp.p     C010, C400"),
    TEST_ENTRY_C(0xD0189001, NULL, "vnrcp.t     C010, C400"),
    TEST_ENTRY_C(0xD0189081, NULL, "vnrcp.q     C010, C400"),
    TEST_ENTRY_C(0xD0191001, NULL, ".word       0xD0191001                   # INVALID     $zero, $t9, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT3>"),
    TEST_ENTRY_C(0xD0191081, NULL, ".word       0xD0191081                   # INVALID     $zero, $t9, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT3>"),
    TEST_ENTRY_C(0xD0199001, NULL, ".word       0xD0199001                   # INVALID     $zero, $t9, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT3>"),
    TEST_ENTRY_C(0xD0199081, NULL, ".word       0xD0199081                   # INVALID     $zero, $t9, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT3>"),
    TEST_ENTRY_C(0xD01A1001, NULL, "vnsin.s     S010, S400"),
    TEST_ENTRY_C(0xD01A1081, NULL, "vnsin.p     C010, C400"),
    TEST_ENTRY_C(0xD01A9001, NULL, "vnsin.t     C010, C400"),
    TEST_ENTRY_C(0xD01A9081, NULL, "vnsin.q     C010, C400"),
    TEST_ENTRY_C(0xD01B1001, NULL, ".word       0xD01B1001                   # INVALID     $zero, $k1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT3>"),
    TEST_ENTRY_C(0xD01B1081, NULL, ".word       0xD01B1081                   # INVALID     $zero, $k1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT3>"),
    TEST_ENTRY_C(0xD01B9001, NULL, ".word       0xD01B9001                   # INVALID     $zero, $k1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT3>"),
    TEST_ENTRY_C(0xD01B9081, NULL, ".word       0xD01B9081                   # INVALID     $zero, $k1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT3>"),
    TEST_ENTRY_C(0xD01C1001, NULL, "vrexp2.s    S010, S400"),
    TEST_ENTRY_C(0xD01C1081, NULL, "vrexp2.p    C010, C400"),
    TEST_ENTRY_C(0xD01C9001, NULL, "vrexp2.t    C010, C400"),
    TEST_ENTRY_C(0xD01C9081, NULL, "vrexp2.q    C010, C400"),
    TEST_ENTRY_C(0xD01D1001, NULL, ".word       0xD01D1001                   # INVALID     $zero, $sp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT3>"),
    TEST_ENTRY_C(0xD01D1081, NULL, ".word       0xD01D1081                   # INVALID     $zero, $sp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT3>"),
    TEST_ENTRY_C(0xD01D9001, NULL, ".word       0xD01D9001                   # INVALID     $zero, $sp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT3>"),
    TEST_ENTRY_C(0xD01D9081, NULL, ".word       0xD01D9081                   # INVALID     $zero, $sp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT3>"),
    TEST_ENTRY_C(0xD01E1001, NULL, ".word       0xD01E1001                   # INVALID     $zero, $fp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT3>"),
    TEST_ENTRY_C(0xD01E1081, NULL, ".word       0xD01E1081                   # INVALID     $zero, $fp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT3>"),
    TEST_ENTRY_C(0xD01E9001, NULL, ".word       0xD01E9001                   # INVALID     $zero, $fp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT3>"),
    TEST_ENTRY_C(0xD01E9081, NULL, ".word       0xD01E9081                   # INVALID     $zero, $fp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT3>"),
    TEST_ENTRY_C(0xD01F1001, NULL, ".word       0xD01F1001                   # INVALID     $zero, $ra, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT3>"),
    TEST_ENTRY_C(0xD01F1081, NULL, ".word       0xD01F1081                   # INVALID     $zero, $ra, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT3>"),
    TEST_ENTRY_C(0xD01F9001, NULL, ".word       0xD01F9001                   # INVALID     $zero, $ra, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT3>"),
    TEST_ENTRY_C(0xD01F9081, NULL, ".word       0xD01F9081                   # INVALID     $zero, $ra, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT3>"),
    TEST_ENTRY_C(0xD0201000, NULL, "vrnds.s     S400"),
    TEST_ENTRY_C(0xD0201001, NULL, ".word       0xD0201001                   # vrnds.s     S400 # 00000001 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0201081, NULL, ".word       0xD0201081                   # INVALID     $at, $zero, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0209001, NULL, ".word       0xD0209001                   # INVALID     $at, $zero, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0209081, NULL, ".word       0xD0209081                   # INVALID     $at, $zero, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0210001, NULL, "vrndi.s     S010"),
    TEST_ENTRY_C(0xD0210081, NULL, "vrndi.p     C010"),
    TEST_ENTRY_C(0xD0218001, NULL, "vrndi.t     C010"),
    TEST_ENTRY_C(0xD0218081, NULL, "vrndi.q     C010"),
    TEST_ENTRY_C(0xD0211001, NULL, ".word       0xD0211001                   # vrndi.s     S010 # 00001000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0211081, NULL, ".word       0xD0211081                   # vrndi.p     C010 # 00001000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0219001, NULL, ".word       0xD0219001                   # vrndi.t     C010 # 00001000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0219081, NULL, ".word       0xD0219081                   # vrndi.q     C010 # 00001000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0220001, NULL, "vrndf1.s    S010"),
    TEST_ENTRY_C(0xD0220081, NULL, "vrndf1.p    C010"),
    TEST_ENTRY_C(0xD0228001, NULL, "vrndf1.t    C010"),
    TEST_ENTRY_C(0xD0228081, NULL, "vrndf1.q    C010"),
    TEST_ENTRY_C(0xD0221001, NULL, ".word       0xD0221001                   # vrndf1.s    S010 # 00001000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0221081, NULL, ".word       0xD0221081                   # vrndf1.p    C010 # 00001000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0229001, NULL, ".word       0xD0229001                   # vrndf1.t    C010 # 00001000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0229081, NULL, ".word       0xD0229081                   # vrndf1.q    C010 # 00001000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0230001, NULL, "vrndf2.s    S010"),
    TEST_ENTRY_C(0xD0230081, NULL, "vrndf2.p    C010"),
    TEST_ENTRY_C(0xD0238001, NULL, "vrndf2.t    C010"),
    TEST_ENTRY_C(0xD0238081, NULL, "vrndf2.q    C010"),
    TEST_ENTRY_C(0xD0231001, NULL, ".word       0xD0231001                   # vrndf2.s    S010 # 00001000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0231081, NULL, ".word       0xD0231081                   # vrndf2.p    C010 # 00001000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0239001, NULL, ".word       0xD0239001                   # vrndf2.t    C010 # 00001000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0239081, NULL, ".word       0xD0239081                   # vrndf2.q    C010 # 00001000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0241001, NULL, ".word       0xD0241001                   # INVALID     $at, $a0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0241081, NULL, ".word       0xD0241081                   # INVALID     $at, $a0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0249001, NULL, ".word       0xD0249001                   # INVALID     $at, $a0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0249081, NULL, ".word       0xD0249081                   # INVALID     $at, $a0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0251001, NULL, ".word       0xD0251001                   # INVALID     $at, $a1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0251081, NULL, ".word       0xD0251081                   # INVALID     $at, $a1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0259001, NULL, ".word       0xD0259001                   # INVALID     $at, $a1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0259081, NULL, ".word       0xD0259081                   # INVALID     $at, $a1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0261001, NULL, ".word       0xD0261001                   # INVALID     $at, $a2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0261081, NULL, ".word       0xD0261081                   # INVALID     $at, $a2, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0269001, NULL, ".word       0xD0269001                   # INVALID     $at, $a2, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0269081, NULL, ".word       0xD0269081                   # INVALID     $at, $a2, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0271001, NULL, ".word       0xD0271001                   # INVALID     $at, $a3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0271081, NULL, ".word       0xD0271081                   # INVALID     $at, $a3, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0279001, NULL, ".word       0xD0279001                   # INVALID     $at, $a3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0279081, NULL, ".word       0xD0279081                   # INVALID     $at, $a3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_RND>"),
    TEST_ENTRY_C(0xD0281001, NULL, ".word       0xD0281001                   # INVALID     $at, $t0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0281081, NULL, ".word       0xD0281081                   # INVALID     $at, $t0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0289001, NULL, ".word       0xD0289001                   # INVALID     $at, $t0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0289081, NULL, ".word       0xD0289081                   # INVALID     $at, $t0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0291001, NULL, ".word       0xD0291001                   # INVALID     $at, $t1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0291081, NULL, ".word       0xD0291081                   # INVALID     $at, $t1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0299001, NULL, ".word       0xD0299001                   # INVALID     $at, $t1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0299081, NULL, ".word       0xD0299081                   # INVALID     $at, $t1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD02A1001, NULL, ".word       0xD02A1001                   # INVALID     $at, $t2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD02A1081, NULL, ".word       0xD02A1081                   # INVALID     $at, $t2, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD02A9001, NULL, ".word       0xD02A9001                   # INVALID     $at, $t2, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD02A9081, NULL, ".word       0xD02A9081                   # INVALID     $at, $t2, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD02B1001, NULL, ".word       0xD02B1001                   # INVALID     $at, $t3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD02B1081, NULL, ".word       0xD02B1081                   # INVALID     $at, $t3, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD02B9001, NULL, ".word       0xD02B9001                   # INVALID     $at, $t3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD02B9081, NULL, ".word       0xD02B9081                   # INVALID     $at, $t3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD02C1001, NULL, ".word       0xD02C1001                   # INVALID     $at, $t4, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD02C1081, NULL, ".word       0xD02C1081                   # INVALID     $at, $t4, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD02C9001, NULL, ".word       0xD02C9001                   # INVALID     $at, $t4, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD02C9081, NULL, ".word       0xD02C9081                   # INVALID     $at, $t4, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD02D1001, NULL, ".word       0xD02D1001                   # INVALID     $at, $t5, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD02D1081, NULL, ".word       0xD02D1081                   # INVALID     $at, $t5, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD02D9001, NULL, ".word       0xD02D9001                   # INVALID     $at, $t5, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD02D9081, NULL, ".word       0xD02D9081                   # INVALID     $at, $t5, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD02E1001, NULL, ".word       0xD02E1001                   # INVALID     $at, $t6, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD02E1081, NULL, ".word       0xD02E1081                   # INVALID     $at, $t6, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD02E9001, NULL, ".word       0xD02E9001                   # INVALID     $at, $t6, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD02E9081, NULL, ".word       0xD02E9081                   # INVALID     $at, $t6, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD02F1001, NULL, ".word       0xD02F1001                   # INVALID     $at, $t7, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD02F1081, NULL, ".word       0xD02F1081                   # INVALID     $at, $t7, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD02F9001, NULL, ".word       0xD02F9001                   # INVALID     $at, $t7, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD02F9081, NULL, ".word       0xD02F9081                   # INVALID     $at, $t7, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0301001, NULL, ".word       0xD0301001                   # INVALID     $at, $s0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0301081, NULL, ".word       0xD0301081                   # INVALID     $at, $s0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0309001, NULL, ".word       0xD0309001                   # INVALID     $at, $s0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0309081, NULL, ".word       0xD0309081                   # INVALID     $at, $s0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0311001, NULL, ".word       0xD0311001                   # INVALID     $at, $s1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0311081, NULL, ".word       0xD0311081                   # INVALID     $at, $s1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0319001, NULL, ".word       0xD0319001                   # INVALID     $at, $s1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0319081, NULL, ".word       0xD0319081                   # INVALID     $at, $s1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0321001, NULL, ".word       0xD0321001                   # INVALID     $at, $s2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0321081, NULL, "vf2h.p      S010, C400"),
    TEST_ENTRY_C(0xD0329001, NULL, ".word       0xD0329001                   # INVALID     $at, $s2, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0329081, NULL, "vf2h.q      C010, C400"),
    TEST_ENTRY_C(0xD0331001, NULL, "vh2f.s      C010, S400"),
    TEST_ENTRY_C(0xD0331081, NULL, "vh2f.p      C010, C400"),
    TEST_ENTRY_C(0xD0339001, NULL, ".word       0xD0339001                   # INVALID     $at, $s3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0339081, NULL, ".word       0xD0339081                   # INVALID     $at, $s3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0341001, NULL, ".word       0xD0341001                   # INVALID     $at, $s4, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0341081, NULL, ".word       0xD0341081                   # INVALID     $at, $s4, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0349001, NULL, ".word       0xD0349001                   # INVALID     $at, $s4, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0349081, NULL, ".word       0xD0349081                   # INVALID     $at, $s4, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0351001, NULL, ".word       0xD0351001                   # INVALID     $at, $s5, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0351081, NULL, ".word       0xD0351081                   # INVALID     $at, $s5, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0359001, NULL, ".word       0xD0359001                   # INVALID     $at, $s5, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0359081, NULL, ".word       0xD0359081                   # INVALID     $at, $s5, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0361001, NULL, "vsbz.s      S010, S400"),
    TEST_ENTRY_C(0xD0361081, NULL, ".word       0xD0361081                   # INVALID     $at, $s6, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0369001, NULL, ".word       0xD0369001                   # INVALID     $at, $s6, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0369081, NULL, ".word       0xD0369081                   # INVALID     $at, $s6, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0371001, NULL, "vlgb.s      S010, S400"),
    TEST_ENTRY_C(0xD0371081, NULL, ".word       0xD0371081                   # INVALID     $at, $s7, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0379001, NULL, ".word       0xD0379001                   # INVALID     $at, $s7, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0379081, NULL, ".word       0xD0379081                   # INVALID     $at, $s7, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTFLT>"),
    TEST_ENTRY_C(0xD0381001, NULL, "vuc2ifs.s   C010, S400"),
    TEST_ENTRY_C(0xD0381081, NULL, ".word       0xD0381081                   # INVALID     $at, $t8, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTINT>"),
    TEST_ENTRY_C(0xD0389001, NULL, ".word       0xD0389001                   # INVALID     $at, $t8, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTINT>"),
    TEST_ENTRY_C(0xD0389081, NULL, ".word       0xD0389081                   # INVALID     $at, $t8, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTINT>"),
    TEST_ENTRY_C(0xD0391001, NULL, "vc2i.s      C010, S400"),
    TEST_ENTRY_C(0xD0391081, NULL, ".word       0xD0391081                   # INVALID     $at, $t9, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTINT>"),
    TEST_ENTRY_C(0xD0399001, NULL, ".word       0xD0399001                   # INVALID     $at, $t9, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTINT>"),
    TEST_ENTRY_C(0xD0399081, NULL, ".word       0xD0399081                   # INVALID     $at, $t9, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTINT>"),
    TEST_ENTRY_C(0xD03A1001, NULL, "vus2i.s     C010, S400"),
    TEST_ENTRY_C(0xD03A1081, NULL, "vus2i.p     C010, C400"),
    TEST_ENTRY_C(0xD03A9001, NULL, ".word       0xD03A9001                   # INVALID     $at, $k0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTINT>"),
    TEST_ENTRY_C(0xD03A9081, NULL, ".word       0xD03A9081                   # INVALID     $at, $k0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTINT>"),
    TEST_ENTRY_C(0xD03B1001, NULL, "vs2i.s      C010, S400"),
    TEST_ENTRY_C(0xD03B1081, NULL, "vs2i.p      C010, C400"),
    TEST_ENTRY_C(0xD03B9001, NULL, ".word       0xD03B9001                   # INVALID     $at, $k1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTINT>"),
    TEST_ENTRY_C(0xD03B9081, NULL, ".word       0xD03B9081                   # INVALID     $at, $k1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTINT>"),
    TEST_ENTRY_C(0xD03C1001, NULL, ".word       0xD03C1001                   # INVALID     $at, $gp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTINT>"),
    TEST_ENTRY_C(0xD03C1081, NULL, ".word       0xD03C1081                   # INVALID     $at, $gp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTINT>"),
    TEST_ENTRY_C(0xD03C9001, NULL, ".word       0xD03C9001                   # INVALID     $at, $gp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTINT>"),
    TEST_ENTRY_C(0xD03C9081, NULL, "vi2uc.q     S010, C400"),
    TEST_ENTRY_C(0xD03D1001, NULL, ".word       0xD03D1001                   # INVALID     $at, $sp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTINT>"),
    TEST_ENTRY_C(0xD03D1081, NULL, ".word       0xD03D1081                   # INVALID     $at, $sp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTINT>"),
    TEST_ENTRY_C(0xD03D9001, NULL, ".word       0xD03D9001                   # INVALID     $at, $sp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTINT>"),
    TEST_ENTRY_C(0xD03D9081, NULL, "vi2c.q      S010, C400"),
    TEST_ENTRY_C(0xD03E1001, NULL, ".word       0xD03E1001                   # INVALID     $at, $fp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTINT>"),
    TEST_ENTRY_C(0xD03E1081, NULL, "vi2us.p     S010, C400"),
    TEST_ENTRY_C(0xD03E9001, NULL, ".word       0xD03E9001                   # INVALID     $at, $fp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTINT>"),
    TEST_ENTRY_C(0xD03E9081, NULL, "vi2us.q     C010, C400"),
    TEST_ENTRY_C(0xD03F1001, NULL, ".word       0xD03F1001                   # INVALID     $at, $ra, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTINT>"),
    TEST_ENTRY_C(0xD03F1081, NULL, "vi2s.p      S010, C400"),
    TEST_ENTRY_C(0xD03F9001, NULL, ".word       0xD03F9001                   # INVALID     $at, $ra, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CVTINT>"),
    TEST_ENTRY_C(0xD03F9081, NULL, "vi2s.q      C010, C400"),
    TEST_ENTRY_C(0xD0401001, NULL, ".word       0xD0401001                   # INVALID     $v0, $zero, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT8>"),
    TEST_ENTRY_C(0xD0401081, NULL, ".word       0xD0401081                   # INVALID     $v0, $zero, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT8>"),
    TEST_ENTRY_C(0xD0409001, NULL, ".word       0xD0409001                   # INVALID     $v0, $zero, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT8>"),
    TEST_ENTRY_C(0xD0409081, NULL, "vsrt1.q     C010, C400"),
    TEST_ENTRY_C(0xD0411001, NULL, ".word       0xD0411001                   # INVALID     $v0, $at, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT8>"),
    TEST_ENTRY_C(0xD0411081, NULL, ".word       0xD0411081                   # INVALID     $v0, $at, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT8>"),
    TEST_ENTRY_C(0xD0419001, NULL, ".word       0xD0419001                   # INVALID     $v0, $at, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT8>"),
    TEST_ENTRY_C(0xD0419081, NULL, "vsrt2.q     C010, C400"),
    TEST_ENTRY_C(0xD0421001, NULL, ".word       0xD0421001                   # INVALID     $v0, $v0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT8>"),
    TEST_ENTRY_C(0xD0421081, NULL, "vbfy1.p     C010, C400"),
    TEST_ENTRY_C(0xD0429001, NULL, ".word       0xD0429001                   # INVALID     $v0, $v0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT8>"),
    TEST_ENTRY_C(0xD0429081, NULL, "vbfy1.q     C010, C400"),
    TEST_ENTRY_C(0xD0431001, NULL, ".word       0xD0431001                   # INVALID     $v0, $v1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT8>"),
    TEST_ENTRY_C(0xD0431081, NULL, ".word       0xD0431081                   # INVALID     $v0, $v1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT8>"),
    TEST_ENTRY_C(0xD0439001, NULL, ".word       0xD0439001                   # INVALID     $v0, $v1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT8>"),
    TEST_ENTRY_C(0xD0439081, NULL, "vbfy2.q     C010, C400"),
    TEST_ENTRY_C(0xD0441001, NULL, "vocp.s      S010, S400"),
    TEST_ENTRY_C(0xD0441081, NULL, "vocp.p      C010, C400"),
    TEST_ENTRY_C(0xD0449001, NULL, "vocp.t      C010, C400"),
    TEST_ENTRY_C(0xD0449081, NULL, "vocp.q      C010, C400"),
    TEST_ENTRY_C(0xD0451001, NULL, "vsocp.s     C010, S400"),
    TEST_ENTRY_C(0xD0451081, NULL, "vsocp.p     C010, C400"),
    TEST_ENTRY_C(0xD0459001, NULL, ".word       0xD0459001                   # INVALID     $v0, $a1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT8>"),
    TEST_ENTRY_C(0xD0459081, NULL, ".word       0xD0459081                   # INVALID     $v0, $a1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT8>"),
    TEST_ENTRY_C(0xD0461001, NULL, ".word       0xD0461001                   # INVALID     $v0, $a2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT8>"),
    TEST_ENTRY_C(0xD0461081, NULL, "vfad.p      S010, C400"),
    TEST_ENTRY_C(0xD0469001, NULL, "vfad.t      S010, C400"),
    TEST_ENTRY_C(0xD0469081, NULL, "vfad.q      S010, C400"),
    TEST_ENTRY_C(0xD0471001, NULL, ".word       0xD0471001                   # INVALID     $v0, $a3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT8>"),
    TEST_ENTRY_C(0xD0471081, NULL, "vavg.p      S010, C400"),
    TEST_ENTRY_C(0xD0479001, NULL, "vavg.t      S010, C400"),
    TEST_ENTRY_C(0xD0479081, NULL, "vavg.q      S010, C400"),
    TEST_ENTRY_C(0xD0481001, NULL, ".word       0xD0481001                   # INVALID     $v0, $t0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD0481081, NULL, ".word       0xD0481081                   # INVALID     $v0, $t0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD0489001, NULL, ".word       0xD0489001                   # INVALID     $v0, $t0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD0489081, NULL, "vsrt3.q     C010, C400"),
    TEST_ENTRY_C(0xD0491001, NULL, ".word       0xD0491001                   # INVALID     $v0, $t1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD0491081, NULL, ".word       0xD0491081                   # INVALID     $v0, $t1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD0499001, NULL, ".word       0xD0499001                   # INVALID     $v0, $t1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD0499081, NULL, "vsrt4.q     C010, C400"),
    TEST_ENTRY_C(0xD04A1001, NULL, "vsgn.s      S010, S400"),
    TEST_ENTRY_C(0xD04A1081, NULL, "vsgn.p      C010, C400"),
    TEST_ENTRY_C(0xD04A9001, NULL, "vsgn.t      C010, C400"),
    TEST_ENTRY_C(0xD04A9081, NULL, "vsgn.q      C010, C400"),
    TEST_ENTRY_C(0xD04B1001, NULL, ".word       0xD04B1001                   # INVALID     $v0, $t3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD04B1081, NULL, ".word       0xD04B1081                   # INVALID     $v0, $t3, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD04B9001, NULL, ".word       0xD04B9001                   # INVALID     $v0, $t3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD04B9081, NULL, ".word       0xD04B9081                   # INVALID     $v0, $t3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD04C1001, NULL, ".word       0xD04C1001                   # INVALID     $v0, $t4, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD04C1081, NULL, ".word       0xD04C1081                   # INVALID     $v0, $t4, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD04C9001, NULL, ".word       0xD04C9001                   # INVALID     $v0, $t4, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD04C9081, NULL, ".word       0xD04C9081                   # INVALID     $v0, $t4, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD04D1001, NULL, ".word       0xD04D1001                   # INVALID     $v0, $t5, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD04D1081, NULL, ".word       0xD04D1081                   # INVALID     $v0, $t5, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD04D9001, NULL, ".word       0xD04D9001                   # INVALID     $v0, $t5, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD04D9081, NULL, ".word       0xD04D9081                   # INVALID     $v0, $t5, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD04E1001, NULL, ".word       0xD04E1001                   # INVALID     $v0, $t6, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD04E1081, NULL, ".word       0xD04E1081                   # INVALID     $v0, $t6, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD04E9001, NULL, ".word       0xD04E9001                   # INVALID     $v0, $t6, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD04E9081, NULL, ".word       0xD04E9081                   # INVALID     $v0, $t6, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD04F1001, NULL, ".word       0xD04F1001                   # INVALID     $v0, $t7, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD04F1081, NULL, ".word       0xD04F1081                   # INVALID     $v0, $t7, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD04F9001, NULL, ".word       0xD04F9001                   # INVALID     $v0, $t7, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),
    TEST_ENTRY_C(0xD04F9081, NULL, ".word       0xD04F9081                   # INVALID     $v0, $t7, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_FMT9>"),

    TEST_ENTRY_C(0xD0508001, NULL, "vmfvc       S010, $128"),
    TEST_ENTRY_C(0xD0508101, NULL, "vmfvc       S010, $129"),
    TEST_ENTRY_C(0xD0508201, NULL, "vmfvc       S010, $130"),
    TEST_ENTRY_C(0xD0508301, NULL, "vmfvc       S010, $131"),
    TEST_ENTRY_C(0xD0508401, NULL, "vmfvc       S010, $132"),
    TEST_ENTRY_C(0xD0508501, NULL, "vmfvc       S010, $133"),
    TEST_ENTRY_C(0xD0508601, NULL, "vmfvc       S010, $134"),
    TEST_ENTRY_C(0xD0508701, NULL, "vmfvc       S010, $135"),
    TEST_ENTRY_C(0xD0508801, NULL, "vmfvc       S010, $136"),
    TEST_ENTRY_C(0xD0508901, NULL, "vmfvc       S010, $137"),
    TEST_ENTRY_C(0xD0508A01, NULL, "vmfvc       S010, $138"),
    TEST_ENTRY_C(0xD0508B01, NULL, "vmfvc       S010, $139"),
    TEST_ENTRY_C(0xD0508C01, NULL, "vmfvc       S010, $140"),
    TEST_ENTRY_C(0xD0508D01, NULL, "vmfvc       S010, $141"),
    TEST_ENTRY_C(0xD0508E01, NULL, "vmfvc       S010, $142"),
    TEST_ENTRY_C(0xD0508F01, NULL, "vmfvc       S010, $143"),
    TEST_ENTRY_C(0xD0509001, NULL, "vmfvc       S010, $144"),
    TEST_ENTRY_C(0xD0509101, NULL, "vmfvc       S010, $145"),
    TEST_ENTRY_C(0xD0509201, NULL, "vmfvc       S010, $146"),
    TEST_ENTRY_C(0xD0509301, NULL, "vmfvc       S010, $147"),
    TEST_ENTRY_C(0xD0509401, NULL, "vmfvc       S010, $148"),
    TEST_ENTRY_C(0xD0509501, NULL, "vmfvc       S010, $149"),
    TEST_ENTRY_C(0xD0509601, NULL, "vmfvc       S010, $150"),
    TEST_ENTRY_C(0xD0509701, NULL, "vmfvc       S010, $151"),
    TEST_ENTRY_C(0xD0509801, NULL, "vmfvc       S010, $152"),
    TEST_ENTRY_C(0xD0509901, NULL, "vmfvc       S010, $153"),
    TEST_ENTRY_C(0xD0509A01, NULL, "vmfvc       S010, $154"),
    TEST_ENTRY_C(0xD0509B01, NULL, "vmfvc       S010, $155"),
    TEST_ENTRY_C(0xD0509C01, NULL, "vmfvc       S010, $156"),
    TEST_ENTRY_C(0xD0509D01, NULL, "vmfvc       S010, $157"),
    TEST_ENTRY_C(0xD0509E01, NULL, "vmfvc       S010, $158"),
    TEST_ENTRY_C(0xD0509F01, NULL, "vmfvc       S010, $159"),
    TEST_ENTRY_C(0xD050A001, NULL, "vmfvc       S010, $160"),
    TEST_ENTRY_C(0xD050A101, NULL, "vmfvc       S010, $161"),
    TEST_ENTRY_C(0xD050A201, NULL, "vmfvc       S010, $162"),
    TEST_ENTRY_C(0xD050A301, NULL, "vmfvc       S010, $163"),
    TEST_ENTRY_C(0xD050A401, NULL, "vmfvc       S010, $164"),
    TEST_ENTRY_C(0xD050A501, NULL, "vmfvc       S010, $165"),
    TEST_ENTRY_C(0xD050A601, NULL, "vmfvc       S010, $166"),
    TEST_ENTRY_C(0xD050A701, NULL, "vmfvc       S010, $167"),
    TEST_ENTRY_C(0xD050A801, NULL, "vmfvc       S010, $168"),
    TEST_ENTRY_C(0xD050A901, NULL, "vmfvc       S010, $169"),
    TEST_ENTRY_C(0xD050AA01, NULL, "vmfvc       S010, $170"),
    TEST_ENTRY_C(0xD050AB01, NULL, "vmfvc       S010, $171"),
    TEST_ENTRY_C(0xD050AC01, NULL, "vmfvc       S010, $172"),
    TEST_ENTRY_C(0xD050AD01, NULL, "vmfvc       S010, $173"),
    TEST_ENTRY_C(0xD050AE01, NULL, "vmfvc       S010, $174"),
    TEST_ENTRY_C(0xD050AF01, NULL, "vmfvc       S010, $175"),
    TEST_ENTRY_C(0xD050B001, NULL, "vmfvc       S010, $176"),
    TEST_ENTRY_C(0xD050B101, NULL, "vmfvc       S010, $177"),
    TEST_ENTRY_C(0xD050B201, NULL, "vmfvc       S010, $178"),
    TEST_ENTRY_C(0xD050B301, NULL, "vmfvc       S010, $179"),
    TEST_ENTRY_C(0xD050B401, NULL, "vmfvc       S010, $180"),
    TEST_ENTRY_C(0xD050B501, NULL, "vmfvc       S010, $181"),
    TEST_ENTRY_C(0xD050B601, NULL, "vmfvc       S010, $182"),
    TEST_ENTRY_C(0xD050B701, NULL, "vmfvc       S010, $183"),
    TEST_ENTRY_C(0xD050B801, NULL, "vmfvc       S010, $184"),
    TEST_ENTRY_C(0xD050B901, NULL, "vmfvc       S010, $185"),
    TEST_ENTRY_C(0xD050BA01, NULL, "vmfvc       S010, $186"),
    TEST_ENTRY_C(0xD050BB01, NULL, "vmfvc       S010, $187"),
    TEST_ENTRY_C(0xD050BC01, NULL, "vmfvc       S010, $188"),
    TEST_ENTRY_C(0xD050BD01, NULL, "vmfvc       S010, $189"),
    TEST_ENTRY_C(0xD050BE01, NULL, "vmfvc       S010, $190"),
    TEST_ENTRY_C(0xD050BF01, NULL, "vmfvc       S010, $191"),
    TEST_ENTRY_C(0xD050C001, NULL, "vmfvc       S010, $192"),
    TEST_ENTRY_C(0xD050C101, NULL, "vmfvc       S010, $193"),
    TEST_ENTRY_C(0xD050C201, NULL, "vmfvc       S010, $194"),
    TEST_ENTRY_C(0xD050C301, NULL, "vmfvc       S010, $195"),
    TEST_ENTRY_C(0xD050C401, NULL, "vmfvc       S010, $196"),
    TEST_ENTRY_C(0xD050C501, NULL, "vmfvc       S010, $197"),
    TEST_ENTRY_C(0xD050C601, NULL, "vmfvc       S010, $198"),
    TEST_ENTRY_C(0xD050C701, NULL, "vmfvc       S010, $199"),
    TEST_ENTRY_C(0xD050C801, NULL, "vmfvc       S010, $200"),
    TEST_ENTRY_C(0xD050C901, NULL, "vmfvc       S010, $201"),
    TEST_ENTRY_C(0xD050CA01, NULL, "vmfvc       S010, $202"),
    TEST_ENTRY_C(0xD050CB01, NULL, "vmfvc       S010, $203"),
    TEST_ENTRY_C(0xD050CC01, NULL, "vmfvc       S010, $204"),
    TEST_ENTRY_C(0xD050CD01, NULL, "vmfvc       S010, $205"),
    TEST_ENTRY_C(0xD050CE01, NULL, "vmfvc       S010, $206"),
    TEST_ENTRY_C(0xD050CF01, NULL, "vmfvc       S010, $207"),
    TEST_ENTRY_C(0xD050D001, NULL, "vmfvc       S010, $208"),
    TEST_ENTRY_C(0xD050D101, NULL, "vmfvc       S010, $209"),
    TEST_ENTRY_C(0xD050D201, NULL, "vmfvc       S010, $210"),
    TEST_ENTRY_C(0xD050D301, NULL, "vmfvc       S010, $211"),
    TEST_ENTRY_C(0xD050D401, NULL, "vmfvc       S010, $212"),
    TEST_ENTRY_C(0xD050D501, NULL, "vmfvc       S010, $213"),
    TEST_ENTRY_C(0xD050D601, NULL, "vmfvc       S010, $214"),
    TEST_ENTRY_C(0xD050D701, NULL, "vmfvc       S010, $215"),
    TEST_ENTRY_C(0xD050D801, NULL, "vmfvc       S010, $216"),
    TEST_ENTRY_C(0xD050D901, NULL, "vmfvc       S010, $217"),
    TEST_ENTRY_C(0xD050DA01, NULL, "vmfvc       S010, $218"),
    TEST_ENTRY_C(0xD050DB01, NULL, "vmfvc       S010, $219"),
    TEST_ENTRY_C(0xD050DC01, NULL, "vmfvc       S010, $220"),
    TEST_ENTRY_C(0xD050DD01, NULL, "vmfvc       S010, $221"),
    TEST_ENTRY_C(0xD050DE01, NULL, "vmfvc       S010, $222"),
    TEST_ENTRY_C(0xD050DF01, NULL, "vmfvc       S010, $223"),
    TEST_ENTRY_C(0xD050E001, NULL, "vmfvc       S010, $224"),
    TEST_ENTRY_C(0xD050E101, NULL, "vmfvc       S010, $225"),
    TEST_ENTRY_C(0xD050E201, NULL, "vmfvc       S010, $226"),
    TEST_ENTRY_C(0xD050E301, NULL, "vmfvc       S010, $227"),
    TEST_ENTRY_C(0xD050E401, NULL, "vmfvc       S010, $228"),
    TEST_ENTRY_C(0xD050E501, NULL, "vmfvc       S010, $229"),
    TEST_ENTRY_C(0xD050E601, NULL, "vmfvc       S010, $230"),
    TEST_ENTRY_C(0xD050E701, NULL, "vmfvc       S010, $231"),
    TEST_ENTRY_C(0xD050E801, NULL, "vmfvc       S010, $232"),
    TEST_ENTRY_C(0xD050E901, NULL, "vmfvc       S010, $233"),
    TEST_ENTRY_C(0xD050EA01, NULL, "vmfvc       S010, $234"),
    TEST_ENTRY_C(0xD050EB01, NULL, "vmfvc       S010, $235"),
    TEST_ENTRY_C(0xD050EC01, NULL, "vmfvc       S010, $236"),
    TEST_ENTRY_C(0xD050ED01, NULL, "vmfvc       S010, $237"),
    TEST_ENTRY_C(0xD050EE01, NULL, "vmfvc       S010, $238"),
    TEST_ENTRY_C(0xD050EF01, NULL, "vmfvc       S010, $239"),
    TEST_ENTRY_C(0xD050F001, NULL, "vmfvc       S010, $240"),
    TEST_ENTRY_C(0xD050F101, NULL, "vmfvc       S010, $241"),
    TEST_ENTRY_C(0xD050F201, NULL, "vmfvc       S010, $242"),
    TEST_ENTRY_C(0xD050F301, NULL, "vmfvc       S010, $243"),
    TEST_ENTRY_C(0xD050F401, NULL, "vmfvc       S010, $244"),
    TEST_ENTRY_C(0xD050F501, NULL, "vmfvc       S010, $245"),
    TEST_ENTRY_C(0xD050F601, NULL, "vmfvc       S010, $246"),
    TEST_ENTRY_C(0xD050F701, NULL, "vmfvc       S010, $247"),
    TEST_ENTRY_C(0xD050F801, NULL, "vmfvc       S010, $248"),
    TEST_ENTRY_C(0xD050F901, NULL, "vmfvc       S010, $249"),
    TEST_ENTRY_C(0xD050FA01, NULL, "vmfvc       S010, $250"),
    TEST_ENTRY_C(0xD050FB01, NULL, "vmfvc       S010, $251"),
    TEST_ENTRY_C(0xD050FC01, NULL, "vmfvc       S010, $252"),
    TEST_ENTRY_C(0xD050FD01, NULL, "vmfvc       S010, $253"),
    TEST_ENTRY_C(0xD050FE01, NULL, "vmfvc       S010, $254"),
    TEST_ENTRY_C(0xD050FF01, NULL, "vmfvc       S010, $255"),

    TEST_ENTRY_C(0xD0510180, NULL, "vmtvc       $128, S010"),
    TEST_ENTRY_C(0xD0510181, NULL, "vmtvc       $129, S010"),
    TEST_ENTRY_C(0xD0510182, NULL, "vmtvc       $130, S010"),
    TEST_ENTRY_C(0xD0510183, NULL, "vmtvc       $131, S010"),
    TEST_ENTRY_C(0xD0510184, NULL, "vmtvc       $132, S010"),
    TEST_ENTRY_C(0xD0510185, NULL, "vmtvc       $133, S010"),
    TEST_ENTRY_C(0xD0510186, NULL, "vmtvc       $134, S010"),
    TEST_ENTRY_C(0xD0510187, NULL, "vmtvc       $135, S010"),
    TEST_ENTRY_C(0xD0510188, NULL, "vmtvc       $136, S010"),
    TEST_ENTRY_C(0xD0510189, NULL, "vmtvc       $137, S010"),
    TEST_ENTRY_C(0xD051018A, NULL, "vmtvc       $138, S010"),
    TEST_ENTRY_C(0xD051018B, NULL, "vmtvc       $139, S010"),
    TEST_ENTRY_C(0xD051018C, NULL, "vmtvc       $140, S010"),
    TEST_ENTRY_C(0xD051018D, NULL, "vmtvc       $141, S010"),
    TEST_ENTRY_C(0xD051018E, NULL, "vmtvc       $142, S010"),
    TEST_ENTRY_C(0xD051018F, NULL, "vmtvc       $143, S010"),
    TEST_ENTRY_C(0xD0510190, NULL, "vmtvc       $144, S010"),
    TEST_ENTRY_C(0xD0510191, NULL, "vmtvc       $145, S010"),
    TEST_ENTRY_C(0xD0510192, NULL, "vmtvc       $146, S010"),
    TEST_ENTRY_C(0xD0510193, NULL, "vmtvc       $147, S010"),
    TEST_ENTRY_C(0xD0510194, NULL, "vmtvc       $148, S010"),
    TEST_ENTRY_C(0xD0510195, NULL, "vmtvc       $149, S010"),
    TEST_ENTRY_C(0xD0510196, NULL, "vmtvc       $150, S010"),
    TEST_ENTRY_C(0xD0510197, NULL, "vmtvc       $151, S010"),
    TEST_ENTRY_C(0xD0510198, NULL, "vmtvc       $152, S010"),
    TEST_ENTRY_C(0xD0510199, NULL, "vmtvc       $153, S010"),
    TEST_ENTRY_C(0xD051019A, NULL, "vmtvc       $154, S010"),
    TEST_ENTRY_C(0xD051019B, NULL, "vmtvc       $155, S010"),
    TEST_ENTRY_C(0xD051019C, NULL, "vmtvc       $156, S010"),
    TEST_ENTRY_C(0xD051019D, NULL, "vmtvc       $157, S010"),
    TEST_ENTRY_C(0xD051019E, NULL, "vmtvc       $158, S010"),
    TEST_ENTRY_C(0xD051019F, NULL, "vmtvc       $159, S010"),
    TEST_ENTRY_C(0xD05101A0, NULL, "vmtvc       $160, S010"),
    TEST_ENTRY_C(0xD05101A1, NULL, "vmtvc       $161, S010"),
    TEST_ENTRY_C(0xD05101A2, NULL, "vmtvc       $162, S010"),
    TEST_ENTRY_C(0xD05101A3, NULL, "vmtvc       $163, S010"),
    TEST_ENTRY_C(0xD05101A4, NULL, "vmtvc       $164, S010"),
    TEST_ENTRY_C(0xD05101A5, NULL, "vmtvc       $165, S010"),
    TEST_ENTRY_C(0xD05101A6, NULL, "vmtvc       $166, S010"),
    TEST_ENTRY_C(0xD05101A7, NULL, "vmtvc       $167, S010"),
    TEST_ENTRY_C(0xD05101A8, NULL, "vmtvc       $168, S010"),
    TEST_ENTRY_C(0xD05101A9, NULL, "vmtvc       $169, S010"),
    TEST_ENTRY_C(0xD05101AA, NULL, "vmtvc       $170, S010"),
    TEST_ENTRY_C(0xD05101AB, NULL, "vmtvc       $171, S010"),
    TEST_ENTRY_C(0xD05101AC, NULL, "vmtvc       $172, S010"),
    TEST_ENTRY_C(0xD05101AD, NULL, "vmtvc       $173, S010"),
    TEST_ENTRY_C(0xD05101AE, NULL, "vmtvc       $174, S010"),
    TEST_ENTRY_C(0xD05101AF, NULL, "vmtvc       $175, S010"),
    TEST_ENTRY_C(0xD05101B0, NULL, "vmtvc       $176, S010"),
    TEST_ENTRY_C(0xD05101B1, NULL, "vmtvc       $177, S010"),
    TEST_ENTRY_C(0xD05101B2, NULL, "vmtvc       $178, S010"),
    TEST_ENTRY_C(0xD05101B3, NULL, "vmtvc       $179, S010"),
    TEST_ENTRY_C(0xD05101B4, NULL, "vmtvc       $180, S010"),
    TEST_ENTRY_C(0xD05101B5, NULL, "vmtvc       $181, S010"),
    TEST_ENTRY_C(0xD05101B6, NULL, "vmtvc       $182, S010"),
    TEST_ENTRY_C(0xD05101B7, NULL, "vmtvc       $183, S010"),
    TEST_ENTRY_C(0xD05101B8, NULL, "vmtvc       $184, S010"),
    TEST_ENTRY_C(0xD05101B9, NULL, "vmtvc       $185, S010"),
    TEST_ENTRY_C(0xD05101BA, NULL, "vmtvc       $186, S010"),
    TEST_ENTRY_C(0xD05101BB, NULL, "vmtvc       $187, S010"),
    TEST_ENTRY_C(0xD05101BC, NULL, "vmtvc       $188, S010"),
    TEST_ENTRY_C(0xD05101BD, NULL, "vmtvc       $189, S010"),
    TEST_ENTRY_C(0xD05101BE, NULL, "vmtvc       $190, S010"),
    TEST_ENTRY_C(0xD05101BF, NULL, "vmtvc       $191, S010"),
    TEST_ENTRY_C(0xD05101C0, NULL, "vmtvc       $192, S010"),
    TEST_ENTRY_C(0xD05101C1, NULL, "vmtvc       $193, S010"),
    TEST_ENTRY_C(0xD05101C2, NULL, "vmtvc       $194, S010"),
    TEST_ENTRY_C(0xD05101C3, NULL, "vmtvc       $195, S010"),
    TEST_ENTRY_C(0xD05101C4, NULL, "vmtvc       $196, S010"),
    TEST_ENTRY_C(0xD05101C5, NULL, "vmtvc       $197, S010"),
    TEST_ENTRY_C(0xD05101C6, NULL, "vmtvc       $198, S010"),
    TEST_ENTRY_C(0xD05101C7, NULL, "vmtvc       $199, S010"),
    TEST_ENTRY_C(0xD05101C8, NULL, "vmtvc       $200, S010"),
    TEST_ENTRY_C(0xD05101C9, NULL, "vmtvc       $201, S010"),
    TEST_ENTRY_C(0xD05101CA, NULL, "vmtvc       $202, S010"),
    TEST_ENTRY_C(0xD05101CB, NULL, "vmtvc       $203, S010"),
    TEST_ENTRY_C(0xD05101CC, NULL, "vmtvc       $204, S010"),
    TEST_ENTRY_C(0xD05101CD, NULL, "vmtvc       $205, S010"),
    TEST_ENTRY_C(0xD05101CE, NULL, "vmtvc       $206, S010"),
    TEST_ENTRY_C(0xD05101CF, NULL, "vmtvc       $207, S010"),
    TEST_ENTRY_C(0xD05101D0, NULL, "vmtvc       $208, S010"),
    TEST_ENTRY_C(0xD05101D1, NULL, "vmtvc       $209, S010"),
    TEST_ENTRY_C(0xD05101D2, NULL, "vmtvc       $210, S010"),
    TEST_ENTRY_C(0xD05101D3, NULL, "vmtvc       $211, S010"),
    TEST_ENTRY_C(0xD05101D4, NULL, "vmtvc       $212, S010"),
    TEST_ENTRY_C(0xD05101D5, NULL, "vmtvc       $213, S010"),
    TEST_ENTRY_C(0xD05101D6, NULL, "vmtvc       $214, S010"),
    TEST_ENTRY_C(0xD05101D7, NULL, "vmtvc       $215, S010"),
    TEST_ENTRY_C(0xD05101D8, NULL, "vmtvc       $216, S010"),
    TEST_ENTRY_C(0xD05101D9, NULL, "vmtvc       $217, S010"),
    TEST_ENTRY_C(0xD05101DA, NULL, "vmtvc       $218, S010"),
    TEST_ENTRY_C(0xD05101DB, NULL, "vmtvc       $219, S010"),
    TEST_ENTRY_C(0xD05101DC, NULL, "vmtvc       $220, S010"),
    TEST_ENTRY_C(0xD05101DD, NULL, "vmtvc       $221, S010"),
    TEST_ENTRY_C(0xD05101DE, NULL, "vmtvc       $222, S010"),
    TEST_ENTRY_C(0xD05101DF, NULL, "vmtvc       $223, S010"),
    TEST_ENTRY_C(0xD05101E0, NULL, "vmtvc       $224, S010"),
    TEST_ENTRY_C(0xD05101E1, NULL, "vmtvc       $225, S010"),
    TEST_ENTRY_C(0xD05101E2, NULL, "vmtvc       $226, S010"),
    TEST_ENTRY_C(0xD05101E3, NULL, "vmtvc       $227, S010"),
    TEST_ENTRY_C(0xD05101E4, NULL, "vmtvc       $228, S010"),
    TEST_ENTRY_C(0xD05101E5, NULL, "vmtvc       $229, S010"),
    TEST_ENTRY_C(0xD05101E6, NULL, "vmtvc       $230, S010"),
    TEST_ENTRY_C(0xD05101E7, NULL, "vmtvc       $231, S010"),
    TEST_ENTRY_C(0xD05101E8, NULL, "vmtvc       $232, S010"),
    TEST_ENTRY_C(0xD05101E9, NULL, "vmtvc       $233, S010"),
    TEST_ENTRY_C(0xD05101EA, NULL, "vmtvc       $234, S010"),
    TEST_ENTRY_C(0xD05101EB, NULL, "vmtvc       $235, S010"),
    TEST_ENTRY_C(0xD05101EC, NULL, "vmtvc       $236, S010"),
    TEST_ENTRY_C(0xD05101ED, NULL, "vmtvc       $237, S010"),
    TEST_ENTRY_C(0xD05101EE, NULL, "vmtvc       $238, S010"),
    TEST_ENTRY_C(0xD05101EF, NULL, "vmtvc       $239, S010"),
    TEST_ENTRY_C(0xD05101F0, NULL, "vmtvc       $240, S010"),
    TEST_ENTRY_C(0xD05101F1, NULL, "vmtvc       $241, S010"),
    TEST_ENTRY_C(0xD05101F2, NULL, "vmtvc       $242, S010"),
    TEST_ENTRY_C(0xD05101F3, NULL, "vmtvc       $243, S010"),
    TEST_ENTRY_C(0xD05101F4, NULL, "vmtvc       $244, S010"),
    TEST_ENTRY_C(0xD05101F5, NULL, "vmtvc       $245, S010"),
    TEST_ENTRY_C(0xD05101F6, NULL, "vmtvc       $246, S010"),
    TEST_ENTRY_C(0xD05101F7, NULL, "vmtvc       $247, S010"),
    TEST_ENTRY_C(0xD05101F8, NULL, "vmtvc       $248, S010"),
    TEST_ENTRY_C(0xD05101F9, NULL, "vmtvc       $249, S010"),
    TEST_ENTRY_C(0xD05101FA, NULL, "vmtvc       $250, S010"),
    TEST_ENTRY_C(0xD05101FB, NULL, "vmtvc       $251, S010"),
    TEST_ENTRY_C(0xD05101FC, NULL, "vmtvc       $252, S010"),
    TEST_ENTRY_C(0xD05101FD, NULL, "vmtvc       $253, S010"),
    TEST_ENTRY_C(0xD05101FE, NULL, "vmtvc       $254, S010"),
    TEST_ENTRY_C(0xD05101FF, NULL, "vmtvc       $255, S010"),

    TEST_ENTRY_C(0xD0521001, NULL, ".word       0xD0521001                   # INVALID     $v0, $s2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CONTROL>"),
    TEST_ENTRY_C(0xD0521081, NULL, ".word       0xD0521081                   # INVALID     $v0, $s2, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CONTROL>"),
    TEST_ENTRY_C(0xD0529001, NULL, ".word       0xD0529001                   # INVALID     $v0, $s2, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CONTROL>"),
    TEST_ENTRY_C(0xD0529081, NULL, ".word       0xD0529081                   # INVALID     $v0, $s2, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CONTROL>"),
    TEST_ENTRY_C(0xD0531001, NULL, ".word       0xD0531001                   # INVALID     $v0, $s3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CONTROL>"),
    TEST_ENTRY_C(0xD0531081, NULL, ".word       0xD0531081                   # INVALID     $v0, $s3, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CONTROL>"),
    TEST_ENTRY_C(0xD0539001, NULL, ".word       0xD0539001                   # INVALID     $v0, $s3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CONTROL>"),
    TEST_ENTRY_C(0xD0539081, NULL, ".word       0xD0539081                   # INVALID     $v0, $s3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CONTROL>"),
    TEST_ENTRY_C(0xD0541001, NULL, ".word       0xD0541001                   # INVALID     $v0, $s4, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CONTROL>"),
    TEST_ENTRY_C(0xD0541081, NULL, ".word       0xD0541081                   # INVALID     $v0, $s4, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CONTROL>"),
    TEST_ENTRY_C(0xD0549001, NULL, ".word       0xD0549001                   # INVALID     $v0, $s4, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CONTROL>"),
    TEST_ENTRY_C(0xD0549081, NULL, ".word       0xD0549081                   # INVALID     $v0, $s4, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CONTROL>"),
    TEST_ENTRY_C(0xD0551001, NULL, ".word       0xD0551001                   # INVALID     $v0, $s5, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CONTROL>"),
    TEST_ENTRY_C(0xD0551081, NULL, ".word       0xD0551081                   # INVALID     $v0, $s5, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CONTROL>"),
    TEST_ENTRY_C(0xD0559001, NULL, ".word       0xD0559001                   # INVALID     $v0, $s5, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CONTROL>"),
    TEST_ENTRY_C(0xD0559081, NULL, ".word       0xD0559081                   # INVALID     $v0, $s5, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CONTROL>"),
    TEST_ENTRY_C(0xD0561001, NULL, ".word       0xD0561001                   # INVALID     $v0, $s6, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CONTROL>"),
    TEST_ENTRY_C(0xD0561081, NULL, ".word       0xD0561081                   # INVALID     $v0, $s6, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CONTROL>"),
    TEST_ENTRY_C(0xD0569001, NULL, ".word       0xD0569001                   # INVALID     $v0, $s6, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CONTROL>"),
    TEST_ENTRY_C(0xD0569081, NULL, ".word       0xD0569081                   # INVALID     $v0, $s6, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CONTROL>"),
    TEST_ENTRY_C(0xD0571001, NULL, ".word       0xD0571001                   # INVALID     $v0, $s7, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CONTROL>"),
    TEST_ENTRY_C(0xD0571081, NULL, ".word       0xD0571081                   # INVALID     $v0, $s7, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CONTROL>"),
    TEST_ENTRY_C(0xD0579001, NULL, ".word       0xD0579001                   # INVALID     $v0, $s7, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CONTROL>"),
    TEST_ENTRY_C(0xD0579081, NULL, ".word       0xD0579081                   # INVALID     $v0, $s7, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_CONTROL>"),
    TEST_ENTRY_C(0xD0581001, NULL, ".word       0xD0581001                   # INVALID     $v0, $t8, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD0581081, NULL, ".word       0xD0581081                   # INVALID     $v0, $t8, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD0589001, NULL, ".word       0xD0589001                   # INVALID     $v0, $t8, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD0589081, NULL, ".word       0xD0589081                   # INVALID     $v0, $t8, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD0591001, NULL, ".word       0xD0591001                   # INVALID     $v0, $t9, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD0591081, NULL, ".word       0xD0591081                   # INVALID     $v0, $t9, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD0599001, NULL, ".word       0xD0599001                   # INVALID     $v0, $t9, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD0599081, NULL, "vt4444.q    C010, C400"),
    TEST_ENTRY_C(0xD05A1001, NULL, ".word       0xD05A1001                   # INVALID     $v0, $k0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD05A1081, NULL, ".word       0xD05A1081                   # INVALID     $v0, $k0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD05A9001, NULL, ".word       0xD05A9001                   # INVALID     $v0, $k0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD05A9081, NULL, "vt5551.q    C010, C400"),
    TEST_ENTRY_C(0xD05B1001, NULL, ".word       0xD05B1001                   # INVALID     $v0, $k1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD05B1081, NULL, ".word       0xD05B1081                   # INVALID     $v0, $k1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD05B9001, NULL, ".word       0xD05B9001                   # INVALID     $v0, $k1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD05B9081, NULL, "vt5650.q    C010, C400"),
    TEST_ENTRY_C(0xD05C1001, NULL, ".word       0xD05C1001                   # INVALID     $v0, $gp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD05C1081, NULL, ".word       0xD05C1081                   # INVALID     $v0, $gp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD05C9001, NULL, ".word       0xD05C9001                   # INVALID     $v0, $gp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD05C9081, NULL, ".word       0xD05C9081                   # INVALID     $v0, $gp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD05D1001, NULL, ".word       0xD05D1001                   # INVALID     $v0, $sp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD05D1081, NULL, ".word       0xD05D1081                   # INVALID     $v0, $sp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD05D9001, NULL, ".word       0xD05D9001                   # INVALID     $v0, $sp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD05D9081, NULL, ".word       0xD05D9081                   # INVALID     $v0, $sp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD05E1001, NULL, ".word       0xD05E1001                   # INVALID     $v0, $fp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD05E1081, NULL, ".word       0xD05E1081                   # INVALID     $v0, $fp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD05E9001, NULL, ".word       0xD05E9001                   # INVALID     $v0, $fp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD05E9081, NULL, ".word       0xD05E9081                   # INVALID     $v0, $fp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD05F1001, NULL, ".word       0xD05F1001                   # INVALID     $v0, $ra, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD05F1081, NULL, ".word       0xD05F1081                   # INVALID     $v0, $ra, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD05F9001, NULL, ".word       0xD05F9001                   # INVALID     $v0, $ra, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD05F9081, NULL, ".word       0xD05F9081                   # INVALID     $v0, $ra, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0_COLOR>"),
    TEST_ENTRY_C(0xD0600001, NULL, "vcst.s      S010, INVALID_0"),
    TEST_ENTRY_C(0xD0600081, NULL, "vcst.p      C010, INVALID_0"),
    TEST_ENTRY_C(0xD0608001, NULL, "vcst.t      C010, INVALID_0"),
    TEST_ENTRY_C(0xD0608081, NULL, "vcst.q      C010, INVALID_0"),
    TEST_ENTRY_C(0xD0610001, NULL, "vcst.s      S010, VFPU_HUGE"),
    TEST_ENTRY_C(0xD0610081, NULL, "vcst.p      C010, VFPU_HUGE"),
    TEST_ENTRY_C(0xD0618001, NULL, "vcst.t      C010, VFPU_HUGE"),
    TEST_ENTRY_C(0xD0618081, NULL, "vcst.q      C010, VFPU_HUGE"),
    TEST_ENTRY_C(0xD0620001, NULL, "vcst.s      S010, VFPU_SQRT2"),
    TEST_ENTRY_C(0xD0620081, NULL, "vcst.p      C010, VFPU_SQRT2"),
    TEST_ENTRY_C(0xD0628001, NULL, "vcst.t      C010, VFPU_SQRT2"),
    TEST_ENTRY_C(0xD0628081, NULL, "vcst.q      C010, VFPU_SQRT2"),
    TEST_ENTRY_C(0xD0630001, NULL, "vcst.s      S010, VFPU_SQRT1_2"),
    TEST_ENTRY_C(0xD0630081, NULL, "vcst.p      C010, VFPU_SQRT1_2"),
    TEST_ENTRY_C(0xD0638001, NULL, "vcst.t      C010, VFPU_SQRT1_2"),
    TEST_ENTRY_C(0xD0638081, NULL, "vcst.q      C010, VFPU_SQRT1_2"),
    TEST_ENTRY_C(0xD0640001, NULL, "vcst.s      S010, VFPU_2_SQRTPI"),
    TEST_ENTRY_C(0xD0640081, NULL, "vcst.p      C010, VFPU_2_SQRTPI"),
    TEST_ENTRY_C(0xD0648001, NULL, "vcst.t      C010, VFPU_2_SQRTPI"),
    TEST_ENTRY_C(0xD0648081, NULL, "vcst.q      C010, VFPU_2_SQRTPI"),
    TEST_ENTRY_C(0xD0650001, NULL, "vcst.s      S010, VFPU_2_PI"),
    TEST_ENTRY_C(0xD0650081, NULL, "vcst.p      C010, VFPU_2_PI"),
    TEST_ENTRY_C(0xD0658001, NULL, "vcst.t      C010, VFPU_2_PI"),
    TEST_ENTRY_C(0xD0658081, NULL, "vcst.q      C010, VFPU_2_PI"),
    TEST_ENTRY_C(0xD0660001, NULL, "vcst.s      S010, VFPU_1_PI"),
    TEST_ENTRY_C(0xD0660081, NULL, "vcst.p      C010, VFPU_1_PI"),
    TEST_ENTRY_C(0xD0668001, NULL, "vcst.t      C010, VFPU_1_PI"),
    TEST_ENTRY_C(0xD0668081, NULL, "vcst.q      C010, VFPU_1_PI"),
    TEST_ENTRY_C(0xD0670001, NULL, "vcst.s      S010, VFPU_PI_4"),
    TEST_ENTRY_C(0xD0670081, NULL, "vcst.p      C010, VFPU_PI_4"),
    TEST_ENTRY_C(0xD0678001, NULL, "vcst.t      C010, VFPU_PI_4"),
    TEST_ENTRY_C(0xD0678081, NULL, "vcst.q      C010, VFPU_PI_4"),
    TEST_ENTRY_C(0xD0680001, NULL, "vcst.s      S010, VFPU_PI_2"),
    TEST_ENTRY_C(0xD0680081, NULL, "vcst.p      C010, VFPU_PI_2"),
    TEST_ENTRY_C(0xD0688001, NULL, "vcst.t      C010, VFPU_PI_2"),
    TEST_ENTRY_C(0xD0688081, NULL, "vcst.q      C010, VFPU_PI_2"),
    TEST_ENTRY_C(0xD0690001, NULL, "vcst.s      S010, VFPU_PI"),
    TEST_ENTRY_C(0xD0690081, NULL, "vcst.p      C010, VFPU_PI"),
    TEST_ENTRY_C(0xD0698001, NULL, "vcst.t      C010, VFPU_PI"),
    TEST_ENTRY_C(0xD0698081, NULL, "vcst.q      C010, VFPU_PI"),
    TEST_ENTRY_C(0xD06A0001, NULL, "vcst.s      S010, VFPU_E"),
    TEST_ENTRY_C(0xD06A0081, NULL, "vcst.p      C010, VFPU_E"),
    TEST_ENTRY_C(0xD06A8001, NULL, "vcst.t      C010, VFPU_E"),
    TEST_ENTRY_C(0xD06A8081, NULL, "vcst.q      C010, VFPU_E"),
    TEST_ENTRY_C(0xD06B0001, NULL, "vcst.s      S010, VFPU_LOG2E"),
    TEST_ENTRY_C(0xD06B0081, NULL, "vcst.p      C010, VFPU_LOG2E"),
    TEST_ENTRY_C(0xD06B8001, NULL, "vcst.t      C010, VFPU_LOG2E"),
    TEST_ENTRY_C(0xD06B8081, NULL, "vcst.q      C010, VFPU_LOG2E"),
    TEST_ENTRY_C(0xD06C0001, NULL, "vcst.s      S010, VFPU_LOG10E"),
    TEST_ENTRY_C(0xD06C0081, NULL, "vcst.p      C010, VFPU_LOG10E"),
    TEST_ENTRY_C(0xD06C8001, NULL, "vcst.t      C010, VFPU_LOG10E"),
    TEST_ENTRY_C(0xD06C8081, NULL, "vcst.q      C010, VFPU_LOG10E"),
    TEST_ENTRY_C(0xD06D0001, NULL, "vcst.s      S010, VFPU_LN2"),
    TEST_ENTRY_C(0xD06D0081, NULL, "vcst.p      C010, VFPU_LN2"),
    TEST_ENTRY_C(0xD06D8001, NULL, "vcst.t      C010, VFPU_LN2"),
    TEST_ENTRY_C(0xD06D8081, NULL, "vcst.q      C010, VFPU_LN2"),
    TEST_ENTRY_C(0xD06E0001, NULL, "vcst.s      S010, VFPU_LN10"),
    TEST_ENTRY_C(0xD06E0081, NULL, "vcst.p      C010, VFPU_LN10"),
    TEST_ENTRY_C(0xD06E8001, NULL, "vcst.t      C010, VFPU_LN10"),
    TEST_ENTRY_C(0xD06E8081, NULL, "vcst.q      C010, VFPU_LN10"),
    TEST_ENTRY_C(0xD06F0001, NULL, "vcst.s      S010, VFPU_2PI"),
    TEST_ENTRY_C(0xD06F0081, NULL, "vcst.p      C010, VFPU_2PI"),
    TEST_ENTRY_C(0xD06F8001, NULL, "vcst.t      C010, VFPU_2PI"),
    TEST_ENTRY_C(0xD06F8081, NULL, "vcst.q      C010, VFPU_2PI"),
    TEST_ENTRY_C(0xD0700001, NULL, "vcst.s      S010, VFPU_PI_6"),
    TEST_ENTRY_C(0xD0700081, NULL, "vcst.p      C010, VFPU_PI_6"),
    TEST_ENTRY_C(0xD0708001, NULL, "vcst.t      C010, VFPU_PI_6"),
    TEST_ENTRY_C(0xD0708081, NULL, "vcst.q      C010, VFPU_PI_6"),
    TEST_ENTRY_C(0xD0710001, NULL, "vcst.s      S010, VFPU_LOG10TWO"),
    TEST_ENTRY_C(0xD0710081, NULL, "vcst.p      C010, VFPU_LOG10TWO"),
    TEST_ENTRY_C(0xD0718001, NULL, "vcst.t      C010, VFPU_LOG10TWO"),
    TEST_ENTRY_C(0xD0718081, NULL, "vcst.q      C010, VFPU_LOG10TWO"),
    TEST_ENTRY_C(0xD0720001, NULL, "vcst.s      S010, VFPU_LOG2TEN"),
    TEST_ENTRY_C(0xD0720081, NULL, "vcst.p      C010, VFPU_LOG2TEN"),
    TEST_ENTRY_C(0xD0728001, NULL, "vcst.t      C010, VFPU_LOG2TEN"),
    TEST_ENTRY_C(0xD0728081, NULL, "vcst.q      C010, VFPU_LOG2TEN"),
    TEST_ENTRY_C(0xD0730001, NULL, "vcst.s      S010, VFPU_SQRT3_2"),
    TEST_ENTRY_C(0xD0730081, NULL, "vcst.p      C010, VFPU_SQRT3_2"),
    TEST_ENTRY_C(0xD0738001, NULL, "vcst.t      C010, VFPU_SQRT3_2"),
    TEST_ENTRY_C(0xD0738081, NULL, "vcst.q      C010, VFPU_SQRT3_2"),
    TEST_ENTRY_C(0xD0740001, NULL, "vcst.s      S010, INVALID_20"),
    TEST_ENTRY_C(0xD0740081, NULL, "vcst.p      C010, INVALID_20"),
    TEST_ENTRY_C(0xD0748001, NULL, "vcst.t      C010, INVALID_20"),
    TEST_ENTRY_C(0xD0748081, NULL, "vcst.q      C010, INVALID_20"),
    TEST_ENTRY_C(0xD0750001, NULL, "vcst.s      S010, INVALID_21"),
    TEST_ENTRY_C(0xD0750081, NULL, "vcst.p      C010, INVALID_21"),
    TEST_ENTRY_C(0xD0758001, NULL, "vcst.t      C010, INVALID_21"),
    TEST_ENTRY_C(0xD0758081, NULL, "vcst.q      C010, INVALID_21"),
    TEST_ENTRY_C(0xD0760001, NULL, "vcst.s      S010, INVALID_22"),
    TEST_ENTRY_C(0xD0760081, NULL, "vcst.p      C010, INVALID_22"),
    TEST_ENTRY_C(0xD0768001, NULL, "vcst.t      C010, INVALID_22"),
    TEST_ENTRY_C(0xD0768081, NULL, "vcst.q      C010, INVALID_22"),
    TEST_ENTRY_C(0xD0770001, NULL, "vcst.s      S010, INVALID_23"),
    TEST_ENTRY_C(0xD0770081, NULL, "vcst.p      C010, INVALID_23"),
    TEST_ENTRY_C(0xD0778001, NULL, "vcst.t      C010, INVALID_23"),
    TEST_ENTRY_C(0xD0778081, NULL, "vcst.q      C010, INVALID_23"),
    TEST_ENTRY_C(0xD0780001, NULL, "vcst.s      S010, INVALID_24"),
    TEST_ENTRY_C(0xD0780081, NULL, "vcst.p      C010, INVALID_24"),
    TEST_ENTRY_C(0xD0788001, NULL, "vcst.t      C010, INVALID_24"),
    TEST_ENTRY_C(0xD0788081, NULL, "vcst.q      C010, INVALID_24"),
    TEST_ENTRY_C(0xD0790001, NULL, "vcst.s      S010, INVALID_25"),
    TEST_ENTRY_C(0xD0790081, NULL, "vcst.p      C010, INVALID_25"),
    TEST_ENTRY_C(0xD0798001, NULL, "vcst.t      C010, INVALID_25"),
    TEST_ENTRY_C(0xD0798081, NULL, "vcst.q      C010, INVALID_25"),
    TEST_ENTRY_C(0xD07A0001, NULL, "vcst.s      S010, INVALID_26"),
    TEST_ENTRY_C(0xD07A0081, NULL, "vcst.p      C010, INVALID_26"),
    TEST_ENTRY_C(0xD07A8001, NULL, "vcst.t      C010, INVALID_26"),
    TEST_ENTRY_C(0xD07A8081, NULL, "vcst.q      C010, INVALID_26"),
    TEST_ENTRY_C(0xD07B0001, NULL, "vcst.s      S010, INVALID_27"),
    TEST_ENTRY_C(0xD07B0081, NULL, "vcst.p      C010, INVALID_27"),
    TEST_ENTRY_C(0xD07B8001, NULL, "vcst.t      C010, INVALID_27"),
    TEST_ENTRY_C(0xD07B8081, NULL, "vcst.q      C010, INVALID_27"),
    TEST_ENTRY_C(0xD07C0001, NULL, "vcst.s      S010, INVALID_28"),
    TEST_ENTRY_C(0xD07C0081, NULL, "vcst.p      C010, INVALID_28"),
    TEST_ENTRY_C(0xD07C8001, NULL, "vcst.t      C010, INVALID_28"),
    TEST_ENTRY_C(0xD07C8081, NULL, "vcst.q      C010, INVALID_28"),
    TEST_ENTRY_C(0xD07D0001, NULL, "vcst.s      S010, INVALID_29"),
    TEST_ENTRY_C(0xD07D0081, NULL, "vcst.p      C010, INVALID_29"),
    TEST_ENTRY_C(0xD07D8001, NULL, "vcst.t      C010, INVALID_29"),
    TEST_ENTRY_C(0xD07D8081, NULL, "vcst.q      C010, INVALID_29"),
    TEST_ENTRY_C(0xD07E0001, NULL, "vcst.s      S010, INVALID_30"),
    TEST_ENTRY_C(0xD07E0081, NULL, "vcst.p      C010, INVALID_30"),
    TEST_ENTRY_C(0xD07E8001, NULL, "vcst.t      C010, INVALID_30"),
    TEST_ENTRY_C(0xD07E8081, NULL, "vcst.q      C010, INVALID_30"),
    TEST_ENTRY_C(0xD07F0001, NULL, "vcst.s      S010, INVALID_31"),
    TEST_ENTRY_C(0xD07F0081, NULL, "vcst.p      C010, INVALID_31"),
    TEST_ENTRY_C(0xD07F8001, NULL, "vcst.t      C010, INVALID_31"),
    TEST_ENTRY_C(0xD07F8081, NULL, "vcst.q      C010, INVALID_31"),
    TEST_ENTRY_C(0xD0801001, NULL, ".word       0xD0801001                   # INVALID     $a0, $zero, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0801081, NULL, ".word       0xD0801081                   # INVALID     $a0, $zero, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0809001, NULL, ".word       0xD0809001                   # INVALID     $a0, $zero, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0809081, NULL, ".word       0xD0809081                   # INVALID     $a0, $zero, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0811001, NULL, ".word       0xD0811001                   # INVALID     $a0, $at, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0811081, NULL, ".word       0xD0811081                   # INVALID     $a0, $at, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0819001, NULL, ".word       0xD0819001                   # INVALID     $a0, $at, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0819081, NULL, ".word       0xD0819081                   # INVALID     $a0, $at, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0821001, NULL, ".word       0xD0821001                   # INVALID     $a0, $v0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0821081, NULL, ".word       0xD0821081                   # INVALID     $a0, $v0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0829001, NULL, ".word       0xD0829001                   # INVALID     $a0, $v0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0829081, NULL, ".word       0xD0829081                   # INVALID     $a0, $v0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0831001, NULL, ".word       0xD0831001                   # INVALID     $a0, $v1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0831081, NULL, ".word       0xD0831081                   # INVALID     $a0, $v1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0839001, NULL, ".word       0xD0839001                   # INVALID     $a0, $v1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0839081, NULL, ".word       0xD0839081                   # INVALID     $a0, $v1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0841001, NULL, ".word       0xD0841001                   # INVALID     $a0, $a0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0841081, NULL, ".word       0xD0841081                   # INVALID     $a0, $a0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0849001, NULL, ".word       0xD0849001                   # INVALID     $a0, $a0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0849081, NULL, ".word       0xD0849081                   # INVALID     $a0, $a0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0851001, NULL, ".word       0xD0851001                   # INVALID     $a0, $a1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0851081, NULL, ".word       0xD0851081                   # INVALID     $a0, $a1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0859001, NULL, ".word       0xD0859001                   # INVALID     $a0, $a1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0859081, NULL, ".word       0xD0859081                   # INVALID     $a0, $a1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0861001, NULL, ".word       0xD0861001                   # INVALID     $a0, $a2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0861081, NULL, ".word       0xD0861081                   # INVALID     $a0, $a2, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0869001, NULL, ".word       0xD0869001                   # INVALID     $a0, $a2, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0869081, NULL, ".word       0xD0869081                   # INVALID     $a0, $a2, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0871001, NULL, ".word       0xD0871001                   # INVALID     $a0, $a3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0871081, NULL, ".word       0xD0871081                   # INVALID     $a0, $a3, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0879001, NULL, ".word       0xD0879001                   # INVALID     $a0, $a3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0879081, NULL, ".word       0xD0879081                   # INVALID     $a0, $a3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0881001, NULL, ".word       0xD0881001                   # INVALID     $a0, $t0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0881081, NULL, ".word       0xD0881081                   # INVALID     $a0, $t0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0889001, NULL, ".word       0xD0889001                   # INVALID     $a0, $t0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0889081, NULL, ".word       0xD0889081                   # INVALID     $a0, $t0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0891001, NULL, ".word       0xD0891001                   # INVALID     $a0, $t1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0891081, NULL, ".word       0xD0891081                   # INVALID     $a0, $t1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0899001, NULL, ".word       0xD0899001                   # INVALID     $a0, $t1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0899081, NULL, ".word       0xD0899081                   # INVALID     $a0, $t1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD08A1001, NULL, ".word       0xD08A1001                   # INVALID     $a0, $t2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD08A1081, NULL, ".word       0xD08A1081                   # INVALID     $a0, $t2, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD08A9001, NULL, ".word       0xD08A9001                   # INVALID     $a0, $t2, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD08A9081, NULL, ".word       0xD08A9081                   # INVALID     $a0, $t2, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD08B1001, NULL, ".word       0xD08B1001                   # INVALID     $a0, $t3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD08B1081, NULL, ".word       0xD08B1081                   # INVALID     $a0, $t3, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD08B9001, NULL, ".word       0xD08B9001                   # INVALID     $a0, $t3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD08B9081, NULL, ".word       0xD08B9081                   # INVALID     $a0, $t3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD08C1001, NULL, ".word       0xD08C1001                   # INVALID     $a0, $t4, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD08C1081, NULL, ".word       0xD08C1081                   # INVALID     $a0, $t4, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD08C9001, NULL, ".word       0xD08C9001                   # INVALID     $a0, $t4, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD08C9081, NULL, ".word       0xD08C9081                   # INVALID     $a0, $t4, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD08D1001, NULL, ".word       0xD08D1001                   # INVALID     $a0, $t5, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD08D1081, NULL, ".word       0xD08D1081                   # INVALID     $a0, $t5, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD08D9001, NULL, ".word       0xD08D9001                   # INVALID     $a0, $t5, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD08D9081, NULL, ".word       0xD08D9081                   # INVALID     $a0, $t5, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD08E1001, NULL, ".word       0xD08E1001                   # INVALID     $a0, $t6, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD08E1081, NULL, ".word       0xD08E1081                   # INVALID     $a0, $t6, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD08E9001, NULL, ".word       0xD08E9001                   # INVALID     $a0, $t6, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD08E9081, NULL, ".word       0xD08E9081                   # INVALID     $a0, $t6, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD08F1001, NULL, ".word       0xD08F1001                   # INVALID     $a0, $t7, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD08F1081, NULL, ".word       0xD08F1081                   # INVALID     $a0, $t7, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD08F9001, NULL, ".word       0xD08F9001                   # INVALID     $a0, $t7, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD08F9081, NULL, ".word       0xD08F9081                   # INVALID     $a0, $t7, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0901001, NULL, ".word       0xD0901001                   # INVALID     $a0, $s0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0901081, NULL, ".word       0xD0901081                   # INVALID     $a0, $s0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0909001, NULL, ".word       0xD0909001                   # INVALID     $a0, $s0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0909081, NULL, ".word       0xD0909081                   # INVALID     $a0, $s0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0911001, NULL, ".word       0xD0911001                   # INVALID     $a0, $s1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0911081, NULL, ".word       0xD0911081                   # INVALID     $a0, $s1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0919001, NULL, ".word       0xD0919001                   # INVALID     $a0, $s1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0919081, NULL, ".word       0xD0919081                   # INVALID     $a0, $s1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0921001, NULL, ".word       0xD0921001                   # INVALID     $a0, $s2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0921081, NULL, ".word       0xD0921081                   # INVALID     $a0, $s2, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0929001, NULL, ".word       0xD0929001                   # INVALID     $a0, $s2, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0929081, NULL, ".word       0xD0929081                   # INVALID     $a0, $s2, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0931001, NULL, ".word       0xD0931001                   # INVALID     $a0, $s3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0931081, NULL, ".word       0xD0931081                   # INVALID     $a0, $s3, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0939001, NULL, ".word       0xD0939001                   # INVALID     $a0, $s3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0939081, NULL, ".word       0xD0939081                   # INVALID     $a0, $s3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0941001, NULL, ".word       0xD0941001                   # INVALID     $a0, $s4, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0941081, NULL, ".word       0xD0941081                   # INVALID     $a0, $s4, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0949001, NULL, ".word       0xD0949001                   # INVALID     $a0, $s4, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0949081, NULL, ".word       0xD0949081                   # INVALID     $a0, $s4, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0951001, NULL, ".word       0xD0951001                   # INVALID     $a0, $s5, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0951081, NULL, ".word       0xD0951081                   # INVALID     $a0, $s5, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0959001, NULL, ".word       0xD0959001                   # INVALID     $a0, $s5, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0959081, NULL, ".word       0xD0959081                   # INVALID     $a0, $s5, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0961001, NULL, ".word       0xD0961001                   # INVALID     $a0, $s6, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0961081, NULL, ".word       0xD0961081                   # INVALID     $a0, $s6, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0969001, NULL, ".word       0xD0969001                   # INVALID     $a0, $s6, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0969081, NULL, ".word       0xD0969081                   # INVALID     $a0, $s6, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0971001, NULL, ".word       0xD0971001                   # INVALID     $a0, $s7, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0971081, NULL, ".word       0xD0971081                   # INVALID     $a0, $s7, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0979001, NULL, ".word       0xD0979001                   # INVALID     $a0, $s7, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0979081, NULL, ".word       0xD0979081                   # INVALID     $a0, $s7, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0981001, NULL, ".word       0xD0981001                   # INVALID     $a0, $t8, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0981081, NULL, ".word       0xD0981081                   # INVALID     $a0, $t8, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0989001, NULL, ".word       0xD0989001                   # INVALID     $a0, $t8, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0989081, NULL, ".word       0xD0989081                   # INVALID     $a0, $t8, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0991001, NULL, ".word       0xD0991001                   # INVALID     $a0, $t9, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0991081, NULL, ".word       0xD0991081                   # INVALID     $a0, $t9, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0999001, NULL, ".word       0xD0999001                   # INVALID     $a0, $t9, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0999081, NULL, ".word       0xD0999081                   # INVALID     $a0, $t9, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD09A1001, NULL, ".word       0xD09A1001                   # INVALID     $a0, $k0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD09A1081, NULL, ".word       0xD09A1081                   # INVALID     $a0, $k0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD09A9001, NULL, ".word       0xD09A9001                   # INVALID     $a0, $k0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD09A9081, NULL, ".word       0xD09A9081                   # INVALID     $a0, $k0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD09B1001, NULL, ".word       0xD09B1001                   # INVALID     $a0, $k1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD09B1081, NULL, ".word       0xD09B1081                   # INVALID     $a0, $k1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD09B9001, NULL, ".word       0xD09B9001                   # INVALID     $a0, $k1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD09B9081, NULL, ".word       0xD09B9081                   # INVALID     $a0, $k1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD09C1001, NULL, ".word       0xD09C1001                   # INVALID     $a0, $gp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD09C1081, NULL, ".word       0xD09C1081                   # INVALID     $a0, $gp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD09C9001, NULL, ".word       0xD09C9001                   # INVALID     $a0, $gp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD09C9081, NULL, ".word       0xD09C9081                   # INVALID     $a0, $gp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD09D1001, NULL, ".word       0xD09D1001                   # INVALID     $a0, $sp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD09D1081, NULL, ".word       0xD09D1081                   # INVALID     $a0, $sp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD09D9001, NULL, ".word       0xD09D9001                   # INVALID     $a0, $sp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD09D9081, NULL, ".word       0xD09D9081                   # INVALID     $a0, $sp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD09E1001, NULL, ".word       0xD09E1001                   # INVALID     $a0, $fp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD09E1081, NULL, ".word       0xD09E1081                   # INVALID     $a0, $fp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD09E9001, NULL, ".word       0xD09E9001                   # INVALID     $a0, $fp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD09E9081, NULL, ".word       0xD09E9081                   # INVALID     $a0, $fp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD09F1001, NULL, ".word       0xD09F1001                   # INVALID     $a0, $ra, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD09F1081, NULL, ".word       0xD09F1081                   # INVALID     $a0, $ra, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD09F9001, NULL, ".word       0xD09F9001                   # INVALID     $a0, $ra, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD09F9081, NULL, ".word       0xD09F9081                   # INVALID     $a0, $ra, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A01001, NULL, ".word       0xD0A01001                   # INVALID     $a1, $zero, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A01081, NULL, ".word       0xD0A01081                   # INVALID     $a1, $zero, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A09001, NULL, ".word       0xD0A09001                   # INVALID     $a1, $zero, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A09081, NULL, ".word       0xD0A09081                   # INVALID     $a1, $zero, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A11001, NULL, ".word       0xD0A11001                   # INVALID     $a1, $at, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A11081, NULL, ".word       0xD0A11081                   # INVALID     $a1, $at, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A19001, NULL, ".word       0xD0A19001                   # INVALID     $a1, $at, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A19081, NULL, ".word       0xD0A19081                   # INVALID     $a1, $at, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A21001, NULL, ".word       0xD0A21001                   # INVALID     $a1, $v0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A21081, NULL, ".word       0xD0A21081                   # INVALID     $a1, $v0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A29001, NULL, ".word       0xD0A29001                   # INVALID     $a1, $v0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A29081, NULL, ".word       0xD0A29081                   # INVALID     $a1, $v0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A31001, NULL, ".word       0xD0A31001                   # INVALID     $a1, $v1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A31081, NULL, ".word       0xD0A31081                   # INVALID     $a1, $v1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A39001, NULL, ".word       0xD0A39001                   # INVALID     $a1, $v1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A39081, NULL, ".word       0xD0A39081                   # INVALID     $a1, $v1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A41001, NULL, ".word       0xD0A41001                   # INVALID     $a1, $a0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A41081, NULL, ".word       0xD0A41081                   # INVALID     $a1, $a0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A49001, NULL, ".word       0xD0A49001                   # INVALID     $a1, $a0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A49081, NULL, ".word       0xD0A49081                   # INVALID     $a1, $a0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A51001, NULL, ".word       0xD0A51001                   # INVALID     $a1, $a1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A51081, NULL, ".word       0xD0A51081                   # INVALID     $a1, $a1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A59001, NULL, ".word       0xD0A59001                   # INVALID     $a1, $a1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A59081, NULL, ".word       0xD0A59081                   # INVALID     $a1, $a1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A61001, NULL, ".word       0xD0A61001                   # INVALID     $a1, $a2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A61081, NULL, ".word       0xD0A61081                   # INVALID     $a1, $a2, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A69001, NULL, ".word       0xD0A69001                   # INVALID     $a1, $a2, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A69081, NULL, ".word       0xD0A69081                   # INVALID     $a1, $a2, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A71001, NULL, ".word       0xD0A71001                   # INVALID     $a1, $a3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A71081, NULL, ".word       0xD0A71081                   # INVALID     $a1, $a3, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A79001, NULL, ".word       0xD0A79001                   # INVALID     $a1, $a3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A79081, NULL, ".word       0xD0A79081                   # INVALID     $a1, $a3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A81001, NULL, ".word       0xD0A81001                   # INVALID     $a1, $t0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A81081, NULL, ".word       0xD0A81081                   # INVALID     $a1, $t0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A89001, NULL, ".word       0xD0A89001                   # INVALID     $a1, $t0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A89081, NULL, ".word       0xD0A89081                   # INVALID     $a1, $t0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A91001, NULL, ".word       0xD0A91001                   # INVALID     $a1, $t1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A91081, NULL, ".word       0xD0A91081                   # INVALID     $a1, $t1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A99001, NULL, ".word       0xD0A99001                   # INVALID     $a1, $t1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0A99081, NULL, ".word       0xD0A99081                   # INVALID     $a1, $t1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0AA1001, NULL, ".word       0xD0AA1001                   # INVALID     $a1, $t2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0AA1081, NULL, ".word       0xD0AA1081                   # INVALID     $a1, $t2, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0AA9001, NULL, ".word       0xD0AA9001                   # INVALID     $a1, $t2, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0AA9081, NULL, ".word       0xD0AA9081                   # INVALID     $a1, $t2, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0AB1001, NULL, ".word       0xD0AB1001                   # INVALID     $a1, $t3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0AB1081, NULL, ".word       0xD0AB1081                   # INVALID     $a1, $t3, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0AB9001, NULL, ".word       0xD0AB9001                   # INVALID     $a1, $t3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0AB9081, NULL, ".word       0xD0AB9081                   # INVALID     $a1, $t3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0AC1001, NULL, ".word       0xD0AC1001                   # INVALID     $a1, $t4, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0AC1081, NULL, ".word       0xD0AC1081                   # INVALID     $a1, $t4, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0AC9001, NULL, ".word       0xD0AC9001                   # INVALID     $a1, $t4, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0AC9081, NULL, ".word       0xD0AC9081                   # INVALID     $a1, $t4, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0AD1001, NULL, ".word       0xD0AD1001                   # INVALID     $a1, $t5, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0AD1081, NULL, ".word       0xD0AD1081                   # INVALID     $a1, $t5, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0AD9001, NULL, ".word       0xD0AD9001                   # INVALID     $a1, $t5, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0AD9081, NULL, ".word       0xD0AD9081                   # INVALID     $a1, $t5, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0AE1001, NULL, ".word       0xD0AE1001                   # INVALID     $a1, $t6, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0AE1081, NULL, ".word       0xD0AE1081                   # INVALID     $a1, $t6, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0AE9001, NULL, ".word       0xD0AE9001                   # INVALID     $a1, $t6, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0AE9081, NULL, ".word       0xD0AE9081                   # INVALID     $a1, $t6, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0AF1001, NULL, ".word       0xD0AF1001                   # INVALID     $a1, $t7, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0AF1081, NULL, ".word       0xD0AF1081                   # INVALID     $a1, $t7, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0AF9001, NULL, ".word       0xD0AF9001                   # INVALID     $a1, $t7, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0AF9081, NULL, ".word       0xD0AF9081                   # INVALID     $a1, $t7, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B01001, NULL, ".word       0xD0B01001                   # INVALID     $a1, $s0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B01081, NULL, ".word       0xD0B01081                   # INVALID     $a1, $s0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B09001, NULL, ".word       0xD0B09001                   # INVALID     $a1, $s0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B09081, NULL, ".word       0xD0B09081                   # INVALID     $a1, $s0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B11001, NULL, ".word       0xD0B11001                   # INVALID     $a1, $s1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B11081, NULL, ".word       0xD0B11081                   # INVALID     $a1, $s1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B19001, NULL, ".word       0xD0B19001                   # INVALID     $a1, $s1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B19081, NULL, ".word       0xD0B19081                   # INVALID     $a1, $s1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B21001, NULL, ".word       0xD0B21001                   # INVALID     $a1, $s2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B21081, NULL, ".word       0xD0B21081                   # INVALID     $a1, $s2, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B29001, NULL, ".word       0xD0B29001                   # INVALID     $a1, $s2, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B29081, NULL, ".word       0xD0B29081                   # INVALID     $a1, $s2, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B31001, NULL, ".word       0xD0B31001                   # INVALID     $a1, $s3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B31081, NULL, ".word       0xD0B31081                   # INVALID     $a1, $s3, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B39001, NULL, ".word       0xD0B39001                   # INVALID     $a1, $s3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B39081, NULL, ".word       0xD0B39081                   # INVALID     $a1, $s3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B41001, NULL, ".word       0xD0B41001                   # INVALID     $a1, $s4, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B41081, NULL, ".word       0xD0B41081                   # INVALID     $a1, $s4, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B49001, NULL, ".word       0xD0B49001                   # INVALID     $a1, $s4, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B49081, NULL, ".word       0xD0B49081                   # INVALID     $a1, $s4, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B51001, NULL, ".word       0xD0B51001                   # INVALID     $a1, $s5, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B51081, NULL, ".word       0xD0B51081                   # INVALID     $a1, $s5, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B59001, NULL, ".word       0xD0B59001                   # INVALID     $a1, $s5, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B59081, NULL, ".word       0xD0B59081                   # INVALID     $a1, $s5, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B61001, NULL, ".word       0xD0B61001                   # INVALID     $a1, $s6, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B61081, NULL, ".word       0xD0B61081                   # INVALID     $a1, $s6, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B69001, NULL, ".word       0xD0B69001                   # INVALID     $a1, $s6, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B69081, NULL, ".word       0xD0B69081                   # INVALID     $a1, $s6, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B71001, NULL, ".word       0xD0B71001                   # INVALID     $a1, $s7, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B71081, NULL, ".word       0xD0B71081                   # INVALID     $a1, $s7, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B79001, NULL, ".word       0xD0B79001                   # INVALID     $a1, $s7, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B79081, NULL, ".word       0xD0B79081                   # INVALID     $a1, $s7, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B81001, NULL, ".word       0xD0B81001                   # INVALID     $a1, $t8, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B81081, NULL, ".word       0xD0B81081                   # INVALID     $a1, $t8, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B89001, NULL, ".word       0xD0B89001                   # INVALID     $a1, $t8, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B89081, NULL, ".word       0xD0B89081                   # INVALID     $a1, $t8, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B91001, NULL, ".word       0xD0B91001                   # INVALID     $a1, $t9, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B91081, NULL, ".word       0xD0B91081                   # INVALID     $a1, $t9, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B99001, NULL, ".word       0xD0B99001                   # INVALID     $a1, $t9, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0B99081, NULL, ".word       0xD0B99081                   # INVALID     $a1, $t9, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0BA1001, NULL, ".word       0xD0BA1001                   # INVALID     $a1, $k0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0BA1081, NULL, ".word       0xD0BA1081                   # INVALID     $a1, $k0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0BA9001, NULL, ".word       0xD0BA9001                   # INVALID     $a1, $k0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0BA9081, NULL, ".word       0xD0BA9081                   # INVALID     $a1, $k0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0BB1001, NULL, ".word       0xD0BB1001                   # INVALID     $a1, $k1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0BB1081, NULL, ".word       0xD0BB1081                   # INVALID     $a1, $k1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0BB9001, NULL, ".word       0xD0BB9001                   # INVALID     $a1, $k1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0BB9081, NULL, ".word       0xD0BB9081                   # INVALID     $a1, $k1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0BC1001, NULL, ".word       0xD0BC1001                   # INVALID     $a1, $gp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0BC1081, NULL, ".word       0xD0BC1081                   # INVALID     $a1, $gp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0BC9001, NULL, ".word       0xD0BC9001                   # INVALID     $a1, $gp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0BC9081, NULL, ".word       0xD0BC9081                   # INVALID     $a1, $gp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0BD1001, NULL, ".word       0xD0BD1001                   # INVALID     $a1, $sp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0BD1081, NULL, ".word       0xD0BD1081                   # INVALID     $a1, $sp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0BD9001, NULL, ".word       0xD0BD9001                   # INVALID     $a1, $sp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0BD9081, NULL, ".word       0xD0BD9081                   # INVALID     $a1, $sp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0BE1001, NULL, ".word       0xD0BE1001                   # INVALID     $a1, $fp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0BE1081, NULL, ".word       0xD0BE1081                   # INVALID     $a1, $fp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0BE9001, NULL, ".word       0xD0BE9001                   # INVALID     $a1, $fp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0BE9081, NULL, ".word       0xD0BE9081                   # INVALID     $a1, $fp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0BF1001, NULL, ".word       0xD0BF1001                   # INVALID     $a1, $ra, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0BF1081, NULL, ".word       0xD0BF1081                   # INVALID     $a1, $ra, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0BF9001, NULL, ".word       0xD0BF9001                   # INVALID     $a1, $ra, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0BF9081, NULL, ".word       0xD0BF9081                   # INVALID     $a1, $ra, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C01001, NULL, ".word       0xD0C01001                   # INVALID     $a2, $zero, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C01081, NULL, ".word       0xD0C01081                   # INVALID     $a2, $zero, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C09001, NULL, ".word       0xD0C09001                   # INVALID     $a2, $zero, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C09081, NULL, ".word       0xD0C09081                   # INVALID     $a2, $zero, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C11001, NULL, ".word       0xD0C11001                   # INVALID     $a2, $at, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C11081, NULL, ".word       0xD0C11081                   # INVALID     $a2, $at, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C19001, NULL, ".word       0xD0C19001                   # INVALID     $a2, $at, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C19081, NULL, ".word       0xD0C19081                   # INVALID     $a2, $at, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C21001, NULL, ".word       0xD0C21001                   # INVALID     $a2, $v0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C21081, NULL, ".word       0xD0C21081                   # INVALID     $a2, $v0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C29001, NULL, ".word       0xD0C29001                   # INVALID     $a2, $v0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C29081, NULL, ".word       0xD0C29081                   # INVALID     $a2, $v0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C31001, NULL, ".word       0xD0C31001                   # INVALID     $a2, $v1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C31081, NULL, ".word       0xD0C31081                   # INVALID     $a2, $v1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C39001, NULL, ".word       0xD0C39001                   # INVALID     $a2, $v1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C39081, NULL, ".word       0xD0C39081                   # INVALID     $a2, $v1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C41001, NULL, ".word       0xD0C41001                   # INVALID     $a2, $a0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C41081, NULL, ".word       0xD0C41081                   # INVALID     $a2, $a0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C49001, NULL, ".word       0xD0C49001                   # INVALID     $a2, $a0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C49081, NULL, ".word       0xD0C49081                   # INVALID     $a2, $a0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C51001, NULL, ".word       0xD0C51001                   # INVALID     $a2, $a1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C51081, NULL, ".word       0xD0C51081                   # INVALID     $a2, $a1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C59001, NULL, ".word       0xD0C59001                   # INVALID     $a2, $a1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C59081, NULL, ".word       0xD0C59081                   # INVALID     $a2, $a1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C61001, NULL, ".word       0xD0C61001                   # INVALID     $a2, $a2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C61081, NULL, ".word       0xD0C61081                   # INVALID     $a2, $a2, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C69001, NULL, ".word       0xD0C69001                   # INVALID     $a2, $a2, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C69081, NULL, ".word       0xD0C69081                   # INVALID     $a2, $a2, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C71001, NULL, ".word       0xD0C71001                   # INVALID     $a2, $a3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C71081, NULL, ".word       0xD0C71081                   # INVALID     $a2, $a3, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C79001, NULL, ".word       0xD0C79001                   # INVALID     $a2, $a3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C79081, NULL, ".word       0xD0C79081                   # INVALID     $a2, $a3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C81001, NULL, ".word       0xD0C81001                   # INVALID     $a2, $t0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C81081, NULL, ".word       0xD0C81081                   # INVALID     $a2, $t0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C89001, NULL, ".word       0xD0C89001                   # INVALID     $a2, $t0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C89081, NULL, ".word       0xD0C89081                   # INVALID     $a2, $t0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C91001, NULL, ".word       0xD0C91001                   # INVALID     $a2, $t1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C91081, NULL, ".word       0xD0C91081                   # INVALID     $a2, $t1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C99001, NULL, ".word       0xD0C99001                   # INVALID     $a2, $t1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0C99081, NULL, ".word       0xD0C99081                   # INVALID     $a2, $t1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0CA1001, NULL, ".word       0xD0CA1001                   # INVALID     $a2, $t2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0CA1081, NULL, ".word       0xD0CA1081                   # INVALID     $a2, $t2, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0CA9001, NULL, ".word       0xD0CA9001                   # INVALID     $a2, $t2, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0CA9081, NULL, ".word       0xD0CA9081                   # INVALID     $a2, $t2, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0CB1001, NULL, ".word       0xD0CB1001                   # INVALID     $a2, $t3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0CB1081, NULL, ".word       0xD0CB1081                   # INVALID     $a2, $t3, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0CB9001, NULL, ".word       0xD0CB9001                   # INVALID     $a2, $t3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0CB9081, NULL, ".word       0xD0CB9081                   # INVALID     $a2, $t3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0CC1001, NULL, ".word       0xD0CC1001                   # INVALID     $a2, $t4, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0CC1081, NULL, ".word       0xD0CC1081                   # INVALID     $a2, $t4, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0CC9001, NULL, ".word       0xD0CC9001                   # INVALID     $a2, $t4, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0CC9081, NULL, ".word       0xD0CC9081                   # INVALID     $a2, $t4, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0CD1001, NULL, ".word       0xD0CD1001                   # INVALID     $a2, $t5, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0CD1081, NULL, ".word       0xD0CD1081                   # INVALID     $a2, $t5, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0CD9001, NULL, ".word       0xD0CD9001                   # INVALID     $a2, $t5, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0CD9081, NULL, ".word       0xD0CD9081                   # INVALID     $a2, $t5, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0CE1001, NULL, ".word       0xD0CE1001                   # INVALID     $a2, $t6, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0CE1081, NULL, ".word       0xD0CE1081                   # INVALID     $a2, $t6, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0CE9001, NULL, ".word       0xD0CE9001                   # INVALID     $a2, $t6, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0CE9081, NULL, ".word       0xD0CE9081                   # INVALID     $a2, $t6, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0CF1001, NULL, ".word       0xD0CF1001                   # INVALID     $a2, $t7, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0CF1081, NULL, ".word       0xD0CF1081                   # INVALID     $a2, $t7, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0CF9001, NULL, ".word       0xD0CF9001                   # INVALID     $a2, $t7, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0CF9081, NULL, ".word       0xD0CF9081                   # INVALID     $a2, $t7, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D01001, NULL, ".word       0xD0D01001                   # INVALID     $a2, $s0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D01081, NULL, ".word       0xD0D01081                   # INVALID     $a2, $s0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D09001, NULL, ".word       0xD0D09001                   # INVALID     $a2, $s0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D09081, NULL, ".word       0xD0D09081                   # INVALID     $a2, $s0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D11001, NULL, ".word       0xD0D11001                   # INVALID     $a2, $s1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D11081, NULL, ".word       0xD0D11081                   # INVALID     $a2, $s1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D19001, NULL, ".word       0xD0D19001                   # INVALID     $a2, $s1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D19081, NULL, ".word       0xD0D19081                   # INVALID     $a2, $s1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D21001, NULL, ".word       0xD0D21001                   # INVALID     $a2, $s2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D21081, NULL, ".word       0xD0D21081                   # INVALID     $a2, $s2, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D29001, NULL, ".word       0xD0D29001                   # INVALID     $a2, $s2, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D29081, NULL, ".word       0xD0D29081                   # INVALID     $a2, $s2, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D31001, NULL, ".word       0xD0D31001                   # INVALID     $a2, $s3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D31081, NULL, ".word       0xD0D31081                   # INVALID     $a2, $s3, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D39001, NULL, ".word       0xD0D39001                   # INVALID     $a2, $s3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D39081, NULL, ".word       0xD0D39081                   # INVALID     $a2, $s3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D41001, NULL, ".word       0xD0D41001                   # INVALID     $a2, $s4, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D41081, NULL, ".word       0xD0D41081                   # INVALID     $a2, $s4, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D49001, NULL, ".word       0xD0D49001                   # INVALID     $a2, $s4, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D49081, NULL, ".word       0xD0D49081                   # INVALID     $a2, $s4, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D51001, NULL, ".word       0xD0D51001                   # INVALID     $a2, $s5, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D51081, NULL, ".word       0xD0D51081                   # INVALID     $a2, $s5, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D59001, NULL, ".word       0xD0D59001                   # INVALID     $a2, $s5, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D59081, NULL, ".word       0xD0D59081                   # INVALID     $a2, $s5, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D61001, NULL, ".word       0xD0D61001                   # INVALID     $a2, $s6, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D61081, NULL, ".word       0xD0D61081                   # INVALID     $a2, $s6, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D69001, NULL, ".word       0xD0D69001                   # INVALID     $a2, $s6, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D69081, NULL, ".word       0xD0D69081                   # INVALID     $a2, $s6, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D71001, NULL, ".word       0xD0D71001                   # INVALID     $a2, $s7, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D71081, NULL, ".word       0xD0D71081                   # INVALID     $a2, $s7, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D79001, NULL, ".word       0xD0D79001                   # INVALID     $a2, $s7, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D79081, NULL, ".word       0xD0D79081                   # INVALID     $a2, $s7, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D81001, NULL, ".word       0xD0D81001                   # INVALID     $a2, $t8, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D81081, NULL, ".word       0xD0D81081                   # INVALID     $a2, $t8, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D89001, NULL, ".word       0xD0D89001                   # INVALID     $a2, $t8, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D89081, NULL, ".word       0xD0D89081                   # INVALID     $a2, $t8, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D91001, NULL, ".word       0xD0D91001                   # INVALID     $a2, $t9, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D91081, NULL, ".word       0xD0D91081                   # INVALID     $a2, $t9, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D99001, NULL, ".word       0xD0D99001                   # INVALID     $a2, $t9, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0D99081, NULL, ".word       0xD0D99081                   # INVALID     $a2, $t9, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0DA1001, NULL, ".word       0xD0DA1001                   # INVALID     $a2, $k0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0DA1081, NULL, ".word       0xD0DA1081                   # INVALID     $a2, $k0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0DA9001, NULL, ".word       0xD0DA9001                   # INVALID     $a2, $k0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0DA9081, NULL, ".word       0xD0DA9081                   # INVALID     $a2, $k0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0DB1001, NULL, ".word       0xD0DB1001                   # INVALID     $a2, $k1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0DB1081, NULL, ".word       0xD0DB1081                   # INVALID     $a2, $k1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0DB9001, NULL, ".word       0xD0DB9001                   # INVALID     $a2, $k1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0DB9081, NULL, ".word       0xD0DB9081                   # INVALID     $a2, $k1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0DC1001, NULL, ".word       0xD0DC1001                   # INVALID     $a2, $gp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0DC1081, NULL, ".word       0xD0DC1081                   # INVALID     $a2, $gp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0DC9001, NULL, ".word       0xD0DC9001                   # INVALID     $a2, $gp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0DC9081, NULL, ".word       0xD0DC9081                   # INVALID     $a2, $gp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0DD1001, NULL, ".word       0xD0DD1001                   # INVALID     $a2, $sp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0DD1081, NULL, ".word       0xD0DD1081                   # INVALID     $a2, $sp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0DD9001, NULL, ".word       0xD0DD9001                   # INVALID     $a2, $sp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0DD9081, NULL, ".word       0xD0DD9081                   # INVALID     $a2, $sp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0DE1001, NULL, ".word       0xD0DE1001                   # INVALID     $a2, $fp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0DE1081, NULL, ".word       0xD0DE1081                   # INVALID     $a2, $fp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0DE9001, NULL, ".word       0xD0DE9001                   # INVALID     $a2, $fp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0DE9081, NULL, ".word       0xD0DE9081                   # INVALID     $a2, $fp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0DF1001, NULL, ".word       0xD0DF1001                   # INVALID     $a2, $ra, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0DF1081, NULL, ".word       0xD0DF1081                   # INVALID     $a2, $ra, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0DF9001, NULL, ".word       0xD0DF9001                   # INVALID     $a2, $ra, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0DF9081, NULL, ".word       0xD0DF9081                   # INVALID     $a2, $ra, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E01001, NULL, ".word       0xD0E01001                   # INVALID     $a3, $zero, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E01081, NULL, ".word       0xD0E01081                   # INVALID     $a3, $zero, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E09001, NULL, ".word       0xD0E09001                   # INVALID     $a3, $zero, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E09081, NULL, ".word       0xD0E09081                   # INVALID     $a3, $zero, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E11001, NULL, ".word       0xD0E11001                   # INVALID     $a3, $at, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E11081, NULL, ".word       0xD0E11081                   # INVALID     $a3, $at, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E19001, NULL, ".word       0xD0E19001                   # INVALID     $a3, $at, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E19081, NULL, ".word       0xD0E19081                   # INVALID     $a3, $at, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E21001, NULL, ".word       0xD0E21001                   # INVALID     $a3, $v0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E21081, NULL, ".word       0xD0E21081                   # INVALID     $a3, $v0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E29001, NULL, ".word       0xD0E29001                   # INVALID     $a3, $v0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E29081, NULL, ".word       0xD0E29081                   # INVALID     $a3, $v0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E31001, NULL, ".word       0xD0E31001                   # INVALID     $a3, $v1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E31081, NULL, ".word       0xD0E31081                   # INVALID     $a3, $v1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E39001, NULL, ".word       0xD0E39001                   # INVALID     $a3, $v1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E39081, NULL, ".word       0xD0E39081                   # INVALID     $a3, $v1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E41001, NULL, ".word       0xD0E41001                   # INVALID     $a3, $a0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E41081, NULL, ".word       0xD0E41081                   # INVALID     $a3, $a0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E49001, NULL, ".word       0xD0E49001                   # INVALID     $a3, $a0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E49081, NULL, ".word       0xD0E49081                   # INVALID     $a3, $a0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E51001, NULL, ".word       0xD0E51001                   # INVALID     $a3, $a1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E51081, NULL, ".word       0xD0E51081                   # INVALID     $a3, $a1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E59001, NULL, ".word       0xD0E59001                   # INVALID     $a3, $a1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E59081, NULL, ".word       0xD0E59081                   # INVALID     $a3, $a1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E61001, NULL, ".word       0xD0E61001                   # INVALID     $a3, $a2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E61081, NULL, ".word       0xD0E61081                   # INVALID     $a3, $a2, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E69001, NULL, ".word       0xD0E69001                   # INVALID     $a3, $a2, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E69081, NULL, ".word       0xD0E69081                   # INVALID     $a3, $a2, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E71001, NULL, ".word       0xD0E71001                   # INVALID     $a3, $a3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E71081, NULL, ".word       0xD0E71081                   # INVALID     $a3, $a3, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E79001, NULL, ".word       0xD0E79001                   # INVALID     $a3, $a3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E79081, NULL, ".word       0xD0E79081                   # INVALID     $a3, $a3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E81001, NULL, ".word       0xD0E81001                   # INVALID     $a3, $t0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E81081, NULL, ".word       0xD0E81081                   # INVALID     $a3, $t0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E89001, NULL, ".word       0xD0E89001                   # INVALID     $a3, $t0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E89081, NULL, ".word       0xD0E89081                   # INVALID     $a3, $t0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E91001, NULL, ".word       0xD0E91001                   # INVALID     $a3, $t1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E91081, NULL, ".word       0xD0E91081                   # INVALID     $a3, $t1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E99001, NULL, ".word       0xD0E99001                   # INVALID     $a3, $t1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0E99081, NULL, ".word       0xD0E99081                   # INVALID     $a3, $t1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0EA1001, NULL, ".word       0xD0EA1001                   # INVALID     $a3, $t2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0EA1081, NULL, ".word       0xD0EA1081                   # INVALID     $a3, $t2, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0EA9001, NULL, ".word       0xD0EA9001                   # INVALID     $a3, $t2, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0EA9081, NULL, ".word       0xD0EA9081                   # INVALID     $a3, $t2, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0EB1001, NULL, ".word       0xD0EB1001                   # INVALID     $a3, $t3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0EB1081, NULL, ".word       0xD0EB1081                   # INVALID     $a3, $t3, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0EB9001, NULL, ".word       0xD0EB9001                   # INVALID     $a3, $t3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0EB9081, NULL, ".word       0xD0EB9081                   # INVALID     $a3, $t3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0EC1001, NULL, ".word       0xD0EC1001                   # INVALID     $a3, $t4, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0EC1081, NULL, ".word       0xD0EC1081                   # INVALID     $a3, $t4, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0EC9001, NULL, ".word       0xD0EC9001                   # INVALID     $a3, $t4, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0EC9081, NULL, ".word       0xD0EC9081                   # INVALID     $a3, $t4, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0ED1001, NULL, ".word       0xD0ED1001                   # INVALID     $a3, $t5, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0ED1081, NULL, ".word       0xD0ED1081                   # INVALID     $a3, $t5, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0ED9001, NULL, ".word       0xD0ED9001                   # INVALID     $a3, $t5, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0ED9081, NULL, ".word       0xD0ED9081                   # INVALID     $a3, $t5, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0EE1001, NULL, ".word       0xD0EE1001                   # INVALID     $a3, $t6, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0EE1081, NULL, ".word       0xD0EE1081                   # INVALID     $a3, $t6, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0EE9001, NULL, ".word       0xD0EE9001                   # INVALID     $a3, $t6, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0EE9081, NULL, ".word       0xD0EE9081                   # INVALID     $a3, $t6, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0EF1001, NULL, ".word       0xD0EF1001                   # INVALID     $a3, $t7, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0EF1081, NULL, ".word       0xD0EF1081                   # INVALID     $a3, $t7, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0EF9001, NULL, ".word       0xD0EF9001                   # INVALID     $a3, $t7, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0EF9081, NULL, ".word       0xD0EF9081                   # INVALID     $a3, $t7, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F01001, NULL, ".word       0xD0F01001                   # INVALID     $a3, $s0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F01081, NULL, ".word       0xD0F01081                   # INVALID     $a3, $s0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F09001, NULL, ".word       0xD0F09001                   # INVALID     $a3, $s0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F09081, NULL, ".word       0xD0F09081                   # INVALID     $a3, $s0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F11001, NULL, ".word       0xD0F11001                   # INVALID     $a3, $s1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F11081, NULL, ".word       0xD0F11081                   # INVALID     $a3, $s1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F19001, NULL, ".word       0xD0F19001                   # INVALID     $a3, $s1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F19081, NULL, ".word       0xD0F19081                   # INVALID     $a3, $s1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F21001, NULL, ".word       0xD0F21001                   # INVALID     $a3, $s2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F21081, NULL, ".word       0xD0F21081                   # INVALID     $a3, $s2, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F29001, NULL, ".word       0xD0F29001                   # INVALID     $a3, $s2, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F29081, NULL, ".word       0xD0F29081                   # INVALID     $a3, $s2, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F31001, NULL, ".word       0xD0F31001                   # INVALID     $a3, $s3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F31081, NULL, ".word       0xD0F31081                   # INVALID     $a3, $s3, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F39001, NULL, ".word       0xD0F39001                   # INVALID     $a3, $s3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F39081, NULL, ".word       0xD0F39081                   # INVALID     $a3, $s3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F41001, NULL, ".word       0xD0F41001                   # INVALID     $a3, $s4, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F41081, NULL, ".word       0xD0F41081                   # INVALID     $a3, $s4, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F49001, NULL, ".word       0xD0F49001                   # INVALID     $a3, $s4, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F49081, NULL, ".word       0xD0F49081                   # INVALID     $a3, $s4, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F51001, NULL, ".word       0xD0F51001                   # INVALID     $a3, $s5, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F51081, NULL, ".word       0xD0F51081                   # INVALID     $a3, $s5, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F59001, NULL, ".word       0xD0F59001                   # INVALID     $a3, $s5, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F59081, NULL, ".word       0xD0F59081                   # INVALID     $a3, $s5, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F61001, NULL, ".word       0xD0F61001                   # INVALID     $a3, $s6, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F61081, NULL, ".word       0xD0F61081                   # INVALID     $a3, $s6, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F69001, NULL, ".word       0xD0F69001                   # INVALID     $a3, $s6, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F69081, NULL, ".word       0xD0F69081                   # INVALID     $a3, $s6, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F71001, NULL, ".word       0xD0F71001                   # INVALID     $a3, $s7, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F71081, NULL, ".word       0xD0F71081                   # INVALID     $a3, $s7, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F79001, NULL, ".word       0xD0F79001                   # INVALID     $a3, $s7, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F79081, NULL, ".word       0xD0F79081                   # INVALID     $a3, $s7, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F81001, NULL, ".word       0xD0F81001                   # INVALID     $a3, $t8, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F81081, NULL, ".word       0xD0F81081                   # INVALID     $a3, $t8, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F89001, NULL, ".word       0xD0F89001                   # INVALID     $a3, $t8, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F89081, NULL, ".word       0xD0F89081                   # INVALID     $a3, $t8, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F91001, NULL, ".word       0xD0F91001                   # INVALID     $a3, $t9, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F91081, NULL, ".word       0xD0F91081                   # INVALID     $a3, $t9, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F99001, NULL, ".word       0xD0F99001                   # INVALID     $a3, $t9, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0F99081, NULL, ".word       0xD0F99081                   # INVALID     $a3, $t9, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0FA1001, NULL, ".word       0xD0FA1001                   # INVALID     $a3, $k0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0FA1081, NULL, ".word       0xD0FA1081                   # INVALID     $a3, $k0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0FA9001, NULL, ".word       0xD0FA9001                   # INVALID     $a3, $k0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0FA9081, NULL, ".word       0xD0FA9081                   # INVALID     $a3, $k0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0FB1001, NULL, ".word       0xD0FB1001                   # INVALID     $a3, $k1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0FB1081, NULL, ".word       0xD0FB1081                   # INVALID     $a3, $k1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0FB9001, NULL, ".word       0xD0FB9001                   # INVALID     $a3, $k1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0FB9081, NULL, ".word       0xD0FB9081                   # INVALID     $a3, $k1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0FC1001, NULL, ".word       0xD0FC1001                   # INVALID     $a3, $gp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0FC1081, NULL, ".word       0xD0FC1081                   # INVALID     $a3, $gp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0FC9001, NULL, ".word       0xD0FC9001                   # INVALID     $a3, $gp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0FC9081, NULL, ".word       0xD0FC9081                   # INVALID     $a3, $gp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0FD1001, NULL, ".word       0xD0FD1001                   # INVALID     $a3, $sp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0FD1081, NULL, ".word       0xD0FD1081                   # INVALID     $a3, $sp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0FD9001, NULL, ".word       0xD0FD9001                   # INVALID     $a3, $sp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0FD9081, NULL, ".word       0xD0FD9081                   # INVALID     $a3, $sp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0FE1001, NULL, ".word       0xD0FE1001                   # INVALID     $a3, $fp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0FE1081, NULL, ".word       0xD0FE1081                   # INVALID     $a3, $fp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0FE9001, NULL, ".word       0xD0FE9001                   # INVALID     $a3, $fp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0FE9081, NULL, ".word       0xD0FE9081                   # INVALID     $a3, $fp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0FF1001, NULL, ".word       0xD0FF1001                   # INVALID     $a3, $ra, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0FF1081, NULL, ".word       0xD0FF1081                   # INVALID     $a3, $ra, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0FF9001, NULL, ".word       0xD0FF9001                   # INVALID     $a3, $ra, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),
    TEST_ENTRY_C(0xD0FF9081, NULL, ".word       0xD0FF9081                   # INVALID     $a3, $ra, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT0>"),

    /* VFPU4 FMT2 */
    TEST_ENTRY_C(0xD2001001, NULL, "vf2in.s     S010, S400, 0"),
    TEST_ENTRY_C(0xD2001081, NULL, "vf2in.p     C010, C400, 0"),
    TEST_ENTRY_C(0xD2009001, NULL, "vf2in.t     C010, C400, 0"),
    TEST_ENTRY_C(0xD2009081, NULL, "vf2in.q     C010, C400, 0"),
    TEST_ENTRY_C(0xD2011001, NULL, "vf2in.s     S010, S400, 1"),
    TEST_ENTRY_C(0xD2011081, NULL, "vf2in.p     C010, C400, 1"),
    TEST_ENTRY_C(0xD2019001, NULL, "vf2in.t     C010, C400, 1"),
    TEST_ENTRY_C(0xD2019081, NULL, "vf2in.q     C010, C400, 1"),
    TEST_ENTRY_C(0xD2021001, NULL, "vf2in.s     S010, S400, 2"),
    TEST_ENTRY_C(0xD2021081, NULL, "vf2in.p     C010, C400, 2"),
    TEST_ENTRY_C(0xD2029001, NULL, "vf2in.t     C010, C400, 2"),
    TEST_ENTRY_C(0xD2029081, NULL, "vf2in.q     C010, C400, 2"),
    TEST_ENTRY_C(0xD2031001, NULL, "vf2in.s     S010, S400, 3"),
    TEST_ENTRY_C(0xD2031081, NULL, "vf2in.p     C010, C400, 3"),
    TEST_ENTRY_C(0xD2039001, NULL, "vf2in.t     C010, C400, 3"),
    TEST_ENTRY_C(0xD2039081, NULL, "vf2in.q     C010, C400, 3"),
    TEST_ENTRY_C(0xD2041001, NULL, "vf2in.s     S010, S400, 4"),
    TEST_ENTRY_C(0xD2041081, NULL, "vf2in.p     C010, C400, 4"),
    TEST_ENTRY_C(0xD2049001, NULL, "vf2in.t     C010, C400, 4"),
    TEST_ENTRY_C(0xD2049081, NULL, "vf2in.q     C010, C400, 4"),
    TEST_ENTRY_C(0xD2051001, NULL, "vf2in.s     S010, S400, 5"),
    TEST_ENTRY_C(0xD2051081, NULL, "vf2in.p     C010, C400, 5"),
    TEST_ENTRY_C(0xD2059001, NULL, "vf2in.t     C010, C400, 5"),
    TEST_ENTRY_C(0xD2059081, NULL, "vf2in.q     C010, C400, 5"),
    TEST_ENTRY_C(0xD2061001, NULL, "vf2in.s     S010, S400, 6"),
    TEST_ENTRY_C(0xD2061081, NULL, "vf2in.p     C010, C400, 6"),
    TEST_ENTRY_C(0xD2069001, NULL, "vf2in.t     C010, C400, 6"),
    TEST_ENTRY_C(0xD2069081, NULL, "vf2in.q     C010, C400, 6"),
    TEST_ENTRY_C(0xD2071001, NULL, "vf2in.s     S010, S400, 7"),
    TEST_ENTRY_C(0xD2071081, NULL, "vf2in.p     C010, C400, 7"),
    TEST_ENTRY_C(0xD2079001, NULL, "vf2in.t     C010, C400, 7"),
    TEST_ENTRY_C(0xD2079081, NULL, "vf2in.q     C010, C400, 7"),
    TEST_ENTRY_C(0xD2081001, NULL, "vf2in.s     S010, S400, 8"),
    TEST_ENTRY_C(0xD2081081, NULL, "vf2in.p     C010, C400, 8"),
    TEST_ENTRY_C(0xD2089001, NULL, "vf2in.t     C010, C400, 8"),
    TEST_ENTRY_C(0xD2089081, NULL, "vf2in.q     C010, C400, 8"),
    TEST_ENTRY_C(0xD2091001, NULL, "vf2in.s     S010, S400, 9"),
    TEST_ENTRY_C(0xD2091081, NULL, "vf2in.p     C010, C400, 9"),
    TEST_ENTRY_C(0xD2099001, NULL, "vf2in.t     C010, C400, 9"),
    TEST_ENTRY_C(0xD2099081, NULL, "vf2in.q     C010, C400, 9"),
    TEST_ENTRY_C(0xD20A1001, NULL, "vf2in.s     S010, S400, 10"),
    TEST_ENTRY_C(0xD20A1081, NULL, "vf2in.p     C010, C400, 10"),
    TEST_ENTRY_C(0xD20A9001, NULL, "vf2in.t     C010, C400, 10"),
    TEST_ENTRY_C(0xD20A9081, NULL, "vf2in.q     C010, C400, 10"),
    TEST_ENTRY_C(0xD20B1001, NULL, "vf2in.s     S010, S400, 11"),
    TEST_ENTRY_C(0xD20B1081, NULL, "vf2in.p     C010, C400, 11"),
    TEST_ENTRY_C(0xD20B9001, NULL, "vf2in.t     C010, C400, 11"),
    TEST_ENTRY_C(0xD20B9081, NULL, "vf2in.q     C010, C400, 11"),
    TEST_ENTRY_C(0xD20C1001, NULL, "vf2in.s     S010, S400, 12"),
    TEST_ENTRY_C(0xD20C1081, NULL, "vf2in.p     C010, C400, 12"),
    TEST_ENTRY_C(0xD20C9001, NULL, "vf2in.t     C010, C400, 12"),
    TEST_ENTRY_C(0xD20C9081, NULL, "vf2in.q     C010, C400, 12"),
    TEST_ENTRY_C(0xD20D1001, NULL, "vf2in.s     S010, S400, 13"),
    TEST_ENTRY_C(0xD20D1081, NULL, "vf2in.p     C010, C400, 13"),
    TEST_ENTRY_C(0xD20D9001, NULL, "vf2in.t     C010, C400, 13"),
    TEST_ENTRY_C(0xD20D9081, NULL, "vf2in.q     C010, C400, 13"),
    TEST_ENTRY_C(0xD20E1001, NULL, "vf2in.s     S010, S400, 14"),
    TEST_ENTRY_C(0xD20E1081, NULL, "vf2in.p     C010, C400, 14"),
    TEST_ENTRY_C(0xD20E9001, NULL, "vf2in.t     C010, C400, 14"),
    TEST_ENTRY_C(0xD20E9081, NULL, "vf2in.q     C010, C400, 14"),
    TEST_ENTRY_C(0xD20F1001, NULL, "vf2in.s     S010, S400, 15"),
    TEST_ENTRY_C(0xD20F1081, NULL, "vf2in.p     C010, C400, 15"),
    TEST_ENTRY_C(0xD20F9001, NULL, "vf2in.t     C010, C400, 15"),
    TEST_ENTRY_C(0xD20F9081, NULL, "vf2in.q     C010, C400, 15"),
    TEST_ENTRY_C(0xD2101001, NULL, "vf2in.s     S010, S400, 16"),
    TEST_ENTRY_C(0xD2101081, NULL, "vf2in.p     C010, C400, 16"),
    TEST_ENTRY_C(0xD2109001, NULL, "vf2in.t     C010, C400, 16"),
    TEST_ENTRY_C(0xD2109081, NULL, "vf2in.q     C010, C400, 16"),
    TEST_ENTRY_C(0xD2111001, NULL, "vf2in.s     S010, S400, 17"),
    TEST_ENTRY_C(0xD2111081, NULL, "vf2in.p     C010, C400, 17"),
    TEST_ENTRY_C(0xD2119001, NULL, "vf2in.t     C010, C400, 17"),
    TEST_ENTRY_C(0xD2119081, NULL, "vf2in.q     C010, C400, 17"),
    TEST_ENTRY_C(0xD2121001, NULL, "vf2in.s     S010, S400, 18"),
    TEST_ENTRY_C(0xD2121081, NULL, "vf2in.p     C010, C400, 18"),
    TEST_ENTRY_C(0xD2129001, NULL, "vf2in.t     C010, C400, 18"),
    TEST_ENTRY_C(0xD2129081, NULL, "vf2in.q     C010, C400, 18"),
    TEST_ENTRY_C(0xD2131001, NULL, "vf2in.s     S010, S400, 19"),
    TEST_ENTRY_C(0xD2131081, NULL, "vf2in.p     C010, C400, 19"),
    TEST_ENTRY_C(0xD2139001, NULL, "vf2in.t     C010, C400, 19"),
    TEST_ENTRY_C(0xD2139081, NULL, "vf2in.q     C010, C400, 19"),
    TEST_ENTRY_C(0xD2141001, NULL, "vf2in.s     S010, S400, 20"),
    TEST_ENTRY_C(0xD2141081, NULL, "vf2in.p     C010, C400, 20"),
    TEST_ENTRY_C(0xD2149001, NULL, "vf2in.t     C010, C400, 20"),
    TEST_ENTRY_C(0xD2149081, NULL, "vf2in.q     C010, C400, 20"),
    TEST_ENTRY_C(0xD2151001, NULL, "vf2in.s     S010, S400, 21"),
    TEST_ENTRY_C(0xD2151081, NULL, "vf2in.p     C010, C400, 21"),
    TEST_ENTRY_C(0xD2159001, NULL, "vf2in.t     C010, C400, 21"),
    TEST_ENTRY_C(0xD2159081, NULL, "vf2in.q     C010, C400, 21"),
    TEST_ENTRY_C(0xD2161001, NULL, "vf2in.s     S010, S400, 22"),
    TEST_ENTRY_C(0xD2161081, NULL, "vf2in.p     C010, C400, 22"),
    TEST_ENTRY_C(0xD2169001, NULL, "vf2in.t     C010, C400, 22"),
    TEST_ENTRY_C(0xD2169081, NULL, "vf2in.q     C010, C400, 22"),
    TEST_ENTRY_C(0xD2171001, NULL, "vf2in.s     S010, S400, 23"),
    TEST_ENTRY_C(0xD2171081, NULL, "vf2in.p     C010, C400, 23"),
    TEST_ENTRY_C(0xD2179001, NULL, "vf2in.t     C010, C400, 23"),
    TEST_ENTRY_C(0xD2179081, NULL, "vf2in.q     C010, C400, 23"),
    TEST_ENTRY_C(0xD2181001, NULL, "vf2in.s     S010, S400, 24"),
    TEST_ENTRY_C(0xD2181081, NULL, "vf2in.p     C010, C400, 24"),
    TEST_ENTRY_C(0xD2189001, NULL, "vf2in.t     C010, C400, 24"),
    TEST_ENTRY_C(0xD2189081, NULL, "vf2in.q     C010, C400, 24"),
    TEST_ENTRY_C(0xD2191001, NULL, "vf2in.s     S010, S400, 25"),
    TEST_ENTRY_C(0xD2191081, NULL, "vf2in.p     C010, C400, 25"),
    TEST_ENTRY_C(0xD2199001, NULL, "vf2in.t     C010, C400, 25"),
    TEST_ENTRY_C(0xD2199081, NULL, "vf2in.q     C010, C400, 25"),
    TEST_ENTRY_C(0xD21A1001, NULL, "vf2in.s     S010, S400, 26"),
    TEST_ENTRY_C(0xD21A1081, NULL, "vf2in.p     C010, C400, 26"),
    TEST_ENTRY_C(0xD21A9001, NULL, "vf2in.t     C010, C400, 26"),
    TEST_ENTRY_C(0xD21A9081, NULL, "vf2in.q     C010, C400, 26"),
    TEST_ENTRY_C(0xD21B1001, NULL, "vf2in.s     S010, S400, 27"),
    TEST_ENTRY_C(0xD21B1081, NULL, "vf2in.p     C010, C400, 27"),
    TEST_ENTRY_C(0xD21B9001, NULL, "vf2in.t     C010, C400, 27"),
    TEST_ENTRY_C(0xD21B9081, NULL, "vf2in.q     C010, C400, 27"),
    TEST_ENTRY_C(0xD21C1001, NULL, "vf2in.s     S010, S400, 28"),
    TEST_ENTRY_C(0xD21C1081, NULL, "vf2in.p     C010, C400, 28"),
    TEST_ENTRY_C(0xD21C9001, NULL, "vf2in.t     C010, C400, 28"),
    TEST_ENTRY_C(0xD21C9081, NULL, "vf2in.q     C010, C400, 28"),
    TEST_ENTRY_C(0xD21D1001, NULL, "vf2in.s     S010, S400, 29"),
    TEST_ENTRY_C(0xD21D1081, NULL, "vf2in.p     C010, C400, 29"),
    TEST_ENTRY_C(0xD21D9001, NULL, "vf2in.t     C010, C400, 29"),
    TEST_ENTRY_C(0xD21D9081, NULL, "vf2in.q     C010, C400, 29"),
    TEST_ENTRY_C(0xD21E1001, NULL, "vf2in.s     S010, S400, 30"),
    TEST_ENTRY_C(0xD21E1081, NULL, "vf2in.p     C010, C400, 30"),
    TEST_ENTRY_C(0xD21E9001, NULL, "vf2in.t     C010, C400, 30"),
    TEST_ENTRY_C(0xD21E9081, NULL, "vf2in.q     C010, C400, 30"),
    TEST_ENTRY_C(0xD21F1001, NULL, "vf2in.s     S010, S400, 31"),
    TEST_ENTRY_C(0xD21F1081, NULL, "vf2in.p     C010, C400, 31"),
    TEST_ENTRY_C(0xD21F9001, NULL, "vf2in.t     C010, C400, 31"),
    TEST_ENTRY_C(0xD21F9081, NULL, "vf2in.q     C010, C400, 31"),
    TEST_ENTRY_C(0xD2201001, NULL, "vf2iz.s     S010, S400, 0"),
    TEST_ENTRY_C(0xD2201081, NULL, "vf2iz.p     C010, C400, 0"),
    TEST_ENTRY_C(0xD2209001, NULL, "vf2iz.t     C010, C400, 0"),
    TEST_ENTRY_C(0xD2209081, NULL, "vf2iz.q     C010, C400, 0"),
    TEST_ENTRY_C(0xD2211001, NULL, "vf2iz.s     S010, S400, 1"),
    TEST_ENTRY_C(0xD2211081, NULL, "vf2iz.p     C010, C400, 1"),
    TEST_ENTRY_C(0xD2219001, NULL, "vf2iz.t     C010, C400, 1"),
    TEST_ENTRY_C(0xD2219081, NULL, "vf2iz.q     C010, C400, 1"),
    TEST_ENTRY_C(0xD2221001, NULL, "vf2iz.s     S010, S400, 2"),
    TEST_ENTRY_C(0xD2221081, NULL, "vf2iz.p     C010, C400, 2"),
    TEST_ENTRY_C(0xD2229001, NULL, "vf2iz.t     C010, C400, 2"),
    TEST_ENTRY_C(0xD2229081, NULL, "vf2iz.q     C010, C400, 2"),
    TEST_ENTRY_C(0xD2231001, NULL, "vf2iz.s     S010, S400, 3"),
    TEST_ENTRY_C(0xD2231081, NULL, "vf2iz.p     C010, C400, 3"),
    TEST_ENTRY_C(0xD2239001, NULL, "vf2iz.t     C010, C400, 3"),
    TEST_ENTRY_C(0xD2239081, NULL, "vf2iz.q     C010, C400, 3"),
    TEST_ENTRY_C(0xD2241001, NULL, "vf2iz.s     S010, S400, 4"),
    TEST_ENTRY_C(0xD2241081, NULL, "vf2iz.p     C010, C400, 4"),
    TEST_ENTRY_C(0xD2249001, NULL, "vf2iz.t     C010, C400, 4"),
    TEST_ENTRY_C(0xD2249081, NULL, "vf2iz.q     C010, C400, 4"),
    TEST_ENTRY_C(0xD2251001, NULL, "vf2iz.s     S010, S400, 5"),
    TEST_ENTRY_C(0xD2251081, NULL, "vf2iz.p     C010, C400, 5"),
    TEST_ENTRY_C(0xD2259001, NULL, "vf2iz.t     C010, C400, 5"),
    TEST_ENTRY_C(0xD2259081, NULL, "vf2iz.q     C010, C400, 5"),
    TEST_ENTRY_C(0xD2261001, NULL, "vf2iz.s     S010, S400, 6"),
    TEST_ENTRY_C(0xD2261081, NULL, "vf2iz.p     C010, C400, 6"),
    TEST_ENTRY_C(0xD2269001, NULL, "vf2iz.t     C010, C400, 6"),
    TEST_ENTRY_C(0xD2269081, NULL, "vf2iz.q     C010, C400, 6"),
    TEST_ENTRY_C(0xD2271001, NULL, "vf2iz.s     S010, S400, 7"),
    TEST_ENTRY_C(0xD2271081, NULL, "vf2iz.p     C010, C400, 7"),
    TEST_ENTRY_C(0xD2279001, NULL, "vf2iz.t     C010, C400, 7"),
    TEST_ENTRY_C(0xD2279081, NULL, "vf2iz.q     C010, C400, 7"),
    TEST_ENTRY_C(0xD2281001, NULL, "vf2iz.s     S010, S400, 8"),
    TEST_ENTRY_C(0xD2281081, NULL, "vf2iz.p     C010, C400, 8"),
    TEST_ENTRY_C(0xD2289001, NULL, "vf2iz.t     C010, C400, 8"),
    TEST_ENTRY_C(0xD2289081, NULL, "vf2iz.q     C010, C400, 8"),
    TEST_ENTRY_C(0xD2291001, NULL, "vf2iz.s     S010, S400, 9"),
    TEST_ENTRY_C(0xD2291081, NULL, "vf2iz.p     C010, C400, 9"),
    TEST_ENTRY_C(0xD2299001, NULL, "vf2iz.t     C010, C400, 9"),
    TEST_ENTRY_C(0xD2299081, NULL, "vf2iz.q     C010, C400, 9"),
    TEST_ENTRY_C(0xD22A1001, NULL, "vf2iz.s     S010, S400, 10"),
    TEST_ENTRY_C(0xD22A1081, NULL, "vf2iz.p     C010, C400, 10"),
    TEST_ENTRY_C(0xD22A9001, NULL, "vf2iz.t     C010, C400, 10"),
    TEST_ENTRY_C(0xD22A9081, NULL, "vf2iz.q     C010, C400, 10"),
    TEST_ENTRY_C(0xD22B1001, NULL, "vf2iz.s     S010, S400, 11"),
    TEST_ENTRY_C(0xD22B1081, NULL, "vf2iz.p     C010, C400, 11"),
    TEST_ENTRY_C(0xD22B9001, NULL, "vf2iz.t     C010, C400, 11"),
    TEST_ENTRY_C(0xD22B9081, NULL, "vf2iz.q     C010, C400, 11"),
    TEST_ENTRY_C(0xD22C1001, NULL, "vf2iz.s     S010, S400, 12"),
    TEST_ENTRY_C(0xD22C1081, NULL, "vf2iz.p     C010, C400, 12"),
    TEST_ENTRY_C(0xD22C9001, NULL, "vf2iz.t     C010, C400, 12"),
    TEST_ENTRY_C(0xD22C9081, NULL, "vf2iz.q     C010, C400, 12"),
    TEST_ENTRY_C(0xD22D1001, NULL, "vf2iz.s     S010, S400, 13"),
    TEST_ENTRY_C(0xD22D1081, NULL, "vf2iz.p     C010, C400, 13"),
    TEST_ENTRY_C(0xD22D9001, NULL, "vf2iz.t     C010, C400, 13"),
    TEST_ENTRY_C(0xD22D9081, NULL, "vf2iz.q     C010, C400, 13"),
    TEST_ENTRY_C(0xD22E1001, NULL, "vf2iz.s     S010, S400, 14"),
    TEST_ENTRY_C(0xD22E1081, NULL, "vf2iz.p     C010, C400, 14"),
    TEST_ENTRY_C(0xD22E9001, NULL, "vf2iz.t     C010, C400, 14"),
    TEST_ENTRY_C(0xD22E9081, NULL, "vf2iz.q     C010, C400, 14"),
    TEST_ENTRY_C(0xD22F1001, NULL, "vf2iz.s     S010, S400, 15"),
    TEST_ENTRY_C(0xD22F1081, NULL, "vf2iz.p     C010, C400, 15"),
    TEST_ENTRY_C(0xD22F9001, NULL, "vf2iz.t     C010, C400, 15"),
    TEST_ENTRY_C(0xD22F9081, NULL, "vf2iz.q     C010, C400, 15"),
    TEST_ENTRY_C(0xD2301001, NULL, "vf2iz.s     S010, S400, 16"),
    TEST_ENTRY_C(0xD2301081, NULL, "vf2iz.p     C010, C400, 16"),
    TEST_ENTRY_C(0xD2309001, NULL, "vf2iz.t     C010, C400, 16"),
    TEST_ENTRY_C(0xD2309081, NULL, "vf2iz.q     C010, C400, 16"),
    TEST_ENTRY_C(0xD2311001, NULL, "vf2iz.s     S010, S400, 17"),
    TEST_ENTRY_C(0xD2311081, NULL, "vf2iz.p     C010, C400, 17"),
    TEST_ENTRY_C(0xD2319001, NULL, "vf2iz.t     C010, C400, 17"),
    TEST_ENTRY_C(0xD2319081, NULL, "vf2iz.q     C010, C400, 17"),
    TEST_ENTRY_C(0xD2321001, NULL, "vf2iz.s     S010, S400, 18"),
    TEST_ENTRY_C(0xD2321081, NULL, "vf2iz.p     C010, C400, 18"),
    TEST_ENTRY_C(0xD2329001, NULL, "vf2iz.t     C010, C400, 18"),
    TEST_ENTRY_C(0xD2329081, NULL, "vf2iz.q     C010, C400, 18"),
    TEST_ENTRY_C(0xD2331001, NULL, "vf2iz.s     S010, S400, 19"),
    TEST_ENTRY_C(0xD2331081, NULL, "vf2iz.p     C010, C400, 19"),
    TEST_ENTRY_C(0xD2339001, NULL, "vf2iz.t     C010, C400, 19"),
    TEST_ENTRY_C(0xD2339081, NULL, "vf2iz.q     C010, C400, 19"),
    TEST_ENTRY_C(0xD2341001, NULL, "vf2iz.s     S010, S400, 20"),
    TEST_ENTRY_C(0xD2341081, NULL, "vf2iz.p     C010, C400, 20"),
    TEST_ENTRY_C(0xD2349001, NULL, "vf2iz.t     C010, C400, 20"),
    TEST_ENTRY_C(0xD2349081, NULL, "vf2iz.q     C010, C400, 20"),
    TEST_ENTRY_C(0xD2351001, NULL, "vf2iz.s     S010, S400, 21"),
    TEST_ENTRY_C(0xD2351081, NULL, "vf2iz.p     C010, C400, 21"),
    TEST_ENTRY_C(0xD2359001, NULL, "vf2iz.t     C010, C400, 21"),
    TEST_ENTRY_C(0xD2359081, NULL, "vf2iz.q     C010, C400, 21"),
    TEST_ENTRY_C(0xD2361001, NULL, "vf2iz.s     S010, S400, 22"),
    TEST_ENTRY_C(0xD2361081, NULL, "vf2iz.p     C010, C400, 22"),
    TEST_ENTRY_C(0xD2369001, NULL, "vf2iz.t     C010, C400, 22"),
    TEST_ENTRY_C(0xD2369081, NULL, "vf2iz.q     C010, C400, 22"),
    TEST_ENTRY_C(0xD2371001, NULL, "vf2iz.s     S010, S400, 23"),
    TEST_ENTRY_C(0xD2371081, NULL, "vf2iz.p     C010, C400, 23"),
    TEST_ENTRY_C(0xD2379001, NULL, "vf2iz.t     C010, C400, 23"),
    TEST_ENTRY_C(0xD2379081, NULL, "vf2iz.q     C010, C400, 23"),
    TEST_ENTRY_C(0xD2381001, NULL, "vf2iz.s     S010, S400, 24"),
    TEST_ENTRY_C(0xD2381081, NULL, "vf2iz.p     C010, C400, 24"),
    TEST_ENTRY_C(0xD2389001, NULL, "vf2iz.t     C010, C400, 24"),
    TEST_ENTRY_C(0xD2389081, NULL, "vf2iz.q     C010, C400, 24"),
    TEST_ENTRY_C(0xD2391001, NULL, "vf2iz.s     S010, S400, 25"),
    TEST_ENTRY_C(0xD2391081, NULL, "vf2iz.p     C010, C400, 25"),
    TEST_ENTRY_C(0xD2399001, NULL, "vf2iz.t     C010, C400, 25"),
    TEST_ENTRY_C(0xD2399081, NULL, "vf2iz.q     C010, C400, 25"),
    TEST_ENTRY_C(0xD23A1001, NULL, "vf2iz.s     S010, S400, 26"),
    TEST_ENTRY_C(0xD23A1081, NULL, "vf2iz.p     C010, C400, 26"),
    TEST_ENTRY_C(0xD23A9001, NULL, "vf2iz.t     C010, C400, 26"),
    TEST_ENTRY_C(0xD23A9081, NULL, "vf2iz.q     C010, C400, 26"),
    TEST_ENTRY_C(0xD23B1001, NULL, "vf2iz.s     S010, S400, 27"),
    TEST_ENTRY_C(0xD23B1081, NULL, "vf2iz.p     C010, C400, 27"),
    TEST_ENTRY_C(0xD23B9001, NULL, "vf2iz.t     C010, C400, 27"),
    TEST_ENTRY_C(0xD23B9081, NULL, "vf2iz.q     C010, C400, 27"),
    TEST_ENTRY_C(0xD23C1001, NULL, "vf2iz.s     S010, S400, 28"),
    TEST_ENTRY_C(0xD23C1081, NULL, "vf2iz.p     C010, C400, 28"),
    TEST_ENTRY_C(0xD23C9001, NULL, "vf2iz.t     C010, C400, 28"),
    TEST_ENTRY_C(0xD23C9081, NULL, "vf2iz.q     C010, C400, 28"),
    TEST_ENTRY_C(0xD23D1001, NULL, "vf2iz.s     S010, S400, 29"),
    TEST_ENTRY_C(0xD23D1081, NULL, "vf2iz.p     C010, C400, 29"),
    TEST_ENTRY_C(0xD23D9001, NULL, "vf2iz.t     C010, C400, 29"),
    TEST_ENTRY_C(0xD23D9081, NULL, "vf2iz.q     C010, C400, 29"),
    TEST_ENTRY_C(0xD23E1001, NULL, "vf2iz.s     S010, S400, 30"),
    TEST_ENTRY_C(0xD23E1081, NULL, "vf2iz.p     C010, C400, 30"),
    TEST_ENTRY_C(0xD23E9001, NULL, "vf2iz.t     C010, C400, 30"),
    TEST_ENTRY_C(0xD23E9081, NULL, "vf2iz.q     C010, C400, 30"),
    TEST_ENTRY_C(0xD23F1001, NULL, "vf2iz.s     S010, S400, 31"),
    TEST_ENTRY_C(0xD23F1081, NULL, "vf2iz.p     C010, C400, 31"),
    TEST_ENTRY_C(0xD23F9001, NULL, "vf2iz.t     C010, C400, 31"),
    TEST_ENTRY_C(0xD23F9081, NULL, "vf2iz.q     C010, C400, 31"),
    TEST_ENTRY_C(0xD2401001, NULL, "vf2iu.s     S010, S400, 0"),
    TEST_ENTRY_C(0xD2401081, NULL, "vf2iu.p     C010, C400, 0"),
    TEST_ENTRY_C(0xD2409001, NULL, "vf2iu.t     C010, C400, 0"),
    TEST_ENTRY_C(0xD2409081, NULL, "vf2iu.q     C010, C400, 0"),
    TEST_ENTRY_C(0xD2411001, NULL, "vf2iu.s     S010, S400, 1"),
    TEST_ENTRY_C(0xD2411081, NULL, "vf2iu.p     C010, C400, 1"),
    TEST_ENTRY_C(0xD2419001, NULL, "vf2iu.t     C010, C400, 1"),
    TEST_ENTRY_C(0xD2419081, NULL, "vf2iu.q     C010, C400, 1"),
    TEST_ENTRY_C(0xD2421001, NULL, "vf2iu.s     S010, S400, 2"),
    TEST_ENTRY_C(0xD2421081, NULL, "vf2iu.p     C010, C400, 2"),
    TEST_ENTRY_C(0xD2429001, NULL, "vf2iu.t     C010, C400, 2"),
    TEST_ENTRY_C(0xD2429081, NULL, "vf2iu.q     C010, C400, 2"),
    TEST_ENTRY_C(0xD2431001, NULL, "vf2iu.s     S010, S400, 3"),
    TEST_ENTRY_C(0xD2431081, NULL, "vf2iu.p     C010, C400, 3"),
    TEST_ENTRY_C(0xD2439001, NULL, "vf2iu.t     C010, C400, 3"),
    TEST_ENTRY_C(0xD2439081, NULL, "vf2iu.q     C010, C400, 3"),
    TEST_ENTRY_C(0xD2441001, NULL, "vf2iu.s     S010, S400, 4"),
    TEST_ENTRY_C(0xD2441081, NULL, "vf2iu.p     C010, C400, 4"),
    TEST_ENTRY_C(0xD2449001, NULL, "vf2iu.t     C010, C400, 4"),
    TEST_ENTRY_C(0xD2449081, NULL, "vf2iu.q     C010, C400, 4"),
    TEST_ENTRY_C(0xD2451001, NULL, "vf2iu.s     S010, S400, 5"),
    TEST_ENTRY_C(0xD2451081, NULL, "vf2iu.p     C010, C400, 5"),
    TEST_ENTRY_C(0xD2459001, NULL, "vf2iu.t     C010, C400, 5"),
    TEST_ENTRY_C(0xD2459081, NULL, "vf2iu.q     C010, C400, 5"),
    TEST_ENTRY_C(0xD2461001, NULL, "vf2iu.s     S010, S400, 6"),
    TEST_ENTRY_C(0xD2461081, NULL, "vf2iu.p     C010, C400, 6"),
    TEST_ENTRY_C(0xD2469001, NULL, "vf2iu.t     C010, C400, 6"),
    TEST_ENTRY_C(0xD2469081, NULL, "vf2iu.q     C010, C400, 6"),
    TEST_ENTRY_C(0xD2471001, NULL, "vf2iu.s     S010, S400, 7"),
    TEST_ENTRY_C(0xD2471081, NULL, "vf2iu.p     C010, C400, 7"),
    TEST_ENTRY_C(0xD2479001, NULL, "vf2iu.t     C010, C400, 7"),
    TEST_ENTRY_C(0xD2479081, NULL, "vf2iu.q     C010, C400, 7"),
    TEST_ENTRY_C(0xD2481001, NULL, "vf2iu.s     S010, S400, 8"),
    TEST_ENTRY_C(0xD2481081, NULL, "vf2iu.p     C010, C400, 8"),
    TEST_ENTRY_C(0xD2489001, NULL, "vf2iu.t     C010, C400, 8"),
    TEST_ENTRY_C(0xD2489081, NULL, "vf2iu.q     C010, C400, 8"),
    TEST_ENTRY_C(0xD2491001, NULL, "vf2iu.s     S010, S400, 9"),
    TEST_ENTRY_C(0xD2491081, NULL, "vf2iu.p     C010, C400, 9"),
    TEST_ENTRY_C(0xD2499001, NULL, "vf2iu.t     C010, C400, 9"),
    TEST_ENTRY_C(0xD2499081, NULL, "vf2iu.q     C010, C400, 9"),
    TEST_ENTRY_C(0xD24A1001, NULL, "vf2iu.s     S010, S400, 10"),
    TEST_ENTRY_C(0xD24A1081, NULL, "vf2iu.p     C010, C400, 10"),
    TEST_ENTRY_C(0xD24A9001, NULL, "vf2iu.t     C010, C400, 10"),
    TEST_ENTRY_C(0xD24A9081, NULL, "vf2iu.q     C010, C400, 10"),
    TEST_ENTRY_C(0xD24B1001, NULL, "vf2iu.s     S010, S400, 11"),
    TEST_ENTRY_C(0xD24B1081, NULL, "vf2iu.p     C010, C400, 11"),
    TEST_ENTRY_C(0xD24B9001, NULL, "vf2iu.t     C010, C400, 11"),
    TEST_ENTRY_C(0xD24B9081, NULL, "vf2iu.q     C010, C400, 11"),
    TEST_ENTRY_C(0xD24C1001, NULL, "vf2iu.s     S010, S400, 12"),
    TEST_ENTRY_C(0xD24C1081, NULL, "vf2iu.p     C010, C400, 12"),
    TEST_ENTRY_C(0xD24C9001, NULL, "vf2iu.t     C010, C400, 12"),
    TEST_ENTRY_C(0xD24C9081, NULL, "vf2iu.q     C010, C400, 12"),
    TEST_ENTRY_C(0xD24D1001, NULL, "vf2iu.s     S010, S400, 13"),
    TEST_ENTRY_C(0xD24D1081, NULL, "vf2iu.p     C010, C400, 13"),
    TEST_ENTRY_C(0xD24D9001, NULL, "vf2iu.t     C010, C400, 13"),
    TEST_ENTRY_C(0xD24D9081, NULL, "vf2iu.q     C010, C400, 13"),
    TEST_ENTRY_C(0xD24E1001, NULL, "vf2iu.s     S010, S400, 14"),
    TEST_ENTRY_C(0xD24E1081, NULL, "vf2iu.p     C010, C400, 14"),
    TEST_ENTRY_C(0xD24E9001, NULL, "vf2iu.t     C010, C400, 14"),
    TEST_ENTRY_C(0xD24E9081, NULL, "vf2iu.q     C010, C400, 14"),
    TEST_ENTRY_C(0xD24F1001, NULL, "vf2iu.s     S010, S400, 15"),
    TEST_ENTRY_C(0xD24F1081, NULL, "vf2iu.p     C010, C400, 15"),
    TEST_ENTRY_C(0xD24F9001, NULL, "vf2iu.t     C010, C400, 15"),
    TEST_ENTRY_C(0xD24F9081, NULL, "vf2iu.q     C010, C400, 15"),
    TEST_ENTRY_C(0xD2501001, NULL, "vf2iu.s     S010, S400, 16"),
    TEST_ENTRY_C(0xD2501081, NULL, "vf2iu.p     C010, C400, 16"),
    TEST_ENTRY_C(0xD2509001, NULL, "vf2iu.t     C010, C400, 16"),
    TEST_ENTRY_C(0xD2509081, NULL, "vf2iu.q     C010, C400, 16"),
    TEST_ENTRY_C(0xD2511001, NULL, "vf2iu.s     S010, S400, 17"),
    TEST_ENTRY_C(0xD2511081, NULL, "vf2iu.p     C010, C400, 17"),
    TEST_ENTRY_C(0xD2519001, NULL, "vf2iu.t     C010, C400, 17"),
    TEST_ENTRY_C(0xD2519081, NULL, "vf2iu.q     C010, C400, 17"),
    TEST_ENTRY_C(0xD2521001, NULL, "vf2iu.s     S010, S400, 18"),
    TEST_ENTRY_C(0xD2521081, NULL, "vf2iu.p     C010, C400, 18"),
    TEST_ENTRY_C(0xD2529001, NULL, "vf2iu.t     C010, C400, 18"),
    TEST_ENTRY_C(0xD2529081, NULL, "vf2iu.q     C010, C400, 18"),
    TEST_ENTRY_C(0xD2531001, NULL, "vf2iu.s     S010, S400, 19"),
    TEST_ENTRY_C(0xD2531081, NULL, "vf2iu.p     C010, C400, 19"),
    TEST_ENTRY_C(0xD2539001, NULL, "vf2iu.t     C010, C400, 19"),
    TEST_ENTRY_C(0xD2539081, NULL, "vf2iu.q     C010, C400, 19"),
    TEST_ENTRY_C(0xD2541001, NULL, "vf2iu.s     S010, S400, 20"),
    TEST_ENTRY_C(0xD2541081, NULL, "vf2iu.p     C010, C400, 20"),
    TEST_ENTRY_C(0xD2549001, NULL, "vf2iu.t     C010, C400, 20"),
    TEST_ENTRY_C(0xD2549081, NULL, "vf2iu.q     C010, C400, 20"),
    TEST_ENTRY_C(0xD2551001, NULL, "vf2iu.s     S010, S400, 21"),
    TEST_ENTRY_C(0xD2551081, NULL, "vf2iu.p     C010, C400, 21"),
    TEST_ENTRY_C(0xD2559001, NULL, "vf2iu.t     C010, C400, 21"),
    TEST_ENTRY_C(0xD2559081, NULL, "vf2iu.q     C010, C400, 21"),
    TEST_ENTRY_C(0xD2561001, NULL, "vf2iu.s     S010, S400, 22"),
    TEST_ENTRY_C(0xD2561081, NULL, "vf2iu.p     C010, C400, 22"),
    TEST_ENTRY_C(0xD2569001, NULL, "vf2iu.t     C010, C400, 22"),
    TEST_ENTRY_C(0xD2569081, NULL, "vf2iu.q     C010, C400, 22"),
    TEST_ENTRY_C(0xD2571001, NULL, "vf2iu.s     S010, S400, 23"),
    TEST_ENTRY_C(0xD2571081, NULL, "vf2iu.p     C010, C400, 23"),
    TEST_ENTRY_C(0xD2579001, NULL, "vf2iu.t     C010, C400, 23"),
    TEST_ENTRY_C(0xD2579081, NULL, "vf2iu.q     C010, C400, 23"),
    TEST_ENTRY_C(0xD2581001, NULL, "vf2iu.s     S010, S400, 24"),
    TEST_ENTRY_C(0xD2581081, NULL, "vf2iu.p     C010, C400, 24"),
    TEST_ENTRY_C(0xD2589001, NULL, "vf2iu.t     C010, C400, 24"),
    TEST_ENTRY_C(0xD2589081, NULL, "vf2iu.q     C010, C400, 24"),
    TEST_ENTRY_C(0xD2591001, NULL, "vf2iu.s     S010, S400, 25"),
    TEST_ENTRY_C(0xD2591081, NULL, "vf2iu.p     C010, C400, 25"),
    TEST_ENTRY_C(0xD2599001, NULL, "vf2iu.t     C010, C400, 25"),
    TEST_ENTRY_C(0xD2599081, NULL, "vf2iu.q     C010, C400, 25"),
    TEST_ENTRY_C(0xD25A1001, NULL, "vf2iu.s     S010, S400, 26"),
    TEST_ENTRY_C(0xD25A1081, NULL, "vf2iu.p     C010, C400, 26"),
    TEST_ENTRY_C(0xD25A9001, NULL, "vf2iu.t     C010, C400, 26"),
    TEST_ENTRY_C(0xD25A9081, NULL, "vf2iu.q     C010, C400, 26"),
    TEST_ENTRY_C(0xD25B1001, NULL, "vf2iu.s     S010, S400, 27"),
    TEST_ENTRY_C(0xD25B1081, NULL, "vf2iu.p     C010, C400, 27"),
    TEST_ENTRY_C(0xD25B9001, NULL, "vf2iu.t     C010, C400, 27"),
    TEST_ENTRY_C(0xD25B9081, NULL, "vf2iu.q     C010, C400, 27"),
    TEST_ENTRY_C(0xD25C1001, NULL, "vf2iu.s     S010, S400, 28"),
    TEST_ENTRY_C(0xD25C1081, NULL, "vf2iu.p     C010, C400, 28"),
    TEST_ENTRY_C(0xD25C9001, NULL, "vf2iu.t     C010, C400, 28"),
    TEST_ENTRY_C(0xD25C9081, NULL, "vf2iu.q     C010, C400, 28"),
    TEST_ENTRY_C(0xD25D1001, NULL, "vf2iu.s     S010, S400, 29"),
    TEST_ENTRY_C(0xD25D1081, NULL, "vf2iu.p     C010, C400, 29"),
    TEST_ENTRY_C(0xD25D9001, NULL, "vf2iu.t     C010, C400, 29"),
    TEST_ENTRY_C(0xD25D9081, NULL, "vf2iu.q     C010, C400, 29"),
    TEST_ENTRY_C(0xD25E1001, NULL, "vf2iu.s     S010, S400, 30"),
    TEST_ENTRY_C(0xD25E1081, NULL, "vf2iu.p     C010, C400, 30"),
    TEST_ENTRY_C(0xD25E9001, NULL, "vf2iu.t     C010, C400, 30"),
    TEST_ENTRY_C(0xD25E9081, NULL, "vf2iu.q     C010, C400, 30"),
    TEST_ENTRY_C(0xD25F1001, NULL, "vf2iu.s     S010, S400, 31"),
    TEST_ENTRY_C(0xD25F1081, NULL, "vf2iu.p     C010, C400, 31"),
    TEST_ENTRY_C(0xD25F9001, NULL, "vf2iu.t     C010, C400, 31"),
    TEST_ENTRY_C(0xD25F9081, NULL, "vf2iu.q     C010, C400, 31"),
    TEST_ENTRY_C(0xD2601001, NULL, "vf2id.s     S010, S400, 0"),
    TEST_ENTRY_C(0xD2601081, NULL, "vf2id.p     C010, C400, 0"),
    TEST_ENTRY_C(0xD2609001, NULL, "vf2id.t     C010, C400, 0"),
    TEST_ENTRY_C(0xD2609081, NULL, "vf2id.q     C010, C400, 0"),
    TEST_ENTRY_C(0xD2611001, NULL, "vf2id.s     S010, S400, 1"),
    TEST_ENTRY_C(0xD2611081, NULL, "vf2id.p     C010, C400, 1"),
    TEST_ENTRY_C(0xD2619001, NULL, "vf2id.t     C010, C400, 1"),
    TEST_ENTRY_C(0xD2619081, NULL, "vf2id.q     C010, C400, 1"),
    TEST_ENTRY_C(0xD2621001, NULL, "vf2id.s     S010, S400, 2"),
    TEST_ENTRY_C(0xD2621081, NULL, "vf2id.p     C010, C400, 2"),
    TEST_ENTRY_C(0xD2629001, NULL, "vf2id.t     C010, C400, 2"),
    TEST_ENTRY_C(0xD2629081, NULL, "vf2id.q     C010, C400, 2"),
    TEST_ENTRY_C(0xD2631001, NULL, "vf2id.s     S010, S400, 3"),
    TEST_ENTRY_C(0xD2631081, NULL, "vf2id.p     C010, C400, 3"),
    TEST_ENTRY_C(0xD2639001, NULL, "vf2id.t     C010, C400, 3"),
    TEST_ENTRY_C(0xD2639081, NULL, "vf2id.q     C010, C400, 3"),
    TEST_ENTRY_C(0xD2641001, NULL, "vf2id.s     S010, S400, 4"),
    TEST_ENTRY_C(0xD2641081, NULL, "vf2id.p     C010, C400, 4"),
    TEST_ENTRY_C(0xD2649001, NULL, "vf2id.t     C010, C400, 4"),
    TEST_ENTRY_C(0xD2649081, NULL, "vf2id.q     C010, C400, 4"),
    TEST_ENTRY_C(0xD2651001, NULL, "vf2id.s     S010, S400, 5"),
    TEST_ENTRY_C(0xD2651081, NULL, "vf2id.p     C010, C400, 5"),
    TEST_ENTRY_C(0xD2659001, NULL, "vf2id.t     C010, C400, 5"),
    TEST_ENTRY_C(0xD2659081, NULL, "vf2id.q     C010, C400, 5"),
    TEST_ENTRY_C(0xD2661001, NULL, "vf2id.s     S010, S400, 6"),
    TEST_ENTRY_C(0xD2661081, NULL, "vf2id.p     C010, C400, 6"),
    TEST_ENTRY_C(0xD2669001, NULL, "vf2id.t     C010, C400, 6"),
    TEST_ENTRY_C(0xD2669081, NULL, "vf2id.q     C010, C400, 6"),
    TEST_ENTRY_C(0xD2671001, NULL, "vf2id.s     S010, S400, 7"),
    TEST_ENTRY_C(0xD2671081, NULL, "vf2id.p     C010, C400, 7"),
    TEST_ENTRY_C(0xD2679001, NULL, "vf2id.t     C010, C400, 7"),
    TEST_ENTRY_C(0xD2679081, NULL, "vf2id.q     C010, C400, 7"),
    TEST_ENTRY_C(0xD2681001, NULL, "vf2id.s     S010, S400, 8"),
    TEST_ENTRY_C(0xD2681081, NULL, "vf2id.p     C010, C400, 8"),
    TEST_ENTRY_C(0xD2689001, NULL, "vf2id.t     C010, C400, 8"),
    TEST_ENTRY_C(0xD2689081, NULL, "vf2id.q     C010, C400, 8"),
    TEST_ENTRY_C(0xD2691001, NULL, "vf2id.s     S010, S400, 9"),
    TEST_ENTRY_C(0xD2691081, NULL, "vf2id.p     C010, C400, 9"),
    TEST_ENTRY_C(0xD2699001, NULL, "vf2id.t     C010, C400, 9"),
    TEST_ENTRY_C(0xD2699081, NULL, "vf2id.q     C010, C400, 9"),
    TEST_ENTRY_C(0xD26A1001, NULL, "vf2id.s     S010, S400, 10"),
    TEST_ENTRY_C(0xD26A1081, NULL, "vf2id.p     C010, C400, 10"),
    TEST_ENTRY_C(0xD26A9001, NULL, "vf2id.t     C010, C400, 10"),
    TEST_ENTRY_C(0xD26A9081, NULL, "vf2id.q     C010, C400, 10"),
    TEST_ENTRY_C(0xD26B1001, NULL, "vf2id.s     S010, S400, 11"),
    TEST_ENTRY_C(0xD26B1081, NULL, "vf2id.p     C010, C400, 11"),
    TEST_ENTRY_C(0xD26B9001, NULL, "vf2id.t     C010, C400, 11"),
    TEST_ENTRY_C(0xD26B9081, NULL, "vf2id.q     C010, C400, 11"),
    TEST_ENTRY_C(0xD26C1001, NULL, "vf2id.s     S010, S400, 12"),
    TEST_ENTRY_C(0xD26C1081, NULL, "vf2id.p     C010, C400, 12"),
    TEST_ENTRY_C(0xD26C9001, NULL, "vf2id.t     C010, C400, 12"),
    TEST_ENTRY_C(0xD26C9081, NULL, "vf2id.q     C010, C400, 12"),
    TEST_ENTRY_C(0xD26D1001, NULL, "vf2id.s     S010, S400, 13"),
    TEST_ENTRY_C(0xD26D1081, NULL, "vf2id.p     C010, C400, 13"),
    TEST_ENTRY_C(0xD26D9001, NULL, "vf2id.t     C010, C400, 13"),
    TEST_ENTRY_C(0xD26D9081, NULL, "vf2id.q     C010, C400, 13"),
    TEST_ENTRY_C(0xD26E1001, NULL, "vf2id.s     S010, S400, 14"),
    TEST_ENTRY_C(0xD26E1081, NULL, "vf2id.p     C010, C400, 14"),
    TEST_ENTRY_C(0xD26E9001, NULL, "vf2id.t     C010, C400, 14"),
    TEST_ENTRY_C(0xD26E9081, NULL, "vf2id.q     C010, C400, 14"),
    TEST_ENTRY_C(0xD26F1001, NULL, "vf2id.s     S010, S400, 15"),
    TEST_ENTRY_C(0xD26F1081, NULL, "vf2id.p     C010, C400, 15"),
    TEST_ENTRY_C(0xD26F9001, NULL, "vf2id.t     C010, C400, 15"),
    TEST_ENTRY_C(0xD26F9081, NULL, "vf2id.q     C010, C400, 15"),
    TEST_ENTRY_C(0xD2701001, NULL, "vf2id.s     S010, S400, 16"),
    TEST_ENTRY_C(0xD2701081, NULL, "vf2id.p     C010, C400, 16"),
    TEST_ENTRY_C(0xD2709001, NULL, "vf2id.t     C010, C400, 16"),
    TEST_ENTRY_C(0xD2709081, NULL, "vf2id.q     C010, C400, 16"),
    TEST_ENTRY_C(0xD2711001, NULL, "vf2id.s     S010, S400, 17"),
    TEST_ENTRY_C(0xD2711081, NULL, "vf2id.p     C010, C400, 17"),
    TEST_ENTRY_C(0xD2719001, NULL, "vf2id.t     C010, C400, 17"),
    TEST_ENTRY_C(0xD2719081, NULL, "vf2id.q     C010, C400, 17"),
    TEST_ENTRY_C(0xD2721001, NULL, "vf2id.s     S010, S400, 18"),
    TEST_ENTRY_C(0xD2721081, NULL, "vf2id.p     C010, C400, 18"),
    TEST_ENTRY_C(0xD2729001, NULL, "vf2id.t     C010, C400, 18"),
    TEST_ENTRY_C(0xD2729081, NULL, "vf2id.q     C010, C400, 18"),
    TEST_ENTRY_C(0xD2731001, NULL, "vf2id.s     S010, S400, 19"),
    TEST_ENTRY_C(0xD2731081, NULL, "vf2id.p     C010, C400, 19"),
    TEST_ENTRY_C(0xD2739001, NULL, "vf2id.t     C010, C400, 19"),
    TEST_ENTRY_C(0xD2739081, NULL, "vf2id.q     C010, C400, 19"),
    TEST_ENTRY_C(0xD2741001, NULL, "vf2id.s     S010, S400, 20"),
    TEST_ENTRY_C(0xD2741081, NULL, "vf2id.p     C010, C400, 20"),
    TEST_ENTRY_C(0xD2749001, NULL, "vf2id.t     C010, C400, 20"),
    TEST_ENTRY_C(0xD2749081, NULL, "vf2id.q     C010, C400, 20"),
    TEST_ENTRY_C(0xD2751001, NULL, "vf2id.s     S010, S400, 21"),
    TEST_ENTRY_C(0xD2751081, NULL, "vf2id.p     C010, C400, 21"),
    TEST_ENTRY_C(0xD2759001, NULL, "vf2id.t     C010, C400, 21"),
    TEST_ENTRY_C(0xD2759081, NULL, "vf2id.q     C010, C400, 21"),
    TEST_ENTRY_C(0xD2761001, NULL, "vf2id.s     S010, S400, 22"),
    TEST_ENTRY_C(0xD2761081, NULL, "vf2id.p     C010, C400, 22"),
    TEST_ENTRY_C(0xD2769001, NULL, "vf2id.t     C010, C400, 22"),
    TEST_ENTRY_C(0xD2769081, NULL, "vf2id.q     C010, C400, 22"),
    TEST_ENTRY_C(0xD2771001, NULL, "vf2id.s     S010, S400, 23"),
    TEST_ENTRY_C(0xD2771081, NULL, "vf2id.p     C010, C400, 23"),
    TEST_ENTRY_C(0xD2779001, NULL, "vf2id.t     C010, C400, 23"),
    TEST_ENTRY_C(0xD2779081, NULL, "vf2id.q     C010, C400, 23"),
    TEST_ENTRY_C(0xD2781001, NULL, "vf2id.s     S010, S400, 24"),
    TEST_ENTRY_C(0xD2781081, NULL, "vf2id.p     C010, C400, 24"),
    TEST_ENTRY_C(0xD2789001, NULL, "vf2id.t     C010, C400, 24"),
    TEST_ENTRY_C(0xD2789081, NULL, "vf2id.q     C010, C400, 24"),
    TEST_ENTRY_C(0xD2791001, NULL, "vf2id.s     S010, S400, 25"),
    TEST_ENTRY_C(0xD2791081, NULL, "vf2id.p     C010, C400, 25"),
    TEST_ENTRY_C(0xD2799001, NULL, "vf2id.t     C010, C400, 25"),
    TEST_ENTRY_C(0xD2799081, NULL, "vf2id.q     C010, C400, 25"),
    TEST_ENTRY_C(0xD27A1001, NULL, "vf2id.s     S010, S400, 26"),
    TEST_ENTRY_C(0xD27A1081, NULL, "vf2id.p     C010, C400, 26"),
    TEST_ENTRY_C(0xD27A9001, NULL, "vf2id.t     C010, C400, 26"),
    TEST_ENTRY_C(0xD27A9081, NULL, "vf2id.q     C010, C400, 26"),
    TEST_ENTRY_C(0xD27B1001, NULL, "vf2id.s     S010, S400, 27"),
    TEST_ENTRY_C(0xD27B1081, NULL, "vf2id.p     C010, C400, 27"),
    TEST_ENTRY_C(0xD27B9001, NULL, "vf2id.t     C010, C400, 27"),
    TEST_ENTRY_C(0xD27B9081, NULL, "vf2id.q     C010, C400, 27"),
    TEST_ENTRY_C(0xD27C1001, NULL, "vf2id.s     S010, S400, 28"),
    TEST_ENTRY_C(0xD27C1081, NULL, "vf2id.p     C010, C400, 28"),
    TEST_ENTRY_C(0xD27C9001, NULL, "vf2id.t     C010, C400, 28"),
    TEST_ENTRY_C(0xD27C9081, NULL, "vf2id.q     C010, C400, 28"),
    TEST_ENTRY_C(0xD27D1001, NULL, "vf2id.s     S010, S400, 29"),
    TEST_ENTRY_C(0xD27D1081, NULL, "vf2id.p     C010, C400, 29"),
    TEST_ENTRY_C(0xD27D9001, NULL, "vf2id.t     C010, C400, 29"),
    TEST_ENTRY_C(0xD27D9081, NULL, "vf2id.q     C010, C400, 29"),
    TEST_ENTRY_C(0xD27E1001, NULL, "vf2id.s     S010, S400, 30"),
    TEST_ENTRY_C(0xD27E1081, NULL, "vf2id.p     C010, C400, 30"),
    TEST_ENTRY_C(0xD27E9001, NULL, "vf2id.t     C010, C400, 30"),
    TEST_ENTRY_C(0xD27E9081, NULL, "vf2id.q     C010, C400, 30"),
    TEST_ENTRY_C(0xD27F1001, NULL, "vf2id.s     S010, S400, 31"),
    TEST_ENTRY_C(0xD27F1081, NULL, "vf2id.p     C010, C400, 31"),
    TEST_ENTRY_C(0xD27F9001, NULL, "vf2id.t     C010, C400, 31"),
    TEST_ENTRY_C(0xD27F9081, NULL, "vf2id.q     C010, C400, 31"),
    TEST_ENTRY_C(0xD2801001, NULL, "vi2f.s      S010, S400, 0"),
    TEST_ENTRY_C(0xD2801081, NULL, "vi2f.p      C010, C400, 0"),
    TEST_ENTRY_C(0xD2809001, NULL, "vi2f.t      C010, C400, 0"),
    TEST_ENTRY_C(0xD2809081, NULL, "vi2f.q      C010, C400, 0"),
    TEST_ENTRY_C(0xD2811001, NULL, "vi2f.s      S010, S400, 1"),
    TEST_ENTRY_C(0xD2811081, NULL, "vi2f.p      C010, C400, 1"),
    TEST_ENTRY_C(0xD2819001, NULL, "vi2f.t      C010, C400, 1"),
    TEST_ENTRY_C(0xD2819081, NULL, "vi2f.q      C010, C400, 1"),
    TEST_ENTRY_C(0xD2821001, NULL, "vi2f.s      S010, S400, 2"),
    TEST_ENTRY_C(0xD2821081, NULL, "vi2f.p      C010, C400, 2"),
    TEST_ENTRY_C(0xD2829001, NULL, "vi2f.t      C010, C400, 2"),
    TEST_ENTRY_C(0xD2829081, NULL, "vi2f.q      C010, C400, 2"),
    TEST_ENTRY_C(0xD2831001, NULL, "vi2f.s      S010, S400, 3"),
    TEST_ENTRY_C(0xD2831081, NULL, "vi2f.p      C010, C400, 3"),
    TEST_ENTRY_C(0xD2839001, NULL, "vi2f.t      C010, C400, 3"),
    TEST_ENTRY_C(0xD2839081, NULL, "vi2f.q      C010, C400, 3"),
    TEST_ENTRY_C(0xD2841001, NULL, "vi2f.s      S010, S400, 4"),
    TEST_ENTRY_C(0xD2841081, NULL, "vi2f.p      C010, C400, 4"),
    TEST_ENTRY_C(0xD2849001, NULL, "vi2f.t      C010, C400, 4"),
    TEST_ENTRY_C(0xD2849081, NULL, "vi2f.q      C010, C400, 4"),
    TEST_ENTRY_C(0xD2851001, NULL, "vi2f.s      S010, S400, 5"),
    TEST_ENTRY_C(0xD2851081, NULL, "vi2f.p      C010, C400, 5"),
    TEST_ENTRY_C(0xD2859001, NULL, "vi2f.t      C010, C400, 5"),
    TEST_ENTRY_C(0xD2859081, NULL, "vi2f.q      C010, C400, 5"),
    TEST_ENTRY_C(0xD2861001, NULL, "vi2f.s      S010, S400, 6"),
    TEST_ENTRY_C(0xD2861081, NULL, "vi2f.p      C010, C400, 6"),
    TEST_ENTRY_C(0xD2869001, NULL, "vi2f.t      C010, C400, 6"),
    TEST_ENTRY_C(0xD2869081, NULL, "vi2f.q      C010, C400, 6"),
    TEST_ENTRY_C(0xD2871001, NULL, "vi2f.s      S010, S400, 7"),
    TEST_ENTRY_C(0xD2871081, NULL, "vi2f.p      C010, C400, 7"),
    TEST_ENTRY_C(0xD2879001, NULL, "vi2f.t      C010, C400, 7"),
    TEST_ENTRY_C(0xD2879081, NULL, "vi2f.q      C010, C400, 7"),
    TEST_ENTRY_C(0xD2881001, NULL, "vi2f.s      S010, S400, 8"),
    TEST_ENTRY_C(0xD2881081, NULL, "vi2f.p      C010, C400, 8"),
    TEST_ENTRY_C(0xD2889001, NULL, "vi2f.t      C010, C400, 8"),
    TEST_ENTRY_C(0xD2889081, NULL, "vi2f.q      C010, C400, 8"),
    TEST_ENTRY_C(0xD2891001, NULL, "vi2f.s      S010, S400, 9"),
    TEST_ENTRY_C(0xD2891081, NULL, "vi2f.p      C010, C400, 9"),
    TEST_ENTRY_C(0xD2899001, NULL, "vi2f.t      C010, C400, 9"),
    TEST_ENTRY_C(0xD2899081, NULL, "vi2f.q      C010, C400, 9"),
    TEST_ENTRY_C(0xD28A1001, NULL, "vi2f.s      S010, S400, 10"),
    TEST_ENTRY_C(0xD28A1081, NULL, "vi2f.p      C010, C400, 10"),
    TEST_ENTRY_C(0xD28A9001, NULL, "vi2f.t      C010, C400, 10"),
    TEST_ENTRY_C(0xD28A9081, NULL, "vi2f.q      C010, C400, 10"),
    TEST_ENTRY_C(0xD28B1001, NULL, "vi2f.s      S010, S400, 11"),
    TEST_ENTRY_C(0xD28B1081, NULL, "vi2f.p      C010, C400, 11"),
    TEST_ENTRY_C(0xD28B9001, NULL, "vi2f.t      C010, C400, 11"),
    TEST_ENTRY_C(0xD28B9081, NULL, "vi2f.q      C010, C400, 11"),
    TEST_ENTRY_C(0xD28C1001, NULL, "vi2f.s      S010, S400, 12"),
    TEST_ENTRY_C(0xD28C1081, NULL, "vi2f.p      C010, C400, 12"),
    TEST_ENTRY_C(0xD28C9001, NULL, "vi2f.t      C010, C400, 12"),
    TEST_ENTRY_C(0xD28C9081, NULL, "vi2f.q      C010, C400, 12"),
    TEST_ENTRY_C(0xD28D1001, NULL, "vi2f.s      S010, S400, 13"),
    TEST_ENTRY_C(0xD28D1081, NULL, "vi2f.p      C010, C400, 13"),
    TEST_ENTRY_C(0xD28D9001, NULL, "vi2f.t      C010, C400, 13"),
    TEST_ENTRY_C(0xD28D9081, NULL, "vi2f.q      C010, C400, 13"),
    TEST_ENTRY_C(0xD28E1001, NULL, "vi2f.s      S010, S400, 14"),
    TEST_ENTRY_C(0xD28E1081, NULL, "vi2f.p      C010, C400, 14"),
    TEST_ENTRY_C(0xD28E9001, NULL, "vi2f.t      C010, C400, 14"),
    TEST_ENTRY_C(0xD28E9081, NULL, "vi2f.q      C010, C400, 14"),
    TEST_ENTRY_C(0xD28F1001, NULL, "vi2f.s      S010, S400, 15"),
    TEST_ENTRY_C(0xD28F1081, NULL, "vi2f.p      C010, C400, 15"),
    TEST_ENTRY_C(0xD28F9001, NULL, "vi2f.t      C010, C400, 15"),
    TEST_ENTRY_C(0xD28F9081, NULL, "vi2f.q      C010, C400, 15"),
    TEST_ENTRY_C(0xD2901001, NULL, "vi2f.s      S010, S400, 16"),
    TEST_ENTRY_C(0xD2901081, NULL, "vi2f.p      C010, C400, 16"),
    TEST_ENTRY_C(0xD2909001, NULL, "vi2f.t      C010, C400, 16"),
    TEST_ENTRY_C(0xD2909081, NULL, "vi2f.q      C010, C400, 16"),
    TEST_ENTRY_C(0xD2911001, NULL, "vi2f.s      S010, S400, 17"),
    TEST_ENTRY_C(0xD2911081, NULL, "vi2f.p      C010, C400, 17"),
    TEST_ENTRY_C(0xD2919001, NULL, "vi2f.t      C010, C400, 17"),
    TEST_ENTRY_C(0xD2919081, NULL, "vi2f.q      C010, C400, 17"),
    TEST_ENTRY_C(0xD2921001, NULL, "vi2f.s      S010, S400, 18"),
    TEST_ENTRY_C(0xD2921081, NULL, "vi2f.p      C010, C400, 18"),
    TEST_ENTRY_C(0xD2929001, NULL, "vi2f.t      C010, C400, 18"),
    TEST_ENTRY_C(0xD2929081, NULL, "vi2f.q      C010, C400, 18"),
    TEST_ENTRY_C(0xD2931001, NULL, "vi2f.s      S010, S400, 19"),
    TEST_ENTRY_C(0xD2931081, NULL, "vi2f.p      C010, C400, 19"),
    TEST_ENTRY_C(0xD2939001, NULL, "vi2f.t      C010, C400, 19"),
    TEST_ENTRY_C(0xD2939081, NULL, "vi2f.q      C010, C400, 19"),
    TEST_ENTRY_C(0xD2941001, NULL, "vi2f.s      S010, S400, 20"),
    TEST_ENTRY_C(0xD2941081, NULL, "vi2f.p      C010, C400, 20"),
    TEST_ENTRY_C(0xD2949001, NULL, "vi2f.t      C010, C400, 20"),
    TEST_ENTRY_C(0xD2949081, NULL, "vi2f.q      C010, C400, 20"),
    TEST_ENTRY_C(0xD2951001, NULL, "vi2f.s      S010, S400, 21"),
    TEST_ENTRY_C(0xD2951081, NULL, "vi2f.p      C010, C400, 21"),
    TEST_ENTRY_C(0xD2959001, NULL, "vi2f.t      C010, C400, 21"),
    TEST_ENTRY_C(0xD2959081, NULL, "vi2f.q      C010, C400, 21"),
    TEST_ENTRY_C(0xD2961001, NULL, "vi2f.s      S010, S400, 22"),
    TEST_ENTRY_C(0xD2961081, NULL, "vi2f.p      C010, C400, 22"),
    TEST_ENTRY_C(0xD2969001, NULL, "vi2f.t      C010, C400, 22"),
    TEST_ENTRY_C(0xD2969081, NULL, "vi2f.q      C010, C400, 22"),
    TEST_ENTRY_C(0xD2971001, NULL, "vi2f.s      S010, S400, 23"),
    TEST_ENTRY_C(0xD2971081, NULL, "vi2f.p      C010, C400, 23"),
    TEST_ENTRY_C(0xD2979001, NULL, "vi2f.t      C010, C400, 23"),
    TEST_ENTRY_C(0xD2979081, NULL, "vi2f.q      C010, C400, 23"),
    TEST_ENTRY_C(0xD2981001, NULL, "vi2f.s      S010, S400, 24"),
    TEST_ENTRY_C(0xD2981081, NULL, "vi2f.p      C010, C400, 24"),
    TEST_ENTRY_C(0xD2989001, NULL, "vi2f.t      C010, C400, 24"),
    TEST_ENTRY_C(0xD2989081, NULL, "vi2f.q      C010, C400, 24"),
    TEST_ENTRY_C(0xD2991001, NULL, "vi2f.s      S010, S400, 25"),
    TEST_ENTRY_C(0xD2991081, NULL, "vi2f.p      C010, C400, 25"),
    TEST_ENTRY_C(0xD2999001, NULL, "vi2f.t      C010, C400, 25"),
    TEST_ENTRY_C(0xD2999081, NULL, "vi2f.q      C010, C400, 25"),
    TEST_ENTRY_C(0xD29A1001, NULL, "vi2f.s      S010, S400, 26"),
    TEST_ENTRY_C(0xD29A1081, NULL, "vi2f.p      C010, C400, 26"),
    TEST_ENTRY_C(0xD29A9001, NULL, "vi2f.t      C010, C400, 26"),
    TEST_ENTRY_C(0xD29A9081, NULL, "vi2f.q      C010, C400, 26"),
    TEST_ENTRY_C(0xD29B1001, NULL, "vi2f.s      S010, S400, 27"),
    TEST_ENTRY_C(0xD29B1081, NULL, "vi2f.p      C010, C400, 27"),
    TEST_ENTRY_C(0xD29B9001, NULL, "vi2f.t      C010, C400, 27"),
    TEST_ENTRY_C(0xD29B9081, NULL, "vi2f.q      C010, C400, 27"),
    TEST_ENTRY_C(0xD29C1001, NULL, "vi2f.s      S010, S400, 28"),
    TEST_ENTRY_C(0xD29C1081, NULL, "vi2f.p      C010, C400, 28"),
    TEST_ENTRY_C(0xD29C9001, NULL, "vi2f.t      C010, C400, 28"),
    TEST_ENTRY_C(0xD29C9081, NULL, "vi2f.q      C010, C400, 28"),
    TEST_ENTRY_C(0xD29D1001, NULL, "vi2f.s      S010, S400, 29"),
    TEST_ENTRY_C(0xD29D1081, NULL, "vi2f.p      C010, C400, 29"),
    TEST_ENTRY_C(0xD29D9001, NULL, "vi2f.t      C010, C400, 29"),
    TEST_ENTRY_C(0xD29D9081, NULL, "vi2f.q      C010, C400, 29"),
    TEST_ENTRY_C(0xD29E1001, NULL, "vi2f.s      S010, S400, 30"),
    TEST_ENTRY_C(0xD29E1081, NULL, "vi2f.p      C010, C400, 30"),
    TEST_ENTRY_C(0xD29E9001, NULL, "vi2f.t      C010, C400, 30"),
    TEST_ENTRY_C(0xD29E9081, NULL, "vi2f.q      C010, C400, 30"),
    TEST_ENTRY_C(0xD29F1001, NULL, "vi2f.s      S010, S400, 31"),
    TEST_ENTRY_C(0xD29F1081, NULL, "vi2f.p      C010, C400, 31"),
    TEST_ENTRY_C(0xD29F9001, NULL, "vi2f.t      C010, C400, 31"),
    TEST_ENTRY_C(0xD29F9081, NULL, "vi2f.q      C010, C400, 31"),
    TEST_ENTRY_C(0xD2A01001, NULL, "vcmovt.s    S010, S400, 0"),
    TEST_ENTRY_C(0xD2A01081, NULL, "vcmovt.p    C010, C400, 0"),
    TEST_ENTRY_C(0xD2A09001, NULL, "vcmovt.t    C010, C400, 0"),
    TEST_ENTRY_C(0xD2A09081, NULL, "vcmovt.q    C010, C400, 0"),
    TEST_ENTRY_C(0xD2A11001, NULL, "vcmovt.s    S010, S400, 1"),
    TEST_ENTRY_C(0xD2A11081, NULL, "vcmovt.p    C010, C400, 1"),
    TEST_ENTRY_C(0xD2A19001, NULL, "vcmovt.t    C010, C400, 1"),
    TEST_ENTRY_C(0xD2A19081, NULL, "vcmovt.q    C010, C400, 1"),
    TEST_ENTRY_C(0xD2A21001, NULL, "vcmovt.s    S010, S400, 2"),
    TEST_ENTRY_C(0xD2A21081, NULL, "vcmovt.p    C010, C400, 2"),
    TEST_ENTRY_C(0xD2A29001, NULL, "vcmovt.t    C010, C400, 2"),
    TEST_ENTRY_C(0xD2A29081, NULL, "vcmovt.q    C010, C400, 2"),
    TEST_ENTRY_C(0xD2A31001, NULL, "vcmovt.s    S010, S400, 3"),
    TEST_ENTRY_C(0xD2A31081, NULL, "vcmovt.p    C010, C400, 3"),
    TEST_ENTRY_C(0xD2A39001, NULL, "vcmovt.t    C010, C400, 3"),
    TEST_ENTRY_C(0xD2A39081, NULL, "vcmovt.q    C010, C400, 3"),
    TEST_ENTRY_C(0xD2A41001, NULL, "vcmovt.s    S010, S400, 4"),
    TEST_ENTRY_C(0xD2A41081, NULL, "vcmovt.p    C010, C400, 4"),
    TEST_ENTRY_C(0xD2A49001, NULL, "vcmovt.t    C010, C400, 4"),
    TEST_ENTRY_C(0xD2A49081, NULL, "vcmovt.q    C010, C400, 4"),
    TEST_ENTRY_C(0xD2A51001, NULL, "vcmovt.s    S010, S400, 5"),
    TEST_ENTRY_C(0xD2A51081, NULL, "vcmovt.p    C010, C400, 5"),
    TEST_ENTRY_C(0xD2A59001, NULL, "vcmovt.t    C010, C400, 5"),
    TEST_ENTRY_C(0xD2A59081, NULL, "vcmovt.q    C010, C400, 5"),
    TEST_ENTRY_C(0xD2A61001, NULL, "vcmovt.s    S010, S400, 6"),
    TEST_ENTRY_C(0xD2A61081, NULL, "vcmovt.p    C010, C400, 6"),
    TEST_ENTRY_C(0xD2A69001, NULL, "vcmovt.t    C010, C400, 6"),
    TEST_ENTRY_C(0xD2A69081, NULL, "vcmovt.q    C010, C400, 6"),
    TEST_ENTRY_C(0xD2A81001, NULL, "vcmovf.s    S010, S400, 0"),
    TEST_ENTRY_C(0xD2A81081, NULL, "vcmovf.p    C010, C400, 0"),
    TEST_ENTRY_C(0xD2A89001, NULL, "vcmovf.t    C010, C400, 0"),
    TEST_ENTRY_C(0xD2A89081, NULL, "vcmovf.q    C010, C400, 0"),
    TEST_ENTRY_C(0xD2A91001, NULL, "vcmovf.s    S010, S400, 1"),
    TEST_ENTRY_C(0xD2A91081, NULL, "vcmovf.p    C010, C400, 1"),
    TEST_ENTRY_C(0xD2A99001, NULL, "vcmovf.t    C010, C400, 1"),
    TEST_ENTRY_C(0xD2A99081, NULL, "vcmovf.q    C010, C400, 1"),
    TEST_ENTRY_C(0xD2AA1001, NULL, "vcmovf.s    S010, S400, 2"),
    TEST_ENTRY_C(0xD2AA1081, NULL, "vcmovf.p    C010, C400, 2"),
    TEST_ENTRY_C(0xD2AA9001, NULL, "vcmovf.t    C010, C400, 2"),
    TEST_ENTRY_C(0xD2AA9081, NULL, "vcmovf.q    C010, C400, 2"),
    TEST_ENTRY_C(0xD2AB1001, NULL, "vcmovf.s    S010, S400, 3"),
    TEST_ENTRY_C(0xD2AB1081, NULL, "vcmovf.p    C010, C400, 3"),
    TEST_ENTRY_C(0xD2AB9001, NULL, "vcmovf.t    C010, C400, 3"),
    TEST_ENTRY_C(0xD2AB9081, NULL, "vcmovf.q    C010, C400, 3"),
    TEST_ENTRY_C(0xD2AC1001, NULL, "vcmovf.s    S010, S400, 4"),
    TEST_ENTRY_C(0xD2AC1081, NULL, "vcmovf.p    C010, C400, 4"),
    TEST_ENTRY_C(0xD2AC9001, NULL, "vcmovf.t    C010, C400, 4"),
    TEST_ENTRY_C(0xD2AC9081, NULL, "vcmovf.q    C010, C400, 4"),
    TEST_ENTRY_C(0xD2AD1001, NULL, "vcmovf.s    S010, S400, 5"),
    TEST_ENTRY_C(0xD2AD1081, NULL, "vcmovf.p    C010, C400, 5"),
    TEST_ENTRY_C(0xD2AD9001, NULL, "vcmovf.t    C010, C400, 5"),
    TEST_ENTRY_C(0xD2AD9081, NULL, "vcmovf.q    C010, C400, 5"),
    TEST_ENTRY_C(0xD2AE1001, NULL, "vcmovf.s    S010, S400, 6"),
    TEST_ENTRY_C(0xD2AE1081, NULL, "vcmovf.p    C010, C400, 6"),
    TEST_ENTRY_C(0xD2AE9001, NULL, "vcmovf.t    C010, C400, 6"),
    TEST_ENTRY_C(0xD2AE9081, NULL, "vcmovf.q    C010, C400, 6"),
    TEST_ENTRY_C(0xD2B01001, NULL, ".word       0xD2B01001                   # INVALID     $s5, $s0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B01081, NULL, ".word       0xD2B01081                   # INVALID     $s5, $s0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B09001, NULL, ".word       0xD2B09001                   # INVALID     $s5, $s0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B09081, NULL, ".word       0xD2B09081                   # INVALID     $s5, $s0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B11001, NULL, ".word       0xD2B11001                   # INVALID     $s5, $s1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B11081, NULL, ".word       0xD2B11081                   # INVALID     $s5, $s1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B19001, NULL, ".word       0xD2B19001                   # INVALID     $s5, $s1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B19081, NULL, ".word       0xD2B19081                   # INVALID     $s5, $s1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B21001, NULL, ".word       0xD2B21001                   # INVALID     $s5, $s2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B21081, NULL, ".word       0xD2B21081                   # INVALID     $s5, $s2, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B29001, NULL, ".word       0xD2B29001                   # INVALID     $s5, $s2, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B29081, NULL, ".word       0xD2B29081                   # INVALID     $s5, $s2, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B31001, NULL, ".word       0xD2B31001                   # INVALID     $s5, $s3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B31081, NULL, ".word       0xD2B31081                   # INVALID     $s5, $s3, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B39001, NULL, ".word       0xD2B39001                   # INVALID     $s5, $s3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B39081, NULL, ".word       0xD2B39081                   # INVALID     $s5, $s3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B41001, NULL, ".word       0xD2B41001                   # INVALID     $s5, $s4, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B41081, NULL, ".word       0xD2B41081                   # INVALID     $s5, $s4, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B49001, NULL, ".word       0xD2B49001                   # INVALID     $s5, $s4, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B49081, NULL, ".word       0xD2B49081                   # INVALID     $s5, $s4, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B51001, NULL, ".word       0xD2B51001                   # INVALID     $s5, $s5, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B51081, NULL, ".word       0xD2B51081                   # INVALID     $s5, $s5, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B59001, NULL, ".word       0xD2B59001                   # INVALID     $s5, $s5, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B59081, NULL, ".word       0xD2B59081                   # INVALID     $s5, $s5, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B61001, NULL, ".word       0xD2B61001                   # INVALID     $s5, $s6, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B61081, NULL, ".word       0xD2B61081                   # INVALID     $s5, $s6, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B69001, NULL, ".word       0xD2B69001                   # INVALID     $s5, $s6, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B69081, NULL, ".word       0xD2B69081                   # INVALID     $s5, $s6, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B81001, NULL, ".word       0xD2B81001                   # INVALID     $s5, $t8, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B81081, NULL, ".word       0xD2B81081                   # INVALID     $s5, $t8, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B89001, NULL, ".word       0xD2B89001                   # INVALID     $s5, $t8, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B89081, NULL, ".word       0xD2B89081                   # INVALID     $s5, $t8, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B91001, NULL, ".word       0xD2B91001                   # INVALID     $s5, $t9, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B91081, NULL, ".word       0xD2B91081                   # INVALID     $s5, $t9, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B99001, NULL, ".word       0xD2B99001                   # INVALID     $s5, $t9, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2B99081, NULL, ".word       0xD2B99081                   # INVALID     $s5, $t9, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2BA1001, NULL, ".word       0xD2BA1001                   # INVALID     $s5, $k0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2BA1081, NULL, ".word       0xD2BA1081                   # INVALID     $s5, $k0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2BA9001, NULL, ".word       0xD2BA9001                   # INVALID     $s5, $k0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2BA9081, NULL, ".word       0xD2BA9081                   # INVALID     $s5, $k0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2BB1001, NULL, ".word       0xD2BB1001                   # INVALID     $s5, $k1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2BB1081, NULL, ".word       0xD2BB1081                   # INVALID     $s5, $k1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2BB9001, NULL, ".word       0xD2BB9001                   # INVALID     $s5, $k1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2BB9081, NULL, ".word       0xD2BB9081                   # INVALID     $s5, $k1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2BC1001, NULL, ".word       0xD2BC1001                   # INVALID     $s5, $gp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2BC1081, NULL, ".word       0xD2BC1081                   # INVALID     $s5, $gp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2BC9001, NULL, ".word       0xD2BC9001                   # INVALID     $s5, $gp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2BC9081, NULL, ".word       0xD2BC9081                   # INVALID     $s5, $gp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2BD1001, NULL, ".word       0xD2BD1001                   # INVALID     $s5, $sp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2BD1081, NULL, ".word       0xD2BD1081                   # INVALID     $s5, $sp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2BD9001, NULL, ".word       0xD2BD9001                   # INVALID     $s5, $sp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2BD9081, NULL, ".word       0xD2BD9081                   # INVALID     $s5, $sp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2BE1001, NULL, ".word       0xD2BE1001                   # INVALID     $s5, $fp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2BE1081, NULL, ".word       0xD2BE1081                   # INVALID     $s5, $fp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2BE9001, NULL, ".word       0xD2BE9001                   # INVALID     $s5, $fp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2BE9081, NULL, ".word       0xD2BE9081                   # INVALID     $s5, $fp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2_CNDMOVE>"),
    TEST_ENTRY_C(0xD2C01001, NULL, ".word       0xD2C01001                   # INVALID     $s6, $zero, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C01081, NULL, ".word       0xD2C01081                   # INVALID     $s6, $zero, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C09001, NULL, ".word       0xD2C09001                   # INVALID     $s6, $zero, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C09081, NULL, ".word       0xD2C09081                   # INVALID     $s6, $zero, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C11001, NULL, ".word       0xD2C11001                   # INVALID     $s6, $at, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C11081, NULL, ".word       0xD2C11081                   # INVALID     $s6, $at, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C19001, NULL, ".word       0xD2C19001                   # INVALID     $s6, $at, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C19081, NULL, ".word       0xD2C19081                   # INVALID     $s6, $at, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C21001, NULL, ".word       0xD2C21001                   # INVALID     $s6, $v0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C21081, NULL, ".word       0xD2C21081                   # INVALID     $s6, $v0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C29001, NULL, ".word       0xD2C29001                   # INVALID     $s6, $v0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C29081, NULL, ".word       0xD2C29081                   # INVALID     $s6, $v0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C31001, NULL, ".word       0xD2C31001                   # INVALID     $s6, $v1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C31081, NULL, ".word       0xD2C31081                   # INVALID     $s6, $v1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C39001, NULL, ".word       0xD2C39001                   # INVALID     $s6, $v1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C39081, NULL, ".word       0xD2C39081                   # INVALID     $s6, $v1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C41001, NULL, ".word       0xD2C41001                   # INVALID     $s6, $a0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C41081, NULL, ".word       0xD2C41081                   # INVALID     $s6, $a0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C49001, NULL, ".word       0xD2C49001                   # INVALID     $s6, $a0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C49081, NULL, ".word       0xD2C49081                   # INVALID     $s6, $a0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C51001, NULL, ".word       0xD2C51001                   # INVALID     $s6, $a1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C51081, NULL, ".word       0xD2C51081                   # INVALID     $s6, $a1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C59001, NULL, ".word       0xD2C59001                   # INVALID     $s6, $a1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C59081, NULL, ".word       0xD2C59081                   # INVALID     $s6, $a1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C61001, NULL, ".word       0xD2C61001                   # INVALID     $s6, $a2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C61081, NULL, ".word       0xD2C61081                   # INVALID     $s6, $a2, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C69001, NULL, ".word       0xD2C69001                   # INVALID     $s6, $a2, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C69081, NULL, ".word       0xD2C69081                   # INVALID     $s6, $a2, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C71001, NULL, ".word       0xD2C71001                   # INVALID     $s6, $a3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C71081, NULL, ".word       0xD2C71081                   # INVALID     $s6, $a3, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C79001, NULL, ".word       0xD2C79001                   # INVALID     $s6, $a3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C79081, NULL, ".word       0xD2C79081                   # INVALID     $s6, $a3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C81001, NULL, ".word       0xD2C81001                   # INVALID     $s6, $t0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C81081, NULL, ".word       0xD2C81081                   # INVALID     $s6, $t0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C89001, NULL, ".word       0xD2C89001                   # INVALID     $s6, $t0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C89081, NULL, ".word       0xD2C89081                   # INVALID     $s6, $t0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C91001, NULL, ".word       0xD2C91001                   # INVALID     $s6, $t1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C91081, NULL, ".word       0xD2C91081                   # INVALID     $s6, $t1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C99001, NULL, ".word       0xD2C99001                   # INVALID     $s6, $t1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2C99081, NULL, ".word       0xD2C99081                   # INVALID     $s6, $t1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2CA1001, NULL, ".word       0xD2CA1001                   # INVALID     $s6, $t2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2CA1081, NULL, ".word       0xD2CA1081                   # INVALID     $s6, $t2, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2CA9001, NULL, ".word       0xD2CA9001                   # INVALID     $s6, $t2, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2CA9081, NULL, ".word       0xD2CA9081                   # INVALID     $s6, $t2, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2CB1001, NULL, ".word       0xD2CB1001                   # INVALID     $s6, $t3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2CB1081, NULL, ".word       0xD2CB1081                   # INVALID     $s6, $t3, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2CB9001, NULL, ".word       0xD2CB9001                   # INVALID     $s6, $t3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2CB9081, NULL, ".word       0xD2CB9081                   # INVALID     $s6, $t3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2CC1001, NULL, ".word       0xD2CC1001                   # INVALID     $s6, $t4, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2CC1081, NULL, ".word       0xD2CC1081                   # INVALID     $s6, $t4, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2CC9001, NULL, ".word       0xD2CC9001                   # INVALID     $s6, $t4, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2CC9081, NULL, ".word       0xD2CC9081                   # INVALID     $s6, $t4, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2CD1001, NULL, ".word       0xD2CD1001                   # INVALID     $s6, $t5, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2CD1081, NULL, ".word       0xD2CD1081                   # INVALID     $s6, $t5, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2CD9001, NULL, ".word       0xD2CD9001                   # INVALID     $s6, $t5, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2CD9081, NULL, ".word       0xD2CD9081                   # INVALID     $s6, $t5, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2CE1001, NULL, ".word       0xD2CE1001                   # INVALID     $s6, $t6, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2CE1081, NULL, ".word       0xD2CE1081                   # INVALID     $s6, $t6, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2CE9001, NULL, ".word       0xD2CE9001                   # INVALID     $s6, $t6, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2CE9081, NULL, ".word       0xD2CE9081                   # INVALID     $s6, $t6, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2CF1001, NULL, ".word       0xD2CF1001                   # INVALID     $s6, $t7, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2CF1081, NULL, ".word       0xD2CF1081                   # INVALID     $s6, $t7, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2CF9001, NULL, ".word       0xD2CF9001                   # INVALID     $s6, $t7, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2CF9081, NULL, ".word       0xD2CF9081                   # INVALID     $s6, $t7, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D01001, NULL, ".word       0xD2D01001                   # INVALID     $s6, $s0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D01081, NULL, ".word       0xD2D01081                   # INVALID     $s6, $s0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D09001, NULL, ".word       0xD2D09001                   # INVALID     $s6, $s0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D09081, NULL, ".word       0xD2D09081                   # INVALID     $s6, $s0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D11001, NULL, ".word       0xD2D11001                   # INVALID     $s6, $s1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D11081, NULL, ".word       0xD2D11081                   # INVALID     $s6, $s1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D19001, NULL, ".word       0xD2D19001                   # INVALID     $s6, $s1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D19081, NULL, ".word       0xD2D19081                   # INVALID     $s6, $s1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D21001, NULL, ".word       0xD2D21001                   # INVALID     $s6, $s2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D21081, NULL, ".word       0xD2D21081                   # INVALID     $s6, $s2, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D29001, NULL, ".word       0xD2D29001                   # INVALID     $s6, $s2, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D29081, NULL, ".word       0xD2D29081                   # INVALID     $s6, $s2, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D31001, NULL, ".word       0xD2D31001                   # INVALID     $s6, $s3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D31081, NULL, ".word       0xD2D31081                   # INVALID     $s6, $s3, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D39001, NULL, ".word       0xD2D39001                   # INVALID     $s6, $s3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D39081, NULL, ".word       0xD2D39081                   # INVALID     $s6, $s3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D41001, NULL, ".word       0xD2D41001                   # INVALID     $s6, $s4, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D41081, NULL, ".word       0xD2D41081                   # INVALID     $s6, $s4, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D49001, NULL, ".word       0xD2D49001                   # INVALID     $s6, $s4, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D49081, NULL, ".word       0xD2D49081                   # INVALID     $s6, $s4, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D51001, NULL, ".word       0xD2D51001                   # INVALID     $s6, $s5, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D51081, NULL, ".word       0xD2D51081                   # INVALID     $s6, $s5, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D59001, NULL, ".word       0xD2D59001                   # INVALID     $s6, $s5, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D59081, NULL, ".word       0xD2D59081                   # INVALID     $s6, $s5, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D61001, NULL, ".word       0xD2D61001                   # INVALID     $s6, $s6, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D61081, NULL, ".word       0xD2D61081                   # INVALID     $s6, $s6, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D69001, NULL, ".word       0xD2D69001                   # INVALID     $s6, $s6, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D69081, NULL, ".word       0xD2D69081                   # INVALID     $s6, $s6, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D71001, NULL, ".word       0xD2D71001                   # INVALID     $s6, $s7, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D71081, NULL, ".word       0xD2D71081                   # INVALID     $s6, $s7, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D79001, NULL, ".word       0xD2D79001                   # INVALID     $s6, $s7, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D79081, NULL, ".word       0xD2D79081                   # INVALID     $s6, $s7, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D81001, NULL, ".word       0xD2D81001                   # INVALID     $s6, $t8, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D81081, NULL, ".word       0xD2D81081                   # INVALID     $s6, $t8, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D89001, NULL, ".word       0xD2D89001                   # INVALID     $s6, $t8, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D89081, NULL, ".word       0xD2D89081                   # INVALID     $s6, $t8, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D91001, NULL, ".word       0xD2D91001                   # INVALID     $s6, $t9, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D91081, NULL, ".word       0xD2D91081                   # INVALID     $s6, $t9, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D99001, NULL, ".word       0xD2D99001                   # INVALID     $s6, $t9, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2D99081, NULL, ".word       0xD2D99081                   # INVALID     $s6, $t9, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2DA1001, NULL, ".word       0xD2DA1001                   # INVALID     $s6, $k0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2DA1081, NULL, ".word       0xD2DA1081                   # INVALID     $s6, $k0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2DA9001, NULL, ".word       0xD2DA9001                   # INVALID     $s6, $k0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2DA9081, NULL, ".word       0xD2DA9081                   # INVALID     $s6, $k0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2DB1001, NULL, ".word       0xD2DB1001                   # INVALID     $s6, $k1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2DB1081, NULL, ".word       0xD2DB1081                   # INVALID     $s6, $k1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2DB9001, NULL, ".word       0xD2DB9001                   # INVALID     $s6, $k1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2DB9081, NULL, ".word       0xD2DB9081                   # INVALID     $s6, $k1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2DC1001, NULL, ".word       0xD2DC1001                   # INVALID     $s6, $gp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2DC1081, NULL, ".word       0xD2DC1081                   # INVALID     $s6, $gp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2DC9001, NULL, ".word       0xD2DC9001                   # INVALID     $s6, $gp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2DC9081, NULL, ".word       0xD2DC9081                   # INVALID     $s6, $gp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2DD1001, NULL, ".word       0xD2DD1001                   # INVALID     $s6, $sp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2DD1081, NULL, ".word       0xD2DD1081                   # INVALID     $s6, $sp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2DD9001, NULL, ".word       0xD2DD9001                   # INVALID     $s6, $sp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2DD9081, NULL, ".word       0xD2DD9081                   # INVALID     $s6, $sp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2DE1001, NULL, ".word       0xD2DE1001                   # INVALID     $s6, $fp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2DE1081, NULL, ".word       0xD2DE1081                   # INVALID     $s6, $fp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2DE9001, NULL, ".word       0xD2DE9001                   # INVALID     $s6, $fp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2DE9081, NULL, ".word       0xD2DE9081                   # INVALID     $s6, $fp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2DF1001, NULL, ".word       0xD2DF1001                   # INVALID     $s6, $ra, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2DF1081, NULL, ".word       0xD2DF1081                   # INVALID     $s6, $ra, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2DF9001, NULL, ".word       0xD2DF9001                   # INVALID     $s6, $ra, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2DF9081, NULL, ".word       0xD2DF9081                   # INVALID     $s6, $ra, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E01001, NULL, ".word       0xD2E01001                   # INVALID     $s7, $zero, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E01081, NULL, ".word       0xD2E01081                   # INVALID     $s7, $zero, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E09001, NULL, ".word       0xD2E09001                   # INVALID     $s7, $zero, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E09081, NULL, ".word       0xD2E09081                   # INVALID     $s7, $zero, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E11001, NULL, ".word       0xD2E11001                   # INVALID     $s7, $at, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E11081, NULL, ".word       0xD2E11081                   # INVALID     $s7, $at, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E19001, NULL, ".word       0xD2E19001                   # INVALID     $s7, $at, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E19081, NULL, ".word       0xD2E19081                   # INVALID     $s7, $at, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E21001, NULL, ".word       0xD2E21001                   # INVALID     $s7, $v0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E21081, NULL, ".word       0xD2E21081                   # INVALID     $s7, $v0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E29001, NULL, ".word       0xD2E29001                   # INVALID     $s7, $v0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E29081, NULL, ".word       0xD2E29081                   # INVALID     $s7, $v0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E31001, NULL, ".word       0xD2E31001                   # INVALID     $s7, $v1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E31081, NULL, ".word       0xD2E31081                   # INVALID     $s7, $v1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E39001, NULL, ".word       0xD2E39001                   # INVALID     $s7, $v1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E39081, NULL, ".word       0xD2E39081                   # INVALID     $s7, $v1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E41001, NULL, ".word       0xD2E41001                   # INVALID     $s7, $a0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E41081, NULL, ".word       0xD2E41081                   # INVALID     $s7, $a0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E49001, NULL, ".word       0xD2E49001                   # INVALID     $s7, $a0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E49081, NULL, ".word       0xD2E49081                   # INVALID     $s7, $a0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E51001, NULL, ".word       0xD2E51001                   # INVALID     $s7, $a1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E51081, NULL, ".word       0xD2E51081                   # INVALID     $s7, $a1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E59001, NULL, ".word       0xD2E59001                   # INVALID     $s7, $a1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E59081, NULL, ".word       0xD2E59081                   # INVALID     $s7, $a1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E61001, NULL, ".word       0xD2E61001                   # INVALID     $s7, $a2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E61081, NULL, ".word       0xD2E61081                   # INVALID     $s7, $a2, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E69001, NULL, ".word       0xD2E69001                   # INVALID     $s7, $a2, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E69081, NULL, ".word       0xD2E69081                   # INVALID     $s7, $a2, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E71001, NULL, ".word       0xD2E71001                   # INVALID     $s7, $a3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E71081, NULL, ".word       0xD2E71081                   # INVALID     $s7, $a3, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E79001, NULL, ".word       0xD2E79001                   # INVALID     $s7, $a3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E79081, NULL, ".word       0xD2E79081                   # INVALID     $s7, $a3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E81001, NULL, ".word       0xD2E81001                   # INVALID     $s7, $t0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E81081, NULL, ".word       0xD2E81081                   # INVALID     $s7, $t0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E89001, NULL, ".word       0xD2E89001                   # INVALID     $s7, $t0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E89081, NULL, ".word       0xD2E89081                   # INVALID     $s7, $t0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E91001, NULL, ".word       0xD2E91001                   # INVALID     $s7, $t1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E91081, NULL, ".word       0xD2E91081                   # INVALID     $s7, $t1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E99001, NULL, ".word       0xD2E99001                   # INVALID     $s7, $t1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2E99081, NULL, ".word       0xD2E99081                   # INVALID     $s7, $t1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2EA1001, NULL, ".word       0xD2EA1001                   # INVALID     $s7, $t2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2EA1081, NULL, ".word       0xD2EA1081                   # INVALID     $s7, $t2, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2EA9001, NULL, ".word       0xD2EA9001                   # INVALID     $s7, $t2, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2EA9081, NULL, ".word       0xD2EA9081                   # INVALID     $s7, $t2, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2EB1001, NULL, ".word       0xD2EB1001                   # INVALID     $s7, $t3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2EB1081, NULL, ".word       0xD2EB1081                   # INVALID     $s7, $t3, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2EB9001, NULL, ".word       0xD2EB9001                   # INVALID     $s7, $t3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2EB9081, NULL, ".word       0xD2EB9081                   # INVALID     $s7, $t3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2EC1001, NULL, ".word       0xD2EC1001                   # INVALID     $s7, $t4, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2EC1081, NULL, ".word       0xD2EC1081                   # INVALID     $s7, $t4, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2EC9001, NULL, ".word       0xD2EC9001                   # INVALID     $s7, $t4, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2EC9081, NULL, ".word       0xD2EC9081                   # INVALID     $s7, $t4, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2ED1001, NULL, ".word       0xD2ED1001                   # INVALID     $s7, $t5, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2ED1081, NULL, ".word       0xD2ED1081                   # INVALID     $s7, $t5, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2ED9001, NULL, ".word       0xD2ED9001                   # INVALID     $s7, $t5, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2ED9081, NULL, ".word       0xD2ED9081                   # INVALID     $s7, $t5, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2EE1001, NULL, ".word       0xD2EE1001                   # INVALID     $s7, $t6, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2EE1081, NULL, ".word       0xD2EE1081                   # INVALID     $s7, $t6, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2EE9001, NULL, ".word       0xD2EE9001                   # INVALID     $s7, $t6, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2EE9081, NULL, ".word       0xD2EE9081                   # INVALID     $s7, $t6, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2EF1001, NULL, ".word       0xD2EF1001                   # INVALID     $s7, $t7, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2EF1081, NULL, ".word       0xD2EF1081                   # INVALID     $s7, $t7, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2EF9001, NULL, ".word       0xD2EF9001                   # INVALID     $s7, $t7, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2EF9081, NULL, ".word       0xD2EF9081                   # INVALID     $s7, $t7, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F01001, NULL, ".word       0xD2F01001                   # INVALID     $s7, $s0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F01081, NULL, ".word       0xD2F01081                   # INVALID     $s7, $s0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F09001, NULL, ".word       0xD2F09001                   # INVALID     $s7, $s0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F09081, NULL, ".word       0xD2F09081                   # INVALID     $s7, $s0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F11001, NULL, ".word       0xD2F11001                   # INVALID     $s7, $s1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F11081, NULL, ".word       0xD2F11081                   # INVALID     $s7, $s1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F19001, NULL, ".word       0xD2F19001                   # INVALID     $s7, $s1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F19081, NULL, ".word       0xD2F19081                   # INVALID     $s7, $s1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F21001, NULL, ".word       0xD2F21001                   # INVALID     $s7, $s2, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F21081, NULL, ".word       0xD2F21081                   # INVALID     $s7, $s2, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F29001, NULL, ".word       0xD2F29001                   # INVALID     $s7, $s2, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F29081, NULL, ".word       0xD2F29081                   # INVALID     $s7, $s2, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F31001, NULL, ".word       0xD2F31001                   # INVALID     $s7, $s3, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F31081, NULL, ".word       0xD2F31081                   # INVALID     $s7, $s3, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F39001, NULL, ".word       0xD2F39001                   # INVALID     $s7, $s3, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F39081, NULL, ".word       0xD2F39081                   # INVALID     $s7, $s3, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F41001, NULL, ".word       0xD2F41001                   # INVALID     $s7, $s4, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F41081, NULL, ".word       0xD2F41081                   # INVALID     $s7, $s4, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F49001, NULL, ".word       0xD2F49001                   # INVALID     $s7, $s4, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F49081, NULL, ".word       0xD2F49081                   # INVALID     $s7, $s4, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F51001, NULL, ".word       0xD2F51001                   # INVALID     $s7, $s5, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F51081, NULL, ".word       0xD2F51081                   # INVALID     $s7, $s5, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F59001, NULL, ".word       0xD2F59001                   # INVALID     $s7, $s5, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F59081, NULL, ".word       0xD2F59081                   # INVALID     $s7, $s5, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F61001, NULL, ".word       0xD2F61001                   # INVALID     $s7, $s6, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F61081, NULL, ".word       0xD2F61081                   # INVALID     $s7, $s6, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F69001, NULL, ".word       0xD2F69001                   # INVALID     $s7, $s6, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F69081, NULL, ".word       0xD2F69081                   # INVALID     $s7, $s6, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F71001, NULL, ".word       0xD2F71001                   # INVALID     $s7, $s7, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F71081, NULL, ".word       0xD2F71081                   # INVALID     $s7, $s7, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F79001, NULL, ".word       0xD2F79001                   # INVALID     $s7, $s7, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F79081, NULL, ".word       0xD2F79081                   # INVALID     $s7, $s7, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F81001, NULL, ".word       0xD2F81001                   # INVALID     $s7, $t8, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F81081, NULL, ".word       0xD2F81081                   # INVALID     $s7, $t8, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F89001, NULL, ".word       0xD2F89001                   # INVALID     $s7, $t8, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F89081, NULL, ".word       0xD2F89081                   # INVALID     $s7, $t8, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F91001, NULL, ".word       0xD2F91001                   # INVALID     $s7, $t9, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F91081, NULL, ".word       0xD2F91081                   # INVALID     $s7, $t9, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F99001, NULL, ".word       0xD2F99001                   # INVALID     $s7, $t9, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2F99081, NULL, ".word       0xD2F99081                   # INVALID     $s7, $t9, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2FA1001, NULL, ".word       0xD2FA1001                   # INVALID     $s7, $k0, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2FA1081, NULL, ".word       0xD2FA1081                   # INVALID     $s7, $k0, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2FA9001, NULL, ".word       0xD2FA9001                   # INVALID     $s7, $k0, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2FA9081, NULL, ".word       0xD2FA9081                   # INVALID     $s7, $k0, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2FB1001, NULL, ".word       0xD2FB1001                   # INVALID     $s7, $k1, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2FB1081, NULL, ".word       0xD2FB1081                   # INVALID     $s7, $k1, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2FB9001, NULL, ".word       0xD2FB9001                   # INVALID     $s7, $k1, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2FB9081, NULL, ".word       0xD2FB9081                   # INVALID     $s7, $k1, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2FC1001, NULL, ".word       0xD2FC1001                   # INVALID     $s7, $gp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2FC1081, NULL, ".word       0xD2FC1081                   # INVALID     $s7, $gp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2FC9001, NULL, ".word       0xD2FC9001                   # INVALID     $s7, $gp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2FC9081, NULL, ".word       0xD2FC9081                   # INVALID     $s7, $gp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2FD1001, NULL, ".word       0xD2FD1001                   # INVALID     $s7, $sp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2FD1081, NULL, ".word       0xD2FD1081                   # INVALID     $s7, $sp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2FD9001, NULL, ".word       0xD2FD9001                   # INVALID     $s7, $sp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2FD9081, NULL, ".word       0xD2FD9081                   # INVALID     $s7, $sp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2FE1001, NULL, ".word       0xD2FE1001                   # INVALID     $s7, $fp, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2FE1081, NULL, ".word       0xD2FE1081                   # INVALID     $s7, $fp, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2FE9001, NULL, ".word       0xD2FE9001                   # INVALID     $s7, $fp, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2FE9081, NULL, ".word       0xD2FE9081                   # INVALID     $s7, $fp, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2FF1001, NULL, ".word       0xD2FF1001                   # INVALID     $s7, $ra, 0x1001 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2FF1081, NULL, ".word       0xD2FF1081                   # INVALID     $s7, $ra, 0x1081 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2FF9001, NULL, ".word       0xD2FF9001                   # INVALID     $s7, $ra, -0x6FFF # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),
    TEST_ENTRY_C(0xD2FF9081, NULL, ".word       0xD2FF9081                   # INVALID     $s7, $ra, -0x6F7F # 00000000 <InstrIdType: R4000ALLEGREX_VFPU4_FMT2>"),


    /* VFPU4 FMT3 */
    TEST_ENTRY_C(0xD3001001, NULL, "vwbn.s      S010, S400, 0"),
    TEST_ENTRY_C(0xD3011001, NULL, "vwbn.s      S010, S400, 1"),
    TEST_ENTRY_C(0xD3021001, NULL, "vwbn.s      S010, S400, 2"),
    TEST_ENTRY_C(0xD3031001, NULL, "vwbn.s      S010, S400, 3"),
    TEST_ENTRY_C(0xD3041001, NULL, "vwbn.s      S010, S400, 4"),
    TEST_ENTRY_C(0xD3051001, NULL, "vwbn.s      S010, S400, 5"),
    TEST_ENTRY_C(0xD3061001, NULL, "vwbn.s      S010, S400, 6"),
    TEST_ENTRY_C(0xD3071001, NULL, "vwbn.s      S010, S400, 7"),
    TEST_ENTRY_C(0xD3081001, NULL, "vwbn.s      S010, S400, 8"),
    TEST_ENTRY_C(0xD3091001, NULL, "vwbn.s      S010, S400, 9"),
    TEST_ENTRY_C(0xD30A1001, NULL, "vwbn.s      S010, S400, 10"),
    TEST_ENTRY_C(0xD30B1001, NULL, "vwbn.s      S010, S400, 11"),
    TEST_ENTRY_C(0xD30C1001, NULL, "vwbn.s      S010, S400, 12"),
    TEST_ENTRY_C(0xD30D1001, NULL, "vwbn.s      S010, S400, 13"),
    TEST_ENTRY_C(0xD30E1001, NULL, "vwbn.s      S010, S400, 14"),
    TEST_ENTRY_C(0xD30F1001, NULL, "vwbn.s      S010, S400, 15"),
    TEST_ENTRY_C(0xD3101001, NULL, "vwbn.s      S010, S400, 16"),
    TEST_ENTRY_C(0xD3111001, NULL, "vwbn.s      S010, S400, 17"),
    TEST_ENTRY_C(0xD3121001, NULL, "vwbn.s      S010, S400, 18"),
    TEST_ENTRY_C(0xD3131001, NULL, "vwbn.s      S010, S400, 19"),
    TEST_ENTRY_C(0xD3141001, NULL, "vwbn.s      S010, S400, 20"),
    TEST_ENTRY_C(0xD3151001, NULL, "vwbn.s      S010, S400, 21"),
    TEST_ENTRY_C(0xD3161001, NULL, "vwbn.s      S010, S400, 22"),
    TEST_ENTRY_C(0xD3171001, NULL, "vwbn.s      S010, S400, 23"),
    TEST_ENTRY_C(0xD3181001, NULL, "vwbn.s      S010, S400, 24"),
    TEST_ENTRY_C(0xD3191001, NULL, "vwbn.s      S010, S400, 25"),
    TEST_ENTRY_C(0xD31A1001, NULL, "vwbn.s      S010, S400, 26"),
    TEST_ENTRY_C(0xD31B1001, NULL, "vwbn.s      S010, S400, 27"),
    TEST_ENTRY_C(0xD31C1001, NULL, "vwbn.s      S010, S400, 28"),
    TEST_ENTRY_C(0xD31D1001, NULL, "vwbn.s      S010, S400, 29"),
    TEST_ENTRY_C(0xD31E1001, NULL, "vwbn.s      S010, S400, 30"),
    TEST_ENTRY_C(0xD31F1001, NULL, "vwbn.s      S010, S400, 31"),
    TEST_ENTRY_C(0xD3201001, NULL, "vwbn.s      S010, S400, 32"),
    TEST_ENTRY_C(0xD3211001, NULL, "vwbn.s      S010, S400, 33"),
    TEST_ENTRY_C(0xD3221001, NULL, "vwbn.s      S010, S400, 34"),
    TEST_ENTRY_C(0xD3231001, NULL, "vwbn.s      S010, S400, 35"),
    TEST_ENTRY_C(0xD3241001, NULL, "vwbn.s      S010, S400, 36"),
    TEST_ENTRY_C(0xD3251001, NULL, "vwbn.s      S010, S400, 37"),
    TEST_ENTRY_C(0xD3261001, NULL, "vwbn.s      S010, S400, 38"),
    TEST_ENTRY_C(0xD3271001, NULL, "vwbn.s      S010, S400, 39"),
    TEST_ENTRY_C(0xD3281001, NULL, "vwbn.s      S010, S400, 40"),
    TEST_ENTRY_C(0xD3291001, NULL, "vwbn.s      S010, S400, 41"),
    TEST_ENTRY_C(0xD32A1001, NULL, "vwbn.s      S010, S400, 42"),
    TEST_ENTRY_C(0xD32B1001, NULL, "vwbn.s      S010, S400, 43"),
    TEST_ENTRY_C(0xD32C1001, NULL, "vwbn.s      S010, S400, 44"),
    TEST_ENTRY_C(0xD32D1001, NULL, "vwbn.s      S010, S400, 45"),
    TEST_ENTRY_C(0xD32E1001, NULL, "vwbn.s      S010, S400, 46"),
    TEST_ENTRY_C(0xD32F1001, NULL, "vwbn.s      S010, S400, 47"),
    TEST_ENTRY_C(0xD3301001, NULL, "vwbn.s      S010, S400, 48"),
    TEST_ENTRY_C(0xD3311001, NULL, "vwbn.s      S010, S400, 49"),
    TEST_ENTRY_C(0xD3321001, NULL, "vwbn.s      S010, S400, 50"),
    TEST_ENTRY_C(0xD3331001, NULL, "vwbn.s      S010, S400, 51"),
    TEST_ENTRY_C(0xD3341001, NULL, "vwbn.s      S010, S400, 52"),
    TEST_ENTRY_C(0xD3351001, NULL, "vwbn.s      S010, S400, 53"),
    TEST_ENTRY_C(0xD3361001, NULL, "vwbn.s      S010, S400, 54"),
    TEST_ENTRY_C(0xD3371001, NULL, "vwbn.s      S010, S400, 55"),
    TEST_ENTRY_C(0xD3381001, NULL, "vwbn.s      S010, S400, 56"),
    TEST_ENTRY_C(0xD3391001, NULL, "vwbn.s      S010, S400, 57"),
    TEST_ENTRY_C(0xD33A1001, NULL, "vwbn.s      S010, S400, 58"),
    TEST_ENTRY_C(0xD33B1001, NULL, "vwbn.s      S010, S400, 59"),
    TEST_ENTRY_C(0xD33C1001, NULL, "vwbn.s      S010, S400, 60"),
    TEST_ENTRY_C(0xD33D1001, NULL, "vwbn.s      S010, S400, 61"),
    TEST_ENTRY_C(0xD33E1001, NULL, "vwbn.s      S010, S400, 62"),
    TEST_ENTRY_C(0xD33F1001, NULL, "vwbn.s      S010, S400, 63"),
    TEST_ENTRY_C(0xD3401001, NULL, "vwbn.s      S010, S400, 64"),
    TEST_ENTRY_C(0xD3411001, NULL, "vwbn.s      S010, S400, 65"),
    TEST_ENTRY_C(0xD3421001, NULL, "vwbn.s      S010, S400, 66"),
    TEST_ENTRY_C(0xD3431001, NULL, "vwbn.s      S010, S400, 67"),
    TEST_ENTRY_C(0xD3441001, NULL, "vwbn.s      S010, S400, 68"),
    TEST_ENTRY_C(0xD3451001, NULL, "vwbn.s      S010, S400, 69"),
    TEST_ENTRY_C(0xD3461001, NULL, "vwbn.s      S010, S400, 70"),
    TEST_ENTRY_C(0xD3471001, NULL, "vwbn.s      S010, S400, 71"),
    TEST_ENTRY_C(0xD3481001, NULL, "vwbn.s      S010, S400, 72"),
    TEST_ENTRY_C(0xD3491001, NULL, "vwbn.s      S010, S400, 73"),
    TEST_ENTRY_C(0xD34A1001, NULL, "vwbn.s      S010, S400, 74"),
    TEST_ENTRY_C(0xD34B1001, NULL, "vwbn.s      S010, S400, 75"),
    TEST_ENTRY_C(0xD34C1001, NULL, "vwbn.s      S010, S400, 76"),
    TEST_ENTRY_C(0xD34D1001, NULL, "vwbn.s      S010, S400, 77"),
    TEST_ENTRY_C(0xD34E1001, NULL, "vwbn.s      S010, S400, 78"),
    TEST_ENTRY_C(0xD34F1001, NULL, "vwbn.s      S010, S400, 79"),
    TEST_ENTRY_C(0xD3501001, NULL, "vwbn.s      S010, S400, 80"),
    TEST_ENTRY_C(0xD3511001, NULL, "vwbn.s      S010, S400, 81"),
    TEST_ENTRY_C(0xD3521001, NULL, "vwbn.s      S010, S400, 82"),
    TEST_ENTRY_C(0xD3531001, NULL, "vwbn.s      S010, S400, 83"),
    TEST_ENTRY_C(0xD3541001, NULL, "vwbn.s      S010, S400, 84"),
    TEST_ENTRY_C(0xD3551001, NULL, "vwbn.s      S010, S400, 85"),
    TEST_ENTRY_C(0xD3561001, NULL, "vwbn.s      S010, S400, 86"),
    TEST_ENTRY_C(0xD3571001, NULL, "vwbn.s      S010, S400, 87"),
    TEST_ENTRY_C(0xD3581001, NULL, "vwbn.s      S010, S400, 88"),
    TEST_ENTRY_C(0xD3591001, NULL, "vwbn.s      S010, S400, 89"),
    TEST_ENTRY_C(0xD35A1001, NULL, "vwbn.s      S010, S400, 90"),
    TEST_ENTRY_C(0xD35B1001, NULL, "vwbn.s      S010, S400, 91"),
    TEST_ENTRY_C(0xD35C1001, NULL, "vwbn.s      S010, S400, 92"),
    TEST_ENTRY_C(0xD35D1001, NULL, "vwbn.s      S010, S400, 93"),
    TEST_ENTRY_C(0xD35E1001, NULL, "vwbn.s      S010, S400, 94"),
    TEST_ENTRY_C(0xD35F1001, NULL, "vwbn.s      S010, S400, 95"),
    TEST_ENTRY_C(0xD3601001, NULL, "vwbn.s      S010, S400, 96"),
    TEST_ENTRY_C(0xD3611001, NULL, "vwbn.s      S010, S400, 97"),
    TEST_ENTRY_C(0xD3621001, NULL, "vwbn.s      S010, S400, 98"),
    TEST_ENTRY_C(0xD3631001, NULL, "vwbn.s      S010, S400, 99"),
    TEST_ENTRY_C(0xD3641001, NULL, "vwbn.s      S010, S400, 100"),
    TEST_ENTRY_C(0xD3651001, NULL, "vwbn.s      S010, S400, 101"),
    TEST_ENTRY_C(0xD3661001, NULL, "vwbn.s      S010, S400, 102"),
    TEST_ENTRY_C(0xD3671001, NULL, "vwbn.s      S010, S400, 103"),
    TEST_ENTRY_C(0xD3681001, NULL, "vwbn.s      S010, S400, 104"),
    TEST_ENTRY_C(0xD3691001, NULL, "vwbn.s      S010, S400, 105"),
    TEST_ENTRY_C(0xD36A1001, NULL, "vwbn.s      S010, S400, 106"),
    TEST_ENTRY_C(0xD36B1001, NULL, "vwbn.s      S010, S400, 107"),
    TEST_ENTRY_C(0xD36C1001, NULL, "vwbn.s      S010, S400, 108"),
    TEST_ENTRY_C(0xD36D1001, NULL, "vwbn.s      S010, S400, 109"),
    TEST_ENTRY_C(0xD36E1001, NULL, "vwbn.s      S010, S400, 110"),
    TEST_ENTRY_C(0xD36F1001, NULL, "vwbn.s      S010, S400, 111"),
    TEST_ENTRY_C(0xD3701001, NULL, "vwbn.s      S010, S400, 112"),
    TEST_ENTRY_C(0xD3711001, NULL, "vwbn.s      S010, S400, 113"),
    TEST_ENTRY_C(0xD3721001, NULL, "vwbn.s      S010, S400, 114"),
    TEST_ENTRY_C(0xD3731001, NULL, "vwbn.s      S010, S400, 115"),
    TEST_ENTRY_C(0xD3741001, NULL, "vwbn.s      S010, S400, 116"),
    TEST_ENTRY_C(0xD3751001, NULL, "vwbn.s      S010, S400, 117"),
    TEST_ENTRY_C(0xD3761001, NULL, "vwbn.s      S010, S400, 118"),
    TEST_ENTRY_C(0xD3771001, NULL, "vwbn.s      S010, S400, 119"),
    TEST_ENTRY_C(0xD3781001, NULL, "vwbn.s      S010, S400, 120"),
    TEST_ENTRY_C(0xD3791001, NULL, "vwbn.s      S010, S400, 121"),
    TEST_ENTRY_C(0xD37A1001, NULL, "vwbn.s      S010, S400, 122"),
    TEST_ENTRY_C(0xD37B1001, NULL, "vwbn.s      S010, S400, 123"),
    TEST_ENTRY_C(0xD37C1001, NULL, "vwbn.s      S010, S400, 124"),
    TEST_ENTRY_C(0xD37D1001, NULL, "vwbn.s      S010, S400, 125"),
    TEST_ENTRY_C(0xD37E1001, NULL, "vwbn.s      S010, S400, 126"),
    TEST_ENTRY_C(0xD37F1001, NULL, "vwbn.s      S010, S400, 127"),
    TEST_ENTRY_C(0xD3801001, NULL, "vwbn.s      S010, S400, 128"),
    TEST_ENTRY_C(0xD3811001, NULL, "vwbn.s      S010, S400, 129"),
    TEST_ENTRY_C(0xD3821001, NULL, "vwbn.s      S010, S400, 130"),
    TEST_ENTRY_C(0xD3831001, NULL, "vwbn.s      S010, S400, 131"),
    TEST_ENTRY_C(0xD3841001, NULL, "vwbn.s      S010, S400, 132"),
    TEST_ENTRY_C(0xD3851001, NULL, "vwbn.s      S010, S400, 133"),
    TEST_ENTRY_C(0xD3861001, NULL, "vwbn.s      S010, S400, 134"),
    TEST_ENTRY_C(0xD3871001, NULL, "vwbn.s      S010, S400, 135"),
    TEST_ENTRY_C(0xD3881001, NULL, "vwbn.s      S010, S400, 136"),
    TEST_ENTRY_C(0xD3891001, NULL, "vwbn.s      S010, S400, 137"),
    TEST_ENTRY_C(0xD38A1001, NULL, "vwbn.s      S010, S400, 138"),
    TEST_ENTRY_C(0xD38B1001, NULL, "vwbn.s      S010, S400, 139"),
    TEST_ENTRY_C(0xD38C1001, NULL, "vwbn.s      S010, S400, 140"),
    TEST_ENTRY_C(0xD38D1001, NULL, "vwbn.s      S010, S400, 141"),
    TEST_ENTRY_C(0xD38E1001, NULL, "vwbn.s      S010, S400, 142"),
    TEST_ENTRY_C(0xD38F1001, NULL, "vwbn.s      S010, S400, 143"),
    TEST_ENTRY_C(0xD3901001, NULL, "vwbn.s      S010, S400, 144"),
    TEST_ENTRY_C(0xD3911001, NULL, "vwbn.s      S010, S400, 145"),
    TEST_ENTRY_C(0xD3921001, NULL, "vwbn.s      S010, S400, 146"),
    TEST_ENTRY_C(0xD3931001, NULL, "vwbn.s      S010, S400, 147"),
    TEST_ENTRY_C(0xD3941001, NULL, "vwbn.s      S010, S400, 148"),
    TEST_ENTRY_C(0xD3951001, NULL, "vwbn.s      S010, S400, 149"),
    TEST_ENTRY_C(0xD3961001, NULL, "vwbn.s      S010, S400, 150"),
    TEST_ENTRY_C(0xD3971001, NULL, "vwbn.s      S010, S400, 151"),
    TEST_ENTRY_C(0xD3981001, NULL, "vwbn.s      S010, S400, 152"),
    TEST_ENTRY_C(0xD3991001, NULL, "vwbn.s      S010, S400, 153"),
    TEST_ENTRY_C(0xD39A1001, NULL, "vwbn.s      S010, S400, 154"),
    TEST_ENTRY_C(0xD39B1001, NULL, "vwbn.s      S010, S400, 155"),
    TEST_ENTRY_C(0xD39C1001, NULL, "vwbn.s      S010, S400, 156"),
    TEST_ENTRY_C(0xD39D1001, NULL, "vwbn.s      S010, S400, 157"),
    TEST_ENTRY_C(0xD39E1001, NULL, "vwbn.s      S010, S400, 158"),
    TEST_ENTRY_C(0xD39F1001, NULL, "vwbn.s      S010, S400, 159"),
    TEST_ENTRY_C(0xD3A01001, NULL, "vwbn.s      S010, S400, 160"),
    TEST_ENTRY_C(0xD3A11001, NULL, "vwbn.s      S010, S400, 161"),
    TEST_ENTRY_C(0xD3A21001, NULL, "vwbn.s      S010, S400, 162"),
    TEST_ENTRY_C(0xD3A31001, NULL, "vwbn.s      S010, S400, 163"),
    TEST_ENTRY_C(0xD3A41001, NULL, "vwbn.s      S010, S400, 164"),
    TEST_ENTRY_C(0xD3A51001, NULL, "vwbn.s      S010, S400, 165"),
    TEST_ENTRY_C(0xD3A61001, NULL, "vwbn.s      S010, S400, 166"),
    TEST_ENTRY_C(0xD3A71001, NULL, "vwbn.s      S010, S400, 167"),
    TEST_ENTRY_C(0xD3A81001, NULL, "vwbn.s      S010, S400, 168"),
    TEST_ENTRY_C(0xD3A91001, NULL, "vwbn.s      S010, S400, 169"),
    TEST_ENTRY_C(0xD3AA1001, NULL, "vwbn.s      S010, S400, 170"),
    TEST_ENTRY_C(0xD3AB1001, NULL, "vwbn.s      S010, S400, 171"),
    TEST_ENTRY_C(0xD3AC1001, NULL, "vwbn.s      S010, S400, 172"),
    TEST_ENTRY_C(0xD3AD1001, NULL, "vwbn.s      S010, S400, 173"),
    TEST_ENTRY_C(0xD3AE1001, NULL, "vwbn.s      S010, S400, 174"),
    TEST_ENTRY_C(0xD3AF1001, NULL, "vwbn.s      S010, S400, 175"),
    TEST_ENTRY_C(0xD3B01001, NULL, "vwbn.s      S010, S400, 176"),
    TEST_ENTRY_C(0xD3B11001, NULL, "vwbn.s      S010, S400, 177"),
    TEST_ENTRY_C(0xD3B21001, NULL, "vwbn.s      S010, S400, 178"),
    TEST_ENTRY_C(0xD3B31001, NULL, "vwbn.s      S010, S400, 179"),
    TEST_ENTRY_C(0xD3B41001, NULL, "vwbn.s      S010, S400, 180"),
    TEST_ENTRY_C(0xD3B51001, NULL, "vwbn.s      S010, S400, 181"),
    TEST_ENTRY_C(0xD3B61001, NULL, "vwbn.s      S010, S400, 182"),
    TEST_ENTRY_C(0xD3B71001, NULL, "vwbn.s      S010, S400, 183"),
    TEST_ENTRY_C(0xD3B81001, NULL, "vwbn.s      S010, S400, 184"),
    TEST_ENTRY_C(0xD3B91001, NULL, "vwbn.s      S010, S400, 185"),
    TEST_ENTRY_C(0xD3BA1001, NULL, "vwbn.s      S010, S400, 186"),
    TEST_ENTRY_C(0xD3BB1001, NULL, "vwbn.s      S010, S400, 187"),
    TEST_ENTRY_C(0xD3BC1001, NULL, "vwbn.s      S010, S400, 188"),
    TEST_ENTRY_C(0xD3BD1001, NULL, "vwbn.s      S010, S400, 189"),
    TEST_ENTRY_C(0xD3BE1001, NULL, "vwbn.s      S010, S400, 190"),
    TEST_ENTRY_C(0xD3BF1001, NULL, "vwbn.s      S010, S400, 191"),
    TEST_ENTRY_C(0xD3C01001, NULL, "vwbn.s      S010, S400, 192"),
    TEST_ENTRY_C(0xD3C11001, NULL, "vwbn.s      S010, S400, 193"),
    TEST_ENTRY_C(0xD3C21001, NULL, "vwbn.s      S010, S400, 194"),
    TEST_ENTRY_C(0xD3C31001, NULL, "vwbn.s      S010, S400, 195"),
    TEST_ENTRY_C(0xD3C41001, NULL, "vwbn.s      S010, S400, 196"),
    TEST_ENTRY_C(0xD3C51001, NULL, "vwbn.s      S010, S400, 197"),
    TEST_ENTRY_C(0xD3C61001, NULL, "vwbn.s      S010, S400, 198"),
    TEST_ENTRY_C(0xD3C71001, NULL, "vwbn.s      S010, S400, 199"),
    TEST_ENTRY_C(0xD3C81001, NULL, "vwbn.s      S010, S400, 200"),
    TEST_ENTRY_C(0xD3C91001, NULL, "vwbn.s      S010, S400, 201"),
    TEST_ENTRY_C(0xD3CA1001, NULL, "vwbn.s      S010, S400, 202"),
    TEST_ENTRY_C(0xD3CB1001, NULL, "vwbn.s      S010, S400, 203"),
    TEST_ENTRY_C(0xD3CC1001, NULL, "vwbn.s      S010, S400, 204"),
    TEST_ENTRY_C(0xD3CD1001, NULL, "vwbn.s      S010, S400, 205"),
    TEST_ENTRY_C(0xD3CE1001, NULL, "vwbn.s      S010, S400, 206"),
    TEST_ENTRY_C(0xD3CF1001, NULL, "vwbn.s      S010, S400, 207"),
    TEST_ENTRY_C(0xD3D01001, NULL, "vwbn.s      S010, S400, 208"),
    TEST_ENTRY_C(0xD3D11001, NULL, "vwbn.s      S010, S400, 209"),
    TEST_ENTRY_C(0xD3D21001, NULL, "vwbn.s      S010, S400, 210"),
    TEST_ENTRY_C(0xD3D31001, NULL, "vwbn.s      S010, S400, 211"),
    TEST_ENTRY_C(0xD3D41001, NULL, "vwbn.s      S010, S400, 212"),
    TEST_ENTRY_C(0xD3D51001, NULL, "vwbn.s      S010, S400, 213"),
    TEST_ENTRY_C(0xD3D61001, NULL, "vwbn.s      S010, S400, 214"),
    TEST_ENTRY_C(0xD3D71001, NULL, "vwbn.s      S010, S400, 215"),
    TEST_ENTRY_C(0xD3D81001, NULL, "vwbn.s      S010, S400, 216"),
    TEST_ENTRY_C(0xD3D91001, NULL, "vwbn.s      S010, S400, 217"),
    TEST_ENTRY_C(0xD3DA1001, NULL, "vwbn.s      S010, S400, 218"),
    TEST_ENTRY_C(0xD3DB1001, NULL, "vwbn.s      S010, S400, 219"),
    TEST_ENTRY_C(0xD3DC1001, NULL, "vwbn.s      S010, S400, 220"),
    TEST_ENTRY_C(0xD3DD1001, NULL, "vwbn.s      S010, S400, 221"),
    TEST_ENTRY_C(0xD3DE1001, NULL, "vwbn.s      S010, S400, 222"),
    TEST_ENTRY_C(0xD3DF1001, NULL, "vwbn.s      S010, S400, 223"),
    TEST_ENTRY_C(0xD3E01001, NULL, "vwbn.s      S010, S400, 224"),
    TEST_ENTRY_C(0xD3E11001, NULL, "vwbn.s      S010, S400, 225"),
    TEST_ENTRY_C(0xD3E21001, NULL, "vwbn.s      S010, S400, 226"),
    TEST_ENTRY_C(0xD3E31001, NULL, "vwbn.s      S010, S400, 227"),
    TEST_ENTRY_C(0xD3E41001, NULL, "vwbn.s      S010, S400, 228"),
    TEST_ENTRY_C(0xD3E51001, NULL, "vwbn.s      S010, S400, 229"),
    TEST_ENTRY_C(0xD3E61001, NULL, "vwbn.s      S010, S400, 230"),
    TEST_ENTRY_C(0xD3E71001, NULL, "vwbn.s      S010, S400, 231"),
    TEST_ENTRY_C(0xD3E81001, NULL, "vwbn.s      S010, S400, 232"),
    TEST_ENTRY_C(0xD3E91001, NULL, "vwbn.s      S010, S400, 233"),
    TEST_ENTRY_C(0xD3EA1001, NULL, "vwbn.s      S010, S400, 234"),
    TEST_ENTRY_C(0xD3EB1001, NULL, "vwbn.s      S010, S400, 235"),
    TEST_ENTRY_C(0xD3EC1001, NULL, "vwbn.s      S010, S400, 236"),
    TEST_ENTRY_C(0xD3ED1001, NULL, "vwbn.s      S010, S400, 237"),
    TEST_ENTRY_C(0xD3EE1001, NULL, "vwbn.s      S010, S400, 238"),
    TEST_ENTRY_C(0xD3EF1001, NULL, "vwbn.s      S010, S400, 239"),
    TEST_ENTRY_C(0xD3F01001, NULL, "vwbn.s      S010, S400, 240"),
    TEST_ENTRY_C(0xD3F11001, NULL, "vwbn.s      S010, S400, 241"),
    TEST_ENTRY_C(0xD3F21001, NULL, "vwbn.s      S010, S400, 242"),
    TEST_ENTRY_C(0xD3F31001, NULL, "vwbn.s      S010, S400, 243"),
    TEST_ENTRY_C(0xD3F41001, NULL, "vwbn.s      S010, S400, 244"),
    TEST_ENTRY_C(0xD3F51001, NULL, "vwbn.s      S010, S400, 245"),
    TEST_ENTRY_C(0xD3F61001, NULL, "vwbn.s      S010, S400, 246"),
    TEST_ENTRY_C(0xD3F71001, NULL, "vwbn.s      S010, S400, 247"),
    TEST_ENTRY_C(0xD3F81001, NULL, "vwbn.s      S010, S400, 248"),
    TEST_ENTRY_C(0xD3F91001, NULL, "vwbn.s      S010, S400, 249"),
    TEST_ENTRY_C(0xD3FA1001, NULL, "vwbn.s      S010, S400, 250"),
    TEST_ENTRY_C(0xD3FB1001, NULL, "vwbn.s      S010, S400, 251"),
    TEST_ENTRY_C(0xD3FC1001, NULL, "vwbn.s      S010, S400, 252"),
    TEST_ENTRY_C(0xD3FD1001, NULL, "vwbn.s      S010, S400, 253"),
    TEST_ENTRY_C(0xD3FE1001, NULL, "vwbn.s      S010, S400, 254"),
    TEST_ENTRY_C(0xD3FF1001, NULL, "vwbn.s      S010, S400, 255"),

    /* VFPU5 */
    TEST_ENTRY_C(0xDC000000, NULL, "vpfxs       X, X, X, X"),
    TEST_ENTRY_C(0xDD000000, NULL, "vpfxt       X, X, X, X"),
    TEST_ENTRY_C(0xDE000000, NULL, "vpfxd       , , , "),
    TEST_ENTRY_C(0xDF400000, NULL, "viim.s      S002, 0"),
    TEST_ENTRY_C(0xDF400001, NULL, "viim.s      S002, 1"),
    TEST_ENTRY_C(0xDF400010, NULL, "viim.s      S002, 16"),
    TEST_ENTRY_C(0xDF400100, NULL, "viim.s      S002, 256"),
    TEST_ENTRY_C(0xDF401000, NULL, "viim.s      S002, 4096"),
    TEST_ENTRY_C(0xDF408000, NULL, "viim.s      S002, -32768"),
    TEST_ENTRY_C(0xDF40F000, NULL, "viim.s      S002, -4096"),
    TEST_ENTRY_C(0xDFC00000, NULL, "vfim.s      S002, 0"),
    TEST_ENTRY_C(0xDFC08000, NULL, "vfim.s      S002, -0"),
    TEST_ENTRY_C(0xDFC01000, NULL, "vfim.s      S002, 0.00048828125"),
    TEST_ENTRY_C(0xDFC0F000, NULL, "vfim.s      S002, -8192"),
    TEST_ENTRY_C(0xDFC05450, NULL, "vfim.s      S002, 69"),
    TEST_ENTRY_C(0xDFC0D450, NULL, "vfim.s      S002, -69"),
    TEST_ENTRY_C(0xDFC03C00, NULL, "vfim.s      S002, 1"),
    TEST_ENTRY_C(0xDFC0BC00, NULL, "vfim.s      S002, -1"),
    TEST_ENTRY_C(0xDFC07C00, NULL, "vfim.s      S002, inf"),
    TEST_ENTRY_C(0xDFC0FC00, NULL, "vfim.s      S002, -inf"),
    TEST_ENTRY_C(0xDFC07BFF, NULL, "vfim.s      S002, 65504"),
    TEST_ENTRY_C(0xDFC0FBFF, NULL, "vfim.s      S002, -65504"),
    TEST_ENTRY_C(0xDFC07EAA, NULL, "vfim.s      S002, nan"),
    TEST_ENTRY_C(0xDFC0FEAA, NULL, "vfim.s      S002, -nan"),

    /* VFPU6 */
    TEST_ENTRY_C(0xF0001040, NULL, ".word       0xF0001040                   # INVALID     $zero, $zero, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF00010C0, NULL, "vmmul.p     M002, E400, M000"),
    TEST_ENTRY_C(0xF0009004, NULL, "vmmul.t     M100, E400, M000"),
    TEST_ENTRY_C(0xF00090C0, NULL, "vmmul.q     M002, E400, M000"),
    TEST_ENTRY_C(0xF0800440, NULL, "vhtfm2.p    C002, M100, C000"),
    TEST_ENTRY_C(0xF08010C0, NULL, "vtfm2.p     C002, M400, C000"),
    TEST_ENTRY_C(0xF0801084, NULL, "vtfm2.p     C100, M400, C000"),
    TEST_ENTRY_C(0xF1008004, NULL, "vtfm3.t     C100, M000, C000"),
    TEST_ENTRY_C(0xF1001084, NULL, "vhtfm3.t    C100, M400, C000"),
    TEST_ENTRY_C(0xF1009004, NULL, "vtfm3.t     C100, M400, C000"),
    TEST_ENTRY_C(0xF1808088, NULL, "vtfm4.q     C200, M000, C000"),
    TEST_ENTRY_C(0xF1809040, NULL, "vhtfm4.q    C002, M400, C000"),
    TEST_ENTRY_C(0xF18090C0, NULL, "vtfm4.q     C002, M400, C000"),
    TEST_ENTRY_C(0xF2001040, NULL, ".word       0xF2001040                   # INVALID     $s0, $zero, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF20010C0, NULL, "vmscl.p     M002, M400, S000"),
    TEST_ENTRY_C(0xF2009040, NULL, "vmscl.t     M001, M400, S000"),
    TEST_ENTRY_C(0xF20090C0, NULL, "vmscl.q     M002, M400, S000"),
    TEST_ENTRY_C(0xF2801040, NULL, ".word       0xF2801040                   # INVALID     $s4, $zero, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF28010C0, NULL, ".word       0xF28010C0                   # INVALID     $s4, $zero, 0x10C0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF2809004, NULL, "vcrsp.t     C100, C400, C000"),
    TEST_ENTRY_C(0xF28090C0, NULL, "vqmul.q     C002, C400, C000"),
    TEST_ENTRY_C(0xF3001040, NULL, ".word       0xF3001040                   # INVALID     $t8, $zero, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF30010C0, NULL, ".word       0xF30010C0                   # INVALID     $t8, $zero, 0x10C0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF3009040, NULL, ".word       0xF3009040                   # INVALID     $t8, $zero, -0x6FC0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF30090C0, NULL, ".word       0xF30090C0                   # INVALID     $t8, $zero, -0x6F40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF3801040, NULL, ".word       0xF3801040                   # INVALID     $gp, $zero, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF38010C0, NULL, "vmmov.p     M002, M400"),
    TEST_ENTRY_C(0xF3809040, NULL, "vmmov.t     M001, M400"),
    TEST_ENTRY_C(0xF38090C0, NULL, "vmmov.q     M002, M400"),
    TEST_ENTRY_C(0xF3811040, NULL, ".word       0xF3811040                   # INVALID     $gp, $at, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF38110C0, NULL, ".word       0xF38110C0                   # INVALID     $gp, $at, 0x10C0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF3819040, NULL, ".word       0xF3819040                   # INVALID     $gp, $at, -0x6FC0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF38190C0, NULL, ".word       0xF38190C0                   # INVALID     $gp, $at, -0x6F40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF3821040, NULL, ".word       0xF3821040                   # INVALID     $gp, $v0, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF38210C0, NULL, ".word       0xF38210C0                   # INVALID     $gp, $v0, 0x10C0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF3829040, NULL, ".word       0xF3829040                   # INVALID     $gp, $v0, -0x6FC0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF38290C0, NULL, ".word       0xF38290C0                   # INVALID     $gp, $v0, -0x6F40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF3831040, NULL, ".word       0xF3831040                   # INVALID     $gp, $v1, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF38300C0, NULL, "vmidt.p     M002"),
    TEST_ENTRY_C(0xF3838040, NULL, "vmidt.t     M001"),
    TEST_ENTRY_C(0xF38380C0, NULL, "vmidt.q     M002"),
    TEST_ENTRY_C(0xF3841040, NULL, ".word       0xF3841040                   # INVALID     $gp, $a0, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF38410C0, NULL, ".word       0xF38410C0                   # INVALID     $gp, $a0, 0x10C0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF3849040, NULL, ".word       0xF3849040                   # INVALID     $gp, $a0, -0x6FC0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF38490C0, NULL, ".word       0xF38490C0                   # INVALID     $gp, $a0, -0x6F40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF3851040, NULL, ".word       0xF3851040                   # INVALID     $gp, $a1, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF38510C0, NULL, ".word       0xF38510C0                   # INVALID     $gp, $a1, 0x10C0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF3859040, NULL, ".word       0xF3859040                   # INVALID     $gp, $a1, -0x6FC0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF38590C0, NULL, ".word       0xF38590C0                   # INVALID     $gp, $a1, -0x6F40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF3861040, NULL, ".word       0xF3861040                   # INVALID     $gp, $a2, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF38610C0, NULL, ".word       0xF38610C0                   # vmzero.p    M002 # 00001000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF3869040, NULL, ".word       0xF3869040                   # vmzero.t    M001 # 00001000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF38690C0, NULL, ".word       0xF38690C0                   # vmzero.q    M002 # 00001000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF3871040, NULL, ".word       0xF3871040                   # INVALID     $gp, $a3, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF38710C0, NULL, ".word       0xF38710C0                   # vmone.p     M002 # 00001000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF3879040, NULL, ".word       0xF3879040                   # vmone.t     M001 # 00001000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF38790C0, NULL, ".word       0xF38790C0                   # vmone.q     M002 # 00001000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7_FMT0>"),
    TEST_ENTRY_C(0xF3A01040, NULL, ".word       0xF3A01040                   # INVALID     $sp, $zero, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7>"),
    TEST_ENTRY_C(0xF3A010C0, NULL, "vrot.p      C002, S400, [C,S]"),
    TEST_ENTRY_C(0xF3A09040, NULL, "vrot.t      C001, S400, [C,S,S]"),
    TEST_ENTRY_C(0xF3A090C0, NULL, "vrot.q      C002, S400, [C,S,S,S]"),
    TEST_ENTRY_C(0xF3C01040, NULL, ".word       0xF3C01040                   # INVALID     $fp, $zero, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7>"),
    TEST_ENTRY_C(0xF3C010C0, NULL, ".word       0xF3C010C0                   # INVALID     $fp, $zero, 0x10C0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7>"),
    TEST_ENTRY_C(0xF3C09040, NULL, ".word       0xF3C09040                   # INVALID     $fp, $zero, -0x6FC0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7>"),
    TEST_ENTRY_C(0xF3C090C0, NULL, ".word       0xF3C090C0                   # INVALID     $fp, $zero, -0x6F40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7>"),
    TEST_ENTRY_C(0xF3E01040, NULL, ".word       0xF3E01040                   # INVALID     $ra, $zero, 0x1040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7>"),
    TEST_ENTRY_C(0xF3E010C0, NULL, ".word       0xF3E010C0                   # INVALID     $ra, $zero, 0x10C0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7>"),
    TEST_ENTRY_C(0xF3E09040, NULL, ".word       0xF3E09040                   # INVALID     $ra, $zero, -0x6FC0 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7>"),
    TEST_ENTRY_C(0xF3E090C0, NULL, ".word       0xF3E090C0                   # INVALID     $ra, $zero, -0x6F40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6_FMT7>"),

    /* VFPU7 */
    TEST_ENTRY_C(0xFFFF040D, NULL, "vflush"),
    TEST_ENTRY_C(0xFFFF0000, NULL, "vnop"),
    TEST_ENTRY_C(0xFFFF0320, NULL, "vsync"),

    /* vmmul */
    TEST_ENTRY_C(0xF0000040, NULL, ".word       0xF0000040                   # INVALID     $zero, $zero, 0x40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0000140, NULL, ".word       0xF0000140                   # INVALID     $zero, $zero, 0x140 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0000240, NULL, ".word       0xF0000240                   # INVALID     $zero, $zero, 0x240 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0000340, NULL, ".word       0xF0000340                   # INVALID     $zero, $zero, 0x340 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0000440, NULL, ".word       0xF0000440                   # INVALID     $zero, $zero, 0x440 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0000540, NULL, ".word       0xF0000540                   # INVALID     $zero, $zero, 0x540 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0000640, NULL, ".word       0xF0000640                   # INVALID     $zero, $zero, 0x640 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0000740, NULL, ".word       0xF0000740                   # INVALID     $zero, $zero, 0x740 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0000840, NULL, ".word       0xF0000840                   # INVALID     $zero, $zero, 0x840 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0000940, NULL, ".word       0xF0000940                   # INVALID     $zero, $zero, 0x940 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0000A40, NULL, ".word       0xF0000A40                   # INVALID     $zero, $zero, 0xA40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0000B40, NULL, ".word       0xF0000B40                   # INVALID     $zero, $zero, 0xB40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0000C40, NULL, ".word       0xF0000C40                   # INVALID     $zero, $zero, 0xC40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0000D40, NULL, ".word       0xF0000D40                   # INVALID     $zero, $zero, 0xD40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0000E40, NULL, ".word       0xF0000E40                   # INVALID     $zero, $zero, 0xE40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0000F40, NULL, ".word       0xF0000F40                   # INVALID     $zero, $zero, 0xF40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0001140, NULL, ".word       0xF0001140                   # INVALID     $zero, $zero, 0x1140 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0001240, NULL, ".word       0xF0001240                   # INVALID     $zero, $zero, 0x1240 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0001340, NULL, ".word       0xF0001340                   # INVALID     $zero, $zero, 0x1340 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0001440, NULL, ".word       0xF0001440                   # INVALID     $zero, $zero, 0x1440 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0001540, NULL, ".word       0xF0001540                   # INVALID     $zero, $zero, 0x1540 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0001640, NULL, ".word       0xF0001640                   # INVALID     $zero, $zero, 0x1640 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0001740, NULL, ".word       0xF0001740                   # INVALID     $zero, $zero, 0x1740 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0001840, NULL, ".word       0xF0001840                   # INVALID     $zero, $zero, 0x1840 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0001940, NULL, ".word       0xF0001940                   # INVALID     $zero, $zero, 0x1940 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0001A40, NULL, ".word       0xF0001A40                   # INVALID     $zero, $zero, 0x1A40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0001B40, NULL, ".word       0xF0001B40                   # INVALID     $zero, $zero, 0x1B40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0001C40, NULL, ".word       0xF0001C40                   # INVALID     $zero, $zero, 0x1C40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0001D40, NULL, ".word       0xF0001D40                   # INVALID     $zero, $zero, 0x1D40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0001E40, NULL, ".word       0xF0001E40                   # INVALID     $zero, $zero, 0x1E40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0001F40, NULL, ".word       0xF0001F40                   # INVALID     $zero, $zero, 0x1F40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0002040, NULL, ".word       0xF0002040                   # INVALID     $zero, $zero, 0x2040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0002140, NULL, ".word       0xF0002140                   # INVALID     $zero, $zero, 0x2140 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0002240, NULL, ".word       0xF0002240                   # INVALID     $zero, $zero, 0x2240 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0002340, NULL, ".word       0xF0002340                   # INVALID     $zero, $zero, 0x2340 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0002440, NULL, ".word       0xF0002440                   # INVALID     $zero, $zero, 0x2440 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0002540, NULL, ".word       0xF0002540                   # INVALID     $zero, $zero, 0x2540 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0002640, NULL, ".word       0xF0002640                   # INVALID     $zero, $zero, 0x2640 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0002740, NULL, ".word       0xF0002740                   # INVALID     $zero, $zero, 0x2740 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0002840, NULL, ".word       0xF0002840                   # INVALID     $zero, $zero, 0x2840 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0002940, NULL, ".word       0xF0002940                   # INVALID     $zero, $zero, 0x2940 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0002A40, NULL, ".word       0xF0002A40                   # INVALID     $zero, $zero, 0x2A40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0002B40, NULL, ".word       0xF0002B40                   # INVALID     $zero, $zero, 0x2B40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0002C40, NULL, ".word       0xF0002C40                   # INVALID     $zero, $zero, 0x2C40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0002D40, NULL, ".word       0xF0002D40                   # INVALID     $zero, $zero, 0x2D40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0002E40, NULL, ".word       0xF0002E40                   # INVALID     $zero, $zero, 0x2E40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0002F40, NULL, ".word       0xF0002F40                   # INVALID     $zero, $zero, 0x2F40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0003040, NULL, ".word       0xF0003040                   # INVALID     $zero, $zero, 0x3040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0003140, NULL, ".word       0xF0003140                   # INVALID     $zero, $zero, 0x3140 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0003240, NULL, ".word       0xF0003240                   # INVALID     $zero, $zero, 0x3240 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0003340, NULL, ".word       0xF0003340                   # INVALID     $zero, $zero, 0x3340 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0003440, NULL, ".word       0xF0003440                   # INVALID     $zero, $zero, 0x3440 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0003540, NULL, ".word       0xF0003540                   # INVALID     $zero, $zero, 0x3540 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0003640, NULL, ".word       0xF0003640                   # INVALID     $zero, $zero, 0x3640 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0003740, NULL, ".word       0xF0003740                   # INVALID     $zero, $zero, 0x3740 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0003840, NULL, ".word       0xF0003840                   # INVALID     $zero, $zero, 0x3840 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0003940, NULL, ".word       0xF0003940                   # INVALID     $zero, $zero, 0x3940 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0003A40, NULL, ".word       0xF0003A40                   # INVALID     $zero, $zero, 0x3A40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0003B40, NULL, ".word       0xF0003B40                   # INVALID     $zero, $zero, 0x3B40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0003C40, NULL, ".word       0xF0003C40                   # INVALID     $zero, $zero, 0x3C40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0003D40, NULL, ".word       0xF0003D40                   # INVALID     $zero, $zero, 0x3D40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0003E40, NULL, ".word       0xF0003E40                   # INVALID     $zero, $zero, 0x3E40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0003F40, NULL, ".word       0xF0003F40                   # INVALID     $zero, $zero, 0x3F40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0004040, NULL, ".word       0xF0004040                   # INVALID     $zero, $zero, 0x4040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0004140, NULL, ".word       0xF0004140                   # INVALID     $zero, $zero, 0x4140 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0004240, NULL, ".word       0xF0004240                   # INVALID     $zero, $zero, 0x4240 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0004340, NULL, ".word       0xF0004340                   # INVALID     $zero, $zero, 0x4340 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0004440, NULL, ".word       0xF0004440                   # INVALID     $zero, $zero, 0x4440 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0004540, NULL, ".word       0xF0004540                   # INVALID     $zero, $zero, 0x4540 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0004640, NULL, ".word       0xF0004640                   # INVALID     $zero, $zero, 0x4640 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0004740, NULL, ".word       0xF0004740                   # INVALID     $zero, $zero, 0x4740 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0004840, NULL, ".word       0xF0004840                   # INVALID     $zero, $zero, 0x4840 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0004940, NULL, ".word       0xF0004940                   # INVALID     $zero, $zero, 0x4940 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0004A40, NULL, ".word       0xF0004A40                   # INVALID     $zero, $zero, 0x4A40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0004B40, NULL, ".word       0xF0004B40                   # INVALID     $zero, $zero, 0x4B40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0004C40, NULL, ".word       0xF0004C40                   # INVALID     $zero, $zero, 0x4C40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0004D40, NULL, ".word       0xF0004D40                   # INVALID     $zero, $zero, 0x4D40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0004E40, NULL, ".word       0xF0004E40                   # INVALID     $zero, $zero, 0x4E40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0004F40, NULL, ".word       0xF0004F40                   # INVALID     $zero, $zero, 0x4F40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0005040, NULL, ".word       0xF0005040                   # INVALID     $zero, $zero, 0x5040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0005140, NULL, ".word       0xF0005140                   # INVALID     $zero, $zero, 0x5140 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0005240, NULL, ".word       0xF0005240                   # INVALID     $zero, $zero, 0x5240 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0005340, NULL, ".word       0xF0005340                   # INVALID     $zero, $zero, 0x5340 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0005440, NULL, ".word       0xF0005440                   # INVALID     $zero, $zero, 0x5440 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0005540, NULL, ".word       0xF0005540                   # INVALID     $zero, $zero, 0x5540 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0005640, NULL, ".word       0xF0005640                   # INVALID     $zero, $zero, 0x5640 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0005740, NULL, ".word       0xF0005740                   # INVALID     $zero, $zero, 0x5740 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0005840, NULL, ".word       0xF0005840                   # INVALID     $zero, $zero, 0x5840 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0005940, NULL, ".word       0xF0005940                   # INVALID     $zero, $zero, 0x5940 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0005A40, NULL, ".word       0xF0005A40                   # INVALID     $zero, $zero, 0x5A40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0005B40, NULL, ".word       0xF0005B40                   # INVALID     $zero, $zero, 0x5B40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0005C40, NULL, ".word       0xF0005C40                   # INVALID     $zero, $zero, 0x5C40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0005D40, NULL, ".word       0xF0005D40                   # INVALID     $zero, $zero, 0x5D40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0005E40, NULL, ".word       0xF0005E40                   # INVALID     $zero, $zero, 0x5E40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0005F40, NULL, ".word       0xF0005F40                   # INVALID     $zero, $zero, 0x5F40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0006040, NULL, ".word       0xF0006040                   # INVALID     $zero, $zero, 0x6040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0006140, NULL, ".word       0xF0006140                   # INVALID     $zero, $zero, 0x6140 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0006240, NULL, ".word       0xF0006240                   # INVALID     $zero, $zero, 0x6240 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0006340, NULL, ".word       0xF0006340                   # INVALID     $zero, $zero, 0x6340 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0006440, NULL, ".word       0xF0006440                   # INVALID     $zero, $zero, 0x6440 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0006540, NULL, ".word       0xF0006540                   # INVALID     $zero, $zero, 0x6540 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0006640, NULL, ".word       0xF0006640                   # INVALID     $zero, $zero, 0x6640 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0006740, NULL, ".word       0xF0006740                   # INVALID     $zero, $zero, 0x6740 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0006840, NULL, ".word       0xF0006840                   # INVALID     $zero, $zero, 0x6840 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0006940, NULL, ".word       0xF0006940                   # INVALID     $zero, $zero, 0x6940 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0006A40, NULL, ".word       0xF0006A40                   # INVALID     $zero, $zero, 0x6A40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0006B40, NULL, ".word       0xF0006B40                   # INVALID     $zero, $zero, 0x6B40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0006C40, NULL, ".word       0xF0006C40                   # INVALID     $zero, $zero, 0x6C40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0006D40, NULL, ".word       0xF0006D40                   # INVALID     $zero, $zero, 0x6D40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0006E40, NULL, ".word       0xF0006E40                   # INVALID     $zero, $zero, 0x6E40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0006F40, NULL, ".word       0xF0006F40                   # INVALID     $zero, $zero, 0x6F40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0007040, NULL, ".word       0xF0007040                   # INVALID     $zero, $zero, 0x7040 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0007140, NULL, ".word       0xF0007140                   # INVALID     $zero, $zero, 0x7140 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0007240, NULL, ".word       0xF0007240                   # INVALID     $zero, $zero, 0x7240 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0007340, NULL, ".word       0xF0007340                   # INVALID     $zero, $zero, 0x7340 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0007440, NULL, ".word       0xF0007440                   # INVALID     $zero, $zero, 0x7440 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0007540, NULL, ".word       0xF0007540                   # INVALID     $zero, $zero, 0x7540 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0007640, NULL, ".word       0xF0007640                   # INVALID     $zero, $zero, 0x7640 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0007740, NULL, ".word       0xF0007740                   # INVALID     $zero, $zero, 0x7740 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0007840, NULL, ".word       0xF0007840                   # INVALID     $zero, $zero, 0x7840 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0007940, NULL, ".word       0xF0007940                   # INVALID     $zero, $zero, 0x7940 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0007A40, NULL, ".word       0xF0007A40                   # INVALID     $zero, $zero, 0x7A40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0007B40, NULL, ".word       0xF0007B40                   # INVALID     $zero, $zero, 0x7B40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0007C40, NULL, ".word       0xF0007C40                   # INVALID     $zero, $zero, 0x7C40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0007D40, NULL, ".word       0xF0007D40                   # INVALID     $zero, $zero, 0x7D40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0007E40, NULL, ".word       0xF0007E40                   # INVALID     $zero, $zero, 0x7E40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF0007F40, NULL, ".word       0xF0007F40                   # INVALID     $zero, $zero, 0x7F40 # 00000000 <InstrIdType: R4000ALLEGREX_VFPU6>"),
    TEST_ENTRY_C(0xF004009F, NULL, "vmmul.p     M730, E000, M100"),
    TEST_ENTRY_C(0xF004019F, NULL, "vmmul.p     M730, E001, M100"),
    TEST_ENTRY_C(0xF004029F, NULL, "vmmul.p     M730, E002, M100"),
    TEST_ENTRY_C(0xF004039F, NULL, "vmmul.p     M730, E003, M100"),
    TEST_ENTRY_C(0xF004049F, NULL, "vmmul.p     M730, E100, M100"),
    TEST_ENTRY_C(0xF004059F, NULL, "vmmul.p     M730, E101, M100"),
    TEST_ENTRY_C(0xF004069F, NULL, "vmmul.p     M730, E102, M100"),
    TEST_ENTRY_C(0xF004079F, NULL, "vmmul.p     M730, E103, M100"),
    TEST_ENTRY_C(0xF004089F, NULL, "vmmul.p     M730, E200, M100"),
    TEST_ENTRY_C(0xF004099F, NULL, "vmmul.p     M730, E201, M100"),
    TEST_ENTRY_C(0xF0040A9F, NULL, "vmmul.p     M730, E202, M100"),
    TEST_ENTRY_C(0xF0040B9F, NULL, "vmmul.p     M730, E203, M100"),
    TEST_ENTRY_C(0xF0040C9F, NULL, "vmmul.p     M730, E300, M100"),
    TEST_ENTRY_C(0xF0040D9F, NULL, "vmmul.p     M730, E301, M100"),
    TEST_ENTRY_C(0xF0040E9F, NULL, "vmmul.p     M730, E302, M100"),
    TEST_ENTRY_C(0xF0040F9F, NULL, "vmmul.p     M730, E303, M100"),
    TEST_ENTRY_C(0xF004109F, NULL, "vmmul.p     M730, E400, M100"),
    TEST_ENTRY_C(0xF004119F, NULL, "vmmul.p     M730, E401, M100"),
    TEST_ENTRY_C(0xF004129F, NULL, "vmmul.p     M730, E402, M100"),
    TEST_ENTRY_C(0xF004139F, NULL, "vmmul.p     M730, E403, M100"),
    TEST_ENTRY_C(0xF004149F, NULL, "vmmul.p     M730, E500, M100"),
    TEST_ENTRY_C(0xF004159F, NULL, "vmmul.p     M730, E501, M100"),
    TEST_ENTRY_C(0xF004169F, NULL, "vmmul.p     M730, E502, M100"),
    TEST_ENTRY_C(0xF004179F, NULL, "vmmul.p     M730, E503, M100"),
    TEST_ENTRY_C(0xF004189F, NULL, "vmmul.p     M730, E600, M100"),
    TEST_ENTRY_C(0xF004199F, NULL, "vmmul.p     M730, E601, M100"),
    TEST_ENTRY_C(0xF0041A9F, NULL, "vmmul.p     M730, E602, M100"),
    TEST_ENTRY_C(0xF0041B9F, NULL, "vmmul.p     M730, E603, M100"),
    TEST_ENTRY_C(0xF0041C88, NULL, "vmmul.p     M200, E700, M100"),
    TEST_ENTRY_C(0xF0041D88, NULL, "vmmul.p     M200, E701, M100"),
    TEST_ENTRY_C(0xF0041E88, NULL, "vmmul.p     M200, E702, M100"),
    TEST_ENTRY_C(0xF0041F88, NULL, "vmmul.p     M200, E703, M100"),
    TEST_ENTRY_C(0xF004209F, NULL, "vmmul.p     M730, M000, M100"),
    TEST_ENTRY_C(0xF004219F, NULL, "vmmul.p     M730, M010, M100"),
    TEST_ENTRY_C(0xF004229F, NULL, "vmmul.p     M730, M020, M100"),
    TEST_ENTRY_C(0xF004239F, NULL, "vmmul.p     M730, M030, M100"),
    TEST_ENTRY_C(0xF004249F, NULL, "vmmul.p     M730, M100, M100"),
    TEST_ENTRY_C(0xF004259F, NULL, "vmmul.p     M730, M110, M100"),
    TEST_ENTRY_C(0xF004269F, NULL, "vmmul.p     M730, M120, M100"),
    TEST_ENTRY_C(0xF004279F, NULL, "vmmul.p     M730, M130, M100"),
    TEST_ENTRY_C(0xF004289F, NULL, "vmmul.p     M730, M200, M100"),
    TEST_ENTRY_C(0xF004299F, NULL, "vmmul.p     M730, M210, M100"),
    TEST_ENTRY_C(0xF0042A9F, NULL, "vmmul.p     M730, M220, M100"),
    TEST_ENTRY_C(0xF0042B9F, NULL, "vmmul.p     M730, M230, M100"),
    TEST_ENTRY_C(0xF0042C9F, NULL, "vmmul.p     M730, M300, M100"),
    TEST_ENTRY_C(0xF0042D9F, NULL, "vmmul.p     M730, M310, M100"),
    TEST_ENTRY_C(0xF0042E9F, NULL, "vmmul.p     M730, M320, M100"),
    TEST_ENTRY_C(0xF0042F9F, NULL, "vmmul.p     M730, M330, M100"),
    TEST_ENTRY_C(0xF004309F, NULL, "vmmul.p     M730, M400, M100"),
    TEST_ENTRY_C(0xF004319F, NULL, "vmmul.p     M730, M410, M100"),
    TEST_ENTRY_C(0xF004329F, NULL, "vmmul.p     M730, M420, M100"),
    TEST_ENTRY_C(0xF004339F, NULL, "vmmul.p     M730, M430, M100"),
    TEST_ENTRY_C(0xF004349F, NULL, "vmmul.p     M730, M500, M100"),
    TEST_ENTRY_C(0xF004359F, NULL, "vmmul.p     M730, M510, M100"),
    TEST_ENTRY_C(0xF004369F, NULL, "vmmul.p     M730, M520, M100"),
    TEST_ENTRY_C(0xF004379F, NULL, "vmmul.p     M730, M530, M100"),
    TEST_ENTRY_C(0xF004389F, NULL, "vmmul.p     M730, M600, M100"),
    TEST_ENTRY_C(0xF004399F, NULL, "vmmul.p     M730, M610, M100"),
    TEST_ENTRY_C(0xF0043A9F, NULL, "vmmul.p     M730, M620, M100"),
    TEST_ENTRY_C(0xF0043B9F, NULL, "vmmul.p     M730, M630, M100"),
    TEST_ENTRY_C(0xF0043C88, NULL, "vmmul.p     M200, M700, M100"),
    TEST_ENTRY_C(0xF0043D88, NULL, "vmmul.p     M200, M710, M100"),
    TEST_ENTRY_C(0xF0043E88, NULL, "vmmul.p     M200, M720, M100"),
    TEST_ENTRY_C(0xF0043F88, NULL, "vmmul.p     M200, M730, M100"),
    TEST_ENTRY_C(0xF004409F, NULL, "vmmul.p     M730, E020, M100"),
    TEST_ENTRY_C(0xF004419F, NULL, "vmmul.p     M730, E021, M100"),
    TEST_ENTRY_C(0xF004429F, NULL, "vmmul.p     M730, E022, M100"),
    TEST_ENTRY_C(0xF004439F, NULL, "vmmul.p     M730, E023, M100"),
    TEST_ENTRY_C(0xF004449F, NULL, "vmmul.p     M730, E120, M100"),
    TEST_ENTRY_C(0xF004459F, NULL, "vmmul.p     M730, E121, M100"),
    TEST_ENTRY_C(0xF004469F, NULL, "vmmul.p     M730, E122, M100"),
    TEST_ENTRY_C(0xF004479F, NULL, "vmmul.p     M730, E123, M100"),
    TEST_ENTRY_C(0xF004489F, NULL, "vmmul.p     M730, E220, M100"),
    TEST_ENTRY_C(0xF004499F, NULL, "vmmul.p     M730, E221, M100"),
    TEST_ENTRY_C(0xF0044A9F, NULL, "vmmul.p     M730, E222, M100"),
    TEST_ENTRY_C(0xF0044B9F, NULL, "vmmul.p     M730, E223, M100"),
    TEST_ENTRY_C(0xF0044C9F, NULL, "vmmul.p     M730, E320, M100"),
    TEST_ENTRY_C(0xF0044D9F, NULL, "vmmul.p     M730, E321, M100"),
    TEST_ENTRY_C(0xF0044E9F, NULL, "vmmul.p     M730, E322, M100"),
    TEST_ENTRY_C(0xF0044F9F, NULL, "vmmul.p     M730, E323, M100"),
    TEST_ENTRY_C(0xF004509F, NULL, "vmmul.p     M730, E420, M100"),
    TEST_ENTRY_C(0xF004519F, NULL, "vmmul.p     M730, E421, M100"),
    TEST_ENTRY_C(0xF004529F, NULL, "vmmul.p     M730, E422, M100"),
    TEST_ENTRY_C(0xF004539F, NULL, "vmmul.p     M730, E423, M100"),
    TEST_ENTRY_C(0xF004549F, NULL, "vmmul.p     M730, E520, M100"),
    TEST_ENTRY_C(0xF004559F, NULL, "vmmul.p     M730, E521, M100"),
    TEST_ENTRY_C(0xF004569F, NULL, "vmmul.p     M730, E522, M100"),
    TEST_ENTRY_C(0xF004579F, NULL, "vmmul.p     M730, E523, M100"),
    TEST_ENTRY_C(0xF004589F, NULL, "vmmul.p     M730, E620, M100"),
    TEST_ENTRY_C(0xF004599F, NULL, "vmmul.p     M730, E621, M100"),
    TEST_ENTRY_C(0xF0045A9F, NULL, "vmmul.p     M730, E622, M100"),
    TEST_ENTRY_C(0xF0045B9F, NULL, "vmmul.p     M730, E623, M100"),
    TEST_ENTRY_C(0xF0045C88, NULL, "vmmul.p     M200, E720, M100"),
    TEST_ENTRY_C(0xF0045D88, NULL, "vmmul.p     M200, E721, M100"),
    TEST_ENTRY_C(0xF0045E88, NULL, "vmmul.p     M200, E722, M100"),
    TEST_ENTRY_C(0xF0045F88, NULL, "vmmul.p     M200, E723, M100"),
    TEST_ENTRY_C(0xF004609F, NULL, "vmmul.p     M730, M002, M100"),
    TEST_ENTRY_C(0xF004619F, NULL, "vmmul.p     M730, M012, M100"),
    TEST_ENTRY_C(0xF004629F, NULL, "vmmul.p     M730, M022, M100"),
    TEST_ENTRY_C(0xF004639F, NULL, "vmmul.p     M730, M032, M100"),
    TEST_ENTRY_C(0xF004649F, NULL, "vmmul.p     M730, M102, M100"),
    TEST_ENTRY_C(0xF004659F, NULL, "vmmul.p     M730, M112, M100"),
    TEST_ENTRY_C(0xF004669F, NULL, "vmmul.p     M730, M122, M100"),
    TEST_ENTRY_C(0xF004679F, NULL, "vmmul.p     M730, M132, M100"),
    TEST_ENTRY_C(0xF004689F, NULL, "vmmul.p     M730, M202, M100"),
    TEST_ENTRY_C(0xF004699F, NULL, "vmmul.p     M730, M212, M100"),
    TEST_ENTRY_C(0xF0046A9F, NULL, "vmmul.p     M730, M222, M100"),
    TEST_ENTRY_C(0xF0046B9F, NULL, "vmmul.p     M730, M232, M100"),
    TEST_ENTRY_C(0xF0046C9F, NULL, "vmmul.p     M730, M302, M100"),
    TEST_ENTRY_C(0xF0046D9F, NULL, "vmmul.p     M730, M312, M100"),
    TEST_ENTRY_C(0xF0046E9F, NULL, "vmmul.p     M730, M322, M100"),
    TEST_ENTRY_C(0xF0046F9F, NULL, "vmmul.p     M730, M332, M100"),
    TEST_ENTRY_C(0xF004709F, NULL, "vmmul.p     M730, M402, M100"),
    TEST_ENTRY_C(0xF004719F, NULL, "vmmul.p     M730, M412, M100"),
    TEST_ENTRY_C(0xF004729F, NULL, "vmmul.p     M730, M422, M100"),
    TEST_ENTRY_C(0xF004739F, NULL, "vmmul.p     M730, M432, M100"),
    TEST_ENTRY_C(0xF004749F, NULL, "vmmul.p     M730, M502, M100"),
    TEST_ENTRY_C(0xF004759F, NULL, "vmmul.p     M730, M512, M100"),
    TEST_ENTRY_C(0xF004769F, NULL, "vmmul.p     M730, M522, M100"),
    TEST_ENTRY_C(0xF004779F, NULL, "vmmul.p     M730, M532, M100"),
    TEST_ENTRY_C(0xF004789F, NULL, "vmmul.p     M730, M602, M100"),
    TEST_ENTRY_C(0xF004799F, NULL, "vmmul.p     M730, M612, M100"),
    TEST_ENTRY_C(0xF0047A9F, NULL, "vmmul.p     M730, M622, M100"),
    TEST_ENTRY_C(0xF0047B9F, NULL, "vmmul.p     M730, M632, M100"),
    TEST_ENTRY_C(0xF0047C88, NULL, "vmmul.p     M200, M702, M100"),
    TEST_ENTRY_C(0xF0047D88, NULL, "vmmul.p     M200, M712, M100"),
    TEST_ENTRY_C(0xF0047E88, NULL, "vmmul.p     M200, M722, M100"),
    TEST_ENTRY_C(0xF0047F88, NULL, "vmmul.p     M200, M732, M100"),
    TEST_ENTRY_C(0xF004801F, NULL, "vmmul.t     M730, E000, M100"),
    TEST_ENTRY_C(0xF004811F, NULL, "vmmul.t     M730, E001, M100"),
    TEST_ENTRY_C(0xF004821F, NULL, "vmmul.t     M730, E002, M100"),
    TEST_ENTRY_C(0xF004831F, NULL, "vmmul.t     M730, E003, M100"),
    TEST_ENTRY_C(0xF004841F, NULL, "vmmul.t     M730, E100, M100"),
    TEST_ENTRY_C(0xF004851F, NULL, "vmmul.t     M730, E101, M100"),
    TEST_ENTRY_C(0xF004861F, NULL, "vmmul.t     M730, E102, M100"),
    TEST_ENTRY_C(0xF004871F, NULL, "vmmul.t     M730, E103, M100"),
    TEST_ENTRY_C(0xF004881F, NULL, "vmmul.t     M730, E200, M100"),
    TEST_ENTRY_C(0xF004891F, NULL, "vmmul.t     M730, E201, M100"),
    TEST_ENTRY_C(0xF0048A1F, NULL, "vmmul.t     M730, E202, M100"),
    TEST_ENTRY_C(0xF0048B1F, NULL, "vmmul.t     M730, E203, M100"),
    TEST_ENTRY_C(0xF0048C1F, NULL, "vmmul.t     M730, E300, M100"),
    TEST_ENTRY_C(0xF0048D1F, NULL, "vmmul.t     M730, E301, M100"),
    TEST_ENTRY_C(0xF0048E1F, NULL, "vmmul.t     M730, E302, M100"),
    TEST_ENTRY_C(0xF0048F1F, NULL, "vmmul.t     M730, E303, M100"),
    TEST_ENTRY_C(0xF004901F, NULL, "vmmul.t     M730, E400, M100"),
    TEST_ENTRY_C(0xF004911F, NULL, "vmmul.t     M730, E401, M100"),
    TEST_ENTRY_C(0xF004921F, NULL, "vmmul.t     M730, E402, M100"),
    TEST_ENTRY_C(0xF004931F, NULL, "vmmul.t     M730, E403, M100"),
    TEST_ENTRY_C(0xF004941F, NULL, "vmmul.t     M730, E500, M100"),
    TEST_ENTRY_C(0xF004951F, NULL, "vmmul.t     M730, E501, M100"),
    TEST_ENTRY_C(0xF004961F, NULL, "vmmul.t     M730, E502, M100"),
    TEST_ENTRY_C(0xF004971F, NULL, "vmmul.t     M730, E503, M100"),
    TEST_ENTRY_C(0xF004981F, NULL, "vmmul.t     M730, E600, M100"),
    TEST_ENTRY_C(0xF004991F, NULL, "vmmul.t     M730, E601, M100"),
    TEST_ENTRY_C(0xF0049A1F, NULL, "vmmul.t     M730, E602, M100"),
    TEST_ENTRY_C(0xF0049B1F, NULL, "vmmul.t     M730, E603, M100"),
    TEST_ENTRY_C(0xF0049C08, NULL, "vmmul.t     M200, E700, M100"),
    TEST_ENTRY_C(0xF0049D08, NULL, "vmmul.t     M200, E701, M100"),
    TEST_ENTRY_C(0xF0049E08, NULL, "vmmul.t     M200, E702, M100"),
    TEST_ENTRY_C(0xF0049F08, NULL, "vmmul.t     M200, E703, M100"),
    TEST_ENTRY_C(0xF004A01F, NULL, "vmmul.t     M730, M000, M100"),
    TEST_ENTRY_C(0xF004A11F, NULL, "vmmul.t     M730, M010, M100"),
    TEST_ENTRY_C(0xF004A21F, NULL, "vmmul.t     M730, M020, M100"),
    TEST_ENTRY_C(0xF004A31F, NULL, "vmmul.t     M730, M030, M100"),
    TEST_ENTRY_C(0xF004A41F, NULL, "vmmul.t     M730, M100, M100"),
    TEST_ENTRY_C(0xF004A51F, NULL, "vmmul.t     M730, M110, M100"),
    TEST_ENTRY_C(0xF004A61F, NULL, "vmmul.t     M730, M120, M100"),
    TEST_ENTRY_C(0xF004A71F, NULL, "vmmul.t     M730, M130, M100"),
    TEST_ENTRY_C(0xF004A81F, NULL, "vmmul.t     M730, M200, M100"),
    TEST_ENTRY_C(0xF004A91F, NULL, "vmmul.t     M730, M210, M100"),
    TEST_ENTRY_C(0xF004AA1F, NULL, "vmmul.t     M730, M220, M100"),
    TEST_ENTRY_C(0xF004AB1F, NULL, "vmmul.t     M730, M230, M100"),
    TEST_ENTRY_C(0xF004AC1F, NULL, "vmmul.t     M730, M300, M100"),
    TEST_ENTRY_C(0xF004AD1F, NULL, "vmmul.t     M730, M310, M100"),
    TEST_ENTRY_C(0xF004AE1F, NULL, "vmmul.t     M730, M320, M100"),
    TEST_ENTRY_C(0xF004AF1F, NULL, "vmmul.t     M730, M330, M100"),
    TEST_ENTRY_C(0xF004B01F, NULL, "vmmul.t     M730, M400, M100"),
    TEST_ENTRY_C(0xF004B11F, NULL, "vmmul.t     M730, M410, M100"),
    TEST_ENTRY_C(0xF004B21F, NULL, "vmmul.t     M730, M420, M100"),
    TEST_ENTRY_C(0xF004B31F, NULL, "vmmul.t     M730, M430, M100"),
    TEST_ENTRY_C(0xF004B41F, NULL, "vmmul.t     M730, M500, M100"),
    TEST_ENTRY_C(0xF004B51F, NULL, "vmmul.t     M730, M510, M100"),
    TEST_ENTRY_C(0xF004B61F, NULL, "vmmul.t     M730, M520, M100"),
    TEST_ENTRY_C(0xF004B71F, NULL, "vmmul.t     M730, M530, M100"),
    TEST_ENTRY_C(0xF004B81F, NULL, "vmmul.t     M730, M600, M100"),
    TEST_ENTRY_C(0xF004B91F, NULL, "vmmul.t     M730, M610, M100"),
    TEST_ENTRY_C(0xF004BA1F, NULL, "vmmul.t     M730, M620, M100"),
    TEST_ENTRY_C(0xF004BB1F, NULL, "vmmul.t     M730, M630, M100"),
    TEST_ENTRY_C(0xF004BC08, NULL, "vmmul.t     M200, M700, M100"),
    TEST_ENTRY_C(0xF004BD08, NULL, "vmmul.t     M200, M710, M100"),
    TEST_ENTRY_C(0xF004BE08, NULL, "vmmul.t     M200, M720, M100"),
    TEST_ENTRY_C(0xF004BF08, NULL, "vmmul.t     M200, M730, M100"),
    TEST_ENTRY_C(0xF004C01F, NULL, "vmmul.t     M730, E010, M100"),
    TEST_ENTRY_C(0xF004C11F, NULL, "vmmul.t     M730, E011, M100"),
    TEST_ENTRY_C(0xF004C21F, NULL, "vmmul.t     M730, E012, M100"),
    TEST_ENTRY_C(0xF004C31F, NULL, "vmmul.t     M730, E013, M100"),
    TEST_ENTRY_C(0xF004C41F, NULL, "vmmul.t     M730, E110, M100"),
    TEST_ENTRY_C(0xF004C51F, NULL, "vmmul.t     M730, E111, M100"),
    TEST_ENTRY_C(0xF004C61F, NULL, "vmmul.t     M730, E112, M100"),
    TEST_ENTRY_C(0xF004C71F, NULL, "vmmul.t     M730, E113, M100"),
    TEST_ENTRY_C(0xF004C81F, NULL, "vmmul.t     M730, E210, M100"),
    TEST_ENTRY_C(0xF004C91F, NULL, "vmmul.t     M730, E211, M100"),
    TEST_ENTRY_C(0xF004CA1F, NULL, "vmmul.t     M730, E212, M100"),
    TEST_ENTRY_C(0xF004CB1F, NULL, "vmmul.t     M730, E213, M100"),
    TEST_ENTRY_C(0xF004CC1F, NULL, "vmmul.t     M730, E310, M100"),
    TEST_ENTRY_C(0xF004CD1F, NULL, "vmmul.t     M730, E311, M100"),
    TEST_ENTRY_C(0xF004CE1F, NULL, "vmmul.t     M730, E312, M100"),
    TEST_ENTRY_C(0xF004CF1F, NULL, "vmmul.t     M730, E313, M100"),
    TEST_ENTRY_C(0xF004D01F, NULL, "vmmul.t     M730, E410, M100"),
    TEST_ENTRY_C(0xF004D11F, NULL, "vmmul.t     M730, E411, M100"),
    TEST_ENTRY_C(0xF004D21F, NULL, "vmmul.t     M730, E412, M100"),
    TEST_ENTRY_C(0xF004D31F, NULL, "vmmul.t     M730, E413, M100"),
    TEST_ENTRY_C(0xF004D41F, NULL, "vmmul.t     M730, E510, M100"),
    TEST_ENTRY_C(0xF004D51F, NULL, "vmmul.t     M730, E511, M100"),
    TEST_ENTRY_C(0xF004D61F, NULL, "vmmul.t     M730, E512, M100"),
    TEST_ENTRY_C(0xF004D71F, NULL, "vmmul.t     M730, E513, M100"),
    TEST_ENTRY_C(0xF004D81F, NULL, "vmmul.t     M730, E610, M100"),
    TEST_ENTRY_C(0xF004D91F, NULL, "vmmul.t     M730, E611, M100"),
    TEST_ENTRY_C(0xF004DA1F, NULL, "vmmul.t     M730, E612, M100"),
    TEST_ENTRY_C(0xF004DB1F, NULL, "vmmul.t     M730, E613, M100"),
    TEST_ENTRY_C(0xF004DC08, NULL, "vmmul.t     M200, E710, M100"),
    TEST_ENTRY_C(0xF004DD08, NULL, "vmmul.t     M200, E711, M100"),
    TEST_ENTRY_C(0xF004DE08, NULL, "vmmul.t     M200, E712, M100"),
    TEST_ENTRY_C(0xF004DF08, NULL, "vmmul.t     M200, E713, M100"),
    TEST_ENTRY_C(0xF004E01F, NULL, "vmmul.t     M730, M001, M100"),
    TEST_ENTRY_C(0xF004E11F, NULL, "vmmul.t     M730, M011, M100"),
    TEST_ENTRY_C(0xF004E21F, NULL, "vmmul.t     M730, M021, M100"),
    TEST_ENTRY_C(0xF004E31F, NULL, "vmmul.t     M730, M031, M100"),
    TEST_ENTRY_C(0xF004E41F, NULL, "vmmul.t     M730, M101, M100"),
    TEST_ENTRY_C(0xF004E51F, NULL, "vmmul.t     M730, M111, M100"),
    TEST_ENTRY_C(0xF004E61F, NULL, "vmmul.t     M730, M121, M100"),
    TEST_ENTRY_C(0xF004E71F, NULL, "vmmul.t     M730, M131, M100"),
    TEST_ENTRY_C(0xF004E81F, NULL, "vmmul.t     M730, M201, M100"),
    TEST_ENTRY_C(0xF004E91F, NULL, "vmmul.t     M730, M211, M100"),
    TEST_ENTRY_C(0xF004EA1F, NULL, "vmmul.t     M730, M221, M100"),
    TEST_ENTRY_C(0xF004EB1F, NULL, "vmmul.t     M730, M231, M100"),
    TEST_ENTRY_C(0xF004EC1F, NULL, "vmmul.t     M730, M301, M100"),
    TEST_ENTRY_C(0xF004ED1F, NULL, "vmmul.t     M730, M311, M100"),
    TEST_ENTRY_C(0xF004EE1F, NULL, "vmmul.t     M730, M321, M100"),
    TEST_ENTRY_C(0xF004EF1F, NULL, "vmmul.t     M730, M331, M100"),
    TEST_ENTRY_C(0xF004F01F, NULL, "vmmul.t     M730, M401, M100"),
    TEST_ENTRY_C(0xF004F11F, NULL, "vmmul.t     M730, M411, M100"),
    TEST_ENTRY_C(0xF004F21F, NULL, "vmmul.t     M730, M421, M100"),
    TEST_ENTRY_C(0xF004F31F, NULL, "vmmul.t     M730, M431, M100"),
    TEST_ENTRY_C(0xF004F41F, NULL, "vmmul.t     M730, M501, M100"),
    TEST_ENTRY_C(0xF004F51F, NULL, "vmmul.t     M730, M511, M100"),
    TEST_ENTRY_C(0xF004F61F, NULL, "vmmul.t     M730, M521, M100"),
    TEST_ENTRY_C(0xF004F71F, NULL, "vmmul.t     M730, M531, M100"),
    TEST_ENTRY_C(0xF004F81F, NULL, "vmmul.t     M730, M601, M100"),
    TEST_ENTRY_C(0xF004F91F, NULL, "vmmul.t     M730, M611, M100"),
    TEST_ENTRY_C(0xF004FA1F, NULL, "vmmul.t     M730, M621, M100"),
    TEST_ENTRY_C(0xF004FB1F, NULL, "vmmul.t     M730, M631, M100"),
    TEST_ENTRY_C(0xF004FC08, NULL, "vmmul.t     M200, M701, M100"),
    TEST_ENTRY_C(0xF004FD08, NULL, "vmmul.t     M200, M711, M100"),
    TEST_ENTRY_C(0xF004FE08, NULL, "vmmul.t     M200, M721, M100"),
    TEST_ENTRY_C(0xF004FF08, NULL, "vmmul.t     M200, M731, M100"),
    TEST_ENTRY_C(0xF004809F, NULL, "vmmul.q     M730, E000, M100"),
    TEST_ENTRY_C(0xF004819F, NULL, "vmmul.q     M730, E001, M100"),
    TEST_ENTRY_C(0xF004829F, NULL, "vmmul.q     M730, E002, M100"),
    TEST_ENTRY_C(0xF004839F, NULL, "vmmul.q     M730, E003, M100"),
    TEST_ENTRY_C(0xF004849F, NULL, "vmmul.q     M730, E100, M100"),
    TEST_ENTRY_C(0xF004859F, NULL, "vmmul.q     M730, E101, M100"),
    TEST_ENTRY_C(0xF004869F, NULL, "vmmul.q     M730, E102, M100"),
    TEST_ENTRY_C(0xF004879F, NULL, "vmmul.q     M730, E103, M100"),
    TEST_ENTRY_C(0xF004889F, NULL, "vmmul.q     M730, E200, M100"),
    TEST_ENTRY_C(0xF004899F, NULL, "vmmul.q     M730, E201, M100"),
    TEST_ENTRY_C(0xF0048A9F, NULL, "vmmul.q     M730, E202, M100"),
    TEST_ENTRY_C(0xF0048B9F, NULL, "vmmul.q     M730, E203, M100"),
    TEST_ENTRY_C(0xF0048C9F, NULL, "vmmul.q     M730, E300, M100"),
    TEST_ENTRY_C(0xF0048D9F, NULL, "vmmul.q     M730, E301, M100"),
    TEST_ENTRY_C(0xF0048E9F, NULL, "vmmul.q     M730, E302, M100"),
    TEST_ENTRY_C(0xF0048F9F, NULL, "vmmul.q     M730, E303, M100"),
    TEST_ENTRY_C(0xF004909F, NULL, "vmmul.q     M730, E400, M100"),
    TEST_ENTRY_C(0xF004919F, NULL, "vmmul.q     M730, E401, M100"),
    TEST_ENTRY_C(0xF004929F, NULL, "vmmul.q     M730, E402, M100"),
    TEST_ENTRY_C(0xF004939F, NULL, "vmmul.q     M730, E403, M100"),
    TEST_ENTRY_C(0xF004949F, NULL, "vmmul.q     M730, E500, M100"),
    TEST_ENTRY_C(0xF004959F, NULL, "vmmul.q     M730, E501, M100"),
    TEST_ENTRY_C(0xF004969F, NULL, "vmmul.q     M730, E502, M100"),
    TEST_ENTRY_C(0xF004979F, NULL, "vmmul.q     M730, E503, M100"),
    TEST_ENTRY_C(0xF004989F, NULL, "vmmul.q     M730, E600, M100"),
    TEST_ENTRY_C(0xF004999F, NULL, "vmmul.q     M730, E601, M100"),
    TEST_ENTRY_C(0xF0049A9F, NULL, "vmmul.q     M730, E602, M100"),
    TEST_ENTRY_C(0xF0049B9F, NULL, "vmmul.q     M730, E603, M100"),
    TEST_ENTRY_C(0xF0049C9F, NULL, "vmmul.q     M730, E700, M100"),
    TEST_ENTRY_C(0xF0049D9F, NULL, "vmmul.q     M730, E701, M100"),
    TEST_ENTRY_C(0xF0049E9F, NULL, "vmmul.q     M730, E702, M100"),
    TEST_ENTRY_C(0xF0049F9F, NULL, "vmmul.q     M730, E703, M100"),
    TEST_ENTRY_C(0xF004A09F, NULL, "vmmul.q     M730, M000, M100"),
    TEST_ENTRY_C(0xF004A19F, NULL, "vmmul.q     M730, M010, M100"),
    TEST_ENTRY_C(0xF004A29F, NULL, "vmmul.q     M730, M020, M100"),
    TEST_ENTRY_C(0xF004A39F, NULL, "vmmul.q     M730, M030, M100"),
    TEST_ENTRY_C(0xF004A49F, NULL, "vmmul.q     M730, M100, M100"),
    TEST_ENTRY_C(0xF004A59F, NULL, "vmmul.q     M730, M110, M100"),
    TEST_ENTRY_C(0xF004A69F, NULL, "vmmul.q     M730, M120, M100"),
    TEST_ENTRY_C(0xF004A79F, NULL, "vmmul.q     M730, M130, M100"),
    TEST_ENTRY_C(0xF004A89F, NULL, "vmmul.q     M730, M200, M100"),
    TEST_ENTRY_C(0xF004A99F, NULL, "vmmul.q     M730, M210, M100"),
    TEST_ENTRY_C(0xF004AA9F, NULL, "vmmul.q     M730, M220, M100"),
    TEST_ENTRY_C(0xF004AB9F, NULL, "vmmul.q     M730, M230, M100"),
    TEST_ENTRY_C(0xF004AC9F, NULL, "vmmul.q     M730, M300, M100"),
    TEST_ENTRY_C(0xF004AD9F, NULL, "vmmul.q     M730, M310, M100"),
    TEST_ENTRY_C(0xF004AE9F, NULL, "vmmul.q     M730, M320, M100"),
    TEST_ENTRY_C(0xF004AF9F, NULL, "vmmul.q     M730, M330, M100"),
    TEST_ENTRY_C(0xF004B09F, NULL, "vmmul.q     M730, M400, M100"),
    TEST_ENTRY_C(0xF004B19F, NULL, "vmmul.q     M730, M410, M100"),
    TEST_ENTRY_C(0xF004B29F, NULL, "vmmul.q     M730, M420, M100"),
    TEST_ENTRY_C(0xF004B39F, NULL, "vmmul.q     M730, M430, M100"),
    TEST_ENTRY_C(0xF004B49F, NULL, "vmmul.q     M730, M500, M100"),
    TEST_ENTRY_C(0xF004B59F, NULL, "vmmul.q     M730, M510, M100"),
    TEST_ENTRY_C(0xF004B69F, NULL, "vmmul.q     M730, M520, M100"),
    TEST_ENTRY_C(0xF004B79F, NULL, "vmmul.q     M730, M530, M100"),
    TEST_ENTRY_C(0xF004B89F, NULL, "vmmul.q     M730, M600, M100"),
    TEST_ENTRY_C(0xF004B99F, NULL, "vmmul.q     M730, M610, M100"),
    TEST_ENTRY_C(0xF004BA9F, NULL, "vmmul.q     M730, M620, M100"),
    TEST_ENTRY_C(0xF004BB9F, NULL, "vmmul.q     M730, M630, M100"),
    TEST_ENTRY_C(0xF004BC9F, NULL, "vmmul.q     M730, M700, M100"),
    TEST_ENTRY_C(0xF004BD9F, NULL, "vmmul.q     M730, M710, M100"),
    TEST_ENTRY_C(0xF004BE9F, NULL, "vmmul.q     M730, M720, M100"),
    TEST_ENTRY_C(0xF004BF9F, NULL, "vmmul.q     M730, M730, M100"),
    TEST_ENTRY_C(0xF004C09F, NULL, "vmmul.q     M730, E020, M100"),
    TEST_ENTRY_C(0xF004C19F, NULL, "vmmul.q     M730, E021, M100"),
    TEST_ENTRY_C(0xF004C29F, NULL, "vmmul.q     M730, E022, M100"),
    TEST_ENTRY_C(0xF004C39F, NULL, "vmmul.q     M730, E023, M100"),
    TEST_ENTRY_C(0xF004C49F, NULL, "vmmul.q     M730, E120, M100"),
    TEST_ENTRY_C(0xF004C59F, NULL, "vmmul.q     M730, E121, M100"),
    TEST_ENTRY_C(0xF004C69F, NULL, "vmmul.q     M730, E122, M100"),
    TEST_ENTRY_C(0xF004C79F, NULL, "vmmul.q     M730, E123, M100"),
    TEST_ENTRY_C(0xF004C89F, NULL, "vmmul.q     M730, E220, M100"),
    TEST_ENTRY_C(0xF004C99F, NULL, "vmmul.q     M730, E221, M100"),
    TEST_ENTRY_C(0xF004CA9F, NULL, "vmmul.q     M730, E222, M100"),
    TEST_ENTRY_C(0xF004CB9F, NULL, "vmmul.q     M730, E223, M100"),
    TEST_ENTRY_C(0xF004CC9F, NULL, "vmmul.q     M730, E320, M100"),
    TEST_ENTRY_C(0xF004CD9F, NULL, "vmmul.q     M730, E321, M100"),
    TEST_ENTRY_C(0xF004CE9F, NULL, "vmmul.q     M730, E322, M100"),
    TEST_ENTRY_C(0xF004CF9F, NULL, "vmmul.q     M730, E323, M100"),
    TEST_ENTRY_C(0xF004D09F, NULL, "vmmul.q     M730, E420, M100"),
    TEST_ENTRY_C(0xF004D19F, NULL, "vmmul.q     M730, E421, M100"),
    TEST_ENTRY_C(0xF004D29F, NULL, "vmmul.q     M730, E422, M100"),
    TEST_ENTRY_C(0xF004D39F, NULL, "vmmul.q     M730, E423, M100"),
    TEST_ENTRY_C(0xF004D49F, NULL, "vmmul.q     M730, E520, M100"),
    TEST_ENTRY_C(0xF004D59F, NULL, "vmmul.q     M730, E521, M100"),
    TEST_ENTRY_C(0xF004D69F, NULL, "vmmul.q     M730, E522, M100"),
    TEST_ENTRY_C(0xF004D79F, NULL, "vmmul.q     M730, E523, M100"),
    TEST_ENTRY_C(0xF004D89F, NULL, "vmmul.q     M730, E620, M100"),
    TEST_ENTRY_C(0xF004D99F, NULL, "vmmul.q     M730, E621, M100"),
    TEST_ENTRY_C(0xF004DA9F, NULL, "vmmul.q     M730, E622, M100"),
    TEST_ENTRY_C(0xF004DB9F, NULL, "vmmul.q     M730, E623, M100"),
    TEST_ENTRY_C(0xF004DC9F, NULL, "vmmul.q     M730, E720, M100"),
    TEST_ENTRY_C(0xF004DD9F, NULL, "vmmul.q     M730, E721, M100"),
    TEST_ENTRY_C(0xF004DE9F, NULL, "vmmul.q     M730, E722, M100"),
    TEST_ENTRY_C(0xF004DF9F, NULL, "vmmul.q     M730, E723, M100"),
    TEST_ENTRY_C(0xF004E09F, NULL, "vmmul.q     M730, M002, M100"),
    TEST_ENTRY_C(0xF004E19F, NULL, "vmmul.q     M730, M012, M100"),
    TEST_ENTRY_C(0xF004E29F, NULL, "vmmul.q     M730, M022, M100"),
    TEST_ENTRY_C(0xF004E39F, NULL, "vmmul.q     M730, M032, M100"),
    TEST_ENTRY_C(0xF004E49F, NULL, "vmmul.q     M730, M102, M100"),
    TEST_ENTRY_C(0xF004E59F, NULL, "vmmul.q     M730, M112, M100"),
    TEST_ENTRY_C(0xF004E69F, NULL, "vmmul.q     M730, M122, M100"),
    TEST_ENTRY_C(0xF004E79F, NULL, "vmmul.q     M730, M132, M100"),
    TEST_ENTRY_C(0xF004E89F, NULL, "vmmul.q     M730, M202, M100"),
    TEST_ENTRY_C(0xF004E99F, NULL, "vmmul.q     M730, M212, M100"),
    TEST_ENTRY_C(0xF004EA9F, NULL, "vmmul.q     M730, M222, M100"),
    TEST_ENTRY_C(0xF004EB9F, NULL, "vmmul.q     M730, M232, M100"),
    TEST_ENTRY_C(0xF004EC9F, NULL, "vmmul.q     M730, M302, M100"),
    TEST_ENTRY_C(0xF004ED9F, NULL, "vmmul.q     M730, M312, M100"),
    TEST_ENTRY_C(0xF004EE9F, NULL, "vmmul.q     M730, M322, M100"),
    TEST_ENTRY_C(0xF004EF9F, NULL, "vmmul.q     M730, M332, M100"),
    TEST_ENTRY_C(0xF004F09F, NULL, "vmmul.q     M730, M402, M100"),
    TEST_ENTRY_C(0xF004F19F, NULL, "vmmul.q     M730, M412, M100"),
    TEST_ENTRY_C(0xF004F29F, NULL, "vmmul.q     M730, M422, M100"),
    TEST_ENTRY_C(0xF004F39F, NULL, "vmmul.q     M730, M432, M100"),
    TEST_ENTRY_C(0xF004F49F, NULL, "vmmul.q     M730, M502, M100"),
    TEST_ENTRY_C(0xF004F59F, NULL, "vmmul.q     M730, M512, M100"),
    TEST_ENTRY_C(0xF004F69F, NULL, "vmmul.q     M730, M522, M100"),
    TEST_ENTRY_C(0xF004F79F, NULL, "vmmul.q     M730, M532, M100"),
    TEST_ENTRY_C(0xF004F89F, NULL, "vmmul.q     M730, M602, M100"),
    TEST_ENTRY_C(0xF004F99F, NULL, "vmmul.q     M730, M612, M100"),
    TEST_ENTRY_C(0xF004FA9F, NULL, "vmmul.q     M730, M622, M100"),
    TEST_ENTRY_C(0xF004FB9F, NULL, "vmmul.q     M730, M632, M100"),
    TEST_ENTRY_C(0xF004FC9F, NULL, "vmmul.q     M730, M702, M100"),
    TEST_ENTRY_C(0xF004FD9F, NULL, "vmmul.q     M730, M712, M100"),
    TEST_ENTRY_C(0xF004FE9F, NULL, "vmmul.q     M730, M722, M100"),
    TEST_ENTRY_C(0xF004FF9F, NULL, "vmmul.q     M730, M732, M100"),

    /* vrot */
    TEST_ENTRY_C(0xF3A0009F, NULL, "vrot.p      C730, S000, [C,S]"),
    TEST_ENTRY_C(0xF3A1009F, NULL, "vrot.p      C730, S000, [S,C]"),
    TEST_ENTRY_C(0xF3A2009F, NULL, "vrot.p      C730, S000, [S,0]"),
    TEST_ENTRY_C(0xF3A3009F, NULL, "vrot.p      C730, S000, [S,0]"),
    TEST_ENTRY_C(0xF3A4009F, NULL, "vrot.p      C730, S000, [C,S]"),
    TEST_ENTRY_C(0xF3A5009F, NULL, "vrot.p      C730, S000, [S,C]"),
    TEST_ENTRY_C(0xF3A6009F, NULL, "vrot.p      C730, S000, [0,S]"),
    TEST_ENTRY_C(0xF3A7009F, NULL, "vrot.p      C730, S000, [0,S]"),
    TEST_ENTRY_C(0xF3A8009F, NULL, "vrot.p      C730, S000, [C,0]"),
    TEST_ENTRY_C(0xF3A9009F, NULL, "vrot.p      C730, S000, [0,C]"),
    TEST_ENTRY_C(0xF3AA009F, NULL, "vrot.p      C730, S000, [S,S]"),
    TEST_ENTRY_C(0xF3AB009F, NULL, "vrot.p      C730, S000, [0,0]"),
    TEST_ENTRY_C(0xF3AC009F, NULL, "vrot.p      C730, S000, [C,0]"),
    TEST_ENTRY_C(0xF3AD009F, NULL, "vrot.p      C730, S000, [0,C]"),
    TEST_ENTRY_C(0xF3AE009F, NULL, "vrot.p      C730, S000, [0,0]"),
    TEST_ENTRY_C(0xF3AF009F, NULL, "vrot.p      C730, S000, [S,S]"),
    TEST_ENTRY_C(0xF3B0009F, NULL, "vrot.p      C730, S000, [C,-S]"),
    TEST_ENTRY_C(0xF3B1009F, NULL, "vrot.p      C730, S000, [-S,C]"),
    TEST_ENTRY_C(0xF3B2009F, NULL, "vrot.p      C730, S000, [-S,0]"),
    TEST_ENTRY_C(0xF3B3009F, NULL, "vrot.p      C730, S000, [-S,0]"),
    TEST_ENTRY_C(0xF3B4009F, NULL, "vrot.p      C730, S000, [C,-S]"),
    TEST_ENTRY_C(0xF3B5009F, NULL, "vrot.p      C730, S000, [-S,C]"),
    TEST_ENTRY_C(0xF3B6009F, NULL, "vrot.p      C730, S000, [0,-S]"),
    TEST_ENTRY_C(0xF3B7009F, NULL, "vrot.p      C730, S000, [0,-S]"),
    TEST_ENTRY_C(0xF3B8009F, NULL, "vrot.p      C730, S000, [C,0]"),
    TEST_ENTRY_C(0xF3B9009F, NULL, "vrot.p      C730, S000, [0,C]"),
    TEST_ENTRY_C(0xF3BA009F, NULL, "vrot.p      C730, S000, [-S,-S]"),
    TEST_ENTRY_C(0xF3BB009F, NULL, "vrot.p      C730, S000, [0,0]"),
    TEST_ENTRY_C(0xF3BC009F, NULL, "vrot.p      C730, S000, [C,0]"),
    TEST_ENTRY_C(0xF3BD009F, NULL, "vrot.p      C730, S000, [0,C]"),
    TEST_ENTRY_C(0xF3BE009F, NULL, "vrot.p      C730, S000, [0,0]"),
    TEST_ENTRY_C(0xF3BF009F, NULL, "vrot.p      C730, S000, [-S,-S]"),
    TEST_ENTRY_C(0xF3A0801F, NULL, "vrot.t      C730, S000, [C,S,S]"),
    TEST_ENTRY_C(0xF3A1801F, NULL, "vrot.t      C730, S000, [S,C,0]"),
    TEST_ENTRY_C(0xF3A2801F, NULL, "vrot.t      C730, S000, [S,0,C]"),
    TEST_ENTRY_C(0xF3A3801F, NULL, "vrot.t      C730, S000, [S,0,0]"),
    TEST_ENTRY_C(0xF3A4801F, NULL, "vrot.t      C730, S000, [C,S,0]"),
    TEST_ENTRY_C(0xF3A5801F, NULL, "vrot.t      C730, S000, [S,C,S]"),
    TEST_ENTRY_C(0xF3A6801F, NULL, "vrot.t      C730, S000, [0,S,C]"),
    TEST_ENTRY_C(0xF3A7801F, NULL, "vrot.t      C730, S000, [0,S,0]"),
    TEST_ENTRY_C(0xF3A8801F, NULL, "vrot.t      C730, S000, [C,0,S]"),
    TEST_ENTRY_C(0xF3A9801F, NULL, "vrot.t      C730, S000, [0,C,S]"),
    TEST_ENTRY_C(0xF3AA801F, NULL, "vrot.t      C730, S000, [S,S,C]"),
    TEST_ENTRY_C(0xF3AB801F, NULL, "vrot.t      C730, S000, [0,0,S]"),
    TEST_ENTRY_C(0xF3AC801F, NULL, "vrot.t      C730, S000, [C,0,0]"),
    TEST_ENTRY_C(0xF3AD801F, NULL, "vrot.t      C730, S000, [0,C,0]"),
    TEST_ENTRY_C(0xF3AE801F, NULL, "vrot.t      C730, S000, [0,0,C]"),
    TEST_ENTRY_C(0xF3AF801F, NULL, "vrot.t      C730, S000, [S,S,S]"),
    TEST_ENTRY_C(0xF3B0801F, NULL, "vrot.t      C730, S000, [C,-S,-S]"),
    TEST_ENTRY_C(0xF3B1801F, NULL, "vrot.t      C730, S000, [-S,C,0]"),
    TEST_ENTRY_C(0xF3B2801F, NULL, "vrot.t      C730, S000, [-S,0,C]"),
    TEST_ENTRY_C(0xF3B3801F, NULL, "vrot.t      C730, S000, [-S,0,0]"),
    TEST_ENTRY_C(0xF3B4801F, NULL, "vrot.t      C730, S000, [C,-S,0]"),
    TEST_ENTRY_C(0xF3B5801F, NULL, "vrot.t      C730, S000, [-S,C,-S]"),
    TEST_ENTRY_C(0xF3B6801F, NULL, "vrot.t      C730, S000, [0,-S,C]"),
    TEST_ENTRY_C(0xF3B7801F, NULL, "vrot.t      C730, S000, [0,-S,0]"),
    TEST_ENTRY_C(0xF3B8801F, NULL, "vrot.t      C730, S000, [C,0,-S]"),
    TEST_ENTRY_C(0xF3B9801F, NULL, "vrot.t      C730, S000, [0,C,-S]"),
    TEST_ENTRY_C(0xF3BA801F, NULL, "vrot.t      C730, S000, [-S,-S,C]"),
    TEST_ENTRY_C(0xF3BB801F, NULL, "vrot.t      C730, S000, [0,0,-S]"),
    TEST_ENTRY_C(0xF3BC801F, NULL, "vrot.t      C730, S000, [C,0,0]"),
    TEST_ENTRY_C(0xF3BD801F, NULL, "vrot.t      C730, S000, [0,C,0]"),
    TEST_ENTRY_C(0xF3BE801F, NULL, "vrot.t      C730, S000, [0,0,C]"),
    TEST_ENTRY_C(0xF3BF801F, NULL, "vrot.t      C730, S000, [-S,-S,-S]"),
    TEST_ENTRY_C(0xF3A0809F, NULL, "vrot.q      C730, S000, [C,S,S,S]"),
    TEST_ENTRY_C(0xF3A1809F, NULL, "vrot.q      C730, S000, [S,C,0,0]"),
    TEST_ENTRY_C(0xF3A2809F, NULL, "vrot.q      C730, S000, [S,0,C,0]"),
    TEST_ENTRY_C(0xF3A3809F, NULL, "vrot.q      C730, S000, [S,0,0,C]"),
    TEST_ENTRY_C(0xF3A4809F, NULL, "vrot.q      C730, S000, [C,S,0,0]"),
    TEST_ENTRY_C(0xF3A5809F, NULL, "vrot.q      C730, S000, [S,C,S,S]"),
    TEST_ENTRY_C(0xF3A6809F, NULL, "vrot.q      C730, S000, [0,S,C,0]"),
    TEST_ENTRY_C(0xF3A7809F, NULL, "vrot.q      C730, S000, [0,S,0,C]"),
    TEST_ENTRY_C(0xF3A8809F, NULL, "vrot.q      C730, S000, [C,0,S,0]"),
    TEST_ENTRY_C(0xF3A9809F, NULL, "vrot.q      C730, S000, [0,C,S,0]"),
    TEST_ENTRY_C(0xF3AA809F, NULL, "vrot.q      C730, S000, [S,S,C,S]"),
    TEST_ENTRY_C(0xF3AB809F, NULL, "vrot.q      C730, S000, [0,0,S,C]"),
    TEST_ENTRY_C(0xF3AC809F, NULL, "vrot.q      C730, S000, [C,0,0,S]"),
    TEST_ENTRY_C(0xF3AD809F, NULL, "vrot.q      C730, S000, [0,C,0,S]"),
    TEST_ENTRY_C(0xF3AE809F, NULL, "vrot.q      C730, S000, [0,0,C,S]"),
    TEST_ENTRY_C(0xF3AF809F, NULL, "vrot.q      C730, S000, [S,S,S,C]"),
    TEST_ENTRY_C(0xF3B0809F, NULL, "vrot.q      C730, S000, [C,-S,-S,-S]"),
    TEST_ENTRY_C(0xF3B1809F, NULL, "vrot.q      C730, S000, [-S,C,0,0]"),
    TEST_ENTRY_C(0xF3B2809F, NULL, "vrot.q      C730, S000, [-S,0,C,0]"),
    TEST_ENTRY_C(0xF3B3809F, NULL, "vrot.q      C730, S000, [-S,0,0,C]"),
    TEST_ENTRY_C(0xF3B4809F, NULL, "vrot.q      C730, S000, [C,-S,0,0]"),
    TEST_ENTRY_C(0xF3B5809F, NULL, "vrot.q      C730, S000, [-S,C,-S,-S]"),
    TEST_ENTRY_C(0xF3B6809F, NULL, "vrot.q      C730, S000, [0,-S,C,0]"),
    TEST_ENTRY_C(0xF3B7809F, NULL, "vrot.q      C730, S000, [0,-S,0,C]"),
    TEST_ENTRY_C(0xF3B8809F, NULL, "vrot.q      C730, S000, [C,0,-S,0]"),
    TEST_ENTRY_C(0xF3B9809F, NULL, "vrot.q      C730, S000, [0,C,-S,0]"),
    TEST_ENTRY_C(0xF3BA809F, NULL, "vrot.q      C730, S000, [-S,-S,C,-S]"),
    TEST_ENTRY_C(0xF3BB809F, NULL, "vrot.q      C730, S000, [0,0,-S,C]"),
    TEST_ENTRY_C(0xF3BC809F, NULL, "vrot.q      C730, S000, [C,0,0,-S]"),
    TEST_ENTRY_C(0xF3BD809F, NULL, "vrot.q      C730, S000, [0,C,0,-S]"),
    TEST_ENTRY_C(0xF3BE809F, NULL, "vrot.q      C730, S000, [0,0,C,-S]"),
    TEST_ENTRY_C(0xF3BF809F, NULL, "vrot.q      C730, S000, [-S,-S,-S,C]"),

    /* vpfxs */
    TEST_ENTRY_C(0xDC000001, NULL, "vpfxs       Y, X, X, X"),
    TEST_ENTRY_C(0xDC000002, NULL, "vpfxs       Z, X, X, X"),
    TEST_ENTRY_C(0xDC000003, NULL, "vpfxs       W, X, X, X"),
    TEST_ENTRY_C(0xDC000100, NULL, "vpfxs       |X|, X, X, X"),
    TEST_ENTRY_C(0xDC000101, NULL, "vpfxs       |Y|, X, X, X"),
    TEST_ENTRY_C(0xDC000102, NULL, "vpfxs       |Z|, X, X, X"),
    TEST_ENTRY_C(0xDC000103, NULL, "vpfxs       |W|, X, X, X"),
    TEST_ENTRY_C(0xDC001000, NULL, "vpfxs       0, X, X, X"),
    TEST_ENTRY_C(0xDC001001, NULL, "vpfxs       1, X, X, X"),
    TEST_ENTRY_C(0xDC001002, NULL, "vpfxs       2, X, X, X"),
    TEST_ENTRY_C(0xDC001003, NULL, "vpfxs       1/2, X, X, X"),
    TEST_ENTRY_C(0xDC001100, NULL, "vpfxs       3, X, X, X"),
    TEST_ENTRY_C(0xDC001101, NULL, "vpfxs       1/3, X, X, X"),
    TEST_ENTRY_C(0xDC001102, NULL, "vpfxs       1/4, X, X, X"),
    TEST_ENTRY_C(0xDC001103, NULL, "vpfxs       1/6, X, X, X"),
    TEST_ENTRY_C(0xDC010000, NULL, "vpfxs       -X, X, X, X"),
    TEST_ENTRY_C(0xDC010001, NULL, "vpfxs       -Y, X, X, X"),
    TEST_ENTRY_C(0xDC010002, NULL, "vpfxs       -Z, X, X, X"),
    TEST_ENTRY_C(0xDC010003, NULL, "vpfxs       -W, X, X, X"),
    TEST_ENTRY_C(0xDC010100, NULL, "vpfxs       -|X|, X, X, X"),
    TEST_ENTRY_C(0xDC010101, NULL, "vpfxs       -|Y|, X, X, X"),
    TEST_ENTRY_C(0xDC010102, NULL, "vpfxs       -|Z|, X, X, X"),
    TEST_ENTRY_C(0xDC010103, NULL, "vpfxs       -|W|, X, X, X"),
    TEST_ENTRY_C(0xDC011000, NULL, "vpfxs       -0, X, X, X"),
    TEST_ENTRY_C(0xDC011001, NULL, "vpfxs       -1, X, X, X"),
    TEST_ENTRY_C(0xDC011002, NULL, "vpfxs       -2, X, X, X"),
    TEST_ENTRY_C(0xDC011003, NULL, "vpfxs       -1/2, X, X, X"),
    TEST_ENTRY_C(0xDC011100, NULL, "vpfxs       -3, X, X, X"),
    TEST_ENTRY_C(0xDC011101, NULL, "vpfxs       -1/3, X, X, X"),
    TEST_ENTRY_C(0xDC011102, NULL, "vpfxs       -1/4, X, X, X"),
    TEST_ENTRY_C(0xDC011103, NULL, "vpfxs       -1/6, X, X, X"),
    TEST_ENTRY_C(0xDC000004, NULL, "vpfxs       X, Y, X, X"),
    TEST_ENTRY_C(0xDC000008, NULL, "vpfxs       X, Z, X, X"),
    TEST_ENTRY_C(0xDC00000C, NULL, "vpfxs       X, W, X, X"),
    TEST_ENTRY_C(0xDC000200, NULL, "vpfxs       X, |X|, X, X"),
    TEST_ENTRY_C(0xDC000204, NULL, "vpfxs       X, |Y|, X, X"),
    TEST_ENTRY_C(0xDC000208, NULL, "vpfxs       X, |Z|, X, X"),
    TEST_ENTRY_C(0xDC00020C, NULL, "vpfxs       X, |W|, X, X"),
    TEST_ENTRY_C(0xDC002000, NULL, "vpfxs       X, 0, X, X"),
    TEST_ENTRY_C(0xDC002004, NULL, "vpfxs       X, 1, X, X"),
    TEST_ENTRY_C(0xDC002008, NULL, "vpfxs       X, 2, X, X"),
    TEST_ENTRY_C(0xDC00200C, NULL, "vpfxs       X, 1/2, X, X"),
    TEST_ENTRY_C(0xDC002200, NULL, "vpfxs       X, 3, X, X"),
    TEST_ENTRY_C(0xDC002204, NULL, "vpfxs       X, 1/3, X, X"),
    TEST_ENTRY_C(0xDC002208, NULL, "vpfxs       X, 1/4, X, X"),
    TEST_ENTRY_C(0xDC00220C, NULL, "vpfxs       X, 1/6, X, X"),
    TEST_ENTRY_C(0xDC020000, NULL, "vpfxs       X, -X, X, X"),
    TEST_ENTRY_C(0xDC020004, NULL, "vpfxs       X, -Y, X, X"),
    TEST_ENTRY_C(0xDC020008, NULL, "vpfxs       X, -Z, X, X"),
    TEST_ENTRY_C(0xDC02000C, NULL, "vpfxs       X, -W, X, X"),
    TEST_ENTRY_C(0xDC020200, NULL, "vpfxs       X, -|X|, X, X"),
    TEST_ENTRY_C(0xDC020204, NULL, "vpfxs       X, -|Y|, X, X"),
    TEST_ENTRY_C(0xDC020208, NULL, "vpfxs       X, -|Z|, X, X"),
    TEST_ENTRY_C(0xDC02020C, NULL, "vpfxs       X, -|W|, X, X"),
    TEST_ENTRY_C(0xDC022000, NULL, "vpfxs       X, -0, X, X"),
    TEST_ENTRY_C(0xDC022004, NULL, "vpfxs       X, -1, X, X"),
    TEST_ENTRY_C(0xDC022008, NULL, "vpfxs       X, -2, X, X"),
    TEST_ENTRY_C(0xDC02200C, NULL, "vpfxs       X, -1/2, X, X"),
    TEST_ENTRY_C(0xDC022200, NULL, "vpfxs       X, -3, X, X"),
    TEST_ENTRY_C(0xDC022204, NULL, "vpfxs       X, -1/3, X, X"),
    TEST_ENTRY_C(0xDC022208, NULL, "vpfxs       X, -1/4, X, X"),
    TEST_ENTRY_C(0xDC02220C, NULL, "vpfxs       X, -1/6, X, X"),
    TEST_ENTRY_C(0xDC000010, NULL, "vpfxs       X, X, Y, X"),
    TEST_ENTRY_C(0xDC000020, NULL, "vpfxs       X, X, Z, X"),
    TEST_ENTRY_C(0xDC000030, NULL, "vpfxs       X, X, W, X"),
    TEST_ENTRY_C(0xDC000400, NULL, "vpfxs       X, X, |X|, X"),
    TEST_ENTRY_C(0xDC000410, NULL, "vpfxs       X, X, |Y|, X"),
    TEST_ENTRY_C(0xDC000420, NULL, "vpfxs       X, X, |Z|, X"),
    TEST_ENTRY_C(0xDC000430, NULL, "vpfxs       X, X, |W|, X"),
    TEST_ENTRY_C(0xDC004000, NULL, "vpfxs       X, X, 0, X"),
    TEST_ENTRY_C(0xDC004010, NULL, "vpfxs       X, X, 1, X"),
    TEST_ENTRY_C(0xDC004020, NULL, "vpfxs       X, X, 2, X"),
    TEST_ENTRY_C(0xDC004030, NULL, "vpfxs       X, X, 1/2, X"),
    TEST_ENTRY_C(0xDC004400, NULL, "vpfxs       X, X, 3, X"),
    TEST_ENTRY_C(0xDC004410, NULL, "vpfxs       X, X, 1/3, X"),
    TEST_ENTRY_C(0xDC004420, NULL, "vpfxs       X, X, 1/4, X"),
    TEST_ENTRY_C(0xDC004430, NULL, "vpfxs       X, X, 1/6, X"),
    TEST_ENTRY_C(0xDC040000, NULL, "vpfxs       X, X, -X, X"),
    TEST_ENTRY_C(0xDC040010, NULL, "vpfxs       X, X, -Y, X"),
    TEST_ENTRY_C(0xDC040020, NULL, "vpfxs       X, X, -Z, X"),
    TEST_ENTRY_C(0xDC040030, NULL, "vpfxs       X, X, -W, X"),
    TEST_ENTRY_C(0xDC040400, NULL, "vpfxs       X, X, -|X|, X"),
    TEST_ENTRY_C(0xDC040410, NULL, "vpfxs       X, X, -|Y|, X"),
    TEST_ENTRY_C(0xDC040420, NULL, "vpfxs       X, X, -|Z|, X"),
    TEST_ENTRY_C(0xDC040430, NULL, "vpfxs       X, X, -|W|, X"),
    TEST_ENTRY_C(0xDC044000, NULL, "vpfxs       X, X, -0, X"),
    TEST_ENTRY_C(0xDC044010, NULL, "vpfxs       X, X, -1, X"),
    TEST_ENTRY_C(0xDC044020, NULL, "vpfxs       X, X, -2, X"),
    TEST_ENTRY_C(0xDC044030, NULL, "vpfxs       X, X, -1/2, X"),
    TEST_ENTRY_C(0xDC044400, NULL, "vpfxs       X, X, -3, X"),
    TEST_ENTRY_C(0xDC044410, NULL, "vpfxs       X, X, -1/3, X"),
    TEST_ENTRY_C(0xDC044420, NULL, "vpfxs       X, X, -1/4, X"),
    TEST_ENTRY_C(0xDC044430, NULL, "vpfxs       X, X, -1/6, X"),
    TEST_ENTRY_C(0xDC000040, NULL, "vpfxs       X, X, X, Y"),
    TEST_ENTRY_C(0xDC000080, NULL, "vpfxs       X, X, X, Z"),
    TEST_ENTRY_C(0xDC0000C0, NULL, "vpfxs       X, X, X, W"),
    TEST_ENTRY_C(0xDC000800, NULL, "vpfxs       X, X, X, |X|"),
    TEST_ENTRY_C(0xDC000840, NULL, "vpfxs       X, X, X, |Y|"),
    TEST_ENTRY_C(0xDC000880, NULL, "vpfxs       X, X, X, |Z|"),
    TEST_ENTRY_C(0xDC0008C0, NULL, "vpfxs       X, X, X, |W|"),
    TEST_ENTRY_C(0xDC008000, NULL, "vpfxs       X, X, X, 0"),
    TEST_ENTRY_C(0xDC008040, NULL, "vpfxs       X, X, X, 1"),
    TEST_ENTRY_C(0xDC008080, NULL, "vpfxs       X, X, X, 2"),
    TEST_ENTRY_C(0xDC0080C0, NULL, "vpfxs       X, X, X, 1/2"),
    TEST_ENTRY_C(0xDC008800, NULL, "vpfxs       X, X, X, 3"),
    TEST_ENTRY_C(0xDC008840, NULL, "vpfxs       X, X, X, 1/3"),
    TEST_ENTRY_C(0xDC008880, NULL, "vpfxs       X, X, X, 1/4"),
    TEST_ENTRY_C(0xDC0088C0, NULL, "vpfxs       X, X, X, 1/6"),
    TEST_ENTRY_C(0xDC080000, NULL, "vpfxs       X, X, X, -X"),
    TEST_ENTRY_C(0xDC080040, NULL, "vpfxs       X, X, X, -Y"),
    TEST_ENTRY_C(0xDC080080, NULL, "vpfxs       X, X, X, -Z"),
    TEST_ENTRY_C(0xDC0800C0, NULL, "vpfxs       X, X, X, -W"),
    TEST_ENTRY_C(0xDC080800, NULL, "vpfxs       X, X, X, -|X|"),
    TEST_ENTRY_C(0xDC080840, NULL, "vpfxs       X, X, X, -|Y|"),
    TEST_ENTRY_C(0xDC080880, NULL, "vpfxs       X, X, X, -|Z|"),
    TEST_ENTRY_C(0xDC0808C0, NULL, "vpfxs       X, X, X, -|W|"),
    TEST_ENTRY_C(0xDC088000, NULL, "vpfxs       X, X, X, -0"),
    TEST_ENTRY_C(0xDC088040, NULL, "vpfxs       X, X, X, -1"),
    TEST_ENTRY_C(0xDC088080, NULL, "vpfxs       X, X, X, -2"),
    TEST_ENTRY_C(0xDC0880C0, NULL, "vpfxs       X, X, X, -1/2"),
    TEST_ENTRY_C(0xDC088800, NULL, "vpfxs       X, X, X, -3"),
    TEST_ENTRY_C(0xDC088840, NULL, "vpfxs       X, X, X, -1/3"),
    TEST_ENTRY_C(0xDC088880, NULL, "vpfxs       X, X, X, -1/4"),
    TEST_ENTRY_C(0xDC0888C0, NULL, "vpfxs       X, X, X, -1/6"),

    /* vpfxt */
    TEST_ENTRY_C(0xDD000001, NULL, "vpfxt       Y, X, X, X"),
    TEST_ENTRY_C(0xDD000002, NULL, "vpfxt       Z, X, X, X"),
    TEST_ENTRY_C(0xDD000003, NULL, "vpfxt       W, X, X, X"),
    TEST_ENTRY_C(0xDD000100, NULL, "vpfxt       |X|, X, X, X"),
    TEST_ENTRY_C(0xDD000101, NULL, "vpfxt       |Y|, X, X, X"),
    TEST_ENTRY_C(0xDD000102, NULL, "vpfxt       |Z|, X, X, X"),
    TEST_ENTRY_C(0xDD000103, NULL, "vpfxt       |W|, X, X, X"),
    TEST_ENTRY_C(0xDD001000, NULL, "vpfxt       0, X, X, X"),
    TEST_ENTRY_C(0xDD001001, NULL, "vpfxt       1, X, X, X"),
    TEST_ENTRY_C(0xDD001002, NULL, "vpfxt       2, X, X, X"),
    TEST_ENTRY_C(0xDD001003, NULL, "vpfxt       1/2, X, X, X"),
    TEST_ENTRY_C(0xDD001100, NULL, "vpfxt       3, X, X, X"),
    TEST_ENTRY_C(0xDD001101, NULL, "vpfxt       1/3, X, X, X"),
    TEST_ENTRY_C(0xDD001102, NULL, "vpfxt       1/4, X, X, X"),
    TEST_ENTRY_C(0xDD001103, NULL, "vpfxt       1/6, X, X, X"),
    TEST_ENTRY_C(0xDD010000, NULL, "vpfxt       -X, X, X, X"),
    TEST_ENTRY_C(0xDD010001, NULL, "vpfxt       -Y, X, X, X"),
    TEST_ENTRY_C(0xDD010002, NULL, "vpfxt       -Z, X, X, X"),
    TEST_ENTRY_C(0xDD010003, NULL, "vpfxt       -W, X, X, X"),
    TEST_ENTRY_C(0xDD010100, NULL, "vpfxt       -|X|, X, X, X"),
    TEST_ENTRY_C(0xDD010101, NULL, "vpfxt       -|Y|, X, X, X"),
    TEST_ENTRY_C(0xDD010102, NULL, "vpfxt       -|Z|, X, X, X"),
    TEST_ENTRY_C(0xDD010103, NULL, "vpfxt       -|W|, X, X, X"),
    TEST_ENTRY_C(0xDD011000, NULL, "vpfxt       -0, X, X, X"),
    TEST_ENTRY_C(0xDD011001, NULL, "vpfxt       -1, X, X, X"),
    TEST_ENTRY_C(0xDD011002, NULL, "vpfxt       -2, X, X, X"),
    TEST_ENTRY_C(0xDD011003, NULL, "vpfxt       -1/2, X, X, X"),
    TEST_ENTRY_C(0xDD011100, NULL, "vpfxt       -3, X, X, X"),
    TEST_ENTRY_C(0xDD011101, NULL, "vpfxt       -1/3, X, X, X"),
    TEST_ENTRY_C(0xDD011102, NULL, "vpfxt       -1/4, X, X, X"),
    TEST_ENTRY_C(0xDD011103, NULL, "vpfxt       -1/6, X, X, X"),
    TEST_ENTRY_C(0xDD000004, NULL, "vpfxt       X, Y, X, X"),
    TEST_ENTRY_C(0xDD000008, NULL, "vpfxt       X, Z, X, X"),
    TEST_ENTRY_C(0xDD00000C, NULL, "vpfxt       X, W, X, X"),
    TEST_ENTRY_C(0xDD000200, NULL, "vpfxt       X, |X|, X, X"),
    TEST_ENTRY_C(0xDD000204, NULL, "vpfxt       X, |Y|, X, X"),
    TEST_ENTRY_C(0xDD000208, NULL, "vpfxt       X, |Z|, X, X"),
    TEST_ENTRY_C(0xDD00020C, NULL, "vpfxt       X, |W|, X, X"),
    TEST_ENTRY_C(0xDD002000, NULL, "vpfxt       X, 0, X, X"),
    TEST_ENTRY_C(0xDD002004, NULL, "vpfxt       X, 1, X, X"),
    TEST_ENTRY_C(0xDD002008, NULL, "vpfxt       X, 2, X, X"),
    TEST_ENTRY_C(0xDD00200C, NULL, "vpfxt       X, 1/2, X, X"),
    TEST_ENTRY_C(0xDD002200, NULL, "vpfxt       X, 3, X, X"),
    TEST_ENTRY_C(0xDD002204, NULL, "vpfxt       X, 1/3, X, X"),
    TEST_ENTRY_C(0xDD002208, NULL, "vpfxt       X, 1/4, X, X"),
    TEST_ENTRY_C(0xDD00220C, NULL, "vpfxt       X, 1/6, X, X"),
    TEST_ENTRY_C(0xDD020000, NULL, "vpfxt       X, -X, X, X"),
    TEST_ENTRY_C(0xDD020004, NULL, "vpfxt       X, -Y, X, X"),
    TEST_ENTRY_C(0xDD020008, NULL, "vpfxt       X, -Z, X, X"),
    TEST_ENTRY_C(0xDD02000C, NULL, "vpfxt       X, -W, X, X"),
    TEST_ENTRY_C(0xDD020200, NULL, "vpfxt       X, -|X|, X, X"),
    TEST_ENTRY_C(0xDD020204, NULL, "vpfxt       X, -|Y|, X, X"),
    TEST_ENTRY_C(0xDD020208, NULL, "vpfxt       X, -|Z|, X, X"),
    TEST_ENTRY_C(0xDD02020C, NULL, "vpfxt       X, -|W|, X, X"),
    TEST_ENTRY_C(0xDD022000, NULL, "vpfxt       X, -0, X, X"),
    TEST_ENTRY_C(0xDD022004, NULL, "vpfxt       X, -1, X, X"),
    TEST_ENTRY_C(0xDD022008, NULL, "vpfxt       X, -2, X, X"),
    TEST_ENTRY_C(0xDD02200C, NULL, "vpfxt       X, -1/2, X, X"),
    TEST_ENTRY_C(0xDD022200, NULL, "vpfxt       X, -3, X, X"),
    TEST_ENTRY_C(0xDD022204, NULL, "vpfxt       X, -1/3, X, X"),
    TEST_ENTRY_C(0xDD022208, NULL, "vpfxt       X, -1/4, X, X"),
    TEST_ENTRY_C(0xDD02220C, NULL, "vpfxt       X, -1/6, X, X"),
    TEST_ENTRY_C(0xDD000010, NULL, "vpfxt       X, X, Y, X"),
    TEST_ENTRY_C(0xDD000020, NULL, "vpfxt       X, X, Z, X"),
    TEST_ENTRY_C(0xDD000030, NULL, "vpfxt       X, X, W, X"),
    TEST_ENTRY_C(0xDD000400, NULL, "vpfxt       X, X, |X|, X"),
    TEST_ENTRY_C(0xDD000410, NULL, "vpfxt       X, X, |Y|, X"),
    TEST_ENTRY_C(0xDD000420, NULL, "vpfxt       X, X, |Z|, X"),
    TEST_ENTRY_C(0xDD000430, NULL, "vpfxt       X, X, |W|, X"),
    TEST_ENTRY_C(0xDD004000, NULL, "vpfxt       X, X, 0, X"),
    TEST_ENTRY_C(0xDD004010, NULL, "vpfxt       X, X, 1, X"),
    TEST_ENTRY_C(0xDD004020, NULL, "vpfxt       X, X, 2, X"),
    TEST_ENTRY_C(0xDD004030, NULL, "vpfxt       X, X, 1/2, X"),
    TEST_ENTRY_C(0xDD004400, NULL, "vpfxt       X, X, 3, X"),
    TEST_ENTRY_C(0xDD004410, NULL, "vpfxt       X, X, 1/3, X"),
    TEST_ENTRY_C(0xDD004420, NULL, "vpfxt       X, X, 1/4, X"),
    TEST_ENTRY_C(0xDD004430, NULL, "vpfxt       X, X, 1/6, X"),
    TEST_ENTRY_C(0xDD040000, NULL, "vpfxt       X, X, -X, X"),
    TEST_ENTRY_C(0xDD040010, NULL, "vpfxt       X, X, -Y, X"),
    TEST_ENTRY_C(0xDD040020, NULL, "vpfxt       X, X, -Z, X"),
    TEST_ENTRY_C(0xDD040030, NULL, "vpfxt       X, X, -W, X"),
    TEST_ENTRY_C(0xDD040400, NULL, "vpfxt       X, X, -|X|, X"),
    TEST_ENTRY_C(0xDD040410, NULL, "vpfxt       X, X, -|Y|, X"),
    TEST_ENTRY_C(0xDD040420, NULL, "vpfxt       X, X, -|Z|, X"),
    TEST_ENTRY_C(0xDD040430, NULL, "vpfxt       X, X, -|W|, X"),
    TEST_ENTRY_C(0xDD044000, NULL, "vpfxt       X, X, -0, X"),
    TEST_ENTRY_C(0xDD044010, NULL, "vpfxt       X, X, -1, X"),
    TEST_ENTRY_C(0xDD044020, NULL, "vpfxt       X, X, -2, X"),
    TEST_ENTRY_C(0xDD044030, NULL, "vpfxt       X, X, -1/2, X"),
    TEST_ENTRY_C(0xDD044400, NULL, "vpfxt       X, X, -3, X"),
    TEST_ENTRY_C(0xDD044410, NULL, "vpfxt       X, X, -1/3, X"),
    TEST_ENTRY_C(0xDD044420, NULL, "vpfxt       X, X, -1/4, X"),
    TEST_ENTRY_C(0xDD044430, NULL, "vpfxt       X, X, -1/6, X"),
    TEST_ENTRY_C(0xDD000040, NULL, "vpfxt       X, X, X, Y"),
    TEST_ENTRY_C(0xDD000080, NULL, "vpfxt       X, X, X, Z"),
    TEST_ENTRY_C(0xDD0000C0, NULL, "vpfxt       X, X, X, W"),
    TEST_ENTRY_C(0xDD000800, NULL, "vpfxt       X, X, X, |X|"),
    TEST_ENTRY_C(0xDD000840, NULL, "vpfxt       X, X, X, |Y|"),
    TEST_ENTRY_C(0xDD000880, NULL, "vpfxt       X, X, X, |Z|"),
    TEST_ENTRY_C(0xDD0008C0, NULL, "vpfxt       X, X, X, |W|"),
    TEST_ENTRY_C(0xDD008000, NULL, "vpfxt       X, X, X, 0"),
    TEST_ENTRY_C(0xDD008040, NULL, "vpfxt       X, X, X, 1"),
    TEST_ENTRY_C(0xDD008080, NULL, "vpfxt       X, X, X, 2"),
    TEST_ENTRY_C(0xDD0080C0, NULL, "vpfxt       X, X, X, 1/2"),
    TEST_ENTRY_C(0xDD008800, NULL, "vpfxt       X, X, X, 3"),
    TEST_ENTRY_C(0xDD008840, NULL, "vpfxt       X, X, X, 1/3"),
    TEST_ENTRY_C(0xDD008880, NULL, "vpfxt       X, X, X, 1/4"),
    TEST_ENTRY_C(0xDD0088C0, NULL, "vpfxt       X, X, X, 1/6"),
    TEST_ENTRY_C(0xDD080000, NULL, "vpfxt       X, X, X, -X"),
    TEST_ENTRY_C(0xDD080040, NULL, "vpfxt       X, X, X, -Y"),
    TEST_ENTRY_C(0xDD080080, NULL, "vpfxt       X, X, X, -Z"),
    TEST_ENTRY_C(0xDD0800C0, NULL, "vpfxt       X, X, X, -W"),
    TEST_ENTRY_C(0xDD080800, NULL, "vpfxt       X, X, X, -|X|"),
    TEST_ENTRY_C(0xDD080840, NULL, "vpfxt       X, X, X, -|Y|"),
    TEST_ENTRY_C(0xDD080880, NULL, "vpfxt       X, X, X, -|Z|"),
    TEST_ENTRY_C(0xDD0808C0, NULL, "vpfxt       X, X, X, -|W|"),
    TEST_ENTRY_C(0xDD088000, NULL, "vpfxt       X, X, X, -0"),
    TEST_ENTRY_C(0xDD088040, NULL, "vpfxt       X, X, X, -1"),
    TEST_ENTRY_C(0xDD088080, NULL, "vpfxt       X, X, X, -2"),
    TEST_ENTRY_C(0xDD0880C0, NULL, "vpfxt       X, X, X, -1/2"),
    TEST_ENTRY_C(0xDD088800, NULL, "vpfxt       X, X, X, -3"),
    TEST_ENTRY_C(0xDD088840, NULL, "vpfxt       X, X, X, -1/3"),
    TEST_ENTRY_C(0xDD088880, NULL, "vpfxt       X, X, X, -1/4"),
    TEST_ENTRY_C(0xDD0888C0, NULL, "vpfxt       X, X, X, -1/6"),

    /* vpfxd */
    TEST_ENTRY_C(0xDE000001, NULL, "vpfxd       0, , , "),
    TEST_ENTRY_C(0xDE000003, NULL, "vpfxd       1, , , "),
    TEST_ENTRY_C(0xDE000100, NULL, "vpfxd       M, , , "),
    TEST_ENTRY_C(0xDE000004, NULL, "vpfxd       , 0, , "),
    TEST_ENTRY_C(0xDE00000C, NULL, "vpfxd       , 1, , "),
    TEST_ENTRY_C(0xDE000200, NULL, "vpfxd       , M, , "),
    TEST_ENTRY_C(0xDE000010, NULL, "vpfxd       , , 0, "),
    TEST_ENTRY_C(0xDE000030, NULL, "vpfxd       , , 1, "),
    TEST_ENTRY_C(0xDE000400, NULL, "vpfxd       , , M, "),
    TEST_ENTRY_C(0xDE000040, NULL, "vpfxd       , , , 0"),
    TEST_ENTRY_C(0xDE0000C0, NULL, "vpfxd       , , , 1"),
    TEST_ENTRY_C(0xDE000800, NULL, "vpfxd       , , , M"),
};

size_t test_entries_len = ARRAY_COUNT(test_entries);
